#!/bin/bash

if [ $# = 0 ] ; then
    echo -e "Please enter the name of a PostScript file."
    exit 1
fi

NAME=$1
RES=${2:-72}
OPTIONS=''
if [ `basename $0 .bash` = 'myps2pdf' ] ; then
    DEV=pdfwrite
    SUF=pdf
    RES=72
    OPTIONS=' -dSAFER -c .setpdfwrite '
elif [ `basename $0 .bash` = 'myps2jpg' ] ; then
    DEV=jpeg
    SUF=jpg
elif [ `basename $0 .bash` = 'myps2png' ] ; then
    DEV=png256
    SUF=png
else
    echo -e "This script must be called as myps2pdf.bash, myps2jpg.bash or as myps2png.bash."
    exit 2
fi

# basename won't work when .ps or .eps files are not in $PWD
#PREFIX1=`basename $NAME .ps`
#PREFIX2=`basename $NAME .eps`
PREFIX1=`echo "$NAME"|sed -e 's/\.ps$//'`
PREFIX2=`echo "$NAME"|sed -e 's/\.eps$//'`

if [ -f $NAME ] ; then
    INFILE=$NAME
    if [ "$NAME" = "${PREFIX1}.ps" ] ; then
	OUTFILE=${PREFIX1}.$SUF
    elif [ "$NAME" = "${PREFIX2}.eps" ] ; then
	OUTFILE=${PREFIX2}.$SUF
    else
	OUTFILE=${NAME}.$SUF
    fi
elif [ -f "${PREFIX1}.ps" ] ; then
    INFILE=${PREFIX1}.ps
    OUTFILE=${PREFIX1}.$SUF
elif [ -f "${PREFIX1}.eps" ] ; then
    INFILE=${PREFIX1}.eps
    OUTFILE=${PREFIX1}.$SUF
elif [ -f "${PREFIX2}.ps" ] ; then
    INFILE=${PREFIX2}.ps
    OUTFILE=${PREFIX2}.$SUF
elif [ -f "${PREFIX2}.eps" ] ; then
    INFILE=${PREFIX2}.eps
    OUTFILE=${PREFIX2}.$SUF
else
    echo "Error!\tNeither $NAME, ${NAME}.ps nor ${NAME}.eps exists."
    exit 3
fi

# Look for bounding box to define X, Y, RESX and RESY
BBOX=`grep '%%BoundingBox: ' $INFILE|head -n1`
BBOX=${BBOX:-'%%BoundingBox: 0 0 612 792'}
X=`echo $RES $BBOX|awk '{printf "%d",$1*$5/72.0}'`
Y=`echo $RES $BBOX|awk '{printf "%d",$1*$6/72.0}'`

echo "Creating $OUTFILE from $INFILE; size ${X}x${Y} and RES $RES."

gs  -dNOPAUSE -dBATCH -q  -g${X}x${Y}   -r$RES -sDEVICE=$DEV  \
 -sOutputFile=$OUTFILE $OPTIONS -f $INFILE
