#!/bin/bash

if [ $# -eq 1 ] ; then
    if [ $1 = 'clean' ] ; then
	rm -vf test-*
	exit 0
    fi
fi

# Test non-interactive programs.

BOX=../../src/boxplot_ng
CT_BOX=../../src/ct_boxplot_ng
OVR_BOX=../../src/overlay_boxplot_ng
SIR_G=../../src/sir_graph

# Test probability modes.

$BOX -pg -jpg -p -c 8 -i 0.002 -o test-boxplot_ng-pf mdv1
$BOX -g -p -c 8 -i 0.002 -o test-boxplot_ng-pf mdv1
$BOX -png -p -c 8 -i 0.002 -o test-boxplot_ng-pf mdv1

$SIR_G -p -ar -fix -prob -lab 10 -n -reg 30 -f -o test-sir_graph-pf mdv1_mfe
$SIR_G -jpg -ar -fix -prob -lab 10 -n -reg 30 -f -o test-sir_graph-pf mdv1_mfe
$SIR_G -ar -fix -prob -lab 10 -n -reg 30 -f -o test-sir_graph-pf -png mdv1_mfe

$OVR_BOX -pg -o test-overlay_boxplot_ng mdv1 mdv1_mfe

# Test mfold type modes

cd mfold

../$BOX -pg -jpg -c 8 -o test-boxplot_ng-mfe mdv1
../$BOX -g -c 8 -o test-boxplot_ng-mfe mdv1
../$BOX -png -c 8 -o test-boxplot_ng-mfe mdv1

../$OVR_BOX -pg -o test-overlay_boxplot_ng-mfe mdv1 mdv1_4

../$CT_BOX -pg -jpg -m 1.5 -o test-ct_boxplot_ng mdv1_1 mdv1_2 mdv1_3 mdv1_4 mdv1_5
../$CT_BOX -g -m 1.5 -o test-ct_boxplot_ng mdv1_1 mdv1_2 mdv1_3 mdv1_4 mdv1_5 mdv1_6
../$CT_BOX -png -m 1.5 -o test-ct_boxplot_ng mdv1_1 mdv1_2 mdv1_3 mdv1_4 mdv1_5 mdv1_6 

../$CT_BOX -pg -jpg  -o test-ct_boxplot_ng_z -z 200 218 4 22 mdv1_1 mdv1_2 mdv1_3 mdv1_4 mdv1_5 mdv1_6
../$CT_BOX -g -o test-ct_boxplot_ng_z -z 200 218 4 22 mdv1_1 mdv1_2 mdv1_3 mdv1_4 mdv1_5 mdv1_6
../$CT_BOX -png  -o test-ct_boxplot_ng_z -z 200 218 4 22 mdv1_1 mdv1_2 mdv1_3 mdv1_4 mdv1_5 mdv1_6

mv test-ct_boxplot_ng* ../
mv test-overlay_boxplot_ng* ../
mv test-boxplot_ng* ../
cd ../

for i in test*.ps ; do
    epstopdf $i
done

if [ $# -gt 0 ] ; then
    if [ $1 = 'pdf' -o $1 = 'PDF' ] ; then
	for i in test*.pdf ; do
	    echo $i
	    acroread $i
	done
    elif [ $1 = 'gif' ] ; then
	for i in test*.gif ; do
	    echo $i
	    xv $i
	done
    elif [ $1 = 'jpg' -o $1 = 'jpeg' ] ; then
	for i in test*.jpg ; do
	    echo $i
	    xv $i
	done
    fi
fi

