#define PROGRAM_NAME "boxplot"

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <limits.h>
#include "util.h"

/* On the Mac, libglut does not exist separately. Search instead for
   the function glutInit. Under Linux, glut.h should be in
   /usr/include/GL and /usr/include is automatically searched.
*/

#ifdef HAVE_GLUTBITMAPCHARACTER
void zoomSetWindow(void);
#if HAVE_LIBGLUT
# include <GL/glut.h>
#else
# include <glut.h>
#endif

#define MAXIMUM_SIZE 30000	/* maximum sequence length */
#define TRUE    1
#define FALSE   0
#define MAIN_POST    1		/* create postcript for main or zoom */
#define ZOOM_POST    2		/* used in general_post, create_post */

#define MESSAGE_NONE   0	/* values for g_type_message */
#define MESSAGE_ENERGY 2
#define MESSAGE_POST   3
#define MESSAGE_GIF    4
#define MESSAGE_PNG    5
#define MESSAGE_JPG    6

int             g_helix_array_size;

struct helix {
	int             row;
	int             column;
	int             diagonal;
	int             color;
	int             energy;
	int             length;
};

struct helix   *g_diag;

/* New stuff added by M. Zuker on Dec. 29, 2006 */
char            g_title[150];
int             g_title_set = FALSE;

int             g_adjust_col_flag; /* for 2 strands, subtract amount
				    * from col to make it look better
				    */ 
int             g_adjust_amount;	

int             g_post_adjust;	/* When true magnify dots in
				   postscript output according to
				   values in boxplot.col */ 
int             g_dot_adjust;
int             g_length;	/* len of RNA sequence, guessed at based on
				 * input file */
int             width;		/* these are the current width and height of
				 * the main window in pixels */
int             height;		
int             width_z, height_z;	/* same for zoom window */

int             first_zoom = 0;	/* used for backing up to previous zoom
				 * window */
/* when right mouse button is clicked, transform coordinates into i,j */
int             g_i_row = 0;
int             g_j_col = 0;	/* for i.j base pair with mouse */
char            g_i_row_s[10], g_j_col_s[10];	/* string of above */

int  win_main, win_zoom, win_text; /* window indentifiers for glut */
int             g_default_color = 0;

/* keep track of the current color for text and
 * borders $$$$$$$$ used in fixcolor()
 * this is now a constant 1 is black, 2 is red
 * 3 is , see fixcolor_plot
 * ______________________________________________________________
 * Notice that most functions are called based on events in the
 * event queue.  They do not call each other.  This results in too
 * many global variables.
 * Variables that could easily be changed for personal preference are
 * usually accompanied by $$$.  Colors in particular
 * __________________________________________________________________ */

/* text window variables _________________________________________ */
int             g_message_set = FALSE;	/* should text display window
					 * show text */
/* window message?
 * It indicates to enter postscript filename or energy increment
 * values below for text_window_message based on g_message_set */
char           *increment_message_energy =
  "How many kcal/mol for the energy cutoff? (/ to abort) ";
char           *increment_message_prob =
  "What is the absolute cutoff? (0.0 to ";
char           *post_message = "Enter name for postscript file (/ to abort) ";
char           *gif_message = "Enter name for gif file (/ to abort) ";
char           *png_message = "Enter name for png file (/ to abort) ";
char           *jpg_message = "Enter name for jpg file (/ to abort) ";

int             g_which_window_i_am_in;

char            text_window_message[80];
char            text_window_input[80];
int             data_entered = FALSE;	/* for text display window */
int             g_type_message;	/* 2 for energy increment
					 * 0 for none */
/* Font variables */
void           *g_font_text = GLUT_BITMAP_TIMES_ROMAN_24; /* initialize font */
void           *g_font_zoom = GLUT_BITMAP_HELVETICA_18;
void           *g_font_main = GLUT_BITMAP_HELVETICA_18;
void           *fonts[] = { /* used by select_font */
  GLUT_BITMAP_HELVETICA_12,
  GLUT_BITMAP_HELVETICA_18,
  GLUT_BITMAP_TIMES_ROMAN_24
};

/* int partial_length,partial_start,partial_end; start length and end
 * for zoom 
 */ 

/* Graphic  variables */
float           g_color_increment;	/* range of energy for each color */
int             g_energy_cutoff;	/* only probabilities <= */
/* g_energy_cutoff will be shown. Note that probabilities have a
 * negative sign. -30 indicates the probabilities >=.3 will be shown 
 */
int  g_chain_len = 1;	/* chains smaller will not be displayed */
int points_plotted, points_plotted_zoom; /* indicates number of points
					  * plotted in each window
					  */ 
/* Zoom Variables */
int g_region_set; /* 0 indicates the top left corner of the zoom
		     region has not been set */ 
int region_t, region_b, region_r, region_l; /* specify boundaries of
					     * the zoom rectangle in
					     * i,j coordinates for
					     * main window or for the
					     * zoom window 
					     */ 
float    region_real_l, region_real_r, region_real_b, region_real_t;
/* screen coordinates of above (i,j)
 * pairs  for the main window  (blue rectangle) */
float    region_real_l_zoom, region_real_r_zoom;
float    region_real_t_zoom, region_real_b_zoom;
/* Screen coordinates  for a zoom rectangle within the zoom window */
int  g_region_set;	/* nonzero indicates the top left
			 * corner of zoom region has been set */
int  g_region_exists;

int  display_l, display_r, display_t, display_b, display_w, display_h;
/* left,right,top,bottom,width and height for zoom window display
 * They are in nucleotide coordinates */
int  saved_display_l, saved_display_r, saved_display_t, saved_display_b;
/* Above used to go back to previous zoom window */

int g_zoom_labels_count;	/* the number of labels used */
int g_zoom_labels_x[50];	/* x coordinate for label i */
int g_zoom_labels_xl[50];	/* used to plot on screen the dot */
int g_zoom_labels_y[50];
int g_zoom_labels_row[50];
int g_zoom_labels_col[50];

char *g_zoom_labels_string[50];	/* value to place at x,y */

/* File Variables _______________________________________________ */
char            rec[90];
FILE           *fp;		/* for .plot file */

char            g_post_filename[120];
char            g_plot_filename[120];
char            g_mplot_filename[120];
char            g_ct_filename[120];
char            g_boxplot_rootname[120];
char            g_sel_filename[120];
char            g_boxseq_filename[120];
char            g_boxct_filename[120];
char            g_seq_filename[120];
char            g_gif_filename[120]; /* depends on -o or input file */
char            g_png_filename[120]; /* depends on -o or input file */
char            g_jpg_filename[120]; /* depends on -o or input file */
int             g_mi_flag;	/* for Mutual Information, makes output look
				 * better */
/* g_mi_flag always sets g_prob flag too */
int             g_prob;	/* set by -p flag */
/* Probabilities are multipled by -1,000,000 and stored as integers.
   Energies are multiplied by 10 and stored as integers */
int             g_opt_prob_flag; /* set to true when some probability
				    of plot file is > 2.0001 */
/* indicates to draw optimal structure for probability in lower triangle */
int             g_grid_flag; /* True for use grid lines False for do not */
int             g_zoom_window_exists = FALSE;
int             g_zoom_window_displayed = FALSE;
int             g_optimal_energy;
int             g_worst_energy;
int             g_post_file;	/* 1 for main, 2 for zoom */

int            *g_diag_count;
int            *g_diag_start;
int             g_helices_in_plot_file;	/* number of entries in file */

int             g_resolution;	/* 72 110 200 or 300 */
int             g_motion_occurred; /* For mouse: determines when to
				    * draw a box or identify a
				    * base pair */

#include "boxplot_setcolor.h"
#include "boxplot_input.h"
#include "boxplot_ps.h"
#if (defined(HAVE_LIBGD) || defined(HAVE_GDIMAGECREATE)) && defined(HAVE_GD_H)
# include "boxplot_img.h"
#endif

/* Initialization Routines for nongraphics */
void bad_arguments(void) {
  printf("Error!\tUsage is:\n");
  printf("boxplot [OPTIONS] name[.plot]\nOptions are\n");
  printf(" -mi  Treat plot file as Mutual Information\n");
  printf(" -p   Treat plot file as probability, default is energy.\n");
  printf("This program requires %s.col, and will look in\n", PROGRAM_NAME);
  printf("the current working directory first.\n");
  try_exit(1);
}

void check_parameters(int argc, char **argv) {
    int             i;
    int             input_filename_defined;
    int             output_filename_defined;
    g_adjust_col_flag = FALSE;
    g_adjust_amount = 0;
    g_prob = FALSE;
    g_mi_flag = FALSE;
    input_filename_defined = FALSE;
    output_filename_defined = FALSE;
    if (argc < 2)
      bad_arguments();
    strcpy(g_plot_filename, argv[argc - 1]);	/* set output file */
    argc = argc - 1;	/* last 1 is input file and has been used */
    i = 1;
  while (i <= (argc - 1)) {
    if(strcmp(argv[i], "-p")==0) {
      g_prob = TRUE;
      i++;
    } else if(strcmp(argv[i], "-mi")==0) {
      g_mi_flag = TRUE;	/* Treat input data as mutual information */
      g_prob = TRUE;	/* treat as probability too */
      i++;
    } else if(strcmp(argv[i], "-t")==0) {
      strcpy(g_title, argv[i + 1]); /* Title must follow */
      g_title[strlen(g_title)] = '\0';
      g_title_set = TRUE;
      i += 2;
    } else if(strcmp(argv[i], "-c")==0) {
      g_number_of_colors = atoi(argv[i + 1]);
      if (g_number_of_colors > 8)
	g_number_of_colors = 8;
      else if(g_number_of_colors < 2)
	g_number_of_colors = 4;
      g_num_color_set=TRUE;
      i += 2;   
    } else {
      printf("Warning!\tFlag %s was not used.\n", argv[i]);
      i++;
    }
  }
  chop_suffix(g_plot_filename, ".plot"); 
  strcpy(g_ct_filename, g_plot_filename);
  strcat(g_ct_filename, ".ct");
  strcpy(g_boxplot_rootname, g_plot_filename);
  strcat(g_boxplot_rootname, "_"PROGRAM_NAME);
  strcpy(g_boxseq_filename, g_boxplot_rootname);
  strcat(g_boxseq_filename, ".seq");
  strcpy(g_sel_filename, g_boxplot_rootname);
  strcat(g_sel_filename, ".sel");
  strcpy(g_boxct_filename, g_boxplot_rootname);
  strcat(g_boxct_filename, ".ct");
  strcpy(g_seq_filename, g_plot_filename);
  strcat(g_seq_filename, ".seq");
  strcpy(g_mplot_filename, g_plot_filename);
  strcat(g_mplot_filename, ".sav");
  if (!output_filename_defined) {
    strcpy(g_post_filename, g_plot_filename);
    strcpy(g_gif_filename, g_plot_filename);
    strcpy(g_png_filename, g_plot_filename);
    strcpy(g_jpg_filename, g_plot_filename);
  }
  /* fix suffixes of file names */
  strcat(g_plot_filename, ".plot");	/* put .plot on */
  chop_suffix(g_post_filename, ".ps");
  strcat(g_post_filename, ".ps");
  chop_suffix(g_gif_filename, ".img");
  strcat(g_gif_filename, ".gif");
  chop_suffix(g_png_filename, ".img");
  strcat(g_png_filename, ".png");
  chop_suffix(g_jpg_filename, ".img");
  strcat(g_jpg_filename, ".jpg");
}

  void initialize_data2(void) {
    g_chain_len = 1;	/* skip chains smaller than this */
    if(!g_num_color_set)
      g_number_of_colors = 4;	/* start with 4 colors $$$$$ */
    g_grid_flag = TRUE;
    width = 800;		/* initialize size of window in pixels */
    height = 800;		/* needed for scale in drawing windows */
    g_energy_cutoff = g_worst_energy;
    if (g_prob != TRUE)
      g_color_increment = ((float) (g_energy_cutoff -	 g_optimal_energy))
	/ (float) (g_number_of_colors - 1);
    initialize_colors();
    g_region_set = FALSE;
    g_region_exists = FALSE;
    g_message_set = FALSE;
    text_window_input[0] = '\0';
    text_window_message[0] = '\0';
    g_type_message = MESSAGE_NONE;
  }

void initialize_data1(int argc, char **argv) {
  int             i;
  g_post_adjust = FALSE;
  g_dot_adjust = FALSE;
  g_zoom_labels_count = 0;	/* initialize to no labels */
  for (i = 0; i < 50; i++) {	/* allocate memory for string */
    g_zoom_labels_string[i] = (char *) malloc(15 * sizeof(char));
    if (g_zoom_labels_string[i] == NULL) {
      printf("Error!\tInsufficient memory for zoom labels\n");
      try_exit(2);
    }
  }
  open_file(argc, argv);
  initialize_len(g_adjust_col_flag, g_adjust_amount);
}

/* Menu routines
 * Most are defined throught GLUT and called from the event queue
 */
void keyboard(unsigned char key, int x, int y) {
  int             string_len1;	/* accepts keys from text window */
  int             string_len2;
  x = y;
  y = x;			/* get rid of warning message */
  if (key == 13) {
    data_entered = TRUE;	/* on <RETURN> mark end of text */
    glutSetWindow(win_text);
    glutPostRedisplay();
    return;
  }
  string_len1 = strlen(text_window_input);
  string_len2 = strlen(text_window_message);
  if (string_len1 > 0)
    if (key == 8) {	/* process backspace or delete */
      text_window_input[string_len1 - 1] = '\0';
      text_window_message[string_len2 - 1] = '\0';
      glutSetWindow(win_text);
      glutPostRedisplay();
      return;
    }
  text_window_input[string_len1] = key;	/* add key to end of input */
  text_window_input[string_len1 + 1] = '\0';
  text_window_message[string_len2] = key;	/* add key to end of message */
  text_window_message[string_len2 + 1] = '\0';
  glutSetWindow(win_text);	/* show the text */
  glutPostRedisplay();
}

void fixcolor_plot(int color) {
  /* for the screen */
  if (color < 0)
    return;
  if (g_number_of_colors >= 7) {
    if ((color >= 1) && (color <= LAST_PROB_COLOR)) {
      glColor3f(g_prob_color[color][0], g_prob_color[color][1],
		g_prob_color[color][2]);
      return;
    }
  }
  if (color < TOTAL_COLORS)
    glColor3f(g_color[color][0], g_color[color][1],
	      g_color[color][2]);
}

void fixcolor_plot_optimal(void) {
  glColor3f(g_color[COLOR_OPTIMAL][0],
	    g_color[COLOR_OPTIMAL][1],
	    g_color[COLOR_OPTIMAL][2]);
}

 void select_font(int newfont) {
   g_font_main = fonts[newfont];
   g_font_zoom = fonts[newfont];
   g_font_text = fonts[newfont];
   glutSetWindow(win_text);
   glutPostRedisplay();
   if (g_zoom_window_displayed==TRUE) {
     zoomSetWindow();
     glutPostRedisplay();
   }
   glutSetWindow(win_main);
   glutPostRedisplay();
 }
 
 void select_font_main(int newfont) {
   g_font_main = fonts[newfont];
   glutPostRedisplay();
 }

void select_font_zoom(int newfont) {
  g_font_zoom = fonts[newfont];
  glutPostRedisplay();
}

void select_font_text(int newfont) {
  g_font_text = fonts[newfont];
  glutPostRedisplay();
}

void select_colors(int color) {
  g_number_of_colors = color;
  if (g_prob != TRUE)
    g_color_increment = 
      ((float) (g_energy_cutoff - g_optimal_energy)) / 
      (float) (g_number_of_colors - 1); 
  fix_colors();
  glutSetWindow(win_main);
  glutPostRedisplay();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutPostRedisplay();
  }
  glutSetWindow(win_text);
  glutPostRedisplay();
}

void fix_cursor(void) {
  /* set cursor back to crosshairs */
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutPostRedisplay();
}

void set_cursor_help(void) {
  glutSetWindow(win_main);
  glutSetCursor(GLUT_CURSOR_HELP);
  glutPostRedisplay();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutSetCursor(GLUT_CURSOR_HELP);
    glutPostRedisplay();
  }
}

void finish_making_post(void) {
  int             error;
  char            title[150];
  if (text_window_input[0] == '/') {
    text_window_input[0] = '\0';
    text_window_message[0] = '\0';
    glutSetWindow(win_text);
    glutPostRedisplay();
    fix_cursor();
    return;
  }
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutSetCursor(GLUT_CURSOR_WAIT);
  }
  glutSetWindow(win_main);
  glutSetCursor(GLUT_CURSOR_WAIT);
  glutPostRedisplay();
  if (strlen(text_window_input) > 0) {
    strcpy(g_post_filename, text_window_input);
    chop_suffix(g_post_filename, ".ps");
    strcat(g_post_filename, ".ps");
  }
  text_window_input[0] = '\0';
  text_window_message[0] = '\0';
  printf("The name for the postscript file is %s\n", g_post_filename);
  error = open_post(g_post_filename);
  if (error) {
    printf("Error creating postscript file\n");
    fix_cursor();
    return;
  }
  if (g_prob) {
    if (g_mi_flag)
      strcpy(title, "Mutual Information Dot Plot for ");
    else
      strcpy(title, "Probability Dot Plot for ");
  } else {
    strcpy(title, "Energy Dot Plot for ");
  }
  if(!g_title_set)
    strcat(title, g_plot_filename);
  else
    strcat(title, g_title);
  general_post(title, g_grid_flag, FALSE, 0, 0, g_post_adjust, 
	       g_zoom_labels_count, g_zoom_labels_row, g_zoom_labels_col);
  fix_cursor();
  glutSetWindow(win_text);
  glutPostRedisplay();
}
 
void finish_making_img_text(int message) {
  char            img_name[120];
  char            title[150];
  int             png_mode, jpg_mode;
  /* use 8.5 x 11 in for image  */
  png_mode = FALSE;
  jpg_mode = FALSE;
  if (text_window_input[0] == '/') { /* abort */
    text_window_input[0] = '\0';
    text_window_message[0] = '\0';
    glutSetWindow(win_text);
    glutPostRedisplay();
    fix_cursor();
    return;
  }
  if (strlen(text_window_input) == 0) {
    if (message == MESSAGE_GIF) {
      strcpy(text_window_input, g_gif_filename);
    } else if (message == MESSAGE_PNG) {
      strcpy(text_window_input, g_png_filename);
    } else {
      strcpy(text_window_input, g_jpg_filename);
    }
  }
  strcpy(img_name, text_window_input);
  text_window_input[0] = '\0';
  text_window_message[0] = '\0';
  chop_suffix(img_name, ".img");
  if (message == MESSAGE_GIF) {
    strcat(img_name, ".gif");
    strcpy(g_gif_filename, img_name);
  } else if (message == MESSAGE_PNG) {
    png_mode = TRUE;
    strcat(img_name, ".png");
    strcpy(g_png_filename, img_name);
  } else {
    strcat(img_name, ".jpg");
    jpg_mode = TRUE;
    strcpy(g_jpg_filename, img_name);
  }
  /* correct message for text window */
  g_message_set = FALSE;
  text_window_message[0] = '\0';
  glutSetWindow(win_text);
  glutPostRedisplay();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutSetCursor(GLUT_CURSOR_WAIT);
  }
  glutSetWindow(win_main);
  glutSetCursor(GLUT_CURSOR_WAIT);
  glutPostRedisplay();
  text_window_input[0] = '\0';
  if (g_prob) {
    if (g_mi_flag)
      strcpy(title, "Mutual Information Dot Plot for ");
    else
      strcpy(title, "Probability Dot Plot for ");
  } else
    strcpy(title, "Energy Dot Plot for ");
  strcat(title, g_plot_filename);
#if (defined(HAVE_LIBGD) || defined(HAVE_GDIMAGECREATE)) && defined(HAVE_GD_H)
  finish_making_img(FALSE, g_resolution, img_name, title, FALSE, FALSE, 0, 0, 
		    g_grid_flag, png_mode, jpg_mode, g_mi_flag, 
		    g_opt_prob_flag, g_zoom_labels_count, g_zoom_labels_row, 
		    g_zoom_labels_col);
#endif
  fix_cursor();
}
 
void set_postscript1(int which_window) {
  if ((which_window == 1) || (which_window == 3))
    g_post_adjust = TRUE;
  else
    g_post_adjust = FALSE;
  if ((which_window < 2) || (g_zoom_window_displayed == FALSE))
    g_post_file = MAIN_POST;	/* create post for main */
  else
    g_post_file = ZOOM_POST;	/* create post for zoom */
  strcpy(text_window_message, post_message);
  strcat(text_window_message, ", default ");
  strcat(text_window_message, g_post_filename);
  strcat(text_window_message, ")? ");
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_POST;
  glutSetWindow(win_text);
  glutPostRedisplay();
  set_cursor_help();
}

void set_gif1(int choice) {
  if (choice == 1)
    g_resolution = 72;
  else if (choice == 2)
    g_resolution = 110;
  else if (choice == 3)
    g_resolution = 200;
  else
    g_resolution = 300;
  strcpy(text_window_message, gif_message);
  strcat(text_window_message, ", default ");
  strcat(text_window_message, g_gif_filename);
  strcat(text_window_message, ")? ");
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_GIF;
  glutSetWindow(win_text);
  glutPostRedisplay();
  set_cursor_help();
}

 void set_png1(int choice) {
   if (choice == 1)
     g_resolution = 72;
   else if (choice == 2)
     g_resolution = 110;
   else if (choice == 3)
     g_resolution = 200;
   else
     g_resolution = 300;
   strcpy(text_window_message, png_message);
   strcat(text_window_message, ", default ");
   strcat(text_window_message, g_png_filename);
   strcat(text_window_message, ")? ");
   g_message_set = TRUE;
   data_entered = FALSE;
   g_type_message = MESSAGE_PNG;
   glutSetWindow(win_text);
   glutPostRedisplay();
   set_cursor_help();
   
 }
 
 void set_jpg1(int choice) {
   if (choice == 1)
     g_resolution = 72;
   else if (choice == 2)
     g_resolution = 110;
   else if (choice == 3)
     g_resolution = 200;
   else
     g_resolution = 300;
   strcpy(text_window_message, jpg_message);
   strcat(text_window_message, ", default ");
   strcat(text_window_message, g_jpg_filename);
   strcat(text_window_message, ")? ");
   g_message_set = TRUE;
   data_entered = FALSE;
   g_type_message = MESSAGE_JPG;
   glutSetWindow(win_text);
   glutPostRedisplay();
   set_cursor_help();
 }

void finish_setting_prob_cutoff(void) {
  float           temp_filter;
  text_window_message[0] = '\0';
  if (text_window_input[0] == '/') {
    text_window_input[0] = '\0';
    text_window_message[0] = '\0';
    fix_cursor();
    glutSetWindow(win_text);
    glutPostRedisplay();
    return;
  }
  sscanf(text_window_input, "%f", &temp_filter);
  g_energy_cutoff = (int) (temp_filter * -1000000);
  text_window_input[0] = '\0';
  if (g_energy_cutoff > g_worst_energy)
    g_energy_cutoff = g_worst_energy;
  fix_colors();
  glutSetWindow(win_text);
  glutPostRedisplay();
  fix_cursor();
}

void finish_setting_energy_cutoff_text(void) {
  float           temp_filter;
  text_window_message[0] = '\0';
  if (text_window_input[0] == '/') {
    text_window_input[0] = '\0';
    text_window_message[0] = '\0';
    fix_cursor();
    glutSetWindow(win_text);
    glutPostRedisplay();
    return;
  }
  sscanf(text_window_input, "%f", &temp_filter);
  finish_setting_energy_cutoff(temp_filter);
  text_window_input[0] = '\0';
  fix_colors();
  glutSetWindow(win_text);
  glutPostRedisplay();
  fix_cursor();
}
 
/* This function is called from the menu and sets the filter */
void set_chain_len(int chain_len_choice) {
  g_zoom_labels_count = 0;	/* reset labeling of points in zoom */
  g_chain_len = chain_len_choice;
  glutSetWindow(win_main);	/* display main */
  glutPostRedisplay();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();	/* display zoom */
    glutPostRedisplay();
  }
  glutSetWindow(win_text);	/* display text window with filter message */
  glutPostRedisplay();
}

/* Plotting functions */
 
void sniff_for_opengl_errors(void) {
  GLenum          errCode;
  const GLubyte  *errString;
  const char     *my_string;
  if ((errCode = glGetError()) != GL_NO_ERROR) {
    errString = gluErrorString(errCode);
    my_string = (char *) errString;
    fprintf(stderr, "OpenGL Error: %s\n", my_string);
  }
}
 
 
void plot_ij(int i_row, int j_col) {
  /* converterts i,j base pair to */
  float           real_x, real_y;	/* real coordinates */
  real_x = 23.0 + (float)((j_col + .5)*700./(g_length+1)); /* and plots them */
  real_y = 749.0 - (float) ((i_row - .5) * 700. / (g_length + 1));
  glVertex2f(real_x, real_y);
}
 
void plot_ij_zoom(int i_row, int j_col, int energy) {
  /* same for zoom window */
  float           size;
  float           real_x, real_y, x_offset, y_offset;
  real_x = 25. + (float) (j_col - display_l + 1) * 700. / (display_w + 1);
  real_y = 50. + (float) (display_b - i_row + 1) * 700. / (display_h + 1);
  if ((display_w > 400) && (display_h > 400)) {
    glVertex2f(real_x, real_y);
  } else {
    if (g_prob == TRUE) {
      if (energy <= -1200000)
	energy = -1200000; 
      size = sqrt((-1.0 * (float) energy) / 1000000.);
    } else
      size = 1.0;
    x_offset = (25.+(float)(j_col+1-display_l+1)*700./(display_w+1)-real_x)/2.;
    y_offset = (50.+(float)(display_b-i_row+2)*700./(display_h+1)-real_y)/2.;
    if (g_prob == TRUE) {
      x_offset = x_offset * size;
      y_offset = y_offset * size;
      if (x_offset < 1)
	x_offset = 1;
      if (y_offset < 1)
	y_offset = 1;
      glBegin(GL_POLYGON);	/* square */
      glVertex2f(real_x - x_offset, real_y - y_offset);
      glVertex2f(real_x - x_offset, real_y + y_offset);
      glVertex2f(real_x + x_offset, real_y + y_offset);
      glVertex2f(real_x + x_offset, real_y - y_offset);
      glEnd();/* this statement produces errors ? */
    } else {
      glBegin(GL_POLYGON);	/* octogon */
      glVertex2f(real_x, real_y - y_offset);
      glVertex2f(real_x - x_offset, real_y - y_offset);
      glVertex2f(real_x - x_offset, real_y);
      glVertex2f(real_x, real_y + y_offset);
      glVertex2f(real_x + x_offset, real_y + y_offset);
      glVertex2f(real_x + x_offset, real_y);
      glEnd();
    }
    sniff_for_opengl_errors();
  }
}

int plot_i_zoom(int i_row) {
  /* convert i_row to y coordinate */
  return (int) (50.+(float)(display_b - i_row + 1) * 700. / (display_h + 1));
}

int plot_j_zoom(int j_col) {
  /* convert col to x coordinate */
  return (int) (25.2+(float) (j_col - display_l + 1) * 700. / (display_w + 1));
}

int plot_i(int i_row) {
  /* converts row to y coordinate */
  return (int) (749.0 - (float) ((i_row - .5) * 700. / (g_length + 1)) + .5);
}

int plot_j(int j_col) {
  /* converts col to x coordinate */
  return (int) (23.0 + (float) ((j_col + .5) * 700. / (g_length + 1)) + .5);
}

void plot_points_zoom(void) {
  int             row, col, new_row, count;
  int             diag, position, old_color, location, new_color;
  int             start_diag, end_diag;
  int             energy;
  old_color = 1;
  fixcolor_plot(old_color);
  points_plotted_zoom = 0;
  start_diag = display_t + display_l - 1;
  end_diag = display_b + display_r - 1;
  if ((display_w > 400) && (display_h > 400))
    glBegin(GL_POINTS);
  for (old_color = g_number_of_colors; old_color >= 1; old_color--) {
    fixcolor_plot(old_color);
    for (diag = start_diag; diag <= end_diag; diag++) {
      position = g_diag_start[diag];
      for (count = 0; (count < g_diag_count[diag])
		       && (g_diag[position + count].row <= display_b) ;
	   count++) {
	location = position + count;
	new_color = g_diag[location].color;
	if (new_color == old_color) {
	  if ((!g_opt_prob_flag)
	      || (g_diag[location].energy >= -20000)) {
	    energy = g_diag[location].energy;
	    if ((g_diag[location].energy <= g_energy_cutoff) &&
		((g_diag[location].length >= g_chain_len) ||
		 (energy == g_optimal_energy))) {
	      new_row = g_diag[location].row;
	      col = g_diag[location].column;
	      if (col >= display_l) {
		for (row = new_row; row < new_row + g_diag[location].length;
		     row++) {
		  if ((row >= display_t) && (row <= display_b)
		      && (col >= display_l) && (col <= display_r)) {
		    plot_ij_zoom(row, col, energy);
		    points_plotted_zoom++;
		  }
		  col--;
		}
	      }
	    }
	  }
	}
      }
    }
  }
  /* plot optimal in lower triangle */
  fixcolor_plot_optimal();
  if (!g_adjust_col_flag) {
    for (diag = start_diag; diag <= end_diag; diag++) {
      position = g_diag_start[diag];
      for (count = 0; ((count < g_diag_count[diag])
		       && (g_diag[position + count].row <= display_r));
	   count++) {
	location = position + count;
	if (g_diag[location].color == 1) {
	  new_row = g_diag[location].row;
	  col = g_diag[location].column;
	  if (col >= display_t) {
	    for (row = new_row; row < new_row + g_diag[location].length;
		 row++) {
	      if ((col >= display_t) && (col <= display_b)
		  && (row >= display_l) && (row <= display_r)) {
		energy = g_diag[location].energy;
		if (g_prob) {
		  if (g_opt_prob_flag) {
		    if (energy <= -20001)
		      plot_ij_zoom(col, row, -1000000);
		  } else
		    plot_ij_zoom(col, row, energy);
		} else
		  plot_ij_zoom(col, row, energy);
	      }
	      col--;
	    }
	  }
	}
      }
    }
  }
  if ((display_w > 400) && (display_h > 400))
    glEnd();
}

void plot_points_opt_prob(void) {
  int             row, col, new_row;
  int             old_color, location, new_color;
  glBegin(GL_POINTS);
  old_color = 1;
  fixcolor_plot_optimal();
  for (location = 0; location < g_helices_in_plot_file; location++) {
    new_color = g_diag[location].color;
    if (new_color == old_color) {
      if (g_diag[location].energy <= -20001) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	for (row = new_row; row < new_row + g_diag[location].length; row++) {
	  plot_ij(col, row);
	  col--;
	}
      }
    }
  }
  glEnd();
}

void plot_points(void) {
  int             row, col, new_row;
  int             old_color, location, new_color;
  old_color = 1;
  fixcolor_plot(old_color);
  glBegin(GL_POINTS);
  points_plotted = 0;
  for (old_color = g_number_of_colors; old_color >= 1; old_color--) {
    fixcolor_plot(old_color);
    for (location = 0; location < g_helices_in_plot_file; location++) {
      new_color = g_diag[location].color;
      if (new_color == old_color) {
	if ((g_diag[location].energy <= g_energy_cutoff) &&
	    ((g_diag[location].length >= g_chain_len) ||
	     (g_diag[location].energy == g_optimal_energy))) {
	  new_row = g_diag[location].row;
	  col = g_diag[location].column;
	  if ((!g_opt_prob_flag)
	      || (g_diag[location].energy >= -20000)) {
	    for (row = new_row; row < new_row + g_diag[location].length;
		 row++) {
	      plot_ij(row, col);
	      if (!g_opt_prob_flag) {
		if ((new_color == 1) && (!g_adjust_col_flag)) {
		  fixcolor_plot_optimal();
		  plot_ij(col, row);
		  fixcolor_plot(old_color);
		}
	      }
	      points_plotted++;
	      col--;
	    }
	  }
	}
      }
    }
  }
  glEnd();
  if (g_opt_prob_flag)
    plot_points_opt_prob();
}
 
void output_main(int x, int y, char *string) {
  /* display string at real coordinates  x,y  */
  int             len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(g_font_main, string[i]);
  }
}

void    output_zoom(int x, int y, char *string) {
  /* display string at real coordinates  x,y  */
  int             len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(g_font_zoom, string[i]);
  }
}

void output_text(int x, int y, char *string) {
  /* display string at real coordinates  x,y  */
  int             len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(g_font_text, string[i]);
  }
}
 
void single_hor_tic(int j_col, int grid_flag) {
  char            string2[20];
  float           x;
  x = (float) plot_j(j_col);
  glBegin(GL_LINES);
  glVertex2f(x, 751.);
  glVertex2f(x, 758.);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COLOR_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(x, 750);
    glVertex2f(x, 43);
    glEnd();
    fixcolor_plot(g_default_color);
  }
  strcpy(string2, num_string_int(j_col));
  output_main(plot_j(j_col) - 10, 762, string2);
}

void plot_hor_tics(void) {
  int             j_col, step, start;
  step = step_fun(1, g_length);
  start = start_fun(1, step);
  single_hor_tic(g_length, FALSE);
  single_hor_tic(1, FALSE);
  for (j_col = start; j_col < (g_length - 2 * step / 3); j_col = j_col+step) {
    single_hor_tic(j_col, g_grid_flag);
  }
}

void single_ver_tic(int i_row, int grid_flag) {
  char            string2[20];
  float           y;
  y = (float) plot_i(i_row);
  glBegin(GL_LINES);
  glVertex2f(727., y);
  glVertex2f(736., y);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COLOR_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(18, y);
    glVertex2f(725., y);
    glEnd();
    fixcolor_plot(g_default_color);
  }
  strcpy(string2, num_string_int(i_row));
  output_main(737, plot_i(i_row) - 5, string2);
}
 
void plot_ver_tics(void) {
  int             i_row, step, start;
  step = step_fun(1, g_length);
  start = start_fun(1, step);
  single_ver_tic(1, FALSE);
  single_ver_tic(g_length, FALSE);
  for (i_row = start; i_row < (g_length - step / 2); i_row = i_row + step) {
    single_ver_tic(i_row, g_grid_flag);
  }
}

void single_hor_tic_zoom(int j_col, int grid_flag) {
  char            string2[10];
  float           x;
  x = (float) plot_j_zoom(j_col);
  glBegin(GL_LINES);
  glVertex2f(x, 751);
  glVertex2f(x, 758);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COLOR_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(x, 749);
    glVertex2f(x, 50);
    glEnd();
    fixcolor_plot(g_default_color);
  }
  strcpy(string2, num_string_int(j_col));
  output_zoom(plot_j_zoom(j_col) - 10, 762, string2);
}

void plot_hor_tics_zoom(void) {
  int             j_col, step;
  int             start;
  step = step_fun(display_l, display_r);
  start = start_fun(display_l, step);
  single_hor_tic_zoom(display_r, FALSE);
  single_hor_tic_zoom(display_l, FALSE);
  for (j_col = start; j_col < (display_r - 2 * step / 3);
       j_col = j_col + step) {
    single_hor_tic_zoom(j_col, g_grid_flag);
  }
}

void single_ver_tic_zoom(int i_row, int grid_flag) {
  char            string2[20];
  float           y;
  y = (float) plot_i_zoom(i_row);
  glBegin(GL_LINES);
  glVertex2f(727., y);
  glVertex2f(736., y);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COLOR_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(23., y);
    glVertex2f(723., y);
    glEnd();
    fixcolor_plot(g_default_color);
  }
  strcpy(string2, num_string_int(i_row));
  output_zoom(737, plot_i_zoom(i_row) - 5, string2);
}

void plot_ver_tics_zoom(void) {
  int             i_row, step, start;
  step = step_fun(display_t, display_b);
  start = start_fun(display_t, step);
  single_ver_tic_zoom(display_t, FALSE);
  single_ver_tic_zoom(display_b, FALSE);
  for (i_row = start; i_row < (display_b - step / 2); i_row = i_row + step) {
    single_ver_tic_zoom(i_row, g_grid_flag);
  }
}

void plot_tics_zoom(void) {
  plot_hor_tics_zoom();
  plot_ver_tics_zoom();
  return;
}

void plot_tics(void) {
  plot_hor_tics();
  plot_ver_tics();
  return;
}

int window_x_convert(int x) {
  /* converts window x coordinates to to  column coordinate of RNA top
   * left for window is 0,0 x is right, y is down i is down, j is
   * right for RNA 
   */  
  float xf;
  xf = x - 23. * width / 800.;
  xf = xf * (g_length + 1) / 700. * 800. / width;
  return (int) (xf);
}

int window_y_convert(int y) {	/* convert window y coordinates to */
  float           yf;	/* to row coordinates */
  int             i_row;
  yf = (float) (y * 800. / height);	/* scale according to window size */
  yf = 799 - yf; /* origin is at top in window, bottom in opengl */
  i_row = (int) ((g_length + 1.) * (749. - yf) / 700. + 1.0);
  /* inverse of plot_ij function 
   * The 1.0 above should be .5 ? but 1.0 works much better */
  return i_row;
}

int window_x_convert_zoom(int x) {	/* converts window x coordinates to */
  float           xf;
  xf = x - 25.*width_z/800.; /* subtract offset to right depending on window */
  xf = xf * (display_w + 1) / 700. * 800. / width_z + display_l - 1; /* size */
  return (int) (xf + .5);
}

int window_y_convert_zoom(int y) {	/* convert window y coordinates to */
  float           yf;	/* RNA i coordinates down */
  yf = y - 50. * height_z / 800; /* subtract offset down depending on window */
  yf = yf * (display_h + 1) / 700 * 800. / height_z + display_t - 1; /* size */
  return (int) (yf + .5);
}

/* display functions */

void display_text(void) {
  float           x_float;
  int             energy;
  char            text_message[150];
  int             color, offset;
  int             gl_row, gl_col;
  int             temp;
  char            location[80], energy_string[60], energy_long_string[80];
  glClear(GL_COLOR_BUFFER_BIT);
  gl_row = g_i_row;
  gl_col = g_j_col;
  if (gl_row > gl_col) {
    temp = gl_col;
    gl_col = gl_row;
    gl_row = temp;
  }
  fixcolor_plot(g_default_color);
  if (g_message_set == TRUE) {
    glutPopWindow();
    fixcolor_plot(COLOR_LABEL);
    strcpy(text_message, text_window_message);
    strcat(text_message, "_");
    output_text(10, 5, text_message);
    fixcolor_plot(g_default_color);
    if (data_entered == TRUE) {
      data_entered = FALSE;
      g_message_set = FALSE;
      if (g_type_message == MESSAGE_ENERGY) {
	if (g_prob != TRUE)
	  finish_setting_energy_cutoff_text();
	else
	  finish_setting_prob_cutoff();
      }
      if (g_type_message == MESSAGE_POST)
	finish_making_post();
      if (g_type_message >= MESSAGE_GIF)
	finish_making_img_text(g_type_message);
      
    }
  } else if (gl_row > 0) {
    energy = energy_read(gl_row, gl_col);
    strcpy(location, "(i,j) base pair =(");
    strcat(location, g_i_row_s);
    strcat(location, ",");
    strcat(location, g_j_col_s);
    strcat(location, ")");
    if (energy != 0) {
      fixcolor_plot(find_color(energy, g_color_increment, g_optimal_energy));
      output_text(2, 50, "**");
    }
    fixcolor_plot(g_default_color);
    output_text(25, 50, location);
    if (g_prob == TRUE) {
      strcpy(energy_long_string, "probability = ");
    } else {
      strcpy(energy_long_string, "Energy = ");
    }
    strcpy(energy_string,
	   num_string_fancy_int(energy));
    strcat(energy_long_string, energy_string);
    if (g_prob == FALSE)
      strcat(energy_long_string, " kcal/mol");
    if (g_opt_prob_flag) {
      if (energy_read_opt_prob(gl_row, gl_col) != 0) {
	strcat(energy_long_string, " Optimal");
      }
    }
    output_text(10, 10, energy_long_string);
  }
  if (g_prob == FALSE) {
    strcpy(location, "Optimal Energy:  ");
    strcat(location, num_string_fancy_int(g_optimal_energy));
    strcat(location, " kcal/mol");
    fixcolor_plot(COLOR_OPTIMAL);
    output_text(10, 180, location);
    fixcolor_plot(g_default_color);
    strcpy(location, "Highest Energy:  ");
    strcat(location, num_string_fancy_int(g_worst_energy));
    strcat(location, " kcal/mol");
    output_text(10, 150, location);
  } else {
    strcpy(location, "Highest: ");
    strcat(location, num_string_fancy_int(g_optimal_energy));
    output_text(10, 180, location);
    strcpy(location, "Lowest: ");
    strcat(location, num_string_fancy_int(g_worst_energy));
    output_text(10, 150, location);
  }
  if (g_chain_len > 1) {
    strcpy(location, "Filter: ");
    strcat(location, num_string_int(g_chain_len));
    output_text(10, 85, location);
  }
  /* display energy information */
  strcpy(location, "Cutoff: ");
  strcat(location, num_string_fancy_int(g_energy_cutoff));
  if (g_prob == FALSE) {
    strcat(location, " or ");
    strcat(location, num_string_fancy_int(g_energy_cutoff - g_optimal_energy));
    strcat(location, " kcal/mol");
  }
  output_text(10, 120, location);
  /* display color scheme */
  if (g_prob == FALSE) {
    x_float = (float) g_optimal_energy;
    for (color = 2, offset = 0; color <= g_number_of_colors;
	 color++, offset = offset + 25) {
      fixcolor_plot(color);
      strcpy(location, num_string_fancy_float(x_float+(color-2)*
					      g_color_increment));
      strcat(location, " < energy <= ");
      strcat(location, num_string_fancy_float(x_float + (color - 1)*
					      g_color_increment));
      output_text(350, 180 - offset, location);
    }
  } else {
    fixcolor_plot(COLOR_OPTIMAL);
    strcpy(location, num_string_float(g__prob[g_number_of_colors - 4][0]));
    if (g_mi_flag)
      strcat(location, " < Mutual Information");
    else
      strcat(location, " < Prob. <= 1");
    output_text(350, 180, location);
    for (color = 2, offset = 25; color <= g_number_of_colors - 1;
	 color++, offset = offset + 22) {
      fixcolor_plot(color);
      strcpy(location,
	     num_string_float(g__prob[g_number_of_colors-4][color-1]));
      if (g_mi_flag)
	strcat(location, " < M.I.  <= ");
      else
	strcat(location, " < Prob. <= ");
      strcat(location,
	     num_string_float(g__prob[g_number_of_colors - 4][color - 2]));
      output_text(350, 180 - offset, location);
    }
    fixcolor_plot(color);
    if (g_mi_flag)
      strcpy(location, "                   M.I.  <= ");
    else
      strcpy(location, "0.000000 < Prob. <= ");
    strcat(location, num_string_float(g__prob[g_number_of_colors - 4]
				      [g_number_of_colors - 2]));
    output_text(350, 180 - offset, location);
  }
  sniff_for_opengl_errors();
  glutSwapBuffers();
}

void display_main(void) {
  char            location[150];
  float           point_size;	/* size of points to plot within box */
  glClear(GL_COLOR_BUFFER_BIT);
  fixcolor_plot(g_default_color);
  glLineWidth(1.0);
  plot_tics();
  glLineWidth(2.0);
  fixcolor_plot(g_default_color);
  glBegin(GL_LINE_LOOP);	/* draw box */
  glVertex2f(18.0, 752.0);
  glVertex2f(726.0, 752.0);
  glVertex2f(726.0, 43.0);
  glVertex2f(18.0, 43.0);
  glEnd();
  glLineWidth(1.0);
  if (!g_adjust_col_flag) {
    glBegin(GL_LINES);	/* draw diagonal */
    glVertex2f(23.0, 752.0);
    glVertex2f(727.0, 47.0);
    glEnd();
  }
  point_size = 700. / (g_length + 1) * height / 800. + .5;
  if (point_size < 1.4)
    point_size = 1.4;
  if ((g_dot_adjust) && (point_size < 1.5))
    point_size = 1.5;
  glPointSize(point_size);
  plot_points();
  fixcolor_plot(g_default_color);
  strcpy(location, "Base Pairs Plotted: ");
  strcat(location, num_string_int(points_plotted));
  output_main(65, 5, location);
  fixcolor_plot_optimal();
  if (g_prob) {
    if (g_opt_prob_flag) {
      strcpy(location, "Lower Triangle:  Optimal Structure");
    } else {
      strcpy(location, "Lower Triangle: Highest Color Range");
    }
  } else {
    strcpy(location, "Lower Triangle: Optimal Energy");
  }
  output_main(350, 5, location);
  glLineWidth(2.0);
  if ((g_region_set == TRUE) || (g_region_exists == TRUE)) {	/* draw zoom */
    fixcolor_plot(COLOR_LABEL);
    glBegin(GL_LINE_LOOP);
    glVertex2f(region_real_l, region_real_t);
    glVertex2f(region_real_r, region_real_t);
    glVertex2f(region_real_r, region_real_b);
    glVertex2f(region_real_l, region_real_b);
    glEnd();
  }
  glutSwapBuffers();
  sniff_for_opengl_errors();
}

void display_zoom(void) {
  float           point_size;
  char            location[60];
  int             right, bottom, top, left, i;
  void           *temp_font;
  glClear(GL_COLOR_BUFFER_BIT);
  glLineWidth(1.0);
  fixcolor_plot(g_default_color);
  plot_tics_zoom();
  point_size = 700./((display_w+display_h)/2)*width_z/800.*height_z/800.+.60;
  if (point_size < 1.4)
    point_size = 1.4;
  if ((g_dot_adjust) && (point_size < 1.5))
    point_size = 1.5;
  glPointSize(point_size);
  plot_points_zoom();
  fixcolor_plot(g_default_color);
  glLineWidth(2.0);
  glBegin(GL_LINE_LOOP);
  glVertex2f(22.0, 751.0);
  glVertex2f(22.0, 48.0);
  glVertex2f(724.0, 48.0);
  glVertex2f(724.0, 751.0);
  glEnd();
  glLineWidth(1.0);
  strcpy(location, "Base Pairs Plotted: ");
  strcat(location, num_string_int(points_plotted_zoom));
  output_zoom(10, 5, location);
  /* draw diagional line to indicate no points below */
  if (!g_adjust_col_flag) {
    if ((display_r >= display_t) && (display_b >= display_l)) {
      if (display_r > display_b) {
	bottom = display_b;
	right = display_b;
      } else {
	right = display_r;
	bottom = display_r;
      }
      if (display_l > display_t) {
	left = display_l;
	top = display_l;
      } else {
	top = display_t;
	left = top;
      }
      glBegin(GL_LINES);
      glVertex2f(plot_j_zoom(left - 1), plot_i_zoom(top - 1));
      glVertex2f(plot_j_zoom(right + 1), plot_i_zoom(bottom + 1));
      glEnd();
    }
  }
  fixcolor_plot(COLOR_LABEL);
  if (g_region_set == TRUE) {	/* draw zoom box */
    glBegin(GL_LINE_LOOP);
    glVertex2f(region_real_l_zoom, region_real_t_zoom);
    glVertex2f(region_real_l_zoom, region_real_b_zoom);
    glVertex2f(region_real_r_zoom, region_real_b_zoom);
    glVertex2f(region_real_r_zoom, region_real_t_zoom);
    glEnd();
  }
  /* draw the labels */
  if (g_zoom_labels_count > 0) {
    temp_font = g_font_zoom;
    g_font_zoom = GLUT_BITMAP_TIMES_ROMAN_10;
    glEnable(GL_POINT_SMOOTH);
    for (i = 0; i < g_zoom_labels_count; i++) {
      output_zoom(g_zoom_labels_x[i], g_zoom_labels_y[i],
		  g_zoom_labels_string[i]);
      
    }
    point_size = 700./((float) (display_w + display_h));
    point_size *= (float) (width_z)/800.;
    point_size *= (float) (height_z)/800. + 0.5;
    glPointSize(point_size);
    glEnable(GL_POINT_SMOOTH);
    glBegin(GL_POINTS);
    for (i = 0; i < g_zoom_labels_count; i++) {
      glVertex2f((float) g_zoom_labels_xl[i], (float) g_zoom_labels_y[i]);
    }
    glEnd();
    glDisable(GL_POINT_SMOOTH);
    g_font_zoom = temp_font;
  }
  glutSwapBuffers();
}

void text_reshape(int w, int h) {
  if (w > 1000) {
    w = 1000;
    glutReshapeWindow(w, h);
  }
  if (h > 300) {
    h = 300;
    glutReshapeWindow(w, h);
  }
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, 600.0, 0.0, 200.0, -1.0, 1.0);
  glMatrixMode(GL_MODELVIEW);
  if (w < 700)
    select_font_text(0);
  else
    select_font_text(1);
}

void reshape(int w, int h) {
  int             current_window, font;
  current_window = glutGetWindow();
  if (w > h) {
    h = w;
    glutReshapeWindow(w, h);
    glutPostRedisplay();
  }
  if (h > w) {
    w = h;
    glutReshapeWindow(w, h);
    glutPostRedisplay();
  }
  if (w > 990) {
    w = 990;
    h = 990;
    glutReshapeWindow(w, h);
  }
  if (current_window == win_main) {
    if (w < h)
      h = w;
    else
      w = h;
  }
  if (current_window == win_main) {
    width = w;
    height = h;
  } else {
    width_z = w;
    height_z = h;
  }
  if (w < 500)
    font = 0;
  else {
    if (w < 900)
      font = 1;
    else
      font = 2;
  }
  if (current_window == win_main)
    select_font_main(font);
  else
    select_font_zoom(font);
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, 800.0, 0.0, 800.0, -1.0, 1.0);	/* 0,0 is origin for
						 * drawing */
  /* lower left is 0,0   up is y, right is x */
  glMatrixMode(GL_MODELVIEW);
}

/* Functions to set zoom rectangle */
void fix_zoom_box(void) {		/* region_real_r, region_r; */
  region_real_l = 23.0 + (float) ((region_l + .5) * 700. / (g_length + 1));
  region_real_r = 23.0 + (float) ((region_r + .5) * 700. / (g_length + 1));
  region_real_b = 749.0 - (float) ((region_b - .5) * 700. / (g_length + 1));
  region_real_t = 749.0 - (float) ((region_t - .5) * 700. / (g_length + 1));
}

void finish_fixing_zoom_box(void) {
  int             temp;	/* make left less than right */
  if (region_r < region_l) {
    temp = region_r;
    region_r = region_l;
    region_l = temp;
    fix_zoom_box();
  }
  if (region_b < region_t) {	/* make top less than bottom */
    temp = region_t;
    region_t = region_b;
    region_b = temp;
    fix_zoom_box();
  }
  if (first_zoom > 0) {
    saved_display_l = display_l;
    saved_display_r = display_r;
    saved_display_t = display_t;
    saved_display_b = display_b;
  } else {
    first_zoom = 1;
    saved_display_l = region_l;
    saved_display_r = region_r;
    saved_display_t = region_t;
    saved_display_b = region_b;
  }
  display_l = region_l;
  display_r = region_r;
  display_t = region_t;
  display_b = region_b;
  display_w = display_r - display_l + 1;
  display_h = display_b - display_t + 1;
}

void fix_zoom_box_zoom(void) {
  region_real_l = 25.0 + (float) region_l *700. / g_length;
  region_real_r = 25.0 + (float) region_r *700. / g_length;
  region_real_b = 50.0 + (float) (g_length - region_b) * 700./g_length;
  region_real_t = 50.0 + (float) (g_length - region_t) * 700/g_length;
  region_real_l_zoom = 25.4+(float)(region_l-display_l+1)*700./(display_w + 1);
  region_real_r_zoom = 25.4+(float)(region_r-display_l+1)*700./(display_w + 1);
  region_real_t_zoom = 50.0+(float)(display_b-region_t+1)*700./(display_h + 1);
  region_real_b_zoom = 50.0+(float)(display_b-region_b+1)*700./(display_h + 1);
  if (region_real_l_zoom < 0.) {
    region_real_l_zoom = 0.;
  };
  if (region_real_r_zoom < 0.) {
    region_real_r_zoom = 0.;
  }
  if (region_real_r_zoom > 799.) {
    region_real_r_zoom = 800.;
  }
  if (region_real_l_zoom > 799.) {
    region_real_l_zoom = 799.;
  }
  if (region_real_t_zoom < 0) {
    region_real_t_zoom = 0;
  }
  if (region_real_t_zoom > 799.) {
    region_real_t_zoom = 799.;
  }
  if (region_real_b_zoom < 0.) {
    region_real_b_zoom = 0.;
  }
  if (region_real_b_zoom > 799.) {
    region_real_b_zoom = 799.;
  }
}

void finish_fixing_zoom_box_zoom(void) {
  int             temp;	/* make left less than right */
  if (region_r < region_l) {
    temp = region_r;
    region_r = region_l;
    region_l = temp;
  }
  if (region_b < region_t) {	/* make top less than bottom */
    temp = region_t;
    region_t = region_b;
    region_b = temp;
  }
  saved_display_l = display_l;
  saved_display_r = display_r;
  saved_display_t = display_t;
  saved_display_b = display_b;
  display_l = region_l;
  display_r = region_r;
  display_t = region_t;
  display_b = region_b;
  display_w = display_r - display_l + 1;
  display_h = display_b - display_t + 1;
  fix_zoom_box_zoom();
}


/* Mouse Functions ___________ */

void fix_key_zoom_window(int x_change, int y_change, int zoom_direction) {
  int             x_offset, y_offset;
  g_zoom_labels_count = 0;	/* reset labels to none */
  if (x_change != 0) {
    x_offset = step_fun(display_l, display_r);
    if (x_offset < 1)
      x_offset = 1;
    x_offset = x_offset * x_change;
    if (x_change < 0) {
      if (display_l == 1)
	return;	/* do not go left when at left edge */
      region_l = display_l + x_offset;
      if (region_l < 1)
	region_l = 1;
      region_r = display_r + x_offset;
      if (region_r < 1)
	region_r = 1;
    }
    if (x_change > 0) {
      if (display_r == g_length)
	return;	/* do go right when at right edge */
      region_r = region_r + x_offset;
      region_l = region_l + x_offset;
      if (region_r > g_length)
	region_r = g_length;
      if (region_l > g_length)
	region_l = g_length;
    }
  }
  if (y_change != 0) {
    y_offset = step_fun(display_t, display_b);
    if (y_offset < 1)
      y_offset = 1;
    y_change = y_change * y_offset;
    if (y_change < 0) {
      if (display_t == 1)
	return;	/* do not go up when at top */
      region_t = display_t + y_change;
      if (region_t < 1)
	region_t = 1;
      region_b = display_b + y_change;
      if (region_b < 1)
	region_b = 1;
    }
    if (y_change > 0) {
      if (display_b == g_length)
	return;	/* do not go up when at top */
      region_t = region_t + y_change;
      region_b = region_b + y_change;
      if (region_b > g_length)
	region_b = g_length;
      if (region_t > g_length)
	region_t = g_length;
    }
  }
  if (zoom_direction) {
    y_offset = step_fun(display_t, display_b);
    if (y_offset < 1)
      y_offset = 1;
    x_offset = step_fun(display_l, display_r);
    if (x_offset < 1)
      x_offset = 1;
    if (zoom_direction < 0) {
      if ((display_b - display_t) < 4) {
	y_offset = 0;
      }
      if ((display_r - display_l) < 4)
	x_offset = 0;
      x_offset = x_offset * -1;
      y_offset = y_offset * -1;
    }
    region_l = display_l - x_offset;
    region_r = display_r + x_offset;
    region_t = display_t - y_offset;
    region_b = display_b + y_offset;
    if (region_l < 1)
      region_l = 1;
    if (region_t < 1)
      region_t = 1;
    if (region_b > g_length)
      region_b = g_length;
    if (region_r > g_length)
      region_r = g_length;
  }
  fix_zoom_box();
  finish_fixing_zoom_box();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutPostRedisplay();
}

static void Key2(int key, int x, int y) {
  int             x_change;
  int             y_change;
  int             zoom_direction;
  if (!g_zoom_window_displayed)
    return;
  x_change = 0;
  y_change = 0;
  zoom_direction = 0;
  switch (key) {
  case GLUT_KEY_LEFT:
    x_change = -1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_RIGHT:
    x_change = 1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_UP:
    y_change = -1.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_DOWN:
    y_change = 1.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);;
    break;
  case GLUT_KEY_PAGE_DOWN:
    zoom_direction = +1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_PAGE_UP:
    zoom_direction = -1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  default:
    return;
  }
}

static void Key2_fast(int key, int x, int y) {
  int             x_change;
  int             y_change;
  int             zoom_direction;
  if (!g_zoom_window_displayed)
    return;
  x_change = 0;
  y_change = 0;
  zoom_direction = 0;
  switch (key) {
  case GLUT_KEY_LEFT:
    x_change = -3;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_RIGHT:
    x_change = 3;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_UP:
    y_change = -3.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_DOWN:
    y_change = 3.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);;
    break;
  case GLUT_KEY_PAGE_DOWN:
    zoom_direction = +1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_PAGE_UP:
    zoom_direction = -1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  default:
    return;
  }
}

void mouse(int button, int state, int x, int y) {
  int             i_row, j_col;
  j_col = window_x_convert(x);
  i_row = window_y_convert(y);
  if (i_row < 1) {
    i_row = 1;
  }
  if (i_row > g_length)
    i_row = g_length;
  if (j_col < 1)
    j_col = 1;
  if (j_col > g_length)
    j_col = g_length;
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
      g_i_row = i_row;
      g_j_col = j_col;
      strcpy(g_i_row_s, num_string_int(i_row));
      strcpy(g_j_col_s, num_string_int(j_col));
      glutSetWindow(win_text);
      glutPostRedisplay();
    }
  }
  if (button == GLUT_MIDDLE_BUTTON) {
    if (state == GLUT_DOWN) {
      if (g_region_set == FALSE) {
	region_l = j_col;
	region_t = i_row;
	region_b = i_row;
	region_r = j_col;
	g_region_set = TRUE;
      } else {
	printf("Within no motion mouse with g_r_s = 1\n");
	region_r = j_col; /* This code is never executed ? */
	region_b = i_row;
      }
      fix_zoom_box();
    } else {
      g_region_set = FALSE;
      g_region_exists = TRUE;
      finish_fixing_zoom_box();
      g_zoom_labels_count = 0;
      g_zoom_window_displayed = TRUE;
      zoomSetWindow();
      glutShowWindow();
      glutPopWindow();
      glutPostRedisplay();
    }
  }
}

void mouse_motion(int x, int y) { /* returns second corner of zoom rectangle as
				   * mouse is moved */
  int             i_row, j_col;
  if (g_region_set == TRUE) {
    j_col = window_x_convert(x);
    i_row = window_y_convert(y);
    if (i_row < 1) {
      i_row = 1;
    }
    if (j_col > g_length)
      j_col = g_length;
    if (i_row > g_length)
      i_row = g_length;
    if (j_col < 1)
      j_col = 1;
    region_r = j_col;
    region_b = i_row;
    fix_zoom_box();
    glutSetWindow(win_main);
    glutPostRedisplay();
  }
}

void mouse_zoom(int button, int state, int x, int y) {
  int   i, i_row, j_col, place, place_old = 0, already_there;
  char  rec[30];
  j_col = window_x_convert_zoom(x);
  i_row = window_y_convert_zoom(y);
  if (i_row < display_t) {
    i_row = display_t;
  }
  if (i_row > display_b)
    i_row = display_b;
  if (j_col < display_l)
    j_col = display_l;
  if (j_col > display_r)
    j_col = display_r;
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
      g_i_row = i_row;
      g_j_col = j_col;
      strcpy(g_i_row_s, num_string_int(i_row));
      strcpy(g_j_col_s, num_string_int(j_col));
      glutSetWindow(win_text);
      glutPostRedisplay();
      if (energy_read(i_row, j_col) <= g_energy_cutoff) {
	strcpy(rec, "<");
	strcat(rec, g_i_row_s);
	strcat(rec, ",");
	strcat(rec, g_j_col_s);
	strcat(rec, ">");
	/*
	 * add this label to the array only if it is
	 * not already there
	 */
	already_there = FALSE;
	for (i = 0; i < g_zoom_labels_count; i++) {
	  if ((g_zoom_labels_row[i] == i_row) &&
	      (g_zoom_labels_col[i] == j_col)) {
	    already_there = TRUE;
	    place_old = i;
	  }
	}
	if (already_there) {	/* remove label and move
				 * last label into this
				 * position */
	  if (g_zoom_labels_count > 1) {
	    place = g_zoom_labels_count - 1;
	    g_zoom_labels_row[place_old] =
	      g_zoom_labels_row[place];
	    g_zoom_labels_col[place_old] =
	      g_zoom_labels_col[place];
	    g_zoom_labels_x[place_old] =
	      g_zoom_labels_x[place];
	    g_zoom_labels_y[place_old] =
	      g_zoom_labels_y[place];
	    g_zoom_labels_xl[place_old] =
	      g_zoom_labels_xl[place];
	    strcpy(g_zoom_labels_string[place_old],
		   g_zoom_labels_string[place]);
	  }
	  g_zoom_labels_count--;
	  if (g_zoom_window_displayed==TRUE) {
	    zoomSetWindow();
	    glutPostRedisplay();
	  }
	} else {
	  if (g_zoom_labels_count < 49)
	    g_zoom_labels_count++;
	  place = g_zoom_labels_count - 1;
	  strcpy(g_zoom_labels_string[place], rec);
	  g_zoom_labels_x[place] =
	    plot_j_zoom(j_col + 1);
	  g_zoom_labels_xl[place] =
	    plot_j_zoom(j_col);
	  g_zoom_labels_y[place] =
	    plot_i_zoom(i_row);
	  g_zoom_labels_row[place] = i_row;
	  g_zoom_labels_col[place] = j_col;
	  if (g_zoom_window_displayed==TRUE) {
	    zoomSetWindow();
	    glutPostRedisplay();
	  }
	}
      }
    }
  }
  if (button == GLUT_MIDDLE_BUTTON) {
    if (state == GLUT_DOWN) {
      if (g_region_set == FALSE) {
	region_l = j_col;
	region_t = i_row;
	region_b = i_row;
	region_r = j_col;
	g_region_set = TRUE;
      } else {
	printf("Within no motion mouse with g_r_s = 1\n");
	region_r = j_col;	/* This code is never executed ? */
	region_b = i_row;
      }
      fix_zoom_box_zoom();
    } else {
      g_region_set = FALSE;
      g_region_exists = TRUE;
      g_zoom_labels_count = 0;
      finish_fixing_zoom_box_zoom();
      zoomSetWindow();
      glutShowWindow();
      glutPostRedisplay();
      glutSetWindow(win_main);
      glutPostRedisplay();
    }
  }
}

void mouse_motion_zoom(int x, int y) {
  int             i_row, j_col;
  if (g_region_set == TRUE) {
    j_col = window_x_convert_zoom(x);
    i_row = window_y_convert_zoom(y);
    if (i_row < display_t) {
      i_row = display_t;
    }
    if (i_row > display_b)
      i_row = display_b;
    if (j_col < display_l)
      j_col = display_l;
    if (j_col > display_r)
      j_col = display_r;
    region_r = j_col;
    region_b = i_row;
    fix_zoom_box_zoom();
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutPostRedisplay();
    } 
  }
}

void create_structure(void) { 
  char            action[120];
  char            number[10];
  int             row, column;
  int             error_occurred;
  FILE           *nafold_fp;
  char            nafold_filename[120];
  /* temp_for_sir_graph.ct is produced as a ct file
   * nafold.input.temp is created and deleted as instructions for
   * creating a ct file by nafold 
   */
  if (g_i_row > 0)
    printf("Structure for base pair(%d,%d)\n", g_i_row, g_j_col);
  else {
    printf("No structure base pair defined %d,%d\n", g_i_row, g_j_col);
    return;
  }
  if (g_i_row > g_j_col) {
    row = g_j_col;
    column = g_i_row;
  } else {
    row = g_i_row;
    column = g_j_col;
  }
  /* remove ct file first */
  printf("Attempting to remove old temp.ct file\n");
  strcpy(action, "rm ");
  strcat(action, "temp_for_sir_graph.ct");
  error_occurred = system(action);
  strcpy(action, "nafold < ");
  printf("**** running script %s ****\n", action);
  /* create input file */
  strcpy(nafold_filename, "nafold.input.temp");
  strcat(action, nafold_filename);
  strcat(action, "\n");
  if ((nafold_fp = fopen(nafold_filename, "w")) == NULL) { /* open a file */
    printf("Could not open file:  %s\n", nafold_filename);
    printf("Cannot generate structure.\n");
    return;
  }
  fprintf(nafold_fp, "2\n\n\n\n");
  fprintf(nafold_fp, "%s", g_mplot_filename);
  fprintf(nafold_fp, "\n\n\nn\ny\ntemp_for_sir_graph.ct\n\n");
  strcpy(number, num_string_int(row));
  fprintf(nafold_fp, "%s", number);
  fprintf(nafold_fp, " ");
  strcpy(number, num_string_int(column));
  fprintf(nafold_fp, "%s\n", number);
  fclose(nafold_fp);
  printf("system string is %s---\n", action);
  error_occurred = system(action);
  if (error_occurred != 0) {
    printf("Error creating new ct file: %s for save file %s\n",
	   "temp_for_sir_graph.ct", g_mplot_filename);
    printf(" %s should be deleted\n", nafold_filename);
  } else {
    strcpy(action, "sir_graph -lij ");
    strcat(action, num_string_int(g_i_row));
    strcat(action, " ");
    strcat(action, num_string_int(g_j_col));
    strcat(action, " temp_for_sir_graph &");
    printf(" Running with sir_graph with command:\n %s\n", action);
    /* remove nafold_filename */
    error_occurred = system(action);
    strcpy(action, "rm ");
    strcat(action, nafold_filename);
    system(action);
  }
}

FILE           *sel_fp;
int create_actual_selected_file(char *file_name) {
  int             i, error_occurred;
  char            action[200];
  if (g_zoom_labels_count < 1) {
    printf("Error!\tNo labeled points selected. Cannot proceed\n");
    return FALSE;
  }
  printf("Creating file %s of selected base pairs\n", file_name);
  if ((sel_fp = fopen(file_name, "w")) == NULL) {
    printf("Error!\tCould not create file %s of selected base pairs\n",
	   file_name);
    return TRUE;
  }
  for (i = 0; i < g_zoom_labels_count; i++) {
    fprintf(sel_fp, "F %5d %5d 1\n", g_zoom_labels_row[i], 
	    g_zoom_labels_col[i]);
  }
  fclose(sel_fp);
  /* sort the labels */
  if (g_zoom_labels_count > 1) {
    strcpy(action, "sort -n -o ");
    strcat(action, file_name);
    strcat(action, " ");
    strcat(action, file_name);
    error_occurred = system(action);
    if (error_occurred != 0) {
      printf("Error occurred trying to sort %s with %s.\n", file_name, action);
    }
  }
  return FALSE;
}

void create_selected_file(void) {
  char            name[120];
  strcpy(name, g_sel_filename);
  /* get name */
  create_actual_selected_file(name);
}

void create_structure_new(void) {
  char            action[250];
  int             error_occurred;
  printf("Create new structure\n");
  /* check that sequence file exists  */
  if ((sel_fp = fopen(g_seq_filename, "r")) == NULL) {
    printf("Error!\tSequence file %s does not exist\n",
	   g_seq_filename);
    printf("Cannot create structure for selected base pairs\n");
    return;
  }
  /* if not return */
  if (create_actual_selected_file(g_sel_filename)) {
    printf("hybrid-ss-min was not run, no structure will be displayed\n");
    return;
  }
  /* Create a symbolic link to the sequence file to avoid overwriting
   * existing files. '-s' forces a new link if an old one exists.
   */
  strcpy(action, "ln -fs ");
  strcat(action, g_seq_filename);
  strcat(action, " ");
  strcat(action, g_boxseq_filename);
  error_occurred = system(action);
  if (error_occurred != 0) {
    printf("Error executing %s", action);
    return;
  }
  /* Run hybrid-ss-min with constraint file */
  strcpy(action, "hybrid-ss-min --NA=RNA3 --constraints="); 
  /*  strcpy(action, "hybrid-ss-min -nRNA -t 10 -T 10 --constraints="); */
  /* strcpy(action, "hybrid-ss-min --NA=DNA -t 4 -T 4 --sodium=0.132 --magnesium=0.002 --constraints="); */
  /*
  strcpy(action, "hybrid-ss-min --NA=DNA -t 37 -T 37 --sodium=0.132
  --magnesium=0.002 --constraints="); 
  */
  strcat(action, g_sel_filename);
  strcat(action, " -o ");
  strcat(action, g_boxplot_rootname);
  strcat(action, " ");
  strcat(action, g_seq_filename);
  printf("Running hybrid-ss-min with:\n%s\n", action);
  error_occurred = system(action);
  if (error_occurred != 0) {
    printf("Error running hybrid-ss-min.");
    /* Cleanup all junk files. */
    strcpy(action, "rm -vf ");
    strcat(action, g_boxplot_rootname);
    strcat(action, ".*" );
    system(action);
  } else {
    /* run sir_graph 
     * First cleanup hybrid-ss-min junk files
     */
    strcpy(action, "rm -vf ");
    strcat(action, g_boxplot_rootname);
    strcat(action, ".[0-9d-r]*" );
    system(action);
    strcpy(action, "sir_graph -bp -lab 10 -ar -aj -force ");
    /*    strcpy(action, "sir_graph -bp -lab 2 -n -f -ar -aj -force "); */
    strcat(action, g_sel_filename);
    strcat(action, " ");
    strcat(action, g_boxplot_rootname);
    strcat(action, " & ");
    printf("Running sir_graph:\n%s\n", action);
    error_occurred = system(action); 
    if (error_occurred) {
      printf("Error!\tCould not run sir_graph.\n");
    }
  }
}

void No_Action_Menu(int msg) {
  if (msg == 1)
    try_exit(0);
  if (msg == 4) { /* M. Zuker Jan. 17, 2008. Option to hide zoom
		     window is added. Changed to destroy later. */
    glutDestroyWindow(win_zoom);
    g_zoom_window_exists = FALSE;
    g_zoom_window_displayed = FALSE;
  }
  if (msg == 2) {
    glutSetWindow(g_which_window_i_am_in);
    glutPopWindow();
  }
  if (msg == 46) {
    create_selected_file();
  }
  if (msg == 47) {
    create_structure_new();
  }
  if (msg == 3) {
    if (g_grid_flag == TRUE)
      g_grid_flag = FALSE;
    else
      g_grid_flag = TRUE;
    glutSetWindow(win_main);	/* display main */
    glutPostRedisplay();
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();	/* display zoom */
      glutPostRedisplay();
    }
  }
  if (msg == 5) {
    g_zoom_labels_count = 0;	/* reset labeling of points
				 * in zoom window */
    strcpy(text_window_message, increment_message_energy);
    g_message_set = TRUE;
    data_entered = FALSE;
    g_type_message = MESSAGE_ENERGY;
    glutSetWindow(win_text);
    glutPostRedisplay();
    set_cursor_help();
  }
  if (msg == 6) {
    g_zoom_labels_count = 0; /* reset labeling of points in zoom window */
    strcpy(text_window_message, increment_message_prob);
    strcat(text_window_message,
	   num_string_float(-((float) g_optimal_energy)/1000000.));
    strcat(text_window_message, " )? (/ to abort) ");
    g_message_set = TRUE;
    data_entered = FALSE;
    g_type_message = MESSAGE_ENERGY;
    glutSetWindow(win_text);
    glutPostRedisplay();
    set_cursor_help();
  }
  if (msg == 7) {		/* previous zoom window */
    region_l = saved_display_l;
    region_r = saved_display_r;
    region_t = saved_display_t;
    region_b = saved_display_b;
    fix_zoom_box();
    finish_fixing_zoom_box();
    g_zoom_labels_count = 0;
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutPostRedisplay();
    }
    glutSetWindow(win_main);
    glutPostRedisplay();
  }
  if (msg == 17) {
    if (g_dot_adjust)
      g_dot_adjust = FALSE;
    else
      g_dot_adjust = TRUE;
    glutSetWindow(win_main);
    glutPostRedisplay();
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutPostRedisplay();
    }
  }
  if (msg == 33) {
    create_structure();
  }
}

void main_entry(int state) {
  if (state == GLUT_ENTERED) {
    /* g_i_row=0; */
    g_which_window_i_am_in = win_main;
  }
}

void zoom_entry(int state) {
  if (state == GLUT_ENTERED) {
    /* g_i_row=0; */
    g_which_window_i_am_in = win_zoom;
  }
}

void text_entry(int state) {
  if (state == GLUT_ENTERED)
    g_which_window_i_am_in = win_text;
}

void make_menu(int window) {	/* initialize graphics for each window */
  int   font_submenu, color_submenu, chain_len_submenu = 0, postscript_submenu;
  int   gif_submenu, png_submenu, jpg_submenu;
  glClearColor(g_color[COLOR_BACKGROUND][0],
	       g_color[COLOR_BACKGROUND][1],
	       g_color[COLOR_BACKGROUND][2], 1.0);
  postscript_submenu = glutCreateMenu(set_postscript1);
  /* M. Zuker March 23, 2008. Appropriate menu settings introduced.
     This ensures that a zoom plot will not be attempted if the zoom
     window does not exit.
  */
  glutAddMenuEntry(".ps for Main Window", 0);
  if (!g_prob)
    glutAddMenuEntry("Alt. .ps for Main Window", 1);
  if (g_zoom_window_displayed == TRUE) {
    glutAddMenuEntry(".ps for Zoom Window", 2);
    if (!g_prob)
      glutAddMenuEntry("Alt .ps for Zoom Window", 3);
  }
  if (g_prob != TRUE) {
    chain_len_submenu = glutCreateMenu(set_chain_len);
    glutAddMenuEntry("Show all base pairs", 1);
    glutAddMenuEntry("Include helices at least 2 long", 2);
    glutAddMenuEntry("Include helices at least 3 long", 3);
    glutAddMenuEntry("Include helices at least 4 long", 4);
    glutAddMenuEntry("Include helices at least 5 long", 5);
    glutAddMenuEntry("Include helices at least 6 long", 6);
    glutAddMenuEntry("Include helices at least 7 long", 7);
    glutAddMenuEntry("Include helices at least 8 long", 8);
    glutAddMenuEntry("Include helices at least 9 long", 9);
    glutAddMenuEntry("Include helices at least 10 long", 10);
  }
  color_submenu = glutCreateMenu(select_colors);
  if (!g_prob) {
    glutAddMenuEntry("2", 2);
    glutAddMenuEntry("3", 3);
  }
  glutAddMenuEntry("4", 4);
  glutAddMenuEntry("5", 5);
  glutAddMenuEntry("6", 6);
  glutAddMenuEntry("7", 7);
  glutAddMenuEntry("8", 8);
  font_submenu = glutCreateMenu(select_font);
  glutAddMenuEntry("Helvetica         12", 0);
  glutAddMenuEntry("Helvetica         18", 1);
  glutAddMenuEntry("Times Roman    24", 2);
  gif_submenu = glutCreateMenu(set_gif1);
  glutAddMenuEntry("Res. 72    or 612  x 792 ", 1);
  glutAddMenuEntry("Res. 110   or 935  x 1210", 2);
  glutAddMenuEntry("Res. 200   or 1700 x 2200", 3);
  glutAddMenuEntry("Res. 300   or 2550 x 3300", 4);
  png_submenu = glutCreateMenu(set_png1);
  glutAddMenuEntry("Res. 72    or 612  x 792 ", 1);
  glutAddMenuEntry("Res. 110   or 935  x 1210", 2);
  glutAddMenuEntry("Res. 200   or 1700 x 2200", 3);
  glutAddMenuEntry("Res. 300   or 2550 x 3300", 4);
  jpg_submenu = glutCreateMenu(set_jpg1);
  glutAddMenuEntry("Res. 72    or 612  x 792 ", 1);
  glutAddMenuEntry("Res. 110   or 935  x 1210", 2);
  glutAddMenuEntry("Res. 200   or 1700 x 2200", 3);
  glutAddMenuEntry("Res. 300   or 2550 x 3300", 4);
  glutCreateMenu(No_Action_Menu);
  if (g_zoom_window_displayed == TRUE)
    glutAddMenuEntry("UnZoom", 7);
  if (g_prob != TRUE)
    glutAddSubMenu("Filter", chain_len_submenu);
  glutAddSubMenu("Number of Colors for plot", color_submenu);
  if (g_prob == TRUE) {
    if (g_mi_flag)
      glutAddMenuEntry("Enter M.I.  cutoff", 6);
    else
      glutAddMenuEntry("Enter Prob. cutoff", 6);
  } else
    glutAddMenuEntry("Enter energy increment value", 5);
  glutAddSubMenu("Create Postscript file", postscript_submenu);
  if (g_zoom_window_displayed == TRUE) {
#if (defined(HAVE_LIBGD) || defined(HAVE_GDIMAGECREATE)) && defined(HAVE_GD_H)
    glutAddSubMenu("Create GIF for Zoom Window", gif_submenu);
#endif
#ifdef HAVE_GDIMAGEPNG
    glutAddSubMenu("Create PNG for Zoom Window", png_submenu);
#endif
#ifdef HAVE_GDIMAGEJPEG
    glutAddSubMenu("Create JPG for Zoom Window", jpg_submenu);
#endif
    glutAddMenuEntry("Create Selected BP File", 46);
  }
  glutAddSubMenu("Font", font_submenu);
  glutAddMenuEntry("Toggle Grid lines On/Off", 3);
  glutAddMenuEntry("Big Dot On/Off", 17);
  glutAddMenuEntry("Pop", 2);
  if (g_zoom_window_displayed == TRUE) {
    glutAddMenuEntry("Structure for base pair", 33);
    glutAddMenuEntry("New mfe structure for base pairs", 47);
    glutAddMenuEntry("Close zoom window.", 4);
  }
  glutAddMenuEntry("Exit", 1);
  glutAttachMenu(GLUT_RIGHT_BUTTON);
}

void gfxinit_nontext(int window) { /* initalize graphics for each window */
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutReshapeFunc(reshape);
  make_menu(window);
}

/* Don't open zoom window until necessary */
void zoomSetWindow(void) {
  char    name_string[140];
  if (g_zoom_window_exists==FALSE) {
    g_zoom_window_exists = TRUE;
    glutInitWindowSize(750, 750);
    glutInitWindowPosition(6, 262);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
    strcpy(name_string, PROGRAM_NAME" Zoom window for ");
    strcat(name_string, g_plot_filename);
    win_zoom = glutCreateWindow(name_string);
    gfxinit_nontext(win_zoom); /*  initialize graphics for zoom window */
    glutDisplayFunc(display_zoom);
    glutEntryFunc(zoom_entry);
    glutMouseFunc(mouse_zoom);
    glutMotionFunc(mouse_motion_zoom);
    glutSpecialFunc(Key2);
    width_z = 800;
    height_z = 800;
  }
  if (g_zoom_window_displayed == TRUE) {
    glutSetWindow(win_zoom);
  } else {
    glutDestroyWindow(win_zoom); /* This is used just once by main */
    g_zoom_window_exists = FALSE;
  }
}
#endif

int main(int argc, char **argv) {
#ifdef HAVE_GLUTBITMAPCHARACTER
  char            name_string[140];
  int             i;
  initialize_data1(argc, argv);
  glutInit(&argc, argv);
  for (i = 1; i < argc; i++) {
    if (!strcmp(argv[i], "-mono")) {
      g_font_main = GLUT_BITMAP_HELVETICA_18;
      g_font_text = GLUT_BITMAP_HELVETICA_18;
      g_font_zoom = GLUT_BITMAP_HELVETICA_18;
    }
  }
  /* zoom window */
  width_z = width; /* Set initial sizes to those of main window */
  height_z = height;
  display_l = display_t = 1;
  display_r = display_b = g_length;
  display_w = display_h = g_length;
  zoomSetWindow();
  /* start of main window */
  glutInitWindowSize(800, 800);
  glutInitWindowPosition(474, 218);
  strcpy(name_string, PROGRAM_NAME" for ");
  strcat(name_string, g_plot_filename);
  if (g_prob) {
    if (g_mi_flag)
      strcat(name_string, " for Mutual Information ");
    else
      strcat(name_string, " for Probability");
  } else
    strcat(name_string, " for Energies");
  win_main = glutCreateWindow(name_string);
  glutMouseFunc(mouse);
  glutMotionFunc(mouse_motion);
  glutSpecialFunc(Key2_fast);
  gfxinit_nontext(win_main);	/* initialize graphics for Main Window */
  glutDisplayFunc(display_main);
  glutEntryFunc(main_entry);
  /* start of text window */
  glutInitWindowSize(800, 200);
  glutInitWindowPosition(6, 31);
  win_text = glutCreateWindow(PROGRAM_NAME", " PACKAGE_STRING);
  make_menu(win_text);
  glutDisplayFunc(display_text);
  glutReshapeFunc(text_reshape);
  glutEntryFunc(text_entry);
  glutKeyboardFunc(keyboard);
  glutMainLoop();		/* start of non-ending loop */
#endif
  printf("%s not available because glut was not found.\n", PROGRAM_NAME);
  printf("Use %s_ng instead.\n", PROGRAM_NAME);
  try_exit(3);
  return 0; /* Keep the stupid compiler happy. */
}
