#define PROGRAM_NAME "ct_boxplot"
#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <limits.h>
#include "util.h"

/* On the Mac, libglut does not exist separately. Search instead for
   the function glutInit. Under Linux, glut.h should be in
   /usr/include/GL and /usr/include is automatically searched.
*/
#ifdef HAVE_GLUTBITMAPCHARACTER
void zoomSetWindow(void);
#if HAVE_LIBGLUT
# include <GL/glut.h>
#else
# include <glut.h>
#endif

#include "ct_boxplot_general.h"
#if (defined(HAVE_LIBGD) || defined(HAVE_GDIMAGECREATE)) && defined(HAVE_GD_H)
# include "ct_boxplot_img.h"
#endif
#include "ct_boxplot_ps.h"
#include "ct_boxplot_read_ct.h"
#include "ct_boxplot_setcolor.h"

#define MESSAGE_NONE   0	/* values for g_type_message */
#define MESSAGE_POST   3
#define MESSAGE_GIF    4
#define MESSAGE_JPG    5
#define MESSAGE_PNG    6

int          *g_gray3_content;
struct helix  g_diag[MAXIMUM_HELICES];
int           g_diag_count[2 * MAXIMUM_SIZE];
int           g_big_dot_flag;
int           g_diag_start[2 * MAXIMUM_SIZE];
int           g_helices_in_plot_file;	/* number of entries in file */
int           len; /* length of RNA sequence, guess from input file */
char          g_sequence_name[120];
int           g_sequence_name_set;
float         g_color[COLOR_TABLE_SIZE][3];
int           g_clear_img_flag, g_ps_hollow_dot ;
int           g_gray3[3];/* for interpreted gray  0=5,1=6,2=7 */
int           width;		/* these are the current width and height of
				 * the main window */
int           height;		/* in pixels */
int           width_z, height_z;	/* same for zoom window */
int           g_forced_stripe_flag;
int           g_lower_structure = -2;	/* default to show partial
					 * and full overlap */
/* when right mouse button is clicked, transform coordinates into i,j */
int           g_i_row = 0;
int           g_j_col = 0;	/* for i.j base pair with mouse */

char          g_i_row_s[10], g_j_col_s[10]; /* string of above */
int           win_main, win_zoom, win_text; /* window indentifiers for glut */
int           g_text_window_size;
int           g_nongray_for_3_flag;
int           g_big_dot_post;

/* ______________________________________________________________
 * Notice that most functions are called based on events in the
 * event queue. They do not call each other.  This results in too many
 * global variables.                                          
 * Variables that could easily be changed for personal preference are 
 * usually accompanied by $$$.  Colors in particular.
 */

/* Font variables  ______________________________________________ */

void    *g_font_text = GLUT_BITMAP_TIMES_ROMAN_24; /* initialize font */ 
void    *g_font_zoom = GLUT_BITMAP_HELVETICA_18;
void    *g_font_main = GLUT_BITMAP_HELVETICA_18;

void    *fonts[] = { /* used by select_font */
  GLUT_BITMAP_HELVETICA_12,
  GLUT_BITMAP_HELVETICA_18,
  GLUT_BITMAP_TIMES_ROMAN_24
};

int             g_last_structure;	/* number of last structure */
/* It is the number of ct files specified + 1
 * colors 2 to g_last structure
 * are used for drawing dots */

/* Value to multiple by size of dots in all output */
float           g_main_dot_mag = 1.0;

/* for zoom */

/* Graphic  variables  ------------------------------------------- */

int     points_plotted, points_plotted_zoom; /* indicates number of
						points plotted in each
						window */  

/* Zoom Variables __________________________________________ */
int     g_region_set; /* 0 indicates the top left corner of the zoom
		       * region has not been set */ 
int     region_t, region_b;
int     region_r, region_l; /* specify boundaries of the zoom
			     * rectangle in i,j coordinates for main
			     * window or for the zoom window
			     */ 
float   region_real_l, region_real_r;
float   region_real_b, region_real_t; /* screen coordinates of above
					 (i,j) pairs  for the main
					 window  (blue rectangle) */ 
float   region_real_l_zoom, region_real_r_zoom;
float   region_real_t_zoom, region_real_b_zoom;
/* Screen coordinates  for a zoom rectangle within the zoom window */
int     saved_display_l, saved_display_r, saved_display_t, saved_display_b;
/* Above are saved versions of region_l, for going to previous zoom window */
int     first_zoom = 0;	/* for back up to previous zoom window */
int     g_region_set; /* nonzero indicates the top left corner of zoom
			 region has been set */ 
int     g_region_exists;
int     display_l, display_r, display_t, display_b, display_w;
int     display_h; /* left, right, top, bottom, width and height for
		      zoom window display in nucleotide coordinates */ 
int     g_zoom_labels_count;	/* the number of labels used */
int     g_zoom_labels_x[50];	/* x coordinate for label i */
int     g_zoom_labels_xl[50];	/* used to plot dot on screen */
int     g_zoom_labels_y[50];
int     g_zoom_labels_row[50];
int     g_zoom_labels_col[50];
char   *g_zoom_labels_string[50]; /* value to place at x,y */

/* File Variables _______________________________________________ */
char    rec[90];
FILE   *fp;		/* for .ct file */
char    g_post_filename[120];
char   *post_message = "Name for ps file (/ to abort)";
char   *img_message = "Enter name for gif file (/ to abort)";
char   *png_message = "Enter name for png file (/ to abort)";
char   *jpg_message = "Enter name for jpg file (/ to abort)";
char    text_window_message[150];
char    text_window_input[150];
int     data_entered = FALSE;	/* for text display window */
int     g_type_message;	/* 2 for energy increment, 0 for none */
int     g_message_set;
int     g_resolution;
int     g_post_file; /* for main or zoom window */
int     g_ct_files; /* number of ct files used for input */
char    filename_ct[MAXIMUM_CT_FILES][255]; /* allow at most MAXIMUM_CT_FILES
					     * filenames for input */ 
char   file_data_ct[MAXIMUM_CT_FILES][255]; /* first line of ct file */
char   filename[120];
char   g_gif_filename[120]; /* this depends on -o or input file */
char   g_png_filename[120]; /* this depends on -o or input file */
char   g_jpg_filename[120]; /* this depends on -o or input file */
int    g_grid_flag;	/* True for use grid lines, False otherwise */
int    g_zoom_window_exists = FALSE;
int    g_zoom_window_displayed = FALSE;

void display_all_helices(void) {
  int             i;
  for (i = 0; i < g_helices_in_plot_file; i++) {
    printf("Helix %d, row %d, column %d, color %d, length %d, diag %d or %d\n",
	   i, g_diag[i].row, g_diag[i].column, g_diag[i].color, 
	   g_diag[i].length, g_diag[i].row + g_diag[i].column - 1, 
	   g_diag[i].diagonal);
  }
  for (i = 1; i <= 2 * len; i++) {
    printf("Diagonal %d starts at %d\n", i, g_diag_start[i]);
  }
}

/* Initialization Routines for nongraphics */

void initialize_data2(void) {
  g_grid_flag = TRUE;
  width = 800;		/* initialize size of window in pixels */
  height = 800;		/* needed for scale in drawing windows */
  initialize_colors(g_color, g_ct_files);
  g_region_set = FALSE;
  g_forced_stripe_flag = FALSE;
  g_nongray_for_3_flag = FALSE;
  g_big_dot_flag = TRUE;
  g_region_exists = FALSE;
  g_message_set = FALSE;
  text_window_input[0] = '\0';
  text_window_message[0] = '\0';
  g_type_message = MESSAGE_NONE;
}

void process_input_files(void) {
  int             i;
  g_zoom_labels_count = 0;	/* initialize to no labels */
  for (i = 0; i < 50; i++) {	/* allocate memory for string */
    g_zoom_labels_string[i] = (char *) malloc(15 * sizeof(char));
    if (g_zoom_labels_string[i] == NULL) {
      printf("Error!\tInsufficient memory for zoom labels.\n");
      try_exit(1);
    }
  }
  len = 0;
  g_helices_in_plot_file = 0;
  read_of_all_ct_files(fp, g_diag_count, g_diag_start, g_ct_files,
		       &g_helices_in_plot_file, &len, filename_ct, g_diag, 
		       file_data_ct, g_sequence_name, g_sequence_name_set);
  if (g_ct_files == 3) {
    g_gray3_content = 
      store_gray_content(g_gray3_content, g_helices_in_plot_file,
			 len, g_diag, g_diag_start);
  }
  initialize_data2();
}

void null_arguments(void) {
  printf("Usage: %s [options] file1[.ct] file2[.ct] ...\nOptions:\n",
	 PROGRAM_NAME);
  printf("-v, -V (display version information)\n");
  printf("-h,    (display this information)\n");
  printf("-b     (Clear background for png/jpg output)\n");
  printf("-o <prefix> (Set output file prefix)\n");
  printf("-d <DIR> (ct files in directory DIR)\n");
  printf("-t <\"TITLE\">  (Specify title in double quotes.)\n");
  try_exit(0);
}

void initialize_data1(int argc, char **argv) {
  char argument[120], ct_directory[120], temp_file_name[140];
  int  i, ct_directory_flag;
  if (argc < 2) {
    printf("Error: data not specified\nRun '%s -h' for help\n", PROGRAM_NAME);
    try_exit(2);
  }
  g_clear_img_flag = FALSE;
  g_ps_hollow_dot = FALSE;
  g_sequence_name_set = FALSE;
  ct_directory_flag = FALSE;
  i = 1;
  g_ct_files = 0;
  strcpy(g_sequence_name, "Name not in first ct file?");
  /* set default filenames */
  strcpy(g_post_filename, "ct_boxplot_out");
  strcpy(g_gif_filename, "ct_boxplot_out");
  strcpy(g_png_filename, "ct_boxplot_out");
  strcpy(g_jpg_filename, "ct_boxplot_out");
  while (i <= (argc - 1)) {
    if (strcmp(argv[i], "-h") == 0) {
      null_arguments();
    } else if (strcmp(argv[i], "-v") == 0 || strcmp(argv[i], "-V") == 0) {
      printf("%s:\t%s\n",PROGRAM_NAME, PACKAGE_STRING);
      try_exit(0);
    } else if (strcmp(argv[i], "-o") == 0) {
      if (i < (argc - 1)) { /* make sure there is an argument to use */
	strcpy(g_post_filename, argv[i + 1]);
	strcpy(g_gif_filename, argv[i + 1]);
	strcpy(g_png_filename, argv[i + 1]);
	strcpy(g_jpg_filename, argv[i + 1]);
	printf("Set output files to %s\n", argv[i + 1]);
      }
      i += 2;
    } else if (strcmp(argv[i], "-b") == 0) {
      g_clear_img_flag = TRUE;
      i++;
    } else if (strcmp(argv[i], "-H") == 0) {
      g_ps_hollow_dot = TRUE;
      i++;
    } else if (strcmp(argv[i], "-d") == 0) {
      if (i < (argc - 1)) { /* make sure there is an argument to use */
	strcpy(ct_directory, argv[i + 1]);
	ct_directory_flag = TRUE;
      }
      i += 2;
    } else if (strcmp(argv[i], "-t") == 0) {
      if (i < (argc - 1)) {
	g_sequence_name_set = TRUE;
	strcpy(g_sequence_name, argv[i + 1]);
	i += 2;
      }
    } else {
      strcpy(argument, argv[i]);
      if (argument[0] == '-') {
	printf("Error!\t%s was not recognized.\n", argument);
	printf("Filenames cannot begin with a hyphen.\n");
	i++;
	printf("Run '%s -h' for help\n", PROGRAM_NAME);
	try_exit(3);
      } else {
	strcpy(filename_ct[g_ct_files], argv[i]);
	if (g_ct_files < MAXIMUM_CT_FILES)
	  g_ct_files++;
	else {
	  printf("Warning!\tToo many ct files.\n");
	  printf("Only %d files will be used.\n", MAXIMUM_CT_FILES);
	}
	i++;
      }
    }
  }
  for (i = 0; i < g_ct_files; i++) {
    chop_suffix(filename_ct[i], ".ct");
    strcat(filename_ct[i], ".ct");
    if (ct_directory_flag) {
      strcpy(temp_file_name, ct_directory);
      strcat(temp_file_name, filename_ct[i]);
      strcpy(filename_ct[i], temp_file_name);
    }
  }
  g_last_structure = g_ct_files + 1;
  strcat(g_post_filename, ".ps");
  strcat(g_gif_filename, ".gif");
  strcat(g_png_filename, ".png");
  strcat(g_jpg_filename, ".jpg");
  process_input_files();
}

/* Menu functions
 * Most are defined throught GLUT and called from the event queue
 */

void keyboard(unsigned char key, int x, int y) {
  int             string_len1;	/* accepts keys from text window */
  int             string_len2;
  x = y;
  y = x;			/* get rid of warning message */
  if (key == 13) {
    data_entered = TRUE;	/* on <RETURN> mark end of text */
    glutSetWindow(win_text);
    glutPostRedisplay();
    return;
  }
  string_len1 = (int) strlen(text_window_input);
  string_len2 = (int) strlen(text_window_message);
  if (string_len1 > 0)
    if (key == 8) {	/* process backspace or delete */
      text_window_input[string_len1 - 1] = '\0';
      text_window_message[string_len2 - 1] = '\0';
      glutSetWindow(win_text);
      glutPostRedisplay();
      return;
    }
  text_window_input[string_len1] = key;	/* add key to end of input */
  text_window_input[string_len1 + 1] = '\0';
  text_window_message[string_len2] = key;	/* add key to end of message */
  text_window_message[string_len2 + 1] = '\0';
  glutSetWindow(win_text);
  glutPostRedisplay();
}

void fixcolor_plot(int color) {	/* for the screen */
  if ((color >= 0) && (color <= (MAXIMUM_CT_FILES + 5)))
    glColor3f(g_color[color][0], g_color[color][1],
	      g_color[color][2]);
}

void select_font(int newfont) {
  g_font_main = fonts[newfont];
  g_font_zoom = fonts[newfont];
  g_font_text = fonts[newfont];
  glutSetWindow(win_text);
  glutPostRedisplay();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutPostRedisplay();
}

void select_font_main(int newfont) {
  g_font_main = fonts[newfont];
  glutPostRedisplay();
}

void select_font_zoom(int newfont) {
  g_font_zoom = fonts[newfont];
  glutPostRedisplay();
}

void select_font_text(int newfont) {
  g_font_text = fonts[newfont];
  glutPostRedisplay();
}

void fix_cursor(void) {	/* set cursor back to crosshairs */
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutPostRedisplay();
}

void set_cursor_help(void) {
  glutSetWindow(win_main);
  glutSetCursor(GLUT_CURSOR_HELP);
  glutPostRedisplay();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutSetCursor(GLUT_CURSOR_HELP);
    glutPostRedisplay();
  }
}

/* from menu, indicate to  create postcript file for window */
void create_post1(int which_window) {
  if (which_window > 5) {
    g_big_dot_post = TRUE;
    which_window -= 10;
  } else
    g_big_dot_post = FALSE;
  if ((which_window == 1) || (g_zoom_window_displayed == FALSE)) {
    g_post_file = MAIN_POST;	/* create post for main */
  } else {
    g_post_file = ZOOM_POST;	/* create post for zoom */
  }
  strcpy(text_window_message, post_message);
  strcat(text_window_message, ", default ");
  strcat(text_window_message, g_post_filename);
  strcat(text_window_message, ")? ");
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_POST;
  glutSetWindow(win_text);
  glutPostRedisplay();
  set_cursor_help();
}

void create_post2(void) {
  int             name_len;
  int             dis_l, dis_r, dis_t, dis_b;
  name_len = strlen(text_window_input);
  if ((text_window_input[0] == '/') && (name_len < 3)) {
    text_window_input[0] = '\0';
    text_window_message[0] = '\0';
    glutSetWindow(win_text);
    glutPostRedisplay();
    fix_cursor();
    return;
  }
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutSetCursor(GLUT_CURSOR_WAIT);
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutSetCursor(GLUT_CURSOR_WAIT);
  glutPostRedisplay();
  if (strlen(text_window_input) > 0) {
    strcpy(g_post_filename, text_window_input);
    chop_suffix(g_post_filename, ".ps");
    strcat(g_post_filename, ".ps");
  }
  text_window_input[0] = '\0';
  text_window_message[0] = '\0';
  if (g_post_file == MAIN_POST) {
    dis_l = 1;
    dis_t = 1;
    dis_r = len;
    dis_b = len;
    g_forced_stripe_flag = FALSE;
  } else {		/* create post for zoom */
    dis_l = display_l;
    dis_t = display_t;
    dis_r = display_r;
    dis_b = display_b;
  }
  finish_making_post(g_big_dot_post, g_post_file, g_post_filename, 
		     g_zoom_labels_count, dis_l, dis_r, dis_t, dis_b, g_color,
		     g_zoom_labels_row, g_zoom_labels_col, 
		     g_zoom_labels_string, g_grid_flag, g_last_structure, 
		     g_diag_start, g_diag_count, g_diag, len, 
		     g_lower_structure, g_sequence_name, file_data_ct, 
		     g_main_dot_mag, g_forced_stripe_flag, 
		     g_nongray_for_3_flag, g_gray3_content);
  glutSetWindow(win_text);
  glutPostRedisplay();
  fix_cursor();
}

void lower_structure_select(int which_structure) {
  g_lower_structure = which_structure;
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutPostRedisplay();
  glutSetWindow(win_text);
  glutPostRedisplay();
}


/* gif creation menu, called from pop up window */
void create_gif1(int choice) {
  if (choice == 1)
    g_resolution = 72;
  else if (choice == 2)
    g_resolution = 110;
  else if (choice == 3)
    g_resolution = 200;
  else
    g_resolution = 300;
  strcpy(text_window_message, img_message);
  strcat(text_window_message, ", default ");
  strcat(text_window_message, g_gif_filename);
  strcat(text_window_message, ")? ");
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_GIF;
  glutSetWindow(win_text);
  glutPostRedisplay();
  set_cursor_help();
}

void create_png1(int choice) {
  if (choice == 1)
    g_resolution = 72;
  else if (choice == 2)
    g_resolution = 110;
  else if (choice == 3)
    g_resolution = 200;
  else
    g_resolution = 300;
  strcpy(text_window_message, png_message);
  strcat(text_window_message, ", default ");
  strcat(text_window_message, g_png_filename);
  strcat(text_window_message, ")? ");
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_PNG;
  glutSetWindow(win_text);
  glutPostRedisplay();
  set_cursor_help();
}

void create_jpg1(int choice) {
  if (choice == 1)
    g_resolution = 72;
  else if (choice == 2)
    g_resolution = 110;
  else if (choice == 3)
    g_resolution = 200;
  else
    g_resolution = 300;
  strcpy(text_window_message, jpg_message);
  strcat(text_window_message, ", default ");
  strcat(text_window_message, g_jpg_filename);
  strcat(text_window_message, ")? ");
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_JPG;
  glutSetWindow(win_text);
  glutPostRedisplay();
  set_cursor_help();
}

void create_img2(int message) {
  int             png_mode;
  int             jpg_mode;
  char            out_filename[120];
  png_mode = FALSE;
  jpg_mode = FALSE;
  if (message == MESSAGE_PNG)
    png_mode = TRUE;
  if (message == MESSAGE_JPG)
    jpg_mode = TRUE;
  if (text_window_input[0] == '/') {
    text_window_input[0] = '\0';
    text_window_message[0] = '\0';
    glutSetWindow(win_text);
    glutPostRedisplay();
    fix_cursor();
    return;
  }
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutSetCursor(GLUT_CURSOR_WAIT);
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutSetCursor(GLUT_CURSOR_WAIT);
  glutPostRedisplay();
  if ((!png_mode) && (!jpg_mode)) {
    if (strlen(text_window_input) > 0) {
      strcpy(g_gif_filename, text_window_input);
      chop_suffix(g_gif_filename, ".img");
      strcat(g_gif_filename, ".gif");
    }
    strcpy(out_filename, g_gif_filename);
  } else if (jpg_mode) {
    if (strlen(text_window_input) > 0) {
      strcpy(g_jpg_filename, text_window_input);
      chop_suffix(g_jpg_filename, ".img");
      strcat(g_jpg_filename, ".jpg");
    }
    strcpy(out_filename, g_jpg_filename);
  } else if (png_mode) {
    if (strlen(text_window_input) > 0) {
      strcpy(g_png_filename, text_window_input);
      chop_suffix(g_png_filename, ".png");
      strcat(g_png_filename, ".png");
    }
    strcpy(out_filename, g_png_filename);
  }
  text_window_input[0] = '\0';
  text_window_message[0] = '\0';
#if (defined(HAVE_LIBGD) || defined(HAVE_GDIMAGECREATE)) && defined(HAVE_GD_H)
  finish_making_img(TRUE, out_filename, g_resolution, g_zoom_labels_count,
		    g_zoom_labels_row, g_zoom_labels_col, g_grid_flag, 
		    g_last_structure, g_diag, g_color, display_l, display_r, 
		    display_t, display_b, g_lower_structure, g_sequence_name,
		    g_main_dot_mag, g_diag_start, g_diag_count, file_data_ct,
		    g_clear_img_flag, FALSE, len, g_forced_stripe_flag, 
		    g_nongray_for_3_flag, g_gray3_content, png_mode, jpg_mode);
#endif
  glutSetWindow(win_text);
  glutPostRedisplay();
  fix_cursor();
}

float g_magnify_options[] = {
  0.5, 1.0, 1.5, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 12.0, 14.0, 
  16.0, 18.0, 20.0, 25.0, 30.0, 35.0, 40.0, 45.0, 50.0, 60.0, 100.0 };

void magnify_menu(int choice) {
  g_main_dot_mag = g_magnify_options[choice];
  glutSetWindow(win_main);
  glutPostRedisplay();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutPostRedisplay();
  }
}

/* Plotting functions _____________________ */

void sniff_for_opengl_errors(char *message) {
  GLenum          errCode;
  const GLubyte  *errString;
  const char     *my_string;
  if ((errCode = glGetError()) != GL_NO_ERROR) {
    errString = gluErrorString(errCode);
    my_string = (char *) errString;
    fprintf(stderr, "GL error: %s at %s \n", my_string, message);
  }
}


void plot_ij(int i_row, int j_col) { /* converts i,j base pair to real
					coordinates  & plots them */ 
  float           real_x, real_y;	
  real_x = 23.0 + (float) ((j_col + .5) * 700. / (len + 1));
  real_y = 749.0 - (float) ((i_row - .5) * 700. / (len + 1));
  glVertex2f(real_x, real_y);
}

void plot_ij_zoom(int i_row, int j_col, int color, int adjust_size) {
  /* same for zoom window */
  float           mag;
  float           adjuster; /* makes different colors different sizes */
  float           real_x, real_y, x_offset, y_offset;
  real_x = 25. + (float) (j_col - display_l + 1) * 700. / (display_w + 1);
  real_y = 50. + (float) (display_b - i_row + 1) * 700. / (display_h + 1);
  x_offset = (25.+(float) (j_col+2-display_l)*700./(display_w + 1)-real_x)/2;
  y_offset = (50.+(float) (display_b-i_row+2)*700./(display_h+1)-real_y)/2;
  mag = g_main_dot_mag;
  if (adjust_size != TRUE) {
    glBegin(GL_POINTS);
    glVertex2f(real_x, real_y);
    glEnd();
    sniff_for_opengl_errors("plotting  a zoom point");
    return;
  }
  x_offset = (25.+(float) (j_col+2-display_l)*700./(display_w+1)-real_x)/2.;
  y_offset = (50.+(float) (display_b-i_row+2)*700./(display_h+1)-real_y)/2.;
  if (x_offset < g_ct_files)
    x_offset = g_ct_files;
  if (adjust_size == TRUE)
    adjuster = (color - 1) * 1. / (float) (g_last_structure - 1);
  else {
    adjuster = 1.0;
  }
  glBegin(GL_POLYGON);	
  /* bot, mid */
  glVertex2f(real_x + x_offset*adjuster*mag, real_y - y_offset*adjuster*mag);
  /* bot, left */
  glVertex2f(real_x - x_offset*g_main_dot_mag, real_y - y_offset * mag);
  /* top, mid */
  glVertex2f(real_x - x_offset*adjuster*mag, real_y + y_offset*adjuster*mag);
  glVertex2f(real_x + x_offset*mag, real_y + y_offset * mag);	/* top,right */
  glEnd();
  sniff_for_opengl_errors("plotting a rectangle from plot_ij_zoom");
}

int plot_i_zoom(int i_row) {	/* convert i_row to y coordinate */
  return (int) (50. + (float) (display_b - i_row + 1)*700./(display_h + 1));
}

int plot_j_zoom(int j_col) { /* convert col to x coordinate */
  return (int) (25.2 + (float) (j_col-display_l+1)*700./(display_w + 1));
}

int plot_i(int i_row) {	/* converts row to y coordinate */
  return (int) (749.0 - (float) ((i_row - .5) * 700. / (len + 1)) + .5);
}

int plot_j(int j_col) {	/* converts col to x coordinate */
  return (int) (23.0 + (float) ((j_col + .5) * 700. / (len + 1)) + .5);
}

void plot_points_zoom(int stripe_flag, int counter[MAXIMUM_CT_FILES + 1],
		      int *lower_basepairs, int gray3[3]) {
  int             row, col, new_row, count;
  int             diag, position, old_color, location, new_color;
  int             start_diag, end_diag;
  int             starting_color, ending_color;
  int             n_col;
  int             column;
  int             i;
  int             count_of_lower_basepairs;
  gray3[0] = 0;
  gray3[1] = 0;
  gray3[2] = 0;
  count_of_lower_basepairs = 0;
  old_color = g_last_structure;
  points_plotted_zoom = 0;
  start_diag = display_t + display_l - 1;
  end_diag = display_b + display_r - 1;
  for (i = 0; i <= g_last_structure; i++)
    counter[i] = 0;
  sniff_for_opengl_errors("start of plot_points_zoom");
  for (old_color = g_last_structure; old_color >= 2; old_color--) {
    fixcolor_plot(old_color);
    for (diag = start_diag; diag <= end_diag; diag++) {
      position = g_diag_start[diag];
      for (count = 0; ((count < g_diag_count[diag]) && 
		       (g_diag[position + count].row <= display_b));
	   count++) {
	location = position + count;
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	if (col >= display_l) {
	  new_color = g_diag[location].color;
	  if (new_color == old_color) {
	    for (row = new_row; row < new_row + g_diag[location].length;
		 row++) {
	      if ((row >= display_t) && (row <= display_b)
		  && (col >= display_l) && (col <= display_r)) {
		plot_ij_zoom(row, col, old_color, stripe_flag);
		points_plotted_zoom++;
		counter[old_color]++;
	      }
	      col--;
	    }
	  }
	}
      }
    }
  }
  sniff_for_opengl_errors("After single structure, before overlap in top");
  if ((g_ct_files > 2) && (!stripe_flag))
    starting_color = COL_PART_OVERLAP;
  else
    starting_color = COL_COMPLETE_OVERLAP;
  for (old_color = starting_color; old_color >= 0; old_color--) {
    fixcolor_plot(old_color);
    if ((old_color == 1) && (g_nongray_for_3_flag))
      fixcolor_plot(5);	/* for overlap of 1 and 2 */
    for (diag = start_diag; diag <= end_diag; diag++) {
      position = g_diag_start[diag];
      for (count = 0; ((count < g_diag_count[diag])
		       && (g_diag[position + count].row <= display_b));
	   count++) {
	location = position + count;
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	if (col >= display_l) {
	  new_color = g_diag[location].color;
	  if (new_color == old_color) {
	    for (row = new_row; row < new_row + g_diag[location].length;
		 row++) {
	      if ((row >= display_t) && (row <= display_b)
		  && (col >= display_l) && (col <= display_r)) {
		plot_ij_zoom(row, col, old_color, FALSE);
		counter[old_color]++;
		if ((g_nongray_for_3_flag) && (old_color == 1))
		  gray3[0]++;
	      }
	      col--;
	    }
	  }
	}
      }
    }
  }
  sniff_for_opengl_errors("about to plot points in lower triangle");
  if (g_lower_structure != -10) {
    old_color = g_lower_structure;
    if (g_lower_structure != -1) {
      if (g_lower_structure == -2) {
	starting_color = COL_PART_OVERLAP;
	ending_color = COL_COMPLETE_OVERLAP;
      } else {
	if (g_lower_structure == -12) {
	  starting_color = COL_COMPLETE_OVERLAP;
	  ending_color = COL_COMPLETE_OVERLAP;
	} else {
	  starting_color = g_lower_structure;
	  ending_color = g_lower_structure;
	}
      }
      for (old_color=starting_color; old_color >= ending_color; old_color--) {
	fixcolor_plot(old_color);
	for (diag = start_diag; diag <= end_diag; diag++) {
	  position = g_diag_start[diag];
	  for (count = 0; ((count < g_diag_count[diag])	&&
			   (g_diag[position + count].row <= display_r)); 
	       count++) {
	    location = position + count;
	    new_row = g_diag[location].row;
	    col = g_diag[location].column;
	    if (col >= display_t) {
	      new_color = g_diag[location].color;
	      if (new_color == old_color) {
		for (row = new_row;
		     row < new_row + g_diag[location].length;
		     row++) {
		  if ((col >= display_t) && (col <= display_b)
		      && (row >= display_l) && (row <= display_r)) {
		    plot_ij_zoom(col, row, old_color, FALSE);
		    count_of_lower_basepairs++;
		  }
		  col--;
		}
	      }
	    }
	  }
	}
      }
    }
  }
  if (g_lower_structure <= -10) { /* plot lower stucture for 3
				     structures, interpreted */ 
    for (diag = start_diag; diag <= end_diag; diag++) {
      position = g_diag_start[diag];
      for (count = 0; ((count < g_diag_count[diag]) && 
		       (g_diag[position + count].row <= display_r)); count++) {
	location = position + count;
	if (g_diag[position + count].color == COL_PART_OVERLAP) {
	  new_row = g_diag[location].row;
	  column = g_diag[location].column;
	  if (column >= display_t) {
	    col = g_gray3_content[row];
	    for (row = new_row; row < new_row + g_diag[location].length;
		 row++) {
	      if ((column >= display_t) && (column <= display_b)
		  && (row >= display_l) && (row <= display_r)) {
		col = g_gray3_content[row];
		if (col == 0)
		  old_color = 5;
		else {
		  if (col < 0) {
		    old_color = 6;
		  } else {
		    old_color = 7;
		  }
		}
		fixcolor_plot(old_color);
		count_of_lower_basepairs++;
		plot_ij_zoom(column, row, COL_PART_OVERLAP, FALSE);
		if (!g_nongray_for_3_flag)
		  gray3[old_color - 5]++;
	      }
	      column--;
	    }
	  }
	}
      }
    }
  }
  /* plot gray interpreted in upper triangle */
  if (g_nongray_for_3_flag) {
    for (row = display_t; row <= display_b; row++) {
      col = g_gray3_content[row];
      if (col != 0) {
	if (col < 0)
	  n_col = -1 * col;
	else
	  n_col = col;
	if ((n_col >= display_l) && (n_col <= display_r)) {
	  if (col > 0) {
	    fixcolor_plot(7);	/* for seq 2 & 3 */
	    gray3[2]++;
	  } else {
	    fixcolor_plot(6);	/* for seq 1 & 3 */
	    gray3[1]++;
	  }
	  plot_ij_zoom(row, n_col, 1, FALSE);
	}
      }
    }
    gray3[0] = gray3[0] - gray3[2] - gray3[1];
  }
  *lower_basepairs = count_of_lower_basepairs;
  sniff_for_opengl_errors("at end of plot points zoom");
}

int plot_points_main(int lower_count, int counter[MAXIMUM_CT_FILES + 1]) {
  int             row, col, column, n_col, new_row;
  int             starting_point;
  int             starting_color;
  int             ending_color;
  int             i;
  int             old_color, location, new_color;
  for (i = 0; i <= g_last_structure; i++)
    counter[i] = 0;
  old_color = 1;
  /* fixcolor_plot(g_last_structure); */
  g_gray3[0] = 0;
  g_gray3[1] = 0;
  g_gray3[2] = 0;
  glBegin(GL_POINTS);
  points_plotted = 0;
  /* dots for each structure in upper triangle */
  for (old_color = g_last_structure; old_color >= 2; old_color--) {
    fixcolor_plot(old_color);
    for (location = 0; location < g_helices_in_plot_file; location++) {
      new_color = g_diag[location].color;
      if (new_color == old_color) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	for (row = new_row; row < new_row + g_diag[location].length; row++) {
	  plot_ij(row, col);
	  col--;
	  counter[new_color]++;
	  points_plotted++;
	}
      }
    }
  }
  /* Draw black dots and gray if necessary */
  if (g_last_structure > 3)
    starting_point = 1;
  else
    starting_point = 0;
  for (old_color = starting_point; old_color >= 0; old_color--) {
    if ((old_color == 1) && (g_nongray_for_3_flag))
      fixcolor_plot(5);	/* for overlap of 1 and 2 */
    else
      fixcolor_plot(old_color);
    for (location = 0; location < g_helices_in_plot_file; location++) {
      new_color = g_diag[location].color;
      if (new_color == old_color) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	for (row = new_row; row < new_row + g_diag[location].length; row++) {
	  plot_ij(row, col);
	  counter[old_color]++;
	  col--;
	}
      }
    }
  }
  if (g_lower_structure != -10) {
    if (g_lower_structure != -1) {
      if (g_lower_structure == -2) {
	starting_color = 1;
	ending_color = 0;
      } else {
	if (g_lower_structure == -12) {
	  starting_color = 0;
	  ending_color = 0;
	} else {
	  starting_color = g_lower_structure;
	  ending_color = g_lower_structure;
	}
      }
      /* Draw a single color below the main diagonal */
      for (old_color = starting_color; old_color >= ending_color;old_color--) {
	fixcolor_plot(old_color);
	for (location = 0; location < g_helices_in_plot_file; location++) {
	  new_color = g_diag[location].color;
	  if (new_color == old_color) {
	    new_row = g_diag[location].row;
	    col = g_diag[location].column;
	    for (row = new_row; row < new_row + g_diag[location].length;
		 row++) {
	      plot_ij(col, row);
	      lower_count++;
	      col--;
	    }
	  }
	}
      }
    }
  }
  if (g_lower_structure <= -10) {
    g_gray3[0] = 0;
    g_gray3[1] = 0;
    g_gray3[2] = 0;
    for (location = 0; location < g_helices_in_plot_file; location++) {
      new_color = g_diag[location].color;
      if (new_color == COL_PART_OVERLAP) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	for (row = new_row; row < new_row + g_diag[location].length; row++) {
	  column = g_gray3_content[row];
	  if (column == 0)
	    old_color = 5;
	  else {
	    if (column < 0) {
	      old_color = 6;
	    } else {
	      old_color = 7;
	    }
	  }
	  fixcolor_plot(old_color);
	  plot_ij(col, row);
	  g_gray3[old_color - 5]++;
	  lower_count++;
	  col--;
	}
      }
    }
  }
  /* plot gray interpreted */
  if (g_nongray_for_3_flag) {
    for (row = 1; row <= len; row++) {
      col = g_gray3_content[row];
      if (col != 0) {
	if (col < 0)
	  n_col = -1 * col;
	else
	  n_col = col;
	if (col > 0) {
	  fixcolor_plot(7);	/* for seq 2 & 3 */
	  if (g_lower_structure != -10)
	    g_gray3[2]++;
	} else {
	  fixcolor_plot(6);	/* for seq 1 & 3 */
	  if (g_lower_structure != -10)
	    g_gray3[1]++;
	}
	plot_ij(row, n_col);
      }
    }
    if (g_lower_structure != -10)
      g_gray3[0] = counter[1] - g_gray3[1] - g_gray3[2];
  }
  glEnd();
  return lower_count;
}

/* display string at real coordinates  x,y  */
void output_main(int x, int y, char *string) { 
  int             len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(g_font_main, string[i]);
  }
}

/* display string at real coordinates  x,y  */
void output_zoom(int x, int y, char *string) {
  int             len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(g_font_zoom, string[i]);
  }
}

/* display string at real coordinates  x,y  */
void output_text(int x, int y, char *string) {
  int             len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(g_font_text, string[i]);
  }
}

void single_hor_tic(int j_col, int grid_flag) {
  char            string2[20];
  float           x;
  char            temp_string[50];
  x = (float) plot_j(j_col);
  glBegin(GL_LINES);
  glVertex2f(x + .25, 751.);
  glVertex2f(x + .25, 758.);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COL_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(x + .25, 750);
    glVertex2f(x + .25, 43);
    glEnd();
    fixcolor_plot(COL_TEXT);
  }
  strcpy(string2, num_string_int(j_col, temp_string));
  output_main(plot_j(j_col) - 10, 762, string2);
}

void plot_hor_tics(void) {
  int             j_col, step, start;
  step = step_fun(1, len);
  start = start_fun(1, step);
  single_hor_tic(len, FALSE);
  single_hor_tic(1, FALSE);
  for (j_col = start; j_col < (len - 2 * step / 3); j_col = j_col + step) {
    single_hor_tic(j_col, g_grid_flag);
  }
}

void single_ver_tic(int i_row, int grid_flag) {
  char            string2[20];
  char            temp_string[50];
  float           y;
  y = (float) plot_i(i_row);
  glBegin(GL_LINES);
  glVertex2f(727., y + .25);
  glVertex2f(736., y + .25);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COL_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(20, y + .25);
    glVertex2f(725., y + .25);
    glEnd();
    fixcolor_plot(COL_TEXT);
  }
  strcpy(string2, num_string_int(i_row, temp_string));
  output_main(737, plot_i(i_row) - 5, string2);
}

void plot_ver_tics(void) {
  int             i_row, step, start;
  step = step_fun(1, len);
  start = start_fun(1, step);
  single_ver_tic(1, FALSE);
  single_ver_tic(len, FALSE);
  for (i_row = start; i_row < (len - step / 2); i_row = i_row + step) {
    single_ver_tic(i_row, g_grid_flag);
  }
}

void single_hor_tic_zoom(int j_col, int grid_flag) {
  char            string2[10];
  char            temp_string[50];
  float           x;
  x = (float) plot_j_zoom(j_col);
  glBegin(GL_LINES);
  glVertex2f(x, 751);
  glVertex2f(x, 758);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COL_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(x, 749);
    glVertex2f(x, 50);
    glEnd();
    fixcolor_plot(COL_TEXT);
  }
  strcpy(string2, num_string_int(j_col, temp_string));
  output_zoom(plot_j_zoom(j_col) - 10, 762, string2);
}

void plot_hor_tics_zoom(void) {
  int             j_col, step;
  int             start;
  step = step_fun(display_l, display_r);
  start = start_fun(display_l, step);
  single_hor_tic_zoom(display_r, FALSE);
  single_hor_tic_zoom(display_l, FALSE);
  for (j_col = start; j_col < (display_r - 2 * step / 3);
       j_col = j_col + step) {
    single_hor_tic_zoom(j_col, g_grid_flag);
  }
}

void single_ver_tic_zoom(int i_row, int grid_flag) {
  char            string2[20];
  char            temp_string[50];
  float           y;
  y = (float) plot_i_zoom(i_row);
  glBegin(GL_LINES);
  glVertex2f(727., y);
  glVertex2f(736., y);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COL_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(24., y);
    glVertex2f(723., y);
    glEnd();
    fixcolor_plot(COL_TEXT);
  }
  strcpy(string2, num_string_int(i_row, temp_string));
  output_zoom(737, plot_i_zoom(i_row) - 5, string2);
}

void plot_ver_tics_zoom(void) {
  int             i_row, step, start;
  step = step_fun(display_t, display_b);
  start = start_fun(display_t, step);
  single_ver_tic_zoom(display_t, FALSE);
  single_ver_tic_zoom(display_b, FALSE);
  for (i_row = start; i_row < (display_b - step / 2); i_row = i_row + step) {
    single_ver_tic_zoom(i_row, g_grid_flag);
  }
}

void plot_tics_zoom(void) {
  plot_hor_tics_zoom();
  plot_ver_tics_zoom();
  return;
}

void plot_tics(void) {
  plot_hor_tics();
  plot_ver_tics();
  return;
}

int window_x_convert(int x) {
  /* Convert window x coordinate to column coordinate of RNA.
     top left for window is 0,0; positivie x is right, positive y is down, 
     i is down, j is right for RNA */
  float xf;
  xf = x - 23. * width / 800.;
  xf = xf * (len + 1) / 700. * 800. / width;
  return (int) (xf);
}

int window_y_convert(int y) {	/* convert window y coordinates to */
  float           yf;	/* to row coordinates */
  int             i_row;
  yf = (float) (y * 800. / height); /* scale according to window size */
  yf = 799 - yf; /* origin is at top in window, bottom in opengl */
  i_row=(int)((len+1.)*(749.-yf)/700.+1.0); /* inverse of plot_ij function */
  /* The 1.0 above should be .5 ? but 1.0 works much better */
  return i_row;
}

int window_x_convert_zoom(int x) {/* converts window x coordinates to */
  float           xf;
  xf = x-25.*width_z/800.; /* subtract offset to right depending on
			      window size */ 
  xf = xf*(display_w+1)/700.*800./width_z + display_l - 1;	
  return (int) (xf + .5);
}

int window_y_convert_zoom(int y) { /* convert window y coordinates to */
  float           yf;	/* RNA i coordinates down */
  yf = y - 50. * height_z / 800;	/* subtract offset down depending on
					 * window size */
  yf = yf * (display_h + 1) / 700 * 800. / height_z + display_t - 1;
  return (int) (yf + .5);
}

/* display functions ___________________________________________________ */

void display_text(void) {
  int             color, offset, stop;
  int             last_color;
  char            location[160];
  char            text_message[150];
  char            base_text[2];
  glClear(GL_COLOR_BUFFER_BIT);
  fixcolor_plot(COL_TEXT);
  strcpy(location, " Name: ");
  strcat(location, g_sequence_name);
  output_text(25, g_text_window_size - 25, location);
  if (g_i_row > 0) {
    strcpy(location, "(i,j) base pair =(");
    strcat(location, g_i_row_s);
    strcat(location, ",");
    strcat(location, g_j_col_s);
    strcat(location, ")");
    fixcolor_plot(COL_TEXT);
    output_text(5, g_text_window_size - 115, location);
    strcpy(location, "(");
    strcpy(base_text, "a");
    base_text[0] = sequence[g_i_row - 1];
    strcat(location, base_text);
    strcat(location, ",");
    base_text[0] = sequence[g_j_col - 1];
    strcat(location, base_text);
    strcat(location, ")");
    output_text(90, g_text_window_size - 135, location);
  }
  /* display color scheme */
  fixcolor_plot(COL_COMPLETE_OVERLAP);
  if (color_read(g_i_row, g_j_col, COL_COMPLETE_OVERLAP,
		 g_diag_start, g_diag) == TRUE)
    output_text(5, g_text_window_size - 50, "**");
  output_text(20, g_text_window_size - 50, "Full Overlap");
  if (g_last_structure > 3) {
    fixcolor_plot(COL_PART_OVERLAP);
    if (color_read(g_i_row, g_j_col, COL_PART_OVERLAP,
		   g_diag_start, g_diag) == TRUE)
      output_text(5, g_text_window_size - 75, "**");
    output_text(20, g_text_window_size - 75, "Partial Overlap");
  }
  if (g_last_structure <= 82) {
    stop = g_last_structure / 2 + 1;
    last_color = g_last_structure;
  } else {
    stop = 82 / 2 + 1;
    last_color = 83;
  }
  for (color = 2, offset = 0; color <= stop;
       color++, offset = offset + 25) {
    fixcolor_plot(color);
    if (color_read(g_i_row, g_j_col, color,
		   g_diag_start, g_diag) == TRUE)
      output_text(180, g_text_window_size - 50 - offset, "**");
    strcpy(location, file_data_ct[color - 2]);
    output_text(200, g_text_window_size - 50 - offset, location);
  }
  for (color = (stop + 1), offset = 0; color <= last_color;
       color++, offset = offset + 25) {
    fixcolor_plot(color);
    if (color_read(g_i_row, g_j_col, color,
		   g_diag_start, g_diag) == TRUE)
      output_text(410, g_text_window_size - 50 - offset, "**");
    strcpy(location, file_data_ct[color - 2]);
    output_text(430, g_text_window_size - 50 - offset, location);
  }
  /* display color for interpreted gray */
  if ((g_nongray_for_3_flag) || (g_lower_structure <= -10)) {
    fixcolor_plot(5);
    output_text(430, 90, "1 & 2");
    fixcolor_plot(6);
    output_text(430, 70, "1 & 3");
    fixcolor_plot(7);
    output_text(430, 50, "2 & 3");
  }
  if (g_message_set == TRUE) {
    glutPopWindow();
    fixcolor_plot(COL_TEXT);
    strcpy(text_message, text_window_message);
    strcat(text_message, "_");
    output_text(10, 5, text_message);
    if (data_entered == TRUE) {
      data_entered = FALSE;
      g_message_set = FALSE;
      if (g_type_message == MESSAGE_POST)
	create_post2();
      if (g_type_message >= MESSAGE_GIF)
	create_img2(g_type_message);
    }
  }
  sniff_for_opengl_errors("at end of display_text");
  glutSwapBuffers();
}

void display_main(void) {
  char            location[60];
  char            temp_string[50];
  int             lower_count;
  int             counter[MAXIMUM_CT_FILES + 1];
  int             i;
  float           l, t, r, b;
  float           point_size;	/* size of points to plot within box */
  l = (float) plot_j(1) + .25;
  t = (float) plot_i(1) + .25;
  r = (float) plot_j(len) + .25;
  b = (float) plot_i(len) + .25;
  lower_count = 0;	/* the number of dots in the lower triangle */
  glClear(GL_COLOR_BUFFER_BIT);
  fixcolor_plot(COL_TEXT);
  glLineWidth(1.0);
  plot_tics();
  glLineWidth(1.0);
  glBegin(GL_LINES);	/* draw diagonal line */
  glVertex2f(l, t);
  glVertex2f(r, b);
  glEnd();
  glLineWidth(2.0);
  fixcolor_plot(COL_TEXT);
  glBegin(GL_LINE_LOOP);	/* draw triangle */
  glVertex2f(18.0, 752.0);
  glVertex2f(18.0, 43.0);
  glVertex2f(726.0, 43.0);
  glVertex2f(726.0, 752.0);
  glEnd();
  point_size = 700. / (len + 1) * height / 800. + .5;
  point_size = point_size * g_main_dot_mag;
  if (g_big_dot_flag) {
    if (point_size < 1.5)
      point_size = 1.5;
  } else {
    if (point_size < 1.0)
      point_size = 1.0;
  }
  glPointSize(point_size);
  lower_count = plot_points_main(lower_count, counter);
  /* display counts for each color */
  for (i = 0; ((i <= g_last_structure) && (i < 10)); i++) {
    if (counter[i] > 0) {
      fixcolor_plot(i);
      strcpy(location, num_string_int(counter[i], temp_string));
      output_main(10 + 80 * i, 23, location);
    }
  }
  fixcolor_plot(COL_TEXT);
  strcpy(location, "Upper Base Pairs: ");
  strcat(location, num_string_int(points_plotted, temp_string));
  output_main(10, 5, location);
  fixcolor_plot(COL_TEXT);
  strcpy(location, PROGRAM_NAME);
  output_main(285, 5, location);
  if ((g_lower_structure == -10) || (g_nongray_for_3_flag) ||
      (g_lower_structure == -12)) {
    fixcolor_plot(1);
    glBegin(GL_LINE_LOOP);	/* bot,mid */
    glVertex2f(460, 20);
    glVertex2f(460, 38);	/* bot,left */
    glVertex2f(745, 38);
    glVertex2f(745, 20);	/* top,right */
    glEnd();
    fixcolor_plot(5);
    strcpy(location, num_string_int(g_gray3[0], temp_string));
    output_main(480, 23, location);
    fixcolor_plot(6);
    strcpy(location, num_string_int(g_gray3[1], temp_string));
    output_main(575, 23, location);
    fixcolor_plot(7);
    strcpy(location, num_string_int(g_gray3[2], temp_string));
    output_main(670, 23, location);
  }
  if (g_lower_structure != -1) {
    strcpy(location, "Lower Dots: ");
    strcat(location, num_string_int(lower_count, temp_string));
    if (g_lower_structure > -1)
      fixcolor_plot(g_lower_structure);
    else
      fixcolor_plot(COL_TEXT);
    output_main(485, 5, location);
  }
  glLineWidth(2.0);
  if ((g_region_set == TRUE) || (g_region_exists == TRUE)) {
    /* draw zoom */
    fixcolor_plot(COL_DOTS);
    glBegin(GL_LINE_LOOP);
    glVertex2f(region_real_l, region_real_t);
    glVertex2f(region_real_r, region_real_t);
    glVertex2f(region_real_r, region_real_b);
    glVertex2f(region_real_l, region_real_b);
    glEnd();
  }
  glutSwapBuffers();
  sniff_for_opengl_errors("end of display_main");
}

void display_zoom_bases_ver(void) {
  int             x, y;
  int             base;
  x = 10;
  for (base = display_t; base <= display_b; base++) {
    y = plot_i_zoom(base) - 2;
    glRasterPos2f(x, y);
    glutBitmapCharacter(g_font_zoom, sequence[base - 1]);
  }
}

void display_zoom_bases_hor(void) {
  int             x, y;
  int             base;
  y = 38;
  for (base = display_l; base <= display_r; base++) {
    x = plot_j_zoom(base) - 2;
    glRasterPos2f(x, y);
    glutBitmapCharacter(g_font_zoom, sequence[base - 1]);
  }
}

void display_zoom(void) {
  float           point_size;
  char            location[60];
  char            temp_string[50];
  int             counter[MAXIMUM_CT_FILES + 1];
  int             right, bottom, top, left, i;
  int             lower_basepairs;
  int             gray3[3];
  void           *temp_font;
  int             stripe_flag;
  /* display_all_helices(); */
  sniff_for_opengl_errors("start of display_zoom");
  glClear(GL_COLOR_BUFFER_BIT);
  glLineWidth(1.0);
  fixcolor_plot(COL_TEXT);
  if (display_h < 80) {
    temp_font = g_font_zoom;
    g_font_zoom = GLUT_BITMAP_TIMES_ROMAN_10;
    display_zoom_bases_ver();
    g_font_zoom = temp_font;
  }
  if (display_w < 80) {
    temp_font = g_font_zoom;
    g_font_zoom = GLUT_BITMAP_TIMES_ROMAN_10;
    display_zoom_bases_hor();
    g_font_zoom = temp_font;
  }
  plot_tics_zoom();
  point_size = 700./((display_w+display_h)/2)*width_z/800.*height_z/800. + .60;
  point_size = point_size * g_main_dot_mag;
  if (g_big_dot_flag) {
    if (point_size < 1.5)
      point_size = 1.5;
  } else {
    if (point_size < 1.0)
      point_size = 1.0;
  }
  /* stripe_flag indicates to draw dots as stripes for overlap */
  if (((point_size >= STRIPE_SCREEN * g_ct_files) ||
       (g_forced_stripe_flag)) &&
      (!g_nongray_for_3_flag) && (g_ct_files > 2))
    stripe_flag = TRUE;
  else
    stripe_flag = FALSE;
  glPointSize(point_size);
  sniff_for_opengl_errors("display zoom before plot points");
  plot_points_zoom(stripe_flag, counter, &lower_basepairs, gray3);
  if (lower_basepairs != 0) {
    fixcolor_plot(COL_TEXT);
    strcpy(location, "Lower Dots: ");
    strcat(location, num_string_int(lower_basepairs, temp_string));
    output_zoom(570, 5, location);
  }
  for (i = 0; ((i <= g_last_structure) && (i < 10)); i++) {
    if (counter[i] > 0) {
      fixcolor_plot(i);
      strcpy(location, num_string_int(counter[i], temp_string));
      output_zoom(10 + 80 * i, 23, location);
    }
  }
  if ((g_lower_structure <= -10) || (g_nongray_for_3_flag)) {
    fixcolor_plot(1);
    glBegin(GL_LINE_LOOP);	/* bot,mid */
    glVertex2f(460, 20);
    glVertex2f(460, 38);	/* bot,left */
    glVertex2f(745, 38);
    glVertex2f(745, 20);	/* top,right */
    glEnd();
    fixcolor_plot(5);
    strcpy(location, num_string_int(gray3[0], temp_string));
    output_zoom(480, 23, location);
    fixcolor_plot(6);
    strcpy(location, num_string_int(gray3[1], temp_string));
    output_zoom(575, 23, location);
    fixcolor_plot(7);
    strcpy(location, num_string_int(gray3[2], temp_string));
    output_zoom(670, 23, location);
  }
  sniff_for_opengl_errors("display zoom after plot points");
  fixcolor_plot(COL_TEXT);
  glLineWidth(2.0);
  glBegin(GL_LINE_LOOP);
  glVertex2f(22.0, 751.0);
  glVertex2f(22.0, 48.0);
  glVertex2f(724.0, 48.0);
  glVertex2f(724.0, 751.0);
  glEnd();
  glLineWidth(1.0);
  strcpy(location, "Upper Base Pairs: ");
  strcat(location, num_string_int(points_plotted_zoom, temp_string));
  if (g_nongray_for_3_flag)
    strcat(location, "        Gray interpreted");
  else {
    if (stripe_flag == TRUE) {
      strcat(location, "        Multicolor Partial Overlap");
      if (g_forced_stripe_flag)
	strcat(location, " Forced");
    } else
      strcat(location, "        Gray Partial Overlap");
  }
  output_zoom(10, 5, location);
  fixcolor_plot(COL_TEXT);
  /* draw diagional line to indicate no points below */
  if ((display_r >= display_t) && (display_b >= display_l)) {
    if (display_r > display_b) {
      bottom = display_b;
      right = display_b;
    } else {
      right = display_r;
      bottom = display_r;
    }
    if (display_l > display_t) {
      left = display_l;
      top = display_l;
    } else {
      top = display_t;
      left = top;
    }
    glBegin(GL_LINES);
    glVertex2f(plot_j_zoom(left - 1), plot_i_zoom(top - 1));
    glVertex2f(plot_j_zoom(right + 1), plot_i_zoom(bottom + 1));
    glEnd();
  }
  fixcolor_plot(COL_DOTS);
  if (g_region_set == TRUE) {	/* draw zoom box */
    glBegin(GL_LINE_LOOP);
    glVertex2f(region_real_l_zoom, region_real_t_zoom);
    glVertex2f(region_real_l_zoom, region_real_b_zoom);
    glVertex2f(region_real_r_zoom, region_real_b_zoom);
    glVertex2f(region_real_r_zoom, region_real_t_zoom);
    glEnd();
  }
  /* draw the labels */
  if (g_zoom_labels_count > 0) {
    temp_font = g_font_zoom;
    g_font_zoom = GLUT_BITMAP_TIMES_ROMAN_10;
    glEnable(GL_POINT_SMOOTH);
    for (i = 0; i < g_zoom_labels_count; i++) {
      output_zoom(g_zoom_labels_x[i], g_zoom_labels_y[i],
		  g_zoom_labels_string[i]);
      
    }
    point_size = 700./((display_w+display_h)/2)*width_z/800.*height_z/800.+.5;
    point_size = point_size * .5;
    glPointSize(point_size);
    glEnable(GL_POINT_SMOOTH);
    glBegin(GL_POINTS);
    for (i = 0; i < g_zoom_labels_count; i++) {
      glVertex2f((float) g_zoom_labels_xl[i],
		 (float) g_zoom_labels_y[i]);
    }
    glEnd();
    glDisable(GL_POINT_SMOOTH);
    g_font_zoom = temp_font;
  }
  glutSwapBuffers();
  sniff_for_opengl_errors("at end of display zoom");
}

void text_reshape(int w, int h) {
  if (w > 1250) {
    w = 1250;
    glutReshapeWindow(w, h);
  }
  if (h > g_text_window_size) {
    h = g_text_window_size;
    glutReshapeWindow(w, h);
  }
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, 600.0, 0.0, g_text_window_size, -1.0, 1.0);
  glMatrixMode(GL_MODELVIEW);
  if (w < 700)
    select_font_text(0);
  else
    select_font_text(1);
}

void reshape(int w, int h) {
  int             current_window, font;
  current_window = glutGetWindow();
  if (w > h) {
    h = w;
    glutReshapeWindow(w, h);
    glutPostRedisplay();
  }
  if (h > w) {
    w = h;
    glutReshapeWindow(w, h);
    glutPostRedisplay();
  }
  if (w > 990) {
    w = 990;
    h = 990;
    glutReshapeWindow(w, h);
  }
  if (current_window == win_main) {
    if (w < h)
      h = w;
    else
      w = h;
  }
  if (current_window == win_main) {
    width = w;
    height = h;
  } else {
    width_z = w;
    height_z = h;
  }
  if (w < 600)
    font = 0;
  else {
    if (w < 900)
      font = 1;
    else
      font = 2;
  }
  if (current_window == win_main)
    select_font_main(font);
  else
    select_font_zoom(font);
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, 800.0, 0.0, 800.0, -1.0, 1.0);	/* 0,0 is origin for
						 * drawing */
  /* lower left is 0,0   up is y, right is x */
  glMatrixMode(GL_MODELVIEW);
}

/* Functions to set zoom rectangle */
void fix_zoom_box(void) { /* region_real_r, region_r; */
  region_real_l = 23.0 + (float) ((region_l + .5) * 700. / (len + 1));
  region_real_r = 23.0 + (float) ((region_r + .5) * 700. / (len + 1));
  region_real_b = 749.0 - (float) ((region_b - .5) * 700. / (len + 1));
  region_real_t = 749.0 - (float) ((region_t - .5) * 700. / (len + 1));
}

void finish_fixing_zoom_box(void) {
  int             temp;	/* make left less than right */
  if (region_r < region_l) {
    temp = region_r;
    region_r = region_l;
    region_l = temp;
    fix_zoom_box();
  }
  if (region_b < region_t) {	/* make top less than bottom */
    temp = region_t;
    region_t = region_b;
    region_b = temp;
    fix_zoom_box();
  }
  if (first_zoom > 0) {
    saved_display_l = display_l;
    saved_display_r = display_r;
    saved_display_t = display_t;
    saved_display_b = display_b;
  } else {
    first_zoom = 1;
    saved_display_l = region_l;
    saved_display_r = region_r;
    saved_display_t = region_t;
    saved_display_b = region_b;
  }
  display_l = region_l;
  display_r = region_r;
  display_t = region_t;
  display_b = region_b;
  display_w = display_r - display_l + 1;
  display_h = display_b - display_t + 1;
}

void fix_zoom_box_zoom(void) {
  region_real_l = 25.0 + (float) region_l *700. / len;
  region_real_r = 25.0 + (float) region_r *700. / len;
  region_real_b = 50.0 + (float) (len - region_b) * 700. / len;
  region_real_t = 50.0 + (float) (len - region_t) * 700 / len;
  region_real_l_zoom = 25.4 + (float)(region_l-display_l+1)*700./(display_w+1);
  region_real_r_zoom = 25.4 + (float)(region_r-display_l+1)*700./(display_w+1);
  region_real_t_zoom = 50.0 + (float)(display_b-region_t+1)*700./(display_h+1);
  region_real_b_zoom = 50.0 + (float)(display_b-region_b+1)*700./(display_h+1);
  if (region_real_l_zoom < 0.) {
    region_real_l_zoom = 0.;
  };
  if (region_real_r_zoom < 0.) {
    region_real_r_zoom = 0.;
  }
  if (region_real_r_zoom > 799.) {
    region_real_r_zoom = 800.;
  }
  if (region_real_l_zoom > 799.) {
    region_real_l_zoom = 799.;
  }
  if (region_real_t_zoom < 0) {
    region_real_t_zoom = 0;
  }
  if (region_real_t_zoom > 799.) {
    region_real_t_zoom = 799.;
  }
  if (region_real_b_zoom < 0.) {
    region_real_b_zoom = 0.;
  }
  if (region_real_b_zoom > 799.) {
    region_real_b_zoom = 799.;
  }
}

void finish_fixing_zoom_box_zoom(void) {
  int             temp;	/* make left less than right */
  if (region_r < region_l) {
    temp = region_r;
    region_r = region_l;
    region_l = temp;
  }
  if (region_b < region_t) {	/* make top less than bottom */
    temp = region_t;
    region_t = region_b;
    region_b = temp;
  }
  saved_display_l = display_l;
  saved_display_r = display_r;
  saved_display_t = display_t;
  saved_display_b = display_b;
  display_l = region_l;
  display_r = region_r;
  display_t = region_t;
  display_b = region_b;
  display_w = display_r - display_l + 1;
  display_h = display_b - display_t + 1;
  fix_zoom_box_zoom();
}

/* Mouse Functions and special key functions  ___________ */

void fix_key_zoom_window(int x_change, int y_change, int zoom_direction) {
  int             x_offset, y_offset;
  g_zoom_labels_count = 0;	/* reset labels to none */
  if (x_change != 0) {
    x_offset = step_fun(display_l, display_r);
    if (x_offset < 1)
      x_offset = 1;
    x_offset = x_offset * x_change;
    if (x_change < 0) {
      if (display_l == 1)
	return;	/* do not go left when at left edge */
      region_l = display_l + x_offset;
      if (region_l < 1)
	region_l = 1;
      region_r = display_r + x_offset;
      if (region_r < 1)
	region_r = 1;
    }
    if (x_change > 0) {
      if (display_r == len)
	return;	/* do go right when at right edge */
      region_r = region_r + x_offset;
      region_l = region_l + x_offset;
      if (region_r > len)
	region_r = len;
      if (region_l > len)
	region_l = len;
    }
  }
  if (y_change != 0) {
    y_offset = step_fun(display_t, display_b);
    if (y_offset < 1)
      y_offset = 1;
    y_change = y_change * y_offset;
    if (y_change < 0) {
      if (display_t == 1)
	return;	/* do not go up when at top */
      region_t = display_t + y_change;
      if (region_t < 1)
	region_t = 1;
      region_b = display_b + y_change;
      if (region_b < 1)
	region_b = 1;
    }
    if (y_change > 0) {
      if (display_b == len)
	return;	/* do not go up when at top */
      region_t = region_t + y_change;
      region_b = region_b + y_change;
      if (region_b > len)
	region_b = len;
      if (region_t > len)
	region_t = len;
    }
  }
  if (zoom_direction) {
    y_offset = step_fun(display_t, display_b);
    if (y_offset < 1)
      y_offset = 1;
    x_offset = step_fun(display_l, display_r);
    if (x_offset < 1)
      x_offset = 1;
    if (zoom_direction < 0) {
      if ((display_b - display_t) < 4) {
	y_offset = 0;
      }
      if ((display_r - display_l) < 4)
	x_offset = 0;
      x_offset = -x_offset;
      y_offset = -y_offset;
    }
    region_l = display_l - x_offset; 
    region_r = display_r + x_offset;
    region_t = display_t - y_offset;
    region_b = display_b + y_offset;
    if (region_l < 1)
      region_l = 1;
    if (region_t < 1)
      region_t = 1;
    if (region_b > len)
      region_b = len;
    if (region_r > len)
      region_r = len;
  }
  fix_zoom_box();
  finish_fixing_zoom_box();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutPostRedisplay();
}

static void Key2(int key, int x, int y) {
  int             x_change;
  int             y_change;
  int             zoom_direction;
  if (!g_zoom_window_displayed)
    return;
  x_change = 0;
  y_change = 0;
  zoom_direction = 0;
  switch (key) {
  case GLUT_KEY_LEFT:
    x_change = -1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_RIGHT:
    x_change = 1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_UP:
    y_change = -1.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_DOWN:
    y_change = 1.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);;
    break;
  case GLUT_KEY_PAGE_DOWN:
    zoom_direction = +1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_PAGE_UP:
    zoom_direction = -1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  default:
    return;
  }
}

static void Key2_fast(int key, int x, int y) {
  int             x_change;
  int             y_change;
  int             zoom_direction;
  if (!g_zoom_window_displayed)
    return;
  x_change = 0;
  y_change = 0;
  zoom_direction = 0;
  switch (key) {
  case GLUT_KEY_LEFT:
    x_change = -3;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_RIGHT:
    x_change = 3;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_UP:
    y_change = -3.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_DOWN:
    y_change = 3.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);;
    break;
  case GLUT_KEY_PAGE_DOWN:
    zoom_direction = +1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_PAGE_UP:
    zoom_direction = -1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  default:
    return;
  }
}

void mouse(int button, int state, int x, int y) {
  int             i_row, j_col;
  char            temp_string[50];
  j_col = window_x_convert(x);
  i_row = window_y_convert(y);
  if (i_row < 1) {
    i_row = 1;
  }
  if (i_row > len)
    i_row = len;
  if (j_col < 1)
    j_col = 1;
  if (j_col > len)
    j_col = len;
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
      g_i_row = i_row;
      g_j_col = j_col;
      strcpy(g_i_row_s, num_string_int(i_row, temp_string));
      strcpy(g_j_col_s, num_string_int(j_col, temp_string));
      glutSetWindow(win_text);
      glutPostRedisplay();
    }
  }
  if (button == GLUT_MIDDLE_BUTTON) {
    if (state == GLUT_DOWN) {
      if (g_region_set == FALSE) {
	region_l = j_col;
	region_t = i_row;
	region_b = i_row;
	region_r = j_col;
	g_region_set = TRUE;
      } else {
	printf("Within no motion mouse with g_region_set = 1\n");
	region_r = j_col;	/* This code is never executed ? */
	region_b = i_row;
      }
      fix_zoom_box();
    } else {
      g_region_set = FALSE;
      g_region_exists = TRUE;
      finish_fixing_zoom_box();
      g_zoom_labels_count = 0;
      g_zoom_window_displayed = TRUE;
      zoomSetWindow();
      glutShowWindow();
      glutPopWindow();
      glutPostRedisplay();
    }
  }
}

void mouse_motion(int x, int y) { /* returns second corner of zoom rectangle as
				   * mouse is moved */
  int             i_row, j_col;
  if (g_region_set == TRUE) {
    j_col = window_x_convert(x);
    i_row = window_y_convert(y);
    if (i_row < 1) {
      i_row = 1;
    }
    if (j_col > len)
      j_col = len;
    if (i_row > len)
      i_row = len;
    if (j_col < 1)
      j_col = 1;
    region_r = j_col;
    region_b = i_row;
    fix_zoom_box();
    glutSetWindow(win_main);
    glutPostRedisplay();
  }
}

void mouse_zoom(int button, int state, int x, int y) {
  int             i_row, j_col;
  int             place;
  char            rec[30];
  char            temp_string[50];
  j_col = window_x_convert_zoom(x);
  i_row = window_y_convert_zoom(y);
  if (i_row < display_t) {
    i_row = display_t;
  }
  if (i_row > display_b)
    i_row = display_b;
  if (j_col < display_l)
    j_col = display_l;
  if (j_col > display_r)
    j_col = display_r;
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
      g_i_row = i_row;
      g_j_col = j_col;
      strcpy(g_i_row_s, num_string_int(i_row, temp_string));
      strcpy(g_j_col_s, num_string_int(j_col, temp_string));
      glutSetWindow(win_text);
      glutPostRedisplay();
      if (color_read(i_row, j_col, -1, g_diag_start, g_diag) >= 0) {
	if (g_zoom_labels_count < 49)
	  g_zoom_labels_count++;
	place = g_zoom_labels_count - 1;
	strcpy(rec, "<");
	strcat(rec, g_i_row_s);
	strcat(rec, ",");
	strcat(rec, g_j_col_s);
	strcat(rec, ">");
	strcpy(g_zoom_labels_string[place], rec);
	g_zoom_labels_x[place] = plot_j_zoom(j_col + 1);
	g_zoom_labels_xl[place] = plot_j_zoom(j_col);
	g_zoom_labels_y[place] = plot_i_zoom(i_row);
	g_zoom_labels_row[place] = i_row;
	g_zoom_labels_col[place] = j_col;
	if (g_zoom_window_displayed==TRUE) {
	  zoomSetWindow();
	  glutPostRedisplay();
	}
      }
    }
  }
  if (button == GLUT_MIDDLE_BUTTON) {
    if (state == GLUT_DOWN) {
      if (g_region_set == FALSE) {
	region_l = j_col;
	region_t = i_row;
	region_b = i_row;
	region_r = j_col;
	g_region_set = TRUE;
      } else {
	printf("Within no motion mouse with g_r_s = 1.\n");
	region_r = j_col;	/* This code is never executed ? */
	region_b = i_row;
      }
      fix_zoom_box_zoom();
    } else {
      g_region_set = FALSE;
      g_region_exists = TRUE;
      g_zoom_labels_count = 0;
      finish_fixing_zoom_box_zoom();
      if (g_zoom_window_displayed==TRUE) {
	zoomSetWindow();
	glutShowWindow();
	glutPostRedisplay();
      }
      glutSetWindow(win_main);
      glutPostRedisplay();
    }
  }
}

void mouse_motion_zoom(int x, int y) {
  int             i_row, j_col;
  if (g_region_set == TRUE) {
    j_col = window_x_convert_zoom(x);
    i_row = window_y_convert_zoom(y);
    if (i_row < display_t) {
      i_row = display_t;
    }
    if (i_row > display_b)
      i_row = display_b;
    if (j_col < display_l)
      j_col = display_l;
    if (j_col > display_r)
      j_col = display_r;
    region_r = j_col;
    region_b = i_row;
    fix_zoom_box_zoom();
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutPostRedisplay();
    }
  }
}

int             g_which_window_i_am_in;

void select_gray_menu(int msg) {
  if (msg == 1) {
    g_forced_stripe_flag = TRUE;
    g_nongray_for_3_flag = FALSE;
    printf("Gray dots forced to multicolor\n");
  }
  if (msg == 2) {
    g_nongray_for_3_flag = TRUE;
    printf("Gray dots interpreted as single colors\n");
    g_forced_stripe_flag = FALSE;
  }
  if (msg < 1) {
    g_forced_stripe_flag = FALSE;
    g_nongray_for_3_flag = FALSE;
    printf("Automatically showing gray or multicolor for partial overlap\n");
  }
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutPostRedisplay();
  glutSetWindow(win_text);
  glutPostRedisplay();
}

void No_Action_Menu(int msg) {
  if (msg == 1) {
    try_exit(0);
  } else if (msg == 2) {
    glutSetWindow(g_which_window_i_am_in);
    glutPopWindow();
  } else if (msg == 3) {
    if (g_grid_flag == TRUE)
      g_grid_flag = FALSE;
    else
      g_grid_flag = TRUE;
    glutSetWindow(win_main);	/* display main */
    glutPostRedisplay();
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutPostRedisplay();
    }
  } else if (msg == 4) {
    glutDestroyWindow(win_zoom);
    g_zoom_window_exists = FALSE;
    g_zoom_window_displayed = FALSE;
  } else if (msg == 73) {
    if (g_big_dot_flag == TRUE)
      g_big_dot_flag = FALSE;
    else
      g_big_dot_flag = TRUE;
    glutPostRedisplay();
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutPostRedisplay();
    }
    glutSetWindow(win_main);	/* display zoom */
    glutPostRedisplay();
  } else if (msg == 7) { /* go back to previous zoom window */
    region_l = saved_display_l;
    region_r = saved_display_r;
    region_t = saved_display_t;
    region_b = saved_display_b;
    fix_zoom_box();
    finish_fixing_zoom_box();
    g_zoom_labels_count = 0;
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutPostRedisplay();
    }
    glutSetWindow(win_main);
    glutPostRedisplay();
  }
}

void main_entry(int state) {
  if (state == GLUT_ENTERED) {
    g_i_row = 0;
    g_which_window_i_am_in = win_main;
  }
}

void zoom_entry(int state) {
  if (state == GLUT_ENTERED) {
    g_i_row = 0;
    g_which_window_i_am_in = win_zoom;
  }
}

void text_entry(int state) {
  if (state == GLUT_ENTERED)
    g_which_window_i_am_in = win_text;
}

void make_menu(int window) {
  /* initialize graphics for each window */
  int             i;
  int             font_submenu, postscript_submenu, lower_structure_submenu;
  int             img_submenu = 0, png_submenu = 0, jpg_submenu = 0;
  int             gray_submenu, magnify_submenu;
  glClearColor(g_color[COL_BACKGROUND][0], g_color[COL_BACKGROUND][1], 
	       g_color[COL_BACKGROUND][2], 1.0);
  lower_structure_submenu = glutCreateMenu(lower_structure_select);
  glutAddMenuEntry("None", -1);
  if (g_last_structure > 3)
    glutAddMenuEntry("Full and Partial Overlap", -2);
  glutAddMenuEntry("Full Overlap", COL_COMPLETE_OVERLAP);
  if (g_last_structure > 3) {
    glutAddMenuEntry("Partial Overlap", COL_PART_OVERLAP);
    if (g_last_structure == 4) {
      glutAddMenuEntry("Partial Overlap Interpreted", -10);
      glutAddMenuEntry("Full and Partial Interpreted", -12);
    }
  }
  for (i = 1; i <= g_ct_files; i++)
    glutAddMenuEntry(file_data_ct[i - 1], i + 1);
  gray_submenu = glutCreateMenu(select_gray_menu);
  glutAddMenuEntry("Automatically switch gray to multicolor", 0);
  glutAddMenuEntry("Show gray as multicolor in zoom", 1);
  if (g_ct_files == 3)
    glutAddMenuEntry("Show gray interpreted for content", 2);
  /* postscript */
  postscript_submenu = glutCreateMenu(create_post1);
  glutAddMenuEntry("For Main Window", MAIN_POST);
  glutAddMenuEntry("Alternate for Main Window", MAIN_POST + 10);
  if (g_zoom_window_displayed) {
    glutAddMenuEntry("For Zoom Window", ZOOM_POST);
    glutAddMenuEntry("Alternate for Zoom Window", ZOOM_POST + 10);
  }
  /* font */
  font_submenu = glutCreateMenu(select_font);
  glutAddMenuEntry("Helvetica   12", 0);
  glutAddMenuEntry("Helvetica   18", 1);
  glutAddMenuEntry("Times Roman 24", 2);
  /* gif */
  if (g_zoom_window_displayed == TRUE) {
    img_submenu = glutCreateMenu(create_gif1);
    glutAddMenuEntry("Res. 72    or 612  x 792 ", 1);
    glutAddMenuEntry("Res. 110   or 935  x 1210", 2);
    glutAddMenuEntry("Res. 200   or 1700 x 2200", 3);
    glutAddMenuEntry("Res. 300   or 2550 x 3300", 4);
    jpg_submenu = glutCreateMenu(create_jpg1);
    glutAddMenuEntry("Res. 72    or 612  x 792 ", 1);
    glutAddMenuEntry("Res. 110   or 935  x 1210", 2);
    glutAddMenuEntry("Res. 200   or 1700 x 2200", 3);
    glutAddMenuEntry("Res. 300   or 2550 x 3300", 4);
    png_submenu = glutCreateMenu(create_png1);
    glutAddMenuEntry("Res. 72    or 612  x 792 ", 1);
    glutAddMenuEntry("Res. 110   or 935  x 1210", 2);
    glutAddMenuEntry("Res. 200   or 1700 x 2200", 3);
    glutAddMenuEntry("Res. 300   or 2550 x 3300", 4);
  }
  /* magnify dot */
  magnify_submenu = glutCreateMenu(magnify_menu);
  glutAddMenuEntry("0.5", 0);
  glutAddMenuEntry("1.0", 1);
  glutAddMenuEntry("1.5", 2);
  glutAddMenuEntry("2.0", 3);
  glutAddMenuEntry("3.0", 4);
  glutAddMenuEntry("4.0", 5);
  glutAddMenuEntry("5.0", 6);
  glutAddMenuEntry("6.0", 7);
  glutAddMenuEntry("7.0", 8);
  glutAddMenuEntry("8.0", 9);
  glutAddMenuEntry("9.0", 10);
  glutAddMenuEntry("10.0", 11);
  glutAddMenuEntry("12.0", 12); 
  glutAddMenuEntry("14.0", 13);
  glutAddMenuEntry("16.0", 14);
  glutAddMenuEntry("18.0", 15);
  glutAddMenuEntry("20.0", 16);
  glutAddMenuEntry("25.0", 17);
  glutAddMenuEntry("30.0", 18);
  glutAddMenuEntry("35.0", 19);
  glutAddMenuEntry("40.0", 20);
  glutAddMenuEntry("45.0", 21);
  glutAddMenuEntry("50.0", 22);
  glutAddMenuEntry("60.0", 23);
  glutAddMenuEntry("100.0", 24);
  glutCreateMenu(No_Action_Menu);
  /* add each menu */
  if (g_zoom_window_displayed == TRUE) {
    glutAddMenuEntry("UnZoom", 7);
    glutAddMenuEntry("Close zoom window", 4);
  }
  glutAddSubMenu("Structure for Lower triangle", lower_structure_submenu);
  if (g_ct_files > 2)
    glutAddSubMenu("Partial Overlap Color", gray_submenu);
  glutAddSubMenu("Create Postscript file", postscript_submenu);
  if (g_zoom_window_displayed == TRUE) {
#if (defined(HAVE_LIBGD) || defined(HAVE_GDIMAGECREATE)) && defined(HAVE_GD_H)
    glutAddSubMenu("Create GIF for Zoom Window", img_submenu);
#endif
#if HAVE_GDIMAGEPNG
    glutAddSubMenu("Create PNG for Zoom Window", png_submenu);
#endif
#if HAVE_GDIMAGEJPEG
    glutAddSubMenu("Create JPG for Zoom Window", jpg_submenu);
#endif
  }
  glutAddSubMenu("Magnify dot size", magnify_submenu);
  glutAddMenuEntry("Big Dot On/Off", 73);
  glutAddMenuEntry("Toggle Grid lines On/Off", 3);
  glutAddSubMenu("Font", font_submenu);
  glutAddMenuEntry("Pop", 2);
  glutAddMenuEntry("Exit", 1);
  glutAttachMenu(GLUT_RIGHT_BUTTON);
}

void gfxinit_nontext(int window) {
  /* initalize graphics for each window */
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutReshapeFunc(reshape);
  make_menu(window);
}

void memory_error(void) {
  printf("Error!\tInsufficient memory to run with sequence length %d.\n", len);
  try_exit(4);
}

void zoomSetWindow(void) {
  if (g_zoom_window_exists==FALSE) {
    g_zoom_window_exists = TRUE;
    /* Zoom window */
    glutInitWindowSize(750, 750);
    glutInitWindowPosition(6, 262);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
    win_zoom = glutCreateWindow(PROGRAM_NAME" Zoom Window");
    gfxinit_nontext(win_zoom);	/* initialize graphics for zoom window */
    glutDisplayFunc(display_zoom);
    glutEntryFunc(zoom_entry);
    glutMouseFunc(mouse_zoom);
    glutMotionFunc(mouse_motion_zoom);
    glutSpecialFunc(Key2);
    width_z = 800;
    height_z = 800;
  }
  if (g_zoom_window_displayed==TRUE) {
    glutSetWindow(win_zoom) ;
  } else {
    glutDestroyWindow(win_zoom);
    g_zoom_window_exists = FALSE;
  }
}
#endif

int main(int argc, char **argv) {
#ifdef HAVE_GLUTBITMAPCHARACTER
  int             i;
  int             last_structure;
  initialize_data1(argc, argv);
  glutInit(&argc, argv);
  for (i = 1; i < argc; i++) {
    if (!strcmp(argv[i], "-mono")) {
      g_font_main = GLUT_BITMAP_HELVETICA_18;
      g_font_text = GLUT_BITMAP_HELVETICA_18;
      g_font_zoom = GLUT_BITMAP_HELVETICA_18;
    }
  }
  if (g_last_structure == 4)
    g_lower_structure = -12; /* default to show full and partial
				overlap interpreted in lower triangle */ 
    /* Zoom window */
  zoomSetWindow();
  /* Main window */
  glutInitWindowSize(800, 800);
  glutInitWindowPosition(474, 218);
  win_main = glutCreateWindow(PROGRAM_NAME" Main Window");
  glutMouseFunc(mouse);
  glutMotionFunc(mouse_motion);
  glutSpecialFunc(Key2_fast);
  gfxinit_nontext(win_main);	/* initialize graphics for Main Window */
  glutDisplayFunc(display_main);
  glutEntryFunc(main_entry);
  g_text_window_size = 200;
  if (g_last_structure > 80)
    last_structure = 80;
  else
    last_structure = g_last_structure;
  if (g_last_structure > 9)
    g_text_window_size = g_text_window_size + 25 *
      ((last_structure - 9) / 2 + 1);
  /* Text window */
  glutInitWindowSize(800, g_text_window_size);
  glutInitWindowPosition(6, 31);
  win_text = glutCreateWindow(PROGRAM_NAME" Text Window");
  make_menu(win_text);
  glutDisplayFunc(display_text);
  glutReshapeFunc(text_reshape);
  glutEntryFunc(text_entry);
  glutKeyboardFunc(keyboard);
  glutMainLoop();
#endif
  printf("%s not available because glut was not found.\n", PROGRAM_NAME);
  printf("Use %s_ng instead.\n", PROGRAM_NAME);
  try_exit(5);
  return 0; /* Keep the stupid compiler happy. */
}
