/* mutplot
 * The sample input is in FASTA format, although embl, genbank,
 * Stanford IG, PIR and GCG formats should be reconized. FASTA format
 * is advised.
 * The input file should contain multiple sequences that are
 * aligned. This means that gap characters are needed.
 * see mutplot.doc for more comments
 */

#define PROGRAM_NAME "mutplot"
#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <limits.h>
#include <sys/unistd.h>
#include "mutplot.h"

#ifdef HAVE_GLUTBITMAPCHARACTER
void zoomSetWindow(void);

#define MAIN_POST    1 /* create postcript for main or zoom */
#define ZOOM_POST    2 /* used in general_post, create_post */
#include "mutplot_ps.h"

#define MAIN_DUMP    1 /* create common dump for main */
#define ZOOM_DUMP    2 /* create common dump for zoom */

#define MESSAGE_NONE   0   /* values for g_type_message */
#define MESSAGE_POST   1   /* see it below */
#define MESSAGE_CUTOFF 2
#define MESSAGE_REFERENCE   3
#define MESSAGE_ID  5
#define MESSAGE_ALIGN_ROW_COL 6
#define MESSAGE_REF_ROW_COL 7
#define MESSAGE_OUT 8

static GLfloat spin = 0.0; /* used to make display spin */

int g_type_of_label_file; /* use 0 for detail of all sequences use sequence 
			   * number otherwise for base pairs for
			   * UNAFold or mfold */ 
int g_first_zoom = 0;  /* used for unzoom, 0 indicates none to undo */
int g_big_dots_flag;
int g_grid_flag;
int g_conserved_bp_flag;

int width, height; /* The current width and height of the main window
		      in pixels */  
int width_z, height_z; /* same for zoom window */

int g_which_window_i_am_in = 1; /* which window the cursor is in */ 

/* when right mouse button is clicked, transform coordinates into i,j */

int g_bigdot_mode; /* draw with bigger dotts when true
		    * edit mutplot.col for dot sizes and color preferences */
/* ______________________________________________________________
 * Notice that most functions are called based on events in the
 * event queue.  They do not call each other.  This results in too  
 * many global variables.                                         
 * Variables that could easily be changed for personal preference are 
 * usually accompanied by $$$.                                     
 * ______________________________________________________________ 
 */

/* Font variables  ______________________________________________ */
void *g_font_text = GLUT_BITMAP_HELVETICA_18; /* initialize font */
void *g_font_zoom = GLUT_BITMAP_HELVETICA_18;
void *g_font_main = GLUT_BITMAP_HELVETICA_18;
				      
void *fonts[] = { /* used by selec_font */
  GLUT_BITMAP_HELVETICA_12,
  GLUT_BITMAP_HELVETICA_18, 
  GLUT_BITMAP_TIMES_ROMAN_24
};

int g_how_close_zoom=0; /* used with search option to determine how
			 * far to move helices, dots */ 

/* Graphic  variables
 * all counts points where all sequences overlap, black dots
 * without all, counts points where has a base pair
 * zoom is for zoom window, other is for main window 
 *
 * Zoom Variables
 * specify boundaries of the zoom rectangle in i,j coordinates for
 * main window or for the zoom window
 */

int region_t = 1, region_b = 1, region_r = 1, region_l = 1; 

float region_real_l,region_real_r,region_real_b,region_real_t;

/* screen coordinates of above (i,j)
 * pairs  for the main window  (blue rectangle)
 */

float region_real_l_zoom,region_real_r_zoom;
float region_real_t_zoom,region_real_b_zoom;
/* Screen coordinates  for a zoom rectangle within the zoom window */ 
int g_region_set=FALSE;  /* nonzero indicates the top left corner of */
/* zoom region has been set */
int g_region_exists=FALSE;

int g_zoom_labels_count=0 ; /* the number of labels used */
int g_zoom_labels_x[50];  /* x coordinate for label i */
int g_zoom_labels_xl[50]; /* used to plot on screen the dot */
int g_zoom_labels_y[50];
int g_zoom_labels_row[50]; /* row,col coordinates for labels */
int g_zoom_labels_col[50];

char g_zoom_labels_string[50][15]; /* value to place at x,y */

/* variables for making insertions in sequences with mouse */

/* File Variables _______________________________________________*/
char rec[90];
FILE *outfp;  /* label file */

int g_prob=FALSE;
int g_zoom_window_exists = FALSE;
int g_zoom_window_displayed = FALSE;
int g_post_file=1; /* 1 for main, 2 for zoom */
                      /* used from text display */
char g_id_list_filename[120]="  ";
char g_out_filename[120]="   ";

int execlp(const char *file, const char *arg, ...); 
void mouse_zoom(int , int , int , int );
void mouse_motion_zoom(int ,int );
void gfxinit_nontext(int );

void initialize_data2(void) {
  g_zoom_labels_count=0; /* initialize to no labels */
  width=800;   /* initialize size of window in pixels */
  height=800;  /* needed for scale in drawing window */
  g_region_set = FALSE;
  g_bigdot_mode = FALSE;
  g_message_set = FALSE;
  g_grid_flag = TRUE;
  text_window_input[0] = 0;
  text_window_message[0] = 0;
  g_type_message = MESSAGE_NONE;
  g_i_row = -1;
  g_j_col = -1;
}

/* 
 * Conversion Routines to display integers and floats 
 */

char g_str[30]="                              ";

/* converts integer to string */
char *num_string_int(int x) {
  int len;
  int i; 
  char spaces[20]="                    ";
  sprintf(g_str,"%d",x);
  len=strlen(g_str);
  len=4-len;
  if(len>0) {
    strcpy(spaces," ");
    for(i=1;i<len;i++)
      strcat(spaces,"  ");
    strcat(spaces,g_str);
    strcpy(g_str,spaces);
  }       
  return g_str;
}

char *num_string_double(double x) { /* converts double to string */
  sprintf(g_str,"%.1f",x);
  return g_str;
}

char *num_string_float(float x) { /* converts float to string */
  sprintf(g_str,"%.2f",x);
  return g_str;
}

char *num_prob_string(int x) { /* divides by 1000 and multiplies by -1 */
  float x_float;
  x_float=(float)x;
  x_float=x_float/1000.;
  x_float=-1*x_float;
  sprintf(g_str,"%4.3f",x_float);
  return g_str;
} 

char *num_energy_string(int x) { /* Divide by 10 and convert to string */
  float x_float;
  x_float=(float)x;
  x_float=x_float/10.;
  if(x_float!=0.)
    sprintf(g_str,"%7.1f",x_float);
  else
    strcpy(g_str,"Undef.");
  return g_str;
}

/* converts integer to string , handles prob or energies*/
char *num_string_fancy_int(int x) {
  float float_x;
  float_x=(float)x;
  if(g_prob==TRUE)
    return strcpy(g_str,num_prob_string(float_x));
  else
    return strcpy(g_str,num_energy_string(float_x));
}

/* converts float to string */
char *num_string_fancy_float(float x) {
  if(g_prob==TRUE)
    return strcpy(g_str,num_prob_string(x));
  else
    return strcpy(g_str, num_energy_string(x));
}

/* tick mark functions
 * find distance between tick marks
 *
 * Menu routines
 * Most are defined through GLUT and called from the event queue
 * cursor functions 
 */
void fix_cursor(void) {
  /* set cursor back to crosshairs */
  if (g_zoom_window_displayed==TRUE) {
    glutSetWindow(win_zoom);
    glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutPostRedisplay();
  glutSetWindow(win_text);
  glutSetCursor(GLUT_CURSOR_LEFT_ARROW);
  glutPostRedisplay();
}

void set_cursor_help(void) {
  glutSetWindow(win_text);
  glutSetCursor(GLUT_CURSOR_HELP);
  glutPopWindow();
  glutPostRedisplay();
}

int g_which_dump_seq;

void keyboard(unsigned char key, int x, int y) {
  int string_len1; /* accepts keys from text window */
  int string_len2;
  x=y;
  y=x; /* get rid of warning message */
  if(key == 13) {
    data_entered = TRUE; /* on <RETURN> mark end of text */
    glutSetWindow(win_text);
    glutPostRedisplay();
    return;
  }
  string_len1 = (int)strlen(text_window_input);
  string_len2 = (int)strlen(text_window_message);
  if(string_len1>0)
    if(key == 8) { /* process backspace or delete*/
      text_window_input[string_len1-1] = '\0';
      text_window_message[string_len2-1] = '\0';
      glutSetWindow(win_text);
      glutPostRedisplay();
      return; 
    }
  text_window_input[string_len1]=key; /* add key to end of input */
  text_window_input[string_len1+1] = '\0';
  text_window_message[string_len2] = key; /* add key to end of message */
  text_window_message[string_len2+1] = '\0';
  glutSetWindow(win_text); /* show the text */
  glutPostRedisplay();
}

void fixcolor_plot(int color) {   /*   used on the screen */
  if((color>=0)&&(color<TOTAL_COLORS))
    glColor3f(g_color[color][0],g_color[color][1], g_color[color][2]);
  else
    glColor3f(g_color[0][0],g_color[0][1], g_color[0][2]);
}

void select_font(int newfont) {
  g_font_main=fonts[newfont];
  g_font_zoom=fonts[newfont];
  g_font_text=fonts[newfont];
  glutSetWindow(win_text);
  glutPostRedisplay();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutPostRedisplay();
}

void select_font_main(int newfont) {
  g_font_main = fonts[newfont];
  glutPostRedisplay();
}

void select_font_zoom(int newfont) {
  g_font_zoom = fonts[newfont];
  glutPostRedisplay();
}

void select_font_text(int newfont) {
  g_font_text = fonts[newfont];
  glutSetWindow(win_text);
  glutPostRedisplay();
}

int open_output_file(char * file_name) {
  if ((outfp = fopen(file_name, "w")) == NULL) {
    printf("Error!\tCould not open file: %s\n", file_name);
    return(1);
  }
  return (0);
}

void get_post_name(void) {
  data_entered = FALSE;
  g_message_set = TRUE;
  strcpy(text_window_message,post_file_message);
  strcat(text_window_message,g_post_filename);
  strcat(text_window_message,")? ");
  g_type_message=MESSAGE_POST;
  glutSetWindow(win_text);
  glutPostRedisplay();
  set_cursor_help();
}

void finish_setting_reference(void) {
  char name_of_sequence[120] = " ";
  int name_len;
  g_message_set = FALSE;
  text_window_message[0]=0;
  if(strlen(text_window_input)==0) {
    glutSetWindow(win_text);
    glutPostRedisplay();
    fix_cursor();
    return;
  }
  strcpy(name_of_sequence,text_window_input);
  text_window_input[0] = '\0';
  name_len = strlen(name_of_sequence);
  if((name_of_sequence[0]=='/') && (name_len==1)) {
    glutSetWindow(win_text);
    glutPostRedisplay();
    fix_cursor();
    return;
  }
   find_sequence_by_name(name_of_sequence);
   glutSetWindow(win_text);
   glutPostRedisplay();
   fix_cursor();
   return;
}

void fix_input(char *text) {
  int length;
  int i;
  length = strlen(text);
  for(i=0;i<length;i++) {
    if(!isdigit(text[i]))
      text[i]=' ';
  }
}

void finish_setting_align_row_col(void) {
  int row, col;
  int temp;
  strcpy(text_window_message,align_row_col_message);
  strcat(text_window_message,num_string_int(g_sequence_length));
  strcat(text_window_message,") ? ");
  if(strlen(text_window_input)<2) {
    text_window_message[0]='\0';
    glutSetWindow(win_text);
    glutPostRedisplay();
    fix_cursor();
    return;
  }
  fix_input(text_window_input);
  sscanf(text_window_input,"%d %d",&row,&col);
  if(row==0 || col==0) {
    text_window_message[0]='\0';
    glutSetWindow(win_text);
    glutPostRedisplay();
    fix_cursor();
  }
  if(row<0)
    row=0;
  if(col<0)
    col=0;
  if(row>g_sequence_length)
    row=g_sequence_length;
  if(col>g_sequence_length)
    col=g_sequence_length;
  if(col<row) {
    temp=row;
    row=col;
    col=temp;
  }
  g_i_row=row;
  g_j_col=col;
  glutSetWindow(win_text);
  glutPostRedisplay();
  fix_cursor();
  return;
}

void finish_setting_ref_seq_row_col(int text_read) {
  int row,col;
  int temp;
  if(text_read) {
    strcpy(text_window_message,ref_seq_row_col_message); 
    strcat(text_window_message,num_string_int(g_ref_sequence_length));
    strcat(text_window_message,") ? ");
    if(strlen(text_window_input)<2) {
      glutSetWindow(win_text);
      glutPostRedisplay();
      fix_cursor();
      return;
    }
    fix_input(text_window_input);
    sscanf(text_window_input,"%d %d",&row,&col);
  } else {
    row = g_input_row;
    col = g_input_col;
  }
  if(row<0) row = 0;
  if(col<0) col = 0;
  if(row > g_ref_sequence_length) row = g_ref_sequence_length;
  if(col > g_ref_sequence_length) col = g_ref_sequence_length;
  if(col < row) {
    temp = row;
    row = col;
    col = temp;
  }
  row = g_reverse_displayed_sequence_numbering[row];
  col = g_reverse_displayed_sequence_numbering[col];
  g_i_row = row;
  g_j_col = col;
  if(text_read) {
    glutSetWindow(win_text);
    glutPostRedisplay();
    fix_cursor();
  }
  return;
}

void finish_making_post(void) {
  int name_len, left, right, top, bottom;
  char post_filename[120] = "         ";
  g_message_set = FALSE;
  text_window_message[0] = 0;
  glutSetWindow(win_text);
  fix_cursor();
  glutPostRedisplay();
  if(strlen(text_window_input)==0)
    strcpy(post_filename,g_post_filename);
  else
    strcpy(post_filename,text_window_input);
  text_window_input[0]='\0';
  name_len=strlen(post_filename);
  if ((post_filename[0]=='/') && (name_len==1)) return;
  if (g_post_file==MAIN_POST) {
    left = 1;
    right = g_sequence_length;
    top = 1;
    bottom = g_sequence_length;
  } else {
    left = display_l;
    right = display_r;
    top = display_t;
    bottom = display_b;
  }
  general_post(post_filename,left,right,top,bottom,g_cutoff,g_color,
	       g_grid_flag,g_big_dots_flag,g_ps_dot_min,g_zoom_labels_count,
	       g_zoom_labels_row,g_zoom_labels_col,g_zoom_labels_string,
	       g_big_matrix,g_post_file,g_conserved_bp_flag);
}

void finish_making_out_file(void) {
  int name_len;
  char out_filename[120];
  g_message_set=FALSE;
  text_window_message[0]=0;
  glutSetWindow(win_text);
  fix_cursor();
  glutPostRedisplay();
  if(strlen(text_window_input)==0)
    strcpy(out_filename,g_out_filename);
  else
    strcpy(out_filename,text_window_input);
  text_window_input[0]='\0';
  name_len=strlen(out_filename);
  if((out_filename[0]=='/')&&(name_len==1))
    return;
  general_out_file(out_filename,g_cutoff,g_big_matrix,g_conserved_bp_flag,
		   g_displayed_sequence_numbering,g_sequence_length);
}

void finish_making_id_list(void) {
  int name_len;
  char id_list_filename[120];
  g_message_set=FALSE;
  text_window_message[0]=0;
  glutSetWindow(win_text);
  glutPostRedisplay();
  fix_cursor();
  if(strlen(text_window_input)==0)
    strcpy(id_list_filename,g_id_list_filename);
  else
    strcpy(id_list_filename,text_window_input);
  text_window_input[0]='\0';
  name_len=strlen(id_list_filename);
  if((id_list_filename[0]=='/')&&(name_len==1))
    return;
  general_id_list(id_list_filename,g_i_row,g_j_col, g_in_align_filename);
}

/* from menu, indicate to  create postcript file for window */
void create_post(int which_window) { 
  if((which_window<2)||(g_zoom_window_displayed==FALSE))
    g_post_file=MAIN_POST; /* create post for main */
  else
    g_post_file=ZOOM_POST; /* create post for zoom */
  if((which_window == 1)||(which_window == 3))
    g_big_dots_flag=TRUE;
  else
    g_big_dots_flag=FALSE;
  get_post_name();
}

void set_cutoff2(void) { /* for energies */
  g_zoom_labels_count = 0; /* reset labeling of points in zoom window */
  strcpy(text_window_message,increment_message);
  strcat(text_window_message,"? ");
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_CUTOFF;
  glutSetWindow(win_text);
  glutPostRedisplay();
  set_cursor_help();
}      

void finish_setting_energy_cutoff(void) {
  float temp_filter;
  text_window_message[0]='\0';
  g_message_set=FALSE; 
  if(text_window_input[0]=='/') {
    text_window_message[0]='\0';
    glutSetWindow(win_text);
    glutPostRedisplay();
    fix_cursor();
    return;
  }
  sscanf(text_window_input,"%f",&temp_filter);
  if(temp_filter<0)
    temp_filter=0;
  if(temp_filter>3.0)
    temp_filter=3.0;
  g_cutoff=(Si)(temp_filter*1000+.5);
  text_window_message[0]='\0';
  glutSetWindow(win_text);
  glutPostRedisplay();
  fix_cursor();
}


/* Plotting functions */

void sniff_for_opengl_errors(void) {
  GLenum errCode;
  const GLubyte *errString;
  const char *my_string;
  if((errCode=glGetError())!=GL_NO_ERROR) {
    errString=gluErrorString(errCode);
    my_string=(char *)errString;
    fprintf(stderr,"mutplot OpenGL Error: %s\n",my_string);
  }
}

/* converts i,j base pair to real coordinates and plots them */
void plot_ij(int i_row,int j_col) { 
  float real_x,real_y;        
  real_x = 23.0 + (float)( (j_col+.5)*700./(g_sequence_length+1));
  real_y = 749.0 - (float) ((i_row-.5)*700./(g_sequence_length+1));
  glVertex2f(real_x,real_y);
}

/* same for zoom window */
void plot_ij_zoom_comp(int i_row,int j_col) {
  float real_x,real_y;
  real_x = 25. + (float) (j_col-display_l+1)*700./(display_w+1);
  real_y = 50. + (float) (display_b-i_row+1)*700./(display_h+1);
  glVertex2f(real_x,real_y);
}

void plot_ij_zoom(int i_row,int j_col,int stripe_flag,int color) {
  float real_x,real_y,x_offset,y_offset,mag,adjuster;
  if((stripe_flag!=0)&&(color==1))
    return;
  real_x = 25. + (float) (j_col-display_l+1)*700./(display_w+1);
  real_y = 50. + (float) (display_b-i_row+1)*700./(display_h+1);
  if(!stripe_flag) {
    if((display_w>300)&&(display_h>300)) {
      glVertex2f(real_x,real_y);
    } else {
      x_offset = 
	(25. + (float) (j_col-display_l+2)*700./(display_w + 1) - real_x)/2;
      y_offset = 
	(50. + (float) (display_b-i_row+2)*700./(display_h + 1) - real_y)/2;
      glBegin(GL_POLYGON);
      glVertex2f(real_x, real_y - y_offset); 
      glVertex2f(real_x - x_offset, real_y-y_offset);
      glVertex2f(real_x - x_offset, real_y);
      glVertex2f(real_x, real_y + y_offset);
      glVertex2f(real_x + x_offset, real_y + y_offset);
      glVertex2f(real_x + x_offset, real_y);
      glEnd(); /*  ? this statement produces errors */
    }
  }  else { /* stripe option */
    x_offset = 
      (25. + (float) (j_col-display_l+2)*700./(display_w + 1) - real_x)/2;
    y_offset = 
      (50. + (float) (display_b-i_row+2)*700./(display_h + 1) - real_y)/2;
    adjuster=1.0;
    mag=1.0;
    glBegin(GL_POLYGON);  /* bot,mid*/
    glVertex2f(real_x+x_offset*adjuster*mag,real_y-y_offset*adjuster*mag);
    glVertex2f(real_x-x_offset*1.0,real_y-y_offset*mag); /* bot,left */
    glVertex2f(real_x-x_offset*adjuster*mag,
	       real_y+y_offset*adjuster*mag); /* top,mid */
    glVertex2f(real_x+x_offset*mag,real_y+y_offset*mag); /* top,right */
    glEnd();
    sniff_for_opengl_errors();
  }
}

int plot_i_zoom(int i_row) { /* convert i_row to y coordinate */
  return (int)(50.2+(float)((display_b-i_row+1)*700./(display_h+1)));
}

int plot_j_zoom(int j_col) { /* convert col to x coordinate */
  return (int) (25.2 + (float) (j_col-display_l+1)*700./(display_w+1));
}

int plot_i(int i_row) { /* converts row to y coordinate */
  return (int) (749.0 - (float) ((i_row-.5)*700./(g_sequence_length+1))+.5);
}

int plot_j(int j_col) { /* converts col to x coordinate */
  return (int) (23.0 + (float) ((j_col+.5)*700./(g_sequence_length+1))+.5);
} 

float plot_i_float(float i_row) { /* converts row to y coordinate */
  return 749.0 - (float) ((i_row-.5)*700./(g_sequence_length+1))+.5;
}

float plot_j_float(float j_col) { /* converts col to x coordinate */
  return 23.0+(float)((j_col+.5)*700./(g_sequence_length+1))+.5;
} 

void plot_points_zoom(float point_size,int stripe_flag) {
  int row, col, color;
  if((display_w>300)&&(display_h>300)) {
    glPointSize(point_size*1.4);
    glBegin(GL_POINTS); /* plot points if they are small */
  }
  for (row=display_t;row<=display_b;row++) {
    for (col=display_l;col<=display_r;col++) {
      if (row<col)
        color = find_dot_color(g_big_matrix[row][col],
			       g_cutoff,g_conserved_bp_flag);
      else
        color = find_dot_color(g_big_matrix[col][row],g_cutoff,
			       g_conserved_bp_flag); 
      if(color>-1) {
	fixcolor_plot(color);
        if(row<col)
	  plot_ij_zoom(row,col,stripe_flag,color);
        else
	  if((row!=col)&&(color<3))
	    plot_ij_zoom(row,col,stripe_flag,color);
      }
    }
  }
  if((display_w>300) && (display_h>300))
    glEnd();
}

void plot_points(void) {
  int row, col, color;
  glBegin(GL_POINTS);
  for (row=1;row<=g_sequence_length;row++) {
    for (col=(row+1);col<=g_sequence_length;col++) {
      color = find_dot_color(g_big_matrix[row][col],g_cutoff,
			     g_conserved_bp_flag);
      if (color>-1) {
	fixcolor_plot(color);
        plot_ij(row,col);
        if(color<3) plot_ij(col,row);
      }
    }
  }
  glEnd();
}

/* display string at real coordinates  x,y  */
void output_main(int x, int y, char *string) { 
  int len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(g_font_main, string[i]);
  }
}

/* display string at real coordinates  x,y  */
void output_main_small(int x, int y, char *string) {
  int len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12,string[i]);
  }
}

/* display string at real coordinates  x,y  */
void output_zoom(int x, int y, char *string) {
  int len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(g_font_zoom, string[i]);
  }
}

/* display string at real coordinates  x,y  */
void output_text(int x, int y, char *string) {
  int len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(g_font_text, string[i]);
  }
}

void single_hor_tic(int j_col,int grid_flag) {
  char string2[20];
  int sequence_position;
  float x;
  x=(float)plot_j(j_col);
  glBegin(GL_LINES);
  glVertex2f(x,751.);
  glVertex2f(x,758.);
  glEnd();
  if(grid_flag==TRUE) {
    fixcolor_plot(COLOR_GRID); /* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(x,750);
    glVertex2f(x,46);
    glEnd();
  }
  sequence_position=g_displayed_sequence_numbering[j_col];
  if(sequence_position!=0)
    strcpy(string2,num_string_int(sequence_position));
  else
    strcpy(string2," - ");
  fixcolor_plot(COLOR_LABEL);
  output_zoom(plot_j(j_col)-10,780,string2);
  fixcolor_plot(COLOR_TEXT);
  strcpy(string2,num_string_int(j_col));
  output_main(plot_j(j_col)-10,762,string2);     
}

void plot_hor_tics(void) { 
  int j_col,step,start;
  step=step_fun(1,g_sequence_length);
  start=start_fun(1,step);
  single_hor_tic(g_sequence_length,FALSE);
  single_hor_tic(1,FALSE);
  for(j_col=start;j_col<(g_sequence_length-2*step/3);j_col=j_col+step) {
    single_hor_tic(j_col,g_grid_flag);
  }
}

void single_ver_tic(int i_row,int grid_flag) { 
  char string2[20];
  float y;
  int sequence_position;
  y=(float)plot_i(i_row);
  glBegin(GL_LINES);
  glVertex2f(727.,y);
  glVertex2f(736.,y);
  glEnd();
  if(grid_flag==TRUE) {
    fixcolor_plot(COLOR_GRID); /* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(21,y);
    glVertex2f(725.,y);
    glEnd();
  }  
  sequence_position=g_displayed_sequence_numbering[i_row];
  if(sequence_position!=0)
    strcpy(string2,num_string_int(sequence_position));
  else
    strcpy(string2," - ");
  fixcolor_plot(COLOR_LABEL);
  output_zoom(737,plot_i(i_row)-25,string2);
  fixcolor_plot(COLOR_TEXT);
  strcpy(string2,num_string_int(i_row));
  output_main(737,plot_i(i_row)-5,string2); 
}

void plot_ver_tics(void) {
  int i_row,step,start;
  step=step_fun(1,g_sequence_length);
  start=start_fun(1,step);
  single_ver_tic(1,FALSE);
  single_ver_tic(g_sequence_length,FALSE);
  for(i_row=start;i_row<(g_sequence_length-step/2);i_row=i_row+step) {
    single_ver_tic(i_row,g_grid_flag);
  }
}

void single_hor_tic_zoom(int j_col,int grid_flag) {
  char string2[10];
  float x;
  int sequence_position;
  x=(float)plot_j_zoom(j_col);
  glBegin(GL_LINES);
  glVertex2f(x,751);
  glVertex2f(x,758);
  glEnd();
  if(grid_flag==TRUE) {
    fixcolor_plot(COLOR_GRID); /* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(x,749);
    glVertex2f(x,50);
    glEnd();
  }
  sequence_position=g_displayed_sequence_numbering[j_col];
  if(sequence_position!=0)
     strcpy(string2,num_string_int(sequence_position));
  else
    strcpy(string2," - ");
  fixcolor_plot(COLOR_LABEL);
  output_zoom(plot_j_zoom(j_col)-10,780,string2);
  fixcolor_plot(COLOR_TEXT);
  strcpy(string2,num_string_int(j_col));
  output_zoom(plot_j_zoom(j_col)-10,758,string2);
}

void plot_hor_tics_zoom(void) { 
  int j_col,step;
  int start;
  step=step_fun(display_l,display_r);
  start=start_fun(display_l,step);
  single_hor_tic_zoom(display_r,FALSE);
  single_hor_tic_zoom(display_l,FALSE);
  for(j_col=start;j_col<=(display_r-2*step/3);j_col=j_col+step) {
    single_hor_tic_zoom(j_col,g_grid_flag);
  }
}

void single_ver_tic_zoom(int i_row,int grid_flag) {
  char string2[20];
  float y;
  int sequence_position;
  y=(float)plot_i_zoom(i_row);
  glBegin(GL_LINES);
  glVertex2f(727.,y);
  glVertex2f(736.,y);
  glEnd();
  if(grid_flag==TRUE) {
    fixcolor_plot(COLOR_GRID); /* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(23.,y);
    glVertex2f(723.,y);
    glEnd();
  }
  sequence_position=g_displayed_sequence_numbering[i_row];
  if(sequence_position!=0)
    strcpy(string2,num_string_int(sequence_position));
  else
    strcpy(string2," - ");
  fixcolor_plot(COLOR_LABEL);
  output_zoom(737,plot_i_zoom(i_row)-25,string2);
  fixcolor_plot(COLOR_TEXT);
  strcpy(string2,num_string_int(i_row));
  output_zoom(737,plot_i_zoom(i_row)-5,string2);
}

void plot_ver_tics_zoom(void) {
  int i_row,step,start;
  step=step_fun(display_t,display_b);
  start=start_fun(display_t,step);
  single_ver_tic_zoom(display_t,FALSE);
  single_ver_tic_zoom(display_b,FALSE);
  for(i_row=start;i_row<(display_b-step/2);i_row=i_row+step) {
    single_ver_tic_zoom(i_row,g_grid_flag);  
  }
}

void plot_tics_zoom(void) { 
  plot_hor_tics_zoom();
  plot_ver_tics_zoom(); 
  return; 
}

void plot_tics(void) {
  plot_hor_tics();
  plot_ver_tics();
  return;
}

/* converts window x coordinate to column coordinate of sequence.
 * top left for window is 0,0; x is right; y is down;
 * i is down and j is right for sequence
 */
int window_x_convert(int x) {
  float xf;
  xf=x-23.*width/800.;  /* subtract offset to right depending on window size */
  xf=xf*(g_sequence_length+1)/700.*800./width;
  return (int) xf;
}

/* convert window y coordinates to sequence i coordinate down */
int window_y_convert(int y) {
  float yf;                 
  int i_row;
  yf=(float)(y*800./height); /* scale according to window size */
  yf=799-yf; /* top is 0 or 800, different from window coordinates */
  i_row=(int)((g_sequence_length+1.)*(749.-yf)/700.+1.0);
  /* inverse of plot_ij function*/
  return i_row;
}

/* converts window x coordinates to a column */
int window_x_convert_zoom(int x) {
  float xf;                        
  float offset_to_window;
  offset_to_window = 5;
  xf=x-(25+offset_to_window)*width_z/(800.+offset_to_window);
  /* subtract offset to right depending on window size */
  xf=xf*(display_w+1)/700.*(800.+offset_to_window)/width_z+display_l-1;
  return (int)(xf+.5);
}

/* convert window y coordinates to sequence i coordinate down */
int window_y_convert_zoom(int y) {
  float yf;                 
  float offset_to_window;
  offset_to_window = 0;
  yf=y-(50.+offset_to_window)*height_z/(800+2*offset_to_window);
  /* subtract offset down depending on window */
  yf=yf*(display_h+1)/700*(800.+2*offset_to_window)/height_z+display_t-1;
  /*size */
  return (int)(yf+.5);
}

/* display functions */

void display_base_frequencies(int row, int col) {
  float fij[5][5];
  set_fij(fij,row,col);
  output_text(320,186,"A");
  output_text(385,186,"C");
  output_text(450,186,"G");
  output_text(515,186,"U");
  output_text(580,186,".");
  output_text(628,186,"Total");
  output_text(265,186,"Fij");
  output_text(260,160,"A");
  output_text(260,135,"C");
  output_text(260,110,"G");
  output_text(260,85,"U");
  output_text(260,60,".");
  output_text(245,35,"Total"); 
  output_text(190,110,num_string_int(row));
  output_text(130,110,"Row:");
  output_text(470,210,num_string_int(col));
  output_text(380,210,"Column:");
  /* row for Total */
  output_text(310,35,num_string_float(fij[1][1] + fij[2][1] + fij[3][1] 
				      + fij[4][1] + fij[0][1]));
  output_text(375,35,num_string_float(fij[1][2] + fij[2][2] + fij[3][2]
				      + fij[4][2] + fij[0][2]));
  output_text(440,35,num_string_float(fij[1][3] + fij[2][3] + fij[3][3]
				      + fij[4][3] + fij[0][3]));
  output_text(505,35,num_string_float(fij[1][4] + fij[2][4] + fij[3][4]
				      + fij[4][4] + fij[0][4]));
  output_text(570,35,num_string_float(fij[1][0] + fij[2][0] + fij[3][0]
				      + fij[4][0] + fij[0][0]));
  /* row for A */
  output_text(310,160,num_string_float(fij[1][1]));
  output_text(375,160,num_string_float(fij[1][2]));
  output_text(440,160,num_string_float(fij[1][3]));
  output_text(505,160,num_string_float(fij[1][4]));
  output_text(570,160,num_string_float(fij[1][0]));
  output_text(635,160,num_string_float(fij[1][1] + fij[1][2] + fij[1][3]
				       + fij[1][4] + fij[1][0]));
  /* row for C */
  output_text(310,135,num_string_float(fij[2][1]));
  output_text(375,135,num_string_float(fij[2][2]));
  output_text(440,135,num_string_float(fij[2][3]));
  output_text(505,135,num_string_float(fij[2][4]));
  output_text(570,135,num_string_float(fij[2][0]));
  output_text(635,135,num_string_float(fij[2][1] + fij[2][2] + fij[2][3] 
				       + fij[2][4] + fij[2][0]));
  /* row for G */
  output_text(310,110,num_string_float(fij[3][1]));
  output_text(375,110,num_string_float(fij[3][2]));
  output_text(440,110,num_string_float(fij[3][3]));
  output_text(505,110,num_string_float(fij[3][4]));
  output_text(570,110,num_string_float(fij[3][0]));
  output_text(635,110,num_string_float(fij[3][1] + fij[3][2] + fij[3][3]
				       + fij[3][4] + fij[3][0]));
  /* row for U */
  output_text(310,85,num_string_float(fij[4][1]));
  output_text(375,85,num_string_float(fij[4][2]));
  output_text(440,85,num_string_float(fij[4][3]));
  output_text(505,85,num_string_float(fij[4][4]));
  output_text(570,85,num_string_float(fij[4][0]));
  output_text(635,85,num_string_float(fij[4][1] + fij[4][2] + fij[4][3] 
				      + fij[4][4] + fij[4][0]));
  /* row for . */
  output_text(310,60,num_string_float(fij[0][1]));
  output_text(375,60,num_string_float(fij[0][2]));
  output_text(440,60,num_string_float(fij[0][3]));
  output_text(505,60,num_string_float(fij[0][4]));
  output_text(570,60,num_string_float(fij[0][0]));
  output_text(635,60,num_string_float(fij[0][1] + fij[0][2] + fij[0][3]
				      + fij[0][4] + fij[0][0]));
  glBegin(GL_LINES);
  glVertex2f(240,180);
  glVertex2f(680,180);
  glEnd(); 
  fixcolor_plot(COLOR_GRID);
  glBegin(GL_LINES);
  glVertex2f(240,155);
  glVertex2f(680,155);
  glVertex2f(240,130);
  glVertex2f(680,130);
  glVertex2f(240,105);
  glVertex2f(680,105); 
  glVertex2f(240,80);
  glVertex2f(680,80);
  glVertex2f(355,204);
  glVertex2f(355,30);
  glVertex2f(420,204);
  glVertex2f(420,30); 
  glVertex2f(485,204);
  glVertex2f(485,30); 
  glVertex2f(550,204);
  glVertex2f(550,30);
  glEnd();
  fixcolor_plot(COLOR_TEXT);
  glBegin(GL_LINES);
  glVertex2f(240,55);
  glVertex2f(680,55);
  glVertex2f(240,30);
  glVertex2f(680,30);
  glVertex2f(295,204);
  glVertex2f(295,30);
  glVertex2f(620,204);
  glVertex2f(620,30);
  glVertex2f(680,204);
  glVertex2f(680,30);
  glVertex2f(240,204);
  glVertex2f(681,204);
  glVertex2f(240,30);
  glVertex2f(240,204);
  glEnd(); 
}

void display_base_counts(int row, int col) {
  int fij[5][5];
 set_fij_int(fij,row,col);
 output_text(320,186,"A");
 output_text(385,186,"C");
 output_text(450,186,"G");
 output_text(515,186,"U");
 output_text(580,186,".");
 output_text(628,186,"Total");
 output_text(260,186,"Fij");
 output_text(260,160,"A");
 output_text(260,135,"C");
 output_text(260,110,"G");
 output_text(260,85,"U");
 output_text(260,60,".");
 output_text(245,35,"Total");
 output_text(190,110,num_string_int(row));
 output_text(130,110,"Row:");
 output_text(470,210,num_string_int(col));
 output_text(380,210,"Column:");
 /* row for Total */
 output_text(310,35,num_string_int(fij[1][1] + fij[2][1] + fij[3][1] 
				   + fij[4][1] + fij[0][1]));
 output_text(375,35,num_string_int(fij[1][2] + fij[2][2] + fij[3][2]
				   + fij[4][2] + fij[0][2]));
 output_text(440,35,num_string_int(fij[1][3] + fij[2][3] + fij[3][3] 
				   + fij[4][3]+fij[0][3]));
 output_text(505,35,num_string_int(fij[1][4] + fij[2][4] + fij[3][4]
				   + fij[4][4] + fij[0][4]));
 output_text(570,35,num_string_int(fij[1][0] + fij[2][0] + fij[3][0]
				   + fij[4][0] + fij[0][0]));
 output_text(635,35,num_string_int(fij[1][0] + fij[2][0] + fij[3][0]
				   + fij[4][0] + fij[0][0] + fij[1][1]
				   + fij[2][1] + fij[3][1] + fij[4][1]
				   + fij[0][1] + fij[1][2] + fij[2][2]
				   + fij[3][2] + fij[4][2] + fij[0][2]
				   + fij[1][3] + fij[2][3] + fij[3][3]
				   + fij[4][3] + fij[0][3] + fij[1][4]
				   + fij[2][4] + fij[3][4] + fij[4][4]
				   + fij[0][4]));
 /* row for A */
 output_text(310,160,num_string_int(fij[1][1]));
 output_text(375,160,num_string_int(fij[1][2]));
 output_text(440,160,num_string_int(fij[1][3]));
 output_text(505,160,num_string_int(fij[1][4]));
 output_text(570,160,num_string_int(fij[1][0]));
 output_text(635,160,num_string_int(fij[1][1] + fij[1][2] + fij[1][3]
				    + fij[1][4] + fij[1][0]));
 /* row for C */
 output_text(310,135,num_string_int(fij[2][1]));
 output_text(375,135,num_string_int(fij[2][2]));
 output_text(440,135,num_string_int(fij[2][3]));
 output_text(505,135,num_string_int(fij[2][4]));
 output_text(570,135,num_string_int(fij[2][0]));
 output_text(635,135,num_string_int(fij[2][1] + fij[2][2] + fij[2][3]
				    + fij[2][4] + fij[2][0]));
 /* row for G */
 output_text(310,110,num_string_int(fij[3][1]));
 output_text(375,110,num_string_int(fij[3][2]));
 output_text(440,110,num_string_int(fij[3][3]));
 output_text(505,110,num_string_int(fij[3][4]));
 output_text(570,110,num_string_int(fij[3][0]));
 output_text(635,110,num_string_int(fij[3][1] + fij[3][2] + fij[3][3]
				    + fij[3][4] + fij[3][0]));
/* row for U */
 output_text(310,85,num_string_int(fij[4][1]));
 output_text(375,85,num_string_int(fij[4][2]));
 output_text(440,85,num_string_int(fij[4][3]));
 output_text(505,85,num_string_int(fij[4][4]));
 output_text(570,85,num_string_int(fij[4][0]));
 output_text(635,85,num_string_int(fij[4][1] + fij[4][2] + fij[4][3]
				   + fij[4][4] + fij[4][0]));
/* row for . */
 output_text(310,60,num_string_int(fij[0][1]));
 output_text(375,60,num_string_int(fij[0][2]));
 output_text(440,60,num_string_int(fij[0][3]));
 output_text(505,60,num_string_int(fij[0][4]));
 output_text(570,60,num_string_int(fij[0][0]));
 output_text(635,60,num_string_int(fij[0][1] + fij[0][2] + fij[0][3]
				   + fij[0][4] + fij[0][0]));
 glBegin(GL_LINES);
 glVertex2f(240,180);
 glVertex2f(680,180);
 glEnd(); 
 fixcolor_plot(COLOR_GRID);
 glBegin(GL_LINES);
 glVertex2f(240,155);
 glVertex2f(680,155);
 glVertex2f(240,130);
 glVertex2f(680,130);
 glVertex2f(240,105);
 glVertex2f(680,105); 
 glVertex2f(240,80);
 glVertex2f(680,80);
 glVertex2f(355,204);
 glVertex2f(355,30 );
 glVertex2f(420,204);
 glVertex2f(420,30 ); 
 glVertex2f(485,204);
 glVertex2f(485,30); 
 glVertex2f(550,204);
 glVertex2f(550,30);
 glEnd();
 fixcolor_plot(COLOR_TEXT);
 glBegin(GL_LINES);
 glVertex2f(240,55);
 glVertex2f(680,55);
 glVertex2f(240,30);
 glVertex2f(680,30);
 glVertex2f(295,204);
 glVertex2f(295,30);
 glVertex2f(620,204);
 glVertex2f(620,30);
 glVertex2f(680,204);
 glVertex2f(680,30);
 glVertex2f(240,204);
 glVertex2f(681,204);
 glVertex2f(240,30);
 glVertex2f(240,204);
 glEnd(); 
}

void display_text_data(void) {
  char text[80];
  float cutoff;
  int row = 1,column = 1;
  fixcolor_plot(COLOR_TEXT);
  output_text(800,182,"Cutoff");
  cutoff=g_cutoff/(float)BIG_CONVERT_FACTOR;
  output_text(880,182,num_string_float(cutoff));
  /* color table */
  if(g_conserved_bp_flag) {
    fixcolor_plot(1);
    output_text(780,152," Conserved Base pair");
  }
  if (g_cutoff<=2000) {
    fixcolor_plot(2);
    if (g_cutoff>=1400) {
      strcpy(text," [");
      strcat(text,num_string_float(cutoff));
      strcat(text,", 2.00]");
      output_text(800,132,text);
    } else {
      output_text(800,132," [1.40, 2.00]");
      fixcolor_plot(3);
      if(g_cutoff>=1000) {
	strcpy(text," [");
	strcat(text,num_string_float(cutoff));
	strcat(text,", 1.40)");
	output_text(800,102,text);
      } else {
	output_text(800,102," [1.00 ,1.40)");
	fixcolor_plot(4);
	if(g_cutoff>=700) {
	  strcpy(text," [");
	  strcat(text,num_string_float(cutoff));
	  strcat(text,", 1.00)");
	  output_text(800,72,text);
	} else {
	  output_text(800,72," [ .70 ,1.00)");
	  fixcolor_plot(5);
	  if(g_cutoff>=400) {
	    strcpy(text," [");
	    strcat(text,num_string_float(cutoff));
	    strcat(text,", 0.70)");
	    output_text(800,42,text);
	  } else {
	    output_text(800,42," [ .40 ,0.70)");
	    fixcolor_plot(6);
	    strcpy(text," [");
	    strcat(text,num_string_float(cutoff));
	    strcat(text,", 0.40)");
	    output_text(800,12,text);
	  }
	}
      }
    }
  }
  fixcolor_plot(COLOR_TEXT);
  if(g_i_row>0) {
    if(g_frequency_flag)
      display_base_frequencies(g_i_row,g_j_col);
    else
      display_base_counts(g_i_row,g_j_col);
    strcpy(text,"(");
    strcat(text,num_string_int(g_i_row));
    strcat(text,",");
    strcat(text,num_string_int(g_j_col));
    strcat(text,")");
    output_text(5,165,text);
    strcpy(text,"value = ");
    if(g_i_row<g_j_col) {
      row=g_i_row;
      column=g_j_col;
    } else {
      column=g_i_row;
      row=g_j_col;
    }
    strcat(text, num_string_float(((float) g_big_matrix[row][column])/
				  (float)BIG_CONVERT_FACTOR));
    output_text(5,140,text);
  }
  fixcolor_plot(COLOR_LABEL);
  strcpy(text,g_data[g_displayed_sequence].name);
  output_text(5,95,text);
  if(g_i_row>0) {
    strcpy(text,"(");
    text[1]=g_data[g_displayed_sequence].sequence[row];
    text[2]=',';
    text[3]=g_data[g_displayed_sequence].sequence[column];
    text[4]=')';
    text[5]='\0';
    output_text(5,71,text);
    strcpy(text,"(");
    strcat(text,num_string_int(g_displayed_sequence_numbering[g_i_row]));
    strcat(text,",");
    strcat(text,num_string_int(g_displayed_sequence_numbering[g_j_col]));
    strcat(text,")");
    output_text(5,51,text);
  } 
}

void display_text(void) { /* int i; */
  char text_message[140];
  sniff_for_opengl_errors();
  glClear(GL_COLOR_BUFFER_BIT); 
  display_text_data(); 
  fixcolor_plot(COLOR_TEXT);
  if(g_message_set==TRUE) {
    glutPopWindow();
    fixcolor_plot(COLOR_LABEL);
    strcpy(text_message,text_window_message);
    strcat(text_message,"_");
    output_text(10,5,text_message);
    if(data_entered==TRUE) {
      data_entered=FALSE;
      g_message_set=FALSE;
      if(g_type_message==MESSAGE_POST)
	finish_making_post();
      if(g_type_message==MESSAGE_ID)
	finish_making_id_list();
      if(g_type_message==MESSAGE_OUT)
	finish_making_out_file(); 
      if(g_type_message==MESSAGE_CUTOFF) {
	finish_setting_energy_cutoff();
	text_window_input[0]='\0';
      }
      if(g_type_message==MESSAGE_REFERENCE) {
	finish_setting_reference();
	text_window_input[0]='\0';
      }              
      if(g_type_message==MESSAGE_ALIGN_ROW_COL) {
	finish_setting_align_row_col();
	g_message_set=TRUE;
	text_window_input[0]='\0';
      }  
      if(g_type_message==MESSAGE_REF_ROW_COL) {
	finish_setting_ref_seq_row_col(TRUE);
	g_message_set=TRUE;
	text_window_input[0]='\0';
      } 
    }
  }  
  sniff_for_opengl_errors();
  glutSwapBuffers(); 
}

void display_main(void) {
  float min_point_size;
  float line_t,line_l,line_r,line_b;
  float point_size; /* size of points to plot within box */
  glClear(GL_COLOR_BUFFER_BIT);
  glPushMatrix();
  glRotatef(360-spin,350.,350.,1.0);
  glLineWidth(1.0);
  fixcolor_plot(COLOR_TEXT);
  plot_tics();
  glLineWidth(2.0);
  glBegin(GL_LINE_LOOP);    /* draw big box */
  glVertex2f(21.0,752.0);
  glVertex2f(21.0,46.0);
  glVertex2f(726.0,46.0);
  glVertex2f(726.0,752.0);
  glEnd();
  glLineWidth(1.0); 
  line_l = plot_j_float(.5);
  line_r = plot_j_float(g_sequence_length+.5);
  line_t = plot_i_float(.5);
  line_b = plot_i_float(g_sequence_length+.5);
  glBegin(GL_LINES); /* draw triangle */
  glVertex2f(line_l,line_t);
  glVertex2f(line_r,line_b); 
  glEnd();
  glLineWidth(2.0);
  point_size=700./(g_sequence_length+1)*width/800.*height/800.+.4;
  if(g_bigdot_mode)
    min_point_size = 2.0;
  else
    min_point_size = 1.0;
  if(point_size<min_point_size)
     point_size=min_point_size;
  glPointSize(point_size);
  plot_points();
  fixcolor_plot(COLOR_TEXT); 
  if(g_region_set==TRUE||(g_region_exists==TRUE)) { /* draw zoom */
    fixcolor_plot(COLOR_LABEL);
    glBegin(GL_LINE_LOOP);
    glVertex2f(region_real_l,region_real_t);
    glVertex2f(region_real_r,region_real_t);
    glVertex2f(region_real_r,region_real_b);
    glVertex2f(region_real_l,region_real_b);
    glEnd();
  }
  glPopMatrix(); /* remove this later */
  glutSwapBuffers(); 
  sniff_for_opengl_errors();
}
  
void display_zoom(void) {
  float min_point_size;
  void *temp_font;
  float point_size;
  int right=display_r, bottom=display_b, top=display_t, left=display_l,i;

  /* stripe_flag is false unless there are more than
   * 2 sequences and the image is zoomed enough to show multiple
   * squares at each spot */
  int stripe_flag=FALSE;
  
  glClear(GL_COLOR_BUFFER_BIT);
  glPushMatrix();
  glRotatef(spin*2.0,0.0,0.0,1.0);
  fixcolor_plot(COLOR_TEXT);
  plot_tics_zoom();
  point_size=700./((display_w+display_h)/2)*width_z/800.*height_z/800.+.60;
  
  if(g_bigdot_mode)
    min_point_size=2.0;
  else
    min_point_size=1.0;
  stripe_flag=FALSE;
  if(point_size<min_point_size)
    point_size=min_point_size;
  glPointSize(point_size);
  plot_points_zoom(point_size,stripe_flag);
  glLineWidth(2.0);
  fixcolor_plot(COLOR_TEXT);
  glBegin(GL_LINE_LOOP);
  glVertex2f(22.0,751.0);
  glVertex2f(22.0,48.0);
  glVertex2f(724.0,48.0);
  glVertex2f(724.0,751.0);
  glEnd();
  glLineWidth(1.0);
  fixcolor_plot(COLOR_TEXT);  
  /* draw diagional line to indicate no points below*/
  if( (display_r>=display_t) && (display_b>=display_l) ) {
    if (display_r>display_b) {
      bottom=display_b;
      right=display_b;
    } else {
      right=display_r;
      bottom=display_r;
    }
    if (display_l>display_t) {
      left=display_l;
      top=display_l;
    } else {
      top=display_t;
      left=top;
    }   
    glBegin(GL_LINES);
    glVertex2f(plot_j_zoom(left-1),plot_i_zoom(top-1));
    glVertex2f(plot_j_zoom(right+1),plot_i_zoom(bottom+1));
    glEnd();
  }
  fixcolor_plot(COLOR_LABEL);
  if(g_region_set) { /* draw zoom box */ 
    fixcolor_plot(COLOR_LABEL);
    glBegin(GL_LINE_LOOP);
    glVertex2f(region_real_l_zoom,region_real_t_zoom);
    glVertex2f(region_real_l_zoom,region_real_b_zoom);
    glVertex2f(region_real_r_zoom,region_real_b_zoom);
    glVertex2f(region_real_r_zoom,region_real_t_zoom);
    glEnd();
  }
  /* draw the labels */
  if(g_zoom_labels_count>0) {
    fixcolor_plot(COLOR_LABEL);
    temp_font=g_font_zoom;
    g_font_zoom=GLUT_BITMAP_TIMES_ROMAN_10;
    glEnable(GL_POINT_SMOOTH);
    for(i=0;i<g_zoom_labels_count;i++) {
      output_zoom(g_zoom_labels_x[i], g_zoom_labels_y[i],
		  g_zoom_labels_string[i]);
    }
    point_size=700./((display_w+display_h)/2)*width_z/800.*height_z/800.+.50;
    point_size=point_size*.5;
    glPointSize(point_size);
    glEnable(GL_POINT_SMOOTH);
    glBegin(GL_POINTS);
    for(i=0;i<g_zoom_labels_count;i++) {
      glVertex2f((float)g_zoom_labels_xl[i],
		 (float)g_zoom_labels_y[i]);
    }
    glEnd();
    glDisable(GL_POINT_SMOOTH);
    g_font_zoom=temp_font;
  }
  glPopMatrix(); /* remove this later */
  sniff_for_opengl_errors();
  glutSwapBuffers();
}

void text_reshape(int w,int h) {
  if(w>1100) {
    w=1100;
    glutReshapeWindow(w,h);
  }
  if(h>300) {
    h=300;
    glutReshapeWindow(w,h);
  }
  glViewport(0,0,(GLsizei) w,(GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0,1000.0,0.0,240.0,-1.0,1.0);
  glMatrixMode(GL_MODELVIEW);
  if(w<850)
    select_font_text(0);
  else
    select_font_text(1);
}

void reshape(int w, int h) {
  int current_window;
  int font;
  float offset;
  if(w>h) {
    h=w;
    glutReshapeWindow(w,h);
    glutPostRedisplay();
  }
  if(h>w) {
    w=h;
    glutReshapeWindow(w,h);
    glutPostRedisplay();
  }
   if(w>990) {
     w=990;
     h=990;
     glutReshapeWindow(w,h);
   }
   current_window = glutGetWindow();
   if(current_window == win_main) {
     if(w<h)
       h=w;
     else
       w=h;
   }
   if(current_window == win_main) {
     width=w;    
     height=h;
   } else {
     width_z=w;
     height_z=h;
   }
   if(w<700)
     font=0;
   else {
     if(w<900)
       font=1;
     else
       font=2;
   }
  if(current_window == win_main)
    select_font_main(font);
  else
    select_font_zoom(font); 
  glViewport(0, 0,(GLsizei)  w,(GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  
  if(current_window == win_main) {
    glOrtho(0,800.0, 0.0,800.0,-1.0,1.0); /* 0,0 is origin for drawing;
					   * lower left is 0,0 ,
					   * up is y and right is x
					   */
  } else {
    offset = -4.0;
    glOrtho(offset,800.0, offset,800.0-offset,-1.0,1.0);
  }
  sniff_for_opengl_errors();
  glMatrixMode(GL_MODELVIEW);
}


/* Functions to set zoom rectangle 
 * region_real_r, region_r; */
void fix_zoom_box(void) {
  region_real_l=23.0+(float)((region_l+.5)*700./(g_sequence_length+1));
  region_real_r=23.0+(float)((region_r+.5)*700./(g_sequence_length+1));
  region_real_b=749.0-(float)((region_b-.5)*700./(g_sequence_length+1));
  region_real_t=749.0-(float)((region_t-.5)*700./(g_sequence_length+1));
}

void finish_fixing_zoom_box(void) { 
  int temp;  /* make left less than right */
  if(region_r<region_l) {
    temp=region_r;
    region_r=region_l;
    region_l=temp;
    fix_zoom_box();
  }
  if(region_b<region_t)  { /* make top less than bottom */
    temp=region_t;
    region_t=region_b;
    region_b=temp;
    fix_zoom_box();
  }
  if(g_first_zoom>0) {
    saved_display_l=display_l;
    saved_display_r=display_r;
    saved_display_t=display_t;
    saved_display_b=display_b;
  } else {
    g_first_zoom=1;
    saved_display_l=region_l;
    saved_display_r=region_r;
    saved_display_t=region_t;
    saved_display_b=region_b;
  }
  display_l=region_l;
  display_r=region_r;
  display_t=region_t;
  display_b=region_b;
  display_w=display_r-display_l+1;
  display_h=display_b-display_t+1;
}

void fix_zoom_box_zoom(void) {
  float left,right,top,bottom;
  region_real_l = 25.0 + (float) region_l*700./g_sequence_length;
  region_real_r = 25.0 + (float) region_r*700./g_sequence_length;
  region_real_b = 50.0 + 
    (float) (g_sequence_length-region_b)*700./g_sequence_length;
  region_real_t = 50.0 + 
    (float) (g_sequence_length-region_t)*700./g_sequence_length;
  left = 25.4 + (float) (region_l-display_l+1)*700./(display_w+1);
  right = 25.4 + (float) (region_r-display_l+1)*700./(display_w+1);
  top = 50. + (float) (display_b-region_t+1)*700./(display_h+1);
  bottom = 50. + (float) (display_b-region_b+1)*700./(display_h+1);
  if (left>=right) {
    left = right + 0.5;
    right += 0.5;
  }
  if(left<0.) left=0.;
  if(right<0.) right=0.;
  if(right>799.) right=800.;
  if(left>799.) left=799.;
  if (top<=bottom) {
    top = bottom + 0.5;
    bottom -= 0.5;
  }
  if(top<0) top=0;
  if(top>799.) top=799.;
  if(bottom<0.) bottom=0.;
  if(bottom>799.) bottom=799.;
  region_real_l_zoom = left;
  region_real_r_zoom = right;
  region_real_t_zoom = top;
  region_real_b_zoom = bottom;
}

void finish_fixing_zoom_box_zoom(void) {
  int temp;  /* make left less than right */
  if(region_r<region_l) {
    temp=region_r;
    region_r=region_l;
    region_l=temp;
  }
  if(region_b<region_t) { /* make top less than bottom */
    temp=region_t;
    region_t=region_b;
    region_b=temp;
  }
  saved_display_l=display_l;
  saved_display_r=display_r;
  saved_display_t=display_t;
  saved_display_b=display_b;
  display_l=region_l;
  display_r=region_r;
  display_t=region_t;
  display_b=region_b;
  display_w=display_r-display_l+1;
  display_h=display_b-display_t+1;
  fix_zoom_box_zoom(); 
}

/* Mouse Functions ___________*/
void mouse(int button, int state, int x, int y) {
  int i_row,j_col;
  j_col=window_x_convert(x);
  i_row=window_y_convert(y);
  if (i_row<1) i_row=1;
  if (i_row>g_sequence_length) i_row=g_sequence_length;
  if (j_col<1) j_col=1;
  if (j_col>g_sequence_length) j_col=g_sequence_length;
  if (button==GLUT_LEFT_BUTTON) {
    if (state==GLUT_DOWN) {
      g_i_row=i_row;
      g_j_col=j_col;  
      glutSetWindow(win_text);
      glutPostRedisplay();  
    }
  } else if (button==GLUT_MIDDLE_BUTTON) {
    if (state==GLUT_DOWN) {
      if (g_region_set==FALSE) {
	region_l=j_col;
	region_t=i_row;
	region_b=i_row;
	region_r=j_col;
	g_region_set=TRUE;
      } else {
	printf("Within no motion mouse with g_region_set = 1\n");
	region_r=j_col; /* This code is never executed ? */
	region_b=i_row;
      }
      fix_zoom_box();
    } else {
      g_region_set=FALSE;
      g_region_exists=TRUE;
      finish_fixing_zoom_box();
      g_zoom_labels_count=0;
      g_zoom_window_displayed=TRUE;
      zoomSetWindow(); 
      glutShowWindow();
      glutPopWindow();
      fix_cursor();
      glutPostRedisplay();
    }
  }
}

/* returns second corner of zoom rectangle as mouse is moved */
void mouse_motion(int x,int y) {
  int i_row,j_col;
  if(g_region_set==TRUE) {
    j_col=window_x_convert(x);
    i_row=window_y_convert(y);
    if(i_row<1) i_row=1;
    if(j_col>g_sequence_length) j_col=g_sequence_length;
    if(i_row>g_sequence_length) i_row=g_sequence_length;
    if(j_col<1) j_col=1;
    region_r=j_col;
    region_b=i_row;
    fix_zoom_box();
    glutSetWindow(win_main);              
    glutPostRedisplay();
  }
}

void check_display_zoom_labels(int i_row, int j_col) {
  int place;
  Si value;
  if(j_col<i_row) return;
  value = g_big_matrix[i_row][j_col];
  if( (value>=g_cutoff) &&
      ( (g_conserved_bp_flag) || (value<=2000) ) ) {
    /* label the point if value is in range */
    if(g_zoom_labels_count<49) g_zoom_labels_count++;
    place=g_zoom_labels_count-1;  
    strcpy(rec,"<");
    strcat(rec,num_string_int(i_row));
    strcat(rec,",");
    strcat(rec,num_string_int(j_col));
    strcat(rec,">");
    strcpy(g_zoom_labels_string[place],rec);
    g_zoom_labels_x[place] = plot_j_zoom(j_col+1);
    g_zoom_labels_xl[place] =	plot_j_zoom(j_col);
    g_zoom_labels_y[place] =	plot_i_zoom(i_row);
    g_zoom_labels_row[place] = i_row;
    g_zoom_labels_col[place] = j_col;
    zoomSetWindow();
    glutPostRedisplay();
    return;  /* only label the point once */
  }
}

void mouse_zoom(int button, int state, int x, int y) {
  int i_row,j_col;
  j_col = window_x_convert_zoom(x);
  i_row = window_y_convert_zoom(y);
  if (i_row<display_t) i_row=display_t;
  if (i_row>display_b) i_row=display_b;
  if (j_col<display_l) j_col=display_l;
  if (j_col>display_r) j_col=display_r;
  if (button==GLUT_LEFT_BUTTON) {
    if(state==GLUT_DOWN) {
      g_i_row=i_row;
      g_j_col=j_col;  
      glutSetWindow(win_text);  
      glutPostRedisplay();
      check_display_zoom_labels(i_row,j_col);
      zoomSetWindow();
      glutPostRedisplay();
    }
  }
  if(button==GLUT_MIDDLE_BUTTON) {
    if(state==GLUT_DOWN) {
      if(g_region_set==FALSE) {
	region_l=j_col;
	region_t=i_row;
	region_b=i_row;
	region_r=j_col;
	g_region_set=TRUE;
      } else {
	printf("Within no motion mouse with g_region_set = 1\n");
	region_r = j_col; /* This code is never executed ? */
	region_b = i_row;
      }
      fix_zoom_box_zoom();
    } else {
      g_region_set = FALSE;
      g_region_exists = TRUE;
      g_zoom_labels_count=0;
      finish_fixing_zoom_box_zoom();
      zoomSetWindow();
      glutShowWindow();
      glutPostRedisplay();
      glutSetWindow(win_main);
      glutPostRedisplay();
    }
  }
}

void mouse_motion_zoom(int x,int y) {
  int i_row,j_col;
  if (g_region_set==TRUE) {
    j_col=window_x_convert_zoom(x);
    i_row=window_y_convert_zoom(y);
    if(i_row<display_t) i_row=display_t;
    if(i_row>display_b) i_row=display_b;
    if(j_col<display_l) j_col=display_l;
    if(j_col>display_r) j_col=display_r;
    region_r = j_col;
    region_b = i_row;
    fix_zoom_box_zoom();
    zoomSetWindow();
    glutPostRedisplay();
  }
}

void main_entry(int state) {
  if (state==GLUT_ENTERED) {
    g_which_window_i_am_in = win_main;
    glutSetWindow(g_which_window_i_am_in);
    glutPopWindow();
  }
}

void zoom_entry(int state) {
  if(state==GLUT_ENTERED) {
    g_which_window_i_am_in = win_zoom;
    glutSetWindow(g_which_window_i_am_in);
    glutPopWindow();
  }
}

void text_entry(int state) {
  if(state==GLUT_ENTERED) {
    g_which_window_i_am_in = win_text;
    glutSetWindow(g_which_window_i_am_in);
    glutPopWindow();
  }
}
void spinDisplay(void) {
  zoomSetWindow();
  spin = spin + 0.2;
  if(spin>360.0)
    spin = spin - 360;
  glutPostRedisplay();
  glutSetWindow(win_main);
  glutPostRedisplay();
}

void undo_insertions(void);

void reload_alignment(void) { 
  char ref[20], ref_row[20], ref_col[20], freq_arg[5];
  char string_cutoff[20], program_name[120]; 
  float float_cutoff;
  int row, col, error; 
  sprintf(ref,"%d",g_displayed_sequence);
  if(g_frequency_flag)
    strcpy(freq_arg,"-f");
  else
    strcpy(freq_arg," ");
  if(g_i_row>0) 
    row=g_displayed_sequence_numbering[g_i_row];
  else
    row=1;
  sprintf(ref_row,"%d",row);
  if(g_j_col>0)
    col=g_displayed_sequence_numbering[g_j_col];
  else
    col=1; 
  sprintf(ref_col,"%d",col);
  float_cutoff=((float)g_cutoff)/1000.;
  sprintf(string_cutoff,"%.4f",float_cutoff);
  strcpy(program_name,g_program_name);
  error = execlp(program_name, g_program_name, "-l", ref_row, ref_col, "-r",
		 ref, freq_arg, "-c", string_cutoff, g_in_align_filename,
		 (char *) 0);
  printf("Error is %d\n",error);
}

void No_Action_Menu(int msg) {
  char integer_value[15];
  if(msg==1)
    try_exit(0);
  if(msg==2) {
    glutSetWindow(g_which_window_i_am_in);
    glutPopWindow();
  } 
  if(msg==4) {
    printf("____________Reloading alignment______________\n");
    reload_alignment();
  }
  if(msg==5) {
    if(g_grid_flag==TRUE)
      g_grid_flag=FALSE;
    else
      g_grid_flag=TRUE;
    glutSetWindow(win_main); /* display main */
    glutPostRedisplay();
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutPostRedisplay();
    }  
  }
  if(msg==7) { /* go back to previous zoom window */
    region_l=saved_display_l;
    region_r=saved_display_r;
    region_t=saved_display_t;
    region_b=saved_display_b;
    if(region_r>g_sequence_length) region_r=g_sequence_length;
    if(region_b>g_sequence_length) region_b=g_sequence_length;
    fix_zoom_box();
    finish_fixing_zoom_box();
    g_zoom_labels_count=0;
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutPostRedisplay();
    }  
    glutSetWindow(win_main);
    glutPostRedisplay();
  }
  if(msg==10) {
    if(g_bigdot_mode==TRUE)
      g_bigdot_mode=FALSE;
    else
      g_bigdot_mode=TRUE;
    glutSetWindow(win_main);
    glutPostRedisplay();
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutPostRedisplay();
    }  
  }
  if(msg==51) {
    if(g_conserved_bp_flag==TRUE)
      g_conserved_bp_flag=FALSE;
    else
      g_conserved_bp_flag=TRUE;
    glutSetWindow(win_main);
    glutPostRedisplay();
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutPostRedisplay();
    }  
  }
  if(msg==63) {
    set_cutoff2();
    return;
  }
  /*
    if(msg==35) {
    zoomSetWindow();
    glutIdleFunc(spinDisplay);
    glutSetWindow(win_main);
    glutIdleFunc(spinDisplay);
    }
    if(msg==36) {
    zoomSetWindow();
    spin=0;
    glutIdleFunc(NULL);
    glutPostRedisplay();
    glutSetWindow(win_main);
    glutIdleFunc(NULL);
    } 
  */
  if(msg==37) {
    if (g_zoom_window_displayed==TRUE) { 
      glutDestroyWindow(win_zoom);
      g_zoom_window_exists = FALSE;
      g_zoom_window_displayed = FALSE;
    }
  }
  if(msg==66) {  /* reset reference sequence */
    strcpy(text_window_message,reference_sequence_message);
    g_message_set=TRUE;
    data_entered=FALSE;
    g_type_message=MESSAGE_REFERENCE;
    glutSetWindow(win_text);
    glutPostRedisplay();
    set_cursor_help();
  }
  if(msg==71)  {  /* reset alignment position for fij */
    strcpy(text_window_message,align_row_col_message);
    strcat(text_window_message,num_string_int(g_sequence_length));
    strcat(text_window_message,") ? ");
    g_message_set=TRUE;
    data_entered=FALSE;
    g_type_message=MESSAGE_ALIGN_ROW_COL;
    glutSetWindow(win_text);
    glutPostRedisplay();
    set_cursor_help();
  }
  if(msg==72) {  /* reset reference sequence position for fij*/
    strcpy(text_window_message,ref_seq_row_col_message); 
    strcat(text_window_message,num_string_int(g_ref_sequence_length));
    strcat(text_window_message,") ? ");
    g_message_set=TRUE;
    data_entered=FALSE;
    g_type_message=MESSAGE_REF_ROW_COL;
    glutSetWindow(win_text);
    glutPostRedisplay();
    set_cursor_help();
  }
  if(msg==57) { /* toggle between counts and frequence in Fij window */
    if(g_frequency_flag==TRUE)
      g_frequency_flag=FALSE;
    else
      g_frequency_flag=TRUE;
    glutSetWindow(win_text);
    glutPostRedisplay();
  }
  if(msg==43) {  /* prepare to save ID's for each base pair type */
    if(g_i_row<1)
      g_i_row=1;
    if(g_j_col<1)
      g_j_col=1;
    data_entered=FALSE;
    g_message_set=TRUE;
    strcpy(text_window_message,id_file_message);
    strcpy(g_id_list_filename,g_in_align_filename);
    strcat(g_id_list_filename,".");
    sprintf(integer_value,"%d",g_i_row);
    strcat(g_id_list_filename,integer_value);
    strcat(g_id_list_filename,"_");
    sprintf(integer_value,"%d",g_j_col);
    strcat(g_id_list_filename,integer_value);
    strcat(text_window_message,g_id_list_filename);
    strcat(text_window_message,")? ");
    g_type_message=MESSAGE_ID;
    glutSetWindow(win_text);
    glutPostRedisplay();
    set_cursor_help();
  }
  if(msg==26) {
    data_entered=FALSE;
    g_message_set=TRUE;
    strcpy(text_window_message,g_out_file_message);
    strcat(text_window_message,g_out_filename);
    strcat(text_window_message,")? ");
    g_type_message=MESSAGE_OUT;
    glutSetWindow(win_text);
    glutPostRedisplay();
    set_cursor_help();
  }
}  

void NoFunction(int junk) {
  if (junk==5)
    printf("Something weird happened!\n");
}

void make_menu(int window) {
  int postscript_submenu;
  glClearColor(g_color[COLOR_BACKGROUND][0], g_color[COLOR_BACKGROUND][1],
               g_color[COLOR_BACKGROUND][2],1.0);
  postscript_submenu = glutCreateMenu(create_post);
  glutAddMenuEntry(".ps for Main Window",0);
  glutAddMenuEntry("Alt .ps for Main Window",1);
  if (g_zoom_window_displayed == TRUE) {
    glutAddMenuEntry(".ps for Zoom Window",2);
    glutAddMenuEntry("Alt .ps for Zoom Window",3);
  }
  glutCreateMenu(No_Action_Menu);
  glutAddMenuEntry("Cutoff",63);  
  if(g_zoom_window_displayed == TRUE)
    glutAddMenuEntry("UnZoom ",7);   
  glutAddMenuEntry("Select Reference Sequence",66);
  glutAddMenuEntry("Toggle Conserved B.P On/Off",51);
  glutAddMenuEntry("Toggle Fij Frequency/Counts",57);
  glutAddMenuEntry("Select Alignment Position",71);
  glutAddMenuEntry("Select Reference Sequence Position",72);
  if(g_zoom_window_displayed == TRUE) {
    /*    glutAddMenuEntry("-------------",35);
	  glutAddMenuEntry("-------------",36); */
    glutAddMenuEntry("Close zoom window", 37);
  }
  glutAddSubMenu("Postscript File",postscript_submenu);
  glutAddMenuEntry("Base pair List File",26);
  glutAddMenuEntry("Save ID list file for current (row,col)",43);
  glutAddMenuEntry("Grid lines On/Off",5);
  glutAddMenuEntry("Big Dot On/Off",10);
  glutAddMenuEntry("Pop",2);
  glutAddMenuEntry("Reload Alignment",4);
  glutAddMenuEntry("Exit",1);
  glutAttachMenu(GLUT_RIGHT_BUTTON);
}

/* initialize graphics for each window */
void gfxinit_nontext(int window) {
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutReshapeFunc(reshape);
  make_menu(window);
}

/* Don't open zoom window until necessary */                              
void zoomSetWindow(void) {
  char data_filename[120]; 
  if (g_zoom_window_exists == FALSE) { 
    g_zoom_window_exists = TRUE;                                            
    width_z = 750;
    height_z = 750;
    glutInitWindowSize(750,750);
    glutInitWindowPosition(6,262);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
    strcpy(data_filename,"Mutplot Zoom for ");                              
    strcat(data_filename,g_in_align_filename); 
    win_zoom = glutCreateWindow(data_filename);
    gfxinit_nontext(win_zoom);  /* initialize graphics for zoom window */
    glutDisplayFunc(display_zoom); 
    glutMouseFunc(mouse_zoom);
    glutMotionFunc(mouse_motion_zoom);                                      
    glutEntryFunc(zoom_entry); 
  }
  if (g_zoom_window_displayed == TRUE) {
    glutSetWindow(win_zoom);
  } else {
    glutDestroyWindow(win_zoom);
    g_zoom_window_exists = FALSE;
  }
}
#endif

int main(int argc, char **argv) {
#ifdef HAVE_GLUTBITMAPCHARACTER
  int i;
  char data_filename[120];
  glutInit(&argc,argv) ;
  initialize_colors();
  first_initialize(argc,argv);
  /* first_initialize will read alignment file, plot files 
   * stores helices in memory according to alignment file */
  initialize_data2(); /* sets graphic global variables set label count
		       * to zero, sets MI cutoff */ 
  if((g_input_reference_sequence>0)&&
     (g_input_reference_sequence<=g_number_sequences)) {
    g_displayed_sequence=g_input_reference_sequence;
    set_numbering(g_displayed_sequence);
  }
  if((g_input_row>0)&&(g_input_col>0)) {
    finish_setting_ref_seq_row_col(FALSE);
  }   
  for (i = 1; i < argc; i++) {
    if (!strcmp(argv[i], "-mono")) {
      g_font_main = GLUT_BITMAP_HELVETICA_18;
      g_font_text = GLUT_BITMAP_HELVETICA_18;
      g_font_zoom = GLUT_BITMAP_HELVETICA_18;
    }
  }
  /* Zoom window - create and immediately destroy. */
  zoomSetWindow();
  /* Display main window */
  glutInitWindowSize(800,800);
  glutInitWindowPosition(474,218);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
  strcpy(data_filename,"Mutplot for ");
  strcat(data_filename,g_in_align_filename);
  win_main = glutCreateWindow(data_filename);
  gfxinit_nontext(win_main);  /* initialize graphics for Main Window */
  glutDisplayFunc(display_main);
  glutMouseFunc(mouse);
  glutMotionFunc(mouse_motion);
  glutEntryFunc(main_entry);
  /* Display text window */
  glutInitWindowSize(1000,240);
  glutInitWindowPosition(6,31);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
  strcpy(data_filename,"Mutplot Information for ");
  strcat(data_filename,g_in_align_filename);
  strcat(data_filename," with ");
  strcat(data_filename,num_string_int(g_number_sequences));
  strcat(data_filename," sequences of length: ");
  strcat(data_filename,num_string_int(g_sequence_length));
  win_text = glutCreateWindow(data_filename);
  glutDisplayFunc(display_text);
  glutReshapeFunc(text_reshape);
  make_menu(win_text);
  glutKeyboardFunc(keyboard);
  glutEntryFunc(text_entry);
  /* Now enter main loop */
  glutMainLoop();
#endif
  printf("mutplot not available because glut was not found.\n");
  try_exit(1);
  return 0;  /* Keep the stupic compiler happy. */
}
