/* This program is defective in many ways in probability mode.
   In the output, there is a text overwrite problem under the dot plot
   in image mode.
   The range of probabilities are always zero. 
   Probabilities stored as -1000 times floats, rather than -1000000
   times floats, as in other programs in the package
*/
#define PROGRAM_NAME "overlay_boxplot"
#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <limits.h>
#include "util.h"

/* On the Mac, libglut does not exist separately. Search instead for
   the function glutInit. Under Linux, glut.h should be in
   /usr/include/GL and /usr/include is automatically searched.
*/
#ifdef HAVE_GLUTBITMAPCHARACTER
void zoomSetWindow(void);
#if HAVE_LIBGLUT
# include <GL/glut.h>
#else
# include <glut.h>
#endif

#include "overlay_boxplot.h"
#include "overlay_boxplot_setcolor.h"
#include "overlay_boxplot_read_ct.h"
#include "overlay_boxplot_ps.h"
#if defined(HAVE_LIBGD) && defined(HAVE_GD_H)
# include "overlay_boxplot_img.h"
#endif

#define MESSAGE_NONE   0	/* values for g_type_message */
#define MESSAGE_ENERGY 2
#define MESSAGE_CLOSE 3
#define MESSAGE_POST   5
#define MESSAGE_GIF    4

struct helix    g_diag[MAXIMUM_HELICES];

int     g_opt_mode; /* true for optimal, coloring
		    * false for ct coloring or older method */
int     g_opt_distance;	/* distance for optimal within ct for opt mode */
int     g_len;	/* Maximum of farthest base pair in ct, plot files */
int     g_plot_len; /* maximum base pair in plot file */
int     g_ct_len; /* maximum base pair in ct file */
int     g_width; /* these are the current width and height of the main 
		  * window in pixels */
int     g_height;	
int     g_width_z = 750, g_height_z = 750; /* same for zoom window */

int     first_zoom = 0;	/* used for backing up to previous zoom window
			 * when right mouse button is clicked,
			 * transform coordinates into i,j 
			 */ 
int     g_i_row = 0;
int     g_j_col = 0;	/* for i.j base pair with mouse */
char    g_i_row_s[10], g_j_col_s[10]; /* string of above */
int     win_main, win_zoom, win_text; 

/* ______________________________________________________________
 * Notice that most functions are called based on events in the  
 * event queue.  They do not call each other.  This results in too
 * many global variables.         
 * Variables that could easily be changed for personal preference are
 * usually accompanied by $$$.  Colors in particular  
 * __________________________________________________________________ 
 */

/* text window variables _________________________________________ 
 *
 * Should text display window show text
 * window message?
 * It indicates to enter postscript filename or energy increment 
 * values below for text_window_message based on g_message_set
 */

int  g_message_set = FALSE;	
char *increment_message_energy = 
"How many kcal/mol for the energy cutoff? (/ to abort) ";
char *increment_message_prob =
"What probability is the absolute cutoff? (.000 to .999)  (/ to abort)  ";
char *close_energy_message = 
"How many kcal/mol for close to optimal (/ to abort) ";
char *close_distance_message =
"How close should optimal base pairs be to ct  (0 to 9) ";
char *post_message = "Enter name for postscript file (/ to abort";
char *img_message = "Enter name for image file (/ to abort";
char text_window_message[80];
char text_window_input[80];
int  data_entered = FALSE;	/* for text display window */
int  g_type_message;	/* 2 for energy increment, 3 for close energy
			 * message and 0 for none */

/* Font variables  ______________________________________________ */

void *g_font_text = GLUT_BITMAP_TIMES_ROMAN_24;	/* initialize font $$$$$$ */
void *g_font_zoom = GLUT_BITMAP_HELVETICA_18;
void *g_font_main = GLUT_BITMAP_HELVETICA_18;
void *fonts[] =	{ /* used by select_font */
  GLUT_BITMAP_HELVETICA_12,
  GLUT_BITMAP_HELVETICA_18,
  GLUT_BITMAP_TIMES_ROMAN_24
};

/* Graphic  variables  ------------------------------------------- */

float g_color_increment;	/* range of energy for each color */
int   g_energy_cutoff;	/* only probabilities <= */
/* g_energy_cutoff will be shown.
 * note that probabilities have a negative sign
 * -30 indicates probabilities >=.3 will be shown 
 */
int   g_chain_len = 1;	/* chains smaller will not be displayed */
int   points_plotted, points_plotted_zoom; /* indicates number of
					    *  points plotted in each
					    *  window */  

/* Zoom Variables __________________________________________ */
int   g_region_set; /* 0 indicates the top left corner of the zoom
		     * region has not been set */ 
int   region_t, region_b, region_r, region_l;	
/* specify boundaries of the zoom  rectangle in i,j coordinates for
 * main window or for the zoom window 
 */
float region_real_l, region_real_r, region_real_b, region_real_t;
/* screen coordinates of above (i,j) pairs for the main window (blue
 * rectangle) */ 
float region_real_l_zoom, region_real_r_zoom, region_real_t_zoom;
float region_real_b_zoom; /* Screen coordinates for a zoom rectangle
			     within the zoom window */ 
int   g_region_exists;

int   display_l, display_r, display_t, display_b, display_w, display_h;
/* left, right, top, bottom, width and height for zoom window display
   in nucleotide coordinates */ 
int   saved_display_l, saved_display_r, saved_display_t, saved_display_b;
/* Above used to go back to previous zoom window */

int   g_zoom_labels_count;	/* the number of labels used */
int   g_zoom_labels_x[50];	/* x coordinate for label i */
int   g_zoom_labels_xl[50];	/* used to plot on screen the dot */
int   g_zoom_labels_y[50];
int   g_zoom_labels_row[50];
int   g_zoom_labels_col[50];

char  *g_zoom_labels_string[50];	/* value to place at x,y */

/* File Variables _______________________________________________ */
char rec[90];
FILE *fp;		/* for .plot file */
char g_post_filename[80];
char g_output_filename[80];	/* as provided by user */
char filename1[80];	/* these two are the plot and ct file */
char filename2[80];	/* Once it is clear which is which, store below */
char g_plot_filename[80];
int  g_ct_red, g_ct_green, g_ct_yellow, g_ct_missed, g_optimal_dots;
int  g_zoom_optimal_dots, g_zoom_ct_missed, g_zoom_ct_red, g_zoom_ct_green;
int  g_zoom_ct_yellow, g_red_gray_overlap, g_yellow_gray_overlap;
char g_ct_filename[80];
char g_img_filename[80];	/* this depends on -o or input file */
char g_img_filename_opt[80];

int  g_prob;
int  g_grid_flag;	/* True for use grid lines False for do not */
int  g_zoom_window_exists = FALSE;
int  g_zoom_window_displayed = FALSE;
int  g_optimal_energy;
int  g_worst_energy;
int  g_post_file;	/* 1 for main, 2 for zoom used from text display */ 
int  *g_diag_count;
int  *g_diag_start;
int   g_ct_diag_count[2 * MAXIMUM_SIZE];
int   g_ct_diag_start[2 * MAXIMUM_SIZE];
int   g_ct_files = 1;	/* Always 1 for this program. It is a
			 * true variable in ct_boxplot */ 
char  filename_ct[MAXIMUM_CT_FILES][80];	/* allow at most
						 * MAXIMUM_CT_FILES */
/* filenames for input */
char  file_data_ct[MAXIMUM_CT_FILES][80];	/* first line of ct file */

struct ct_helix g_ct_diag[MAXIMUM_HELICES];
char  g_sequence_name[120];
int   g_sequence_name_set;
int   g_helices_in_plot_file;	/* number of entries in file */
int   g_helices_in_ct_file;
int   g_resolution;	/* 72 110 200 or 300 */
/* _________________________________________________________________ */
/* given a row, col return the helix length of the helix at the position */
/* in the plot file */
/* return 0 if no helix is defined */
int chain_length_read(int row, int col) {
  int   diag, diag_start, diag_end, i, temp_row;
  /* perhaps switch row,col when row>col */
  if (row > col) {
    temp_row = col;
    col = row;
    row = temp_row;
  }
  diag = col + row - 1;
  diag_start = g_diag_start[diag];
  diag_end = g_diag_start[diag + 1];
  if ((row > g_plot_len) || (col > g_plot_len))
    return 0;
  for (i = diag_start; i < diag_end; i++) {
    temp_row = g_diag[i].row;
    if (temp_row > row)
      return INT_MIN;	/* entries in diagonal are sorted */
    if ((temp_row <= row) && (g_diag[i].row + g_diag[i].length - 1 >= row))
      return g_diag[i].length;
  }
  return 0;
}

/* given a row and column, return the energy of that position */

int energy_read(int row, int col) {
  int   diag, diag_start, diag_end, i, temp_row;
  /* perhaps switch row,col when row>col */
  if (row > col) {
    temp_row = col;
    col = row;
    row = temp_row;
  }
  diag = col + row - 1;
  diag_start = g_diag_start[diag];
  diag_end = g_diag_start[diag + 1];
  if ((row > g_plot_len) || (col > g_plot_len))
    return INT_MIN;
  for (i = diag_start; i < diag_end; i++) {
    temp_row = g_diag[i].row;
    if (temp_row > row)
      return INT_MIN;	/* entries in diagonal are sorted */
    if ((temp_row <= row) && (g_diag[i].row + g_diag[i].length - 1 >= row))
      return g_diag[i].energy;
  }
  return INT_MIN;
}
/* given a row and column, return TRUE if  a ct dot exists  */
int ct_read(int row, int col) {
  int   diag, diag_start, diag_end, i, temp_row;
  diag = col + row - 1;
  diag_start = g_ct_diag_start[diag];
  diag_end = g_ct_diag_start[diag + 1];
  if ((row > g_ct_len) || (col > g_ct_len))
    return FALSE;
  for (i = diag_start; i < diag_end; i++) {
    temp_row = g_ct_diag[i].row;
    if (temp_row > row)
      return FALSE;	/* entries in diagonal are sorted */
    if ((temp_row <= row) && (g_ct_diag[i].row+g_ct_diag[i].length-1 >= row)) {
      return TRUE;
    }
  }
  return FALSE;
}

/* ____________________________________________________________________ */
/* Initialization Routines for nongraphics */
void check_output_file(int user_output_flag, char *output_filename) {
  if (user_output_flag) {
    strcpy(g_post_filename, output_filename);
    strcpy(g_img_filename, output_filename);
  } else {
    strcpy(g_post_filename, g_plot_filename);
    strcpy(g_img_filename, g_plot_filename);
    chop_suffix(g_post_filename, ".plot"); /* remove .plot if present */
    chop_suffix(g_img_filename, ".plot");
    chop_suffix(g_img_filename, ".img"); /* remove .gif, .jpg or .png */
  }
  strcpy(g_img_filename_opt, g_img_filename);
  strcat(g_img_filename, ".ovr-ct.gif");
  strcat(g_img_filename_opt, ".ovr-opt.gif");
}

void insufficient_arguments(void) {
  printf("Insufficient arguments:\n");
  printf("[ Usage:     %s name1.plot name2.ct            ]\n", PROGRAM_NAME);
  printf("[        or  %s -o name_out name1.plot name2.ct]\n", PROGRAM_NAME);
  printf("[         (.plot or .ct suffixes optional)                  ]\n");
  printf("[                                                           ]\n");
  printf("[           Valid arguments                                 ]\n");
  printf("[ -o name_out  Specify name of output file as               ]\n");
  printf("[              name_out.ovr.ps or name_out.ovr.png          ]\n");
  printf("[ -t \"TITLE\"   Specify TITLE as title, (Quotes Required)  ]\n");
  try_exit(1);
}

void open_plot_file(char *filename1, char *filename2) {
  char  filename[120];
  strcpy(filename, filename1);
  chop_suffix(filename, ".plot"); /* strip off .plot if it is there */
  strcat(filename, ".plot");	/* put .plot on */
  printf("Trying to open %s as a plot file.\n", filename);
  if ((fp = fopen(filename, "r")) == NULL) {
    strcpy(filename, filename2);
    chop_suffix(filename, ".plot");
    strcat(filename, ".plot");
    printf("Trying to open  %s as a plot file.\n", filename);
    if ((fp = fopen(filename, "r")) == NULL) {
      printf("Error!\tCould not open a plot file.\n");
      insufficient_arguments();
    } else {
      strcpy(g_plot_filename, filename);
      strcpy(g_ct_filename, filename1);
    }
  } else {
    strcpy(g_plot_filename, filename);
    strcpy(g_ct_filename, filename2);
  }
  chop_suffix(g_ct_filename, ".ct");
  strcat(g_ct_filename, ".ct");
}

int compare_helices(const void *h1, const void *h2) {
  int   dif;
  struct helix   *helix1, *helix2;
  helix1 = (struct helix *) h1;
  helix2 = (struct helix *) h2;
  dif = (*helix1).diagonal - (*helix2).diagonal; /* return <0 if diag1<diag2 */
  if (dif == 0)		/* on same diagonal */
    dif = (*helix1).row - (*helix2).row; /* return <0 if row1<row2 or >0 if > */
  return dif;
}

void sort_helices(void) {
  qsort(g_diag, g_helices_in_plot_file,
	sizeof(struct helix), compare_helices);
}

void print_helix(int number_of_helices) {
  int   i;
  for (i = 0; i < number_of_helices; i++) {
    printf("diag=%d, row=%d, color=%d, column=%d, energy=%d, length=%d\n",
	   g_diag[i].diagonal, g_diag[i].row, g_diag[i].color, 
	   g_diag[i].column, g_diag[i].energy, g_diag[i].length);
  }
}

void fix_counts(void) {
  int   i;
  for (i = 0; i < g_helices_in_plot_file; i++) {
    g_diag_count[g_diag[i].diagonal]++;	/* update the count for this
					 * diagonal */ 
  }
}

void fix_colors(void) {
  /* set the colors for each line of input */
  int   i;
  for (i = 0; i < g_helices_in_plot_file; i++) {
    g_diag[i].color = find_color(g_diag[i].energy);
  }
}
void initialize_data2(void) {
  g_chain_len = 1;	/* skip chains smaller than this */
  g_number_of_colors = 4;	/* start with 4 colors $$$$$ */
  g_grid_flag = TRUE;
  g_width = 800;	/* initialize size of window in pixels */
  g_height = 800;	/* needed for scale in drawing windows */
  g_energy_cutoff = g_worst_energy;
  if (g_prob != TRUE)
    g_color_increment = ((float) (g_energy_cutoff-g_optimal_energy)) /
      ((float) (g_number_of_colors - 1));
  initialize_colors();
  g_region_set = FALSE;
  g_region_exists = FALSE;
  g_message_set = FALSE;
  g_opt_mode = FALSE;
  g_opt_distance = 0;
  text_window_input[0] = '\0';
  text_window_message[0] = '\0';
  g_type_message = MESSAGE_NONE;
}

void initialize_len(void) {
  /* g_worst_energy and
   * g_optimal_energy are also set */
  int   level, length, row, col, energy, diag;
  int   i;
  /* diag is the diagonal that each row,col corresponds to */
  float rec_energy;
  g_helices_in_plot_file = 0;
  if (fgets(rec, 90, fp) == NULL) {
    printf("Error!\tPlot file is completely empty\n");
    try_exit(2);
  }
  g_zoom_labels_count = 0;	/* initialize to no labels */
  for (i = 0; i < 50; i++) {	/* allocate memory for string */
    g_zoom_labels_string[i] = (char *) malloc(15 * sizeof(char));
    if (g_zoom_labels_string[i] == NULL) {
      printf("Error!\tInsufficient memory for zoom labels\n");
      try_exit(3);
    }
  }
  g_plot_len = 0;
  /* Read the plot file */
  g_worst_energy = INT_MIN;
  g_optimal_energy = INT_MAX;
  while (fgets(rec, 90, fp) != NULL) {
    sscanf(rec, "%d%d%d%d%f", &level, &length, &row, &col, &rec_energy);
    if (g_plot_len < col)
      g_plot_len = col;
    g_diag[g_helices_in_plot_file].length = length;
    g_diag[g_helices_in_plot_file].row = row;
    g_diag[g_helices_in_plot_file].column = col;
    if (rec_energy > 0.0)
      energy = (int) (-1 * (1000. * rec_energy + .50));
    else
      energy = (int) (rec_energy);
    g_diag[g_helices_in_plot_file].energy = energy;
    if (rec_energy > 0)
      g_prob = TRUE;
    if (energy < g_optimal_energy)
      g_optimal_energy = energy;
    if (energy > g_worst_energy)
      g_worst_energy = energy;
    diag = row + col - 1;
    g_diag[g_helices_in_plot_file].diagonal = diag;
    g_helices_in_plot_file++;
    if (g_helices_in_plot_file == MAXIMUM_HELICES) {
      printf("Error!\tThere are more than %d helices in the plot file.\n",
	     MAXIMUM_HELICES);
      printf("Increase the constant, MAXIMUM_HELICES, in %s.c", PROGRAM_NAME);
      printf(" and recompile.\n");
      try_exit(4);
    }
  }
  printf("There are %d helices in the plot file.\n", g_helices_in_plot_file);
  if (g_helices_in_plot_file == 0) {
    printf("Error!\tThe plot file contains no helices\n");
    try_exit(5);
  }
  /*
   * glob_diag_count counts the total number of entries in each diagonal
   */
  fclose(fp);
  /* Sort the plot file */
  sort_helices();
  g_diag_start = (int *) malloc((2 * g_plot_len) * sizeof(int));
  if (g_diag_start == NULL)
    printf("Error!\tInsufficient memory for g_diag_start\n");
  g_diag_count = (int *) malloc((2 * g_plot_len) * sizeof(int));
  if (g_diag_count == NULL)
    printf("Error!\tInsufficient memory for g_diag_count");
  for (i = 0; i <= (2 * g_plot_len - 1); i++) g_diag_count[i] = 0;
  /* fix the count of diagonal for each entry in plot file */
  initialize_data2();
  fix_colors();
  g_diag_start[1] = 0;
  fix_counts();
  for (i = 2; i <= (2 * g_plot_len - 1); i++) {
    g_diag_start[i] = g_diag_start[i - 1] + g_diag_count[i - 1];
  }
  /* g_diag_start counts forward into each array for the start of
   * each diagonal */
}

void open_ct_file(char *ct_filename) {
  g_ct_len = 0;
  printf("The farthest base pair in the plot file is %d.\n", g_plot_len);
  strcpy(filename_ct[0], ct_filename);
  g_helices_in_ct_file = 0;
  read_of_all_ct_files(fp, g_ct_diag_count, g_ct_diag_start, g_ct_files,
		       &g_helices_in_ct_file, &g_ct_len, filename_ct, 
		       g_ct_diag, file_data_ct, g_sequence_name,
		       g_sequence_name_set);
  if (g_ct_len > g_plot_len) {
    printf("The ct file has a base pair beyond any helix in the plot file\n");
    g_len = g_ct_len;
  } else {
    g_len = g_plot_len;
  }
}

void initialize_data1(int argc, char **argv) {
  int   i, files_used;
  int   user_output_flag;
  g_prob = FALSE;
  user_output_flag = FALSE;
  g_sequence_name_set = FALSE;
  files_used = 0;
  i = 1;
  while (i < argc) {
    if (strcmp(argv[i], "-o") == 0) {
      if (i < (argc - 1)) {
	user_output_flag = TRUE;
	strcpy(g_output_filename, argv[i + 1]);
	i++;
      } else {
	printf("Error!\tThe output file name must follow -o.\n");
      }
    } else if (strcmp(argv[i], "-t") == 0) {
      if (i < (argc - 1)) {
	g_sequence_name_set = TRUE;
	strcpy(g_sequence_name, argv[i + 1]);
	i++;
      }
    } else if (files_used == 0) {
      strcpy(filename1, argv[i]);
      files_used++;
    } else if (files_used == 1) {
      strcpy(filename2, argv[i]);
      files_used++;
    } else {
      printf("Error!\tExtra parameter %s ignored.\n", argv[i]);
      printf("%s and %s were assumed to be input files\n", 
	     filename1, filename2);
    }
    i++;
  }
  if (files_used < 2) {
    printf("Error\tYou must supply both plot and ct file names.\n");
    insufficient_arguments();
  }
  check_output_file(user_output_flag, g_output_filename);
  open_plot_file(filename1, filename2);
  check_output_file(user_output_flag, g_output_filename);
  initialize_len();
  open_ct_file(g_ct_filename);
}

/* Menu routines
 * Most are defined throught GLUT and called from the event queue */

void keyboard(unsigned char key, int x, int y) {
  int   string_len1;	/* accepts keys from text window */
  int   string_len2;
  x = y;
  y = x;			/* get rid of warning message */
  if (key == 13) {
    data_entered = TRUE;	/* on <RETURN> mark end of text */
    glutSetWindow(win_text);
    glutPostRedisplay();
    return;
  }
  string_len1 = (int) strlen(text_window_input);
  string_len2 = (int) strlen(text_window_message);
  if (string_len1 > 0)
    if (key == 8) {	/* process backspace or delete */
      text_window_input[string_len1 - 1] = '\0';
      text_window_message[string_len2 - 1] = '\0';
      glutSetWindow(win_text);
      glutPostRedisplay();
      return;
    }
  text_window_input[string_len1] = key;	/* add key to end of input */
  text_window_input[string_len1 + 1] = '\0';
  text_window_message[string_len2] = key;	/* add key to end of message */
  text_window_message[string_len2 + 1] = '\0';
  glutSetWindow(win_text);	/* show the text */
  glutPostRedisplay();
}

void fixcolor_plot(int color) {
  /* for the screen */
  if ((color >= 0) && (color < TOTAL_COLORS))
    glColor3f(g_color[color][0], g_color[color][1],
	      g_color[color][2]);
}

void select_font(int newfont) {
  g_font_main = fonts[newfont];
  g_font_zoom = fonts[newfont];
  g_font_text = fonts[newfont];
  glutSetWindow(win_text);
  glutPostRedisplay();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutPostRedisplay();
}

void select_font_main(int newfont) {
  g_font_main = fonts[newfont];
  glutPostRedisplay();
}

void select_font_zoom(int newfont) {
  g_font_zoom = fonts[newfont];
  glutPostRedisplay();
}

void select_font_text(int newfont) {
  g_font_text = fonts[newfont];
  glutPostRedisplay();
}

void fix_cursor(void) {
  /* set cursor back to crosshairs */
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutPostRedisplay();
}

void set_cursor_help(void) {
  glutSetWindow(win_main);
  glutSetCursor(GLUT_CURSOR_HELP);
  glutPostRedisplay();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutSetCursor(GLUT_CURSOR_HELP);
    glutPostRedisplay();
  }
}

/* ___________________postscript functions ______________________________ */

void finish_making_post(void) {
  int   width;
  int   height;
  char  filename[120];
  int   left, right, top, bottom;

  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutSetCursor(GLUT_CURSOR_WAIT);
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutSetCursor(GLUT_CURSOR_WAIT);
  glutPostRedisplay();
  if (g_post_file == MAIN_POST) {
    width = g_len;
    height = g_len;
    left = 1;
    top = 1;
    bottom = g_len;
    right = g_len;
  } else {
    width = display_w;
    height = display_h;
    left = display_l;
    right = display_r;
    bottom = display_b;
    top = display_t;
  }
  if (strlen(text_window_input) > 0) {
    strcpy(filename, text_window_input);
    chop_suffix(filename, ".ps");
    text_window_input[0] = '\0';
    text_window_message[0] = '\0';
  } else
    strcpy(filename, g_post_filename);
  if (g_opt_mode)
    strcat(filename, ".ovr-opt.ps");
  else
    strcat(filename, ".ovr-ct.ps");
  printf("The name for the postscript file is %s\n", filename);
  general_post(g_opt_mode, filename, g_post_file, width, height,
	       g_plot_filename, g_ct_filename, g_optimal_energy,
	       g_energy_cutoff, g_zoom_labels_count, g_prob,
	       g_chain_len, g_number_of_colors, g_color_increment,
	       g__prob, g_zoom_labels_string, g_zoom_labels_row, 
	       g_zoom_labels_col, left, right, top, bottom, g_diag_start, 
	       g_diag_count, g_diag, g_grid_flag, g_plot_len, g_ct_len,
	       g_dot_size, g_ct_diag_start, g_ct_diag_count, g_ct_diag,
	       g_sequence_name, g_sequence_name_set, g_close_optimal, 
	       g_worst_energy, g_color, g_opt_distance, g_ct_basepair);
  glutSetWindow(win_text);
  glutPostRedisplay();
  fix_cursor();
}

void create_post1(int which_window) {
  if ((which_window == 1) || (g_zoom_window_displayed == FALSE))
    g_post_file = MAIN_POST;	/* create post for main */
  else
    g_post_file = ZOOM_POST;	/* create post for zoom */
  strcpy(text_window_message, post_message);
  strcat(text_window_message, ", default ");
  strcat(text_window_message, g_post_filename);
  if (g_opt_mode)
    strcat(text_window_message, ".ovr-opt.ps");
  else
    strcat(text_window_message, ".ovr-ct.ps");
  strcat(text_window_message, ")? ");
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_POST;
  glutSetWindow(win_text);
  glutPostRedisplay();
  set_cursor_help();
}

/* Create an image file */
/* Image creation menu, called from pop up window */
void img_menu(int choice) {
  char *filename;
  if (choice == 1)
    g_resolution = 72;
  else if (choice == 2)
    g_resolution = 110;
  else if (choice == 3)
    g_resolution = 200;
  else
    g_resolution = 300;
  if (g_opt_mode)
    filename = g_img_filename_opt;
  else
    filename = g_img_filename;
  g_message_set = FALSE;
  text_window_message[0] = '\0';
  glutSetWindow(win_text);
  glutPostRedisplay();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutSetCursor(GLUT_CURSOR_WAIT);
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutSetCursor(GLUT_CURSOR_WAIT);
  glutPostRedisplay();
  text_window_input[0] = '\0';
#if defined(HAVE_LIBGD) && defined(HAVE_GD_H)
  finish_making_img(g_opt_mode, FALSE,filename, g_plot_filename, g_ct_filename,
		    g_optimal_energy, g_energy_cutoff, g_zoom_labels_count,
		    g_prob, g_chain_len, g_number_of_colors, 
		    g_color_increment, g__prob, g_zoom_labels_row, 
		    g_zoom_labels_col, display_l, display_r, display_t, 
		    display_b, g_diag_start, g_diag_count, g_diag, g_grid_flag,
		    g_plot_len, g_ct_len, g_dot_size, g_ct_diag_start, 
		    g_ct_diag_count, g_ct_diag, g_sequence_name, 
		    g_sequence_name_set, g_resolution, g_close_optimal, 
		    g_worst_energy, g_color, g_opt_distance, g_ct_basepair);
#endif
  fix_cursor();
}

void set_distance_cutoff(void) {
  strcpy(text_window_message, close_distance_message);
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_CLOSE;
  glutSetWindow(win_text);
  glutPostRedisplay();
  set_cursor_help();
}

void set_energy_cutoff1(void) { /* for probabilities */
  g_zoom_labels_count = 0; /* reset labeling of points in zoom window */
  strcpy(text_window_message, increment_message_prob);
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_ENERGY;
  glutSetWindow(win_text);
  glutPostRedisplay();
  set_cursor_help();
}

void set_energy_cutoff2(void) {
  /* for energies */
  g_zoom_labels_count = 0;	/* reset labeling of points in zoom
				 * window */
  strcpy(text_window_message, increment_message_energy);
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_ENERGY;
  glutSetWindow(win_text);
  glutPostRedisplay();
  set_cursor_help();
}

void set_close_range(void) {
  g_zoom_labels_count = 0;
  if (g_opt_mode)
    strcpy(text_window_message, close_distance_message);
  else
    strcpy(text_window_message, close_energy_message);
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_CLOSE;
  glutSetWindow(win_text);
  glutPostRedisplay();
  set_cursor_help();
}

void finish_setting_prob_cutoff(void) {
  float temp_filter;
  text_window_message[0] = '\0';
  if (text_window_input[0] == '/') {
    text_window_input[0] = '\0';
    text_window_message[0] = '\0';
    glutSetWindow(win_text);
    glutPostRedisplay();
    return;
  }
  sscanf(text_window_input, "%f", &temp_filter);
  g_energy_cutoff = (int) (temp_filter * -1000);
  text_window_input[0] = '\0';
  if (g_energy_cutoff > g_worst_energy)
    g_energy_cutoff = g_worst_energy;
  if (g_energy_cutoff < g_optimal_energy)
    g_energy_cutoff = g_optimal_energy;
  fix_colors();
  glutSetWindow(win_text);
  glutPostRedisplay();
  fix_cursor();
}

void finish_setting_close_distance(void) {
  float temp_filter;
  text_window_message[0] = '\0';
  if (text_window_input[0] == '/') {
    text_window_input[0] = '\0';
    text_window_message[0] = '\0';
    glutSetWindow(win_text);
    glutPostRedisplay();
    fix_cursor();
    return;
  }
  sscanf(text_window_input, "%f", &temp_filter);
  g_opt_distance = (int) (temp_filter + .05);
  if (g_opt_distance >= 9)
    g_opt_distance = 9;
  if (g_opt_distance < 0)
    g_opt_distance = 0;
  text_window_input[0] = '\0';
    /* enter extra programming to adjust distance */
  glutSetWindow(win_text);
  glutPostRedisplay();
  fix_cursor();
}

void finish_setting_energy_cutoff(void) {
  float temp_filter;
  text_window_message[0] = '\0';
  if (text_window_input[0] == '/') {
    text_window_input[0] = '\0';
    text_window_message[0] = '\0';
    glutSetWindow(win_text);
    glutPostRedisplay();
    fix_cursor();
    return;
  }
  sscanf(text_window_input, "%f", &temp_filter);
  g_energy_cutoff = (int) (temp_filter * 10);
  if (g_energy_cutoff >= 0)
    g_energy_cutoff = g_optimal_energy + g_energy_cutoff;
  text_window_input[0] = '\0';
  if (g_energy_cutoff > g_worst_energy)
    g_energy_cutoff = g_worst_energy;
  if (g_energy_cutoff < g_optimal_energy)
    g_energy_cutoff = g_optimal_energy;
  g_color_increment = ((float) (g_energy_cutoff - g_optimal_energy)) / 
    ((float) (g_number_of_colors - 1));
  fix_colors();
  glutSetWindow(win_text);
  glutPostRedisplay();
  fix_cursor();
}

void finish_setting_close(void) {
  float temp_filter;
  int   energy_value;
  text_window_message[0] = '\0';
  if (text_window_input[0] == '/') {
    text_window_input[0] = '\0';
    text_window_message[0] = '\0';
    fix_cursor();
    glutSetWindow(win_text);
    glutPostRedisplay();
    return;
  }
  sscanf(text_window_input, "%f", &temp_filter);
  energy_value = temp_filter * 10;
  if (g_close_optimal > 0)
    energy_value = energy_value + g_optimal_energy;
  text_window_input[0] = '\0';
  if (energy_value > g_worst_energy)
    energy_value = g_worst_energy;
  if (energy_value < g_optimal_energy)
    energy_value = g_optimal_energy;

  if ((energy_value==g_optimal_energy) || (g_optimal_energy==g_worst_energy))
    g_close_optimal = 0.0;
  else
    g_close_optimal = ((float) (g_optimal_energy - energy_value)) /
      ((float) (g_optimal_energy - g_worst_energy));
  glutSetWindow(win_text);
  glutPostRedisplay();
  fix_cursor();
}

/* This function is called from the menu and sets the filter */
void set_chain_len(int chain_len_choice) {
  g_zoom_labels_count = 0;	/* reset labeling of points in zoom */
  g_chain_len = chain_len_choice;
  glutSetWindow(win_main);	/* display main */
  glutPostRedisplay();
  if (g_zoom_window_displayed==TRUE) {	/* display zoom */
    zoomSetWindow();
    glutPostRedisplay();
  }
  glutSetWindow(win_text); /* display text window with filter message */
  glutPostRedisplay();
}

/* Plotting functions _____________________ */

void sniff_for_opengl_errors(char *message) {
  GLenum          errCode;
  const GLubyte  *errString;
  const char     *my_string;
  if ((errCode = glGetError()) != GL_NO_ERROR) {
    errString = gluErrorString(errCode);
    my_string = (char *) errString;
    fprintf(stderr, "OpenGL Error: %s\n ", my_string);
    fprintf(stderr, "Error from %s\n", message);
  }
}

void plot_ij(int i_row, int j_col) {
  /* converterts i,j base pair to */
  float real_x, real_y;
  real_x = 23.0 + (float) ((j_col + .5) * 700. / (g_len + 1));
  real_y = 749.0 - (float) ((i_row - .5) * 700. / (g_len + 1));
  glVertex2f(real_x, real_y);
}

/* for ct dots */
void opt_plot_ij_zoom(int i_row, int j_col, int energy, int square_flag, 
		      float size_multiplier) {
  float size;
  float real_x, real_y, x_offset, y_offset;
  real_x = 25. + (float) (j_col - display_l + 1) * 700. / (display_w + 1);
  real_y = 50. + (float) (display_b - i_row + 1) * 700. / (display_h + 1);
  if (!square_flag) {
    glVertex2f(real_x, real_y);
    return;
  } else {
    if (g_prob == TRUE) {
      size = sqrt((-1.0 * (float) energy) / 1000.);
    } else
      size = 1.0;
    size = size * size_multiplier;
    x_offset = (25. + (float) (j_col + 1 - display_l + 1) * 700. / 
		(display_w + 1) - real_x) / 2;
    y_offset = (50. + (float) (display_b - (i_row - 1) + 1) * 700. / 
		(display_h + 1) - real_y) / 2;
    if (g_prob == TRUE) {
      x_offset = x_offset * size;
      y_offset = y_offset * size;
      if (x_offset < 1)
	x_offset = 1;
      if (y_offset < 1)
	y_offset = 1;
      glBegin(GL_POLYGON);	/* square */
      glVertex2f(real_x - x_offset, real_y - y_offset);
      glVertex2f(real_x - x_offset, real_y + y_offset);
      glVertex2f(real_x + x_offset, real_y + y_offset);
      glVertex2f(real_x + x_offset, real_y - y_offset);
      glEnd();/* this statement produces errors ? */
    } else {
      x_offset = x_offset * size;
      y_offset = y_offset * size;
      glBegin(GL_POLYGON);	/* square */
      /* make square */
      glVertex2f(real_x - x_offset, real_y - y_offset);
      glVertex2f(real_x - x_offset, real_y + y_offset);
      glVertex2f(real_x + x_offset, real_y + y_offset);
      glVertex2f(real_x + x_offset, real_y - y_offset);
      glEnd();
    }
  }
  sniff_for_opengl_errors("At end of opt plot ij zoom\n");
}

void plot_ij_zoom(int i_row, int j_col, int energy, int ct_flag, 
		  float size_multiplier) {
  float size;
  float real_x, real_y, x_offset, y_offset;
  real_x = 25. + (float) (j_col - display_l + 1) * 700. / (display_w + 1);
  real_y = 50. + (float) (display_b - i_row + 1) * 700. / (display_h + 1);
  if ((display_w > 200) && (display_h > 200)) {
    glVertex2f(real_x, real_y);
    return;
  } else {
    if (g_prob == TRUE) {
      size = sqrt((-1.0 * (float) energy) / 1000.);
    } else
      size = 1.0;
    size = size * size_multiplier;
    x_offset = (25. + (float) (j_col + 1 - display_l + 1) * 700. / 
		(display_w + 1) - real_x) / 2;
    y_offset = (50. + (float) (display_b - (i_row - 1) + 1) * 700. / 
		(display_h + 1) - real_y) / 2;
    if (g_prob == TRUE) {
      x_offset = x_offset * size;
      y_offset = y_offset * size;
      if (x_offset < 1)
	x_offset = 1;
      if (y_offset < 1)
	y_offset = 1;
      glBegin(GL_POLYGON);	/* square */
      glVertex2f(real_x - x_offset, real_y - y_offset);
      glVertex2f(real_x - x_offset, real_y + y_offset);
      glVertex2f(real_x + x_offset, real_y + y_offset);
      glVertex2f(real_x + x_offset, real_y - y_offset);
      glEnd();/* this statement produces errors ? */
    } else {
      x_offset = x_offset * size;
      y_offset = y_offset * size;
      glBegin(GL_POLYGON);	/* octogon */
      if (ct_flag) {	/* make square */
	glVertex2f(real_x - x_offset, real_y - y_offset);
	glVertex2f(real_x - x_offset, real_y + y_offset);
	glVertex2f(real_x + x_offset, real_y + y_offset);
	glVertex2f(real_x + x_offset, real_y - y_offset);
      } else { /* make a hexagon of sorts */
	glVertex2f(real_x, real_y - y_offset);
	glVertex2f(real_x - x_offset, real_y - y_offset);
	glVertex2f(real_x - x_offset, real_y);
	glVertex2f(real_x, real_y + y_offset);
	glVertex2f(real_x + x_offset, real_y + y_offset);
	glVertex2f(real_x + x_offset, real_y);
      }
      glEnd();
    }
  }
  sniff_for_opengl_errors("at end of plot ij zoom\n");
}

int plot_i_zoom(int i_row) {	/* convert i_row to y coordinate */
  return (int) (50. + (float) (display_b - i_row + 1) * 700. / (display_h + 1));
}

int plot_j_zoom(int j_col) { /* convert col to x coordinate */
  return (int) (25.2 + (float) (j_col - display_l + 1)*700. / (display_w + 1));
}

int plot_i(int i_row) {	/* converts row to y coordinate */
  return (int) (749.0 - (float) ((i_row - .5) * 700. / (g_len + 1)) + .5);
}

int plot_j(int j_col) {	/* converts col to x coordinate */
  return (int) (23.0 + (float) ((j_col + .5) * 700. / (g_len + 1)) + .5);
}

int find_color(int energy) {
  int   result;
  if (g_prob == TRUE)
    return (find_color_prob(energy));
  if (energy == g_optimal_energy)
    return (1);
  result = (int)
    (((float) (energy - g_optimal_energy)) / g_color_increment +
     1.9999);
  return (result);
  /* 1.99 should be right, 1.98 is too big, 1.90 is too small */
}

void plot_points_zoom(float point_size) {
  int   row, col, new_row, count;
  int   diag, position, old_color, location, new_color;
  int   start_diag, end_diag;
  int   color_optimal;	/* for the optimal dots of plot file */
  float size_multiplier;
  float big_dot_size;
  int   type, current_type;
  int   energy;
  int   plot_display_b, plot_display_r;
  int   ct_display_b, ct_display_r;
  int   ct_type;
  g_zoom_ct_red = 0;
  g_zoom_ct_green = 0;
  g_zoom_ct_yellow = 0;
  old_color = COLOR_PLOT_OPTIMAL;
  color_optimal = COLOR_PLOT_OPTIMAL;
  fixcolor_plot(color_optimal);
  points_plotted_zoom = 0;
  /* keep these within bounds of plot file */
  if (display_b > g_plot_len)
    plot_display_b = g_plot_len;
  else
    plot_display_b = display_b;
  if (display_r > g_plot_len)
    plot_display_r = g_plot_len;
  else
    plot_display_r = display_r;
  g_zoom_optimal_dots = 0;
  start_diag = display_t + display_l - 1;
  end_diag = plot_display_b + plot_display_r - 1;
  /* plot black,gray in upper triangle */
  glPointSize(point_size * g_dot_size[0]);
  sniff_for_opengl_errors("Just before plot black,gray of  plot points zoom\n");
  if ((display_w > 200) && (display_h > 200))
    glBegin(GL_POINTS);
  for (diag = start_diag; diag <= end_diag; diag++) {
    position = g_diag_start[diag];
    for (count = 0; ((count < g_diag_count[diag])
		     && (g_diag[position + count].row <= display_b));
	 count++) {
      location = position + count;
      if (((g_diag[location].energy <= g_energy_cutoff) &&
	   (g_diag[location].length >= g_chain_len)) ||
	  (g_diag[location].energy == g_optimal_energy)) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	if (col >= display_l) {
	  new_color = g_diag[location].color;
	  if (new_color != old_color) {
	    fixcolor_plot(new_color);
	    old_color = new_color;
	  }
	  for (row = new_row; row < new_row + g_diag[location].length;
	       row++) {
	    if ((row >= display_t) && (row <= display_b)
		&& (col >= display_l) && (col <= display_r)) {
	      plot_ij_zoom(row, col,
			   g_diag[location].energy, TRUE, g_dot_size[0]);
	      if (g_diag[location].energy == g_optimal_energy)
		g_zoom_optimal_dots++;
	      points_plotted_zoom++;
	    }
	    col--;
	  }
	}
      }
    }
  }
  if ((display_w > 200) && (display_h > 200))
    glEnd();
  /* plot black in lower triangle */
  glPointSize(point_size * g_dot_size[0]);
  if ((display_w > 200) && (display_h > 200))
    glBegin(GL_POINTS);
  fixcolor_plot(COLOR_PLOT_OPTIMAL);
  old_color = 1;
  for (diag = start_diag; diag <= end_diag; diag++) {
    position = g_diag_start[diag];
    for (count = 0; ((count < g_diag_count[diag])
		     && (g_diag[position + count].row <= display_r));
	 count++) {
      location = position + count;
      if (g_diag[location].energy == g_optimal_energy) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	if (col >= display_t) {
	  for (row = new_row; row < new_row + g_diag[location].length;row++) {
	    if ((col >= display_t) && (col <= display_b)
		&& (row >= display_l) && (row <= display_r)) {
	      plot_ij_zoom(col,row,g_diag[location].energy,TRUE,g_dot_size[0]);
	    }
	    col--;
	  }
	}
      }
    }
  }
  if ((display_w > 200) && (display_h > 200))
    glEnd();
  /* keep these within bounds of plot file */
  if (display_b > g_ct_len)
    ct_display_b = g_ct_len;
  else
    ct_display_b = display_b;
  if (display_r > g_ct_len)
    ct_display_r = g_ct_len;
  else
    ct_display_r = display_r;
  end_diag = ct_display_b + ct_display_r - 1;
  /* for ct dots, red, yellow, green */
  sniff_for_opengl_errors("Just before plot red, yellow, green of plot points zoom");
  for (current_type = 2; current_type >= 0; current_type--) {
    size_multiplier = g_dot_size[current_type + 1];
    fixcolor_plot(current_type + COLOR_OPTIMAL);
    big_dot_size = size_multiplier * point_size;
    if (big_dot_size < 3.1)
      big_dot_size = 3.1;
    glPointSize(big_dot_size);
    if ((display_w > 200) && (display_h > 200))
      glBegin(GL_POINTS);
    for (diag = start_diag; diag <= end_diag; diag++) {
      position = g_ct_diag_start[diag];
      for (count = 0; ((count < g_ct_diag_count[diag])
		       && (g_ct_diag[position + count].row <= display_b));
	   count++) {
	location = position + count;
	new_row = g_ct_diag[location].row;
	col = g_ct_diag[location].column;
	energy = g_ct_diag[location].energy;
	type = set_ct_color(energy, g_optimal_energy,
			    g_worst_energy, g_close_optimal);
	if (type == current_type) {
	  if (col >= display_l) {
	    for (row = new_row; row < new_row + g_ct_diag[location].length;
		 row++) {
	      if ((row >= display_t) && (row <= display_b)
		  && (col >= display_l) && (col <= display_r)) {
		plot_ij_zoom(row, col, -5, TRUE, size_multiplier);
		if (type == 0)
		  g_zoom_ct_green++;
		else {
		  if (type == 1)
		    g_zoom_ct_yellow++;
		  else
		    g_zoom_ct_red++;
		}
	      }
	      col--;
	    }
	  }
	}
      }
    }
    if ((display_w > 200) && (display_h > 200))
      glEnd();
  }
  /*
   * for ct dots, plot gray on top of red, yellow, green when big
   * enough
   */
  sniff_for_opengl_errors("Just before plot gray on top of  plot points zoom");
  g_red_gray_overlap = 0;
  g_yellow_gray_overlap = 0;
  old_color = COLOR_OPTIMAL;
  glPointSize(g_dot_size[0] * point_size);
  if ((display_w > 200) && (display_h > 200))
    glBegin(GL_POINTS);
  for (diag = start_diag; diag <= end_diag; diag++) {
    position = g_ct_diag_start[diag];
    for (count = 0; ((count < g_ct_diag_count[diag])
		     && (g_ct_diag[position + count].row <= display_b));
	 count++) {
      location = position + count;
      new_row = g_ct_diag[location].row;
      col = g_ct_diag[location].column;
      new_color = g_ct_diag[location].color;
      if (new_color < COLOR_OPTIMAL) {
	if (new_color != old_color) {
	  fixcolor_plot(new_color);
	  old_color = new_color;
	}
	if (col >= display_l) {
	  energy = g_ct_diag[location].energy;
	  if (energy <= g_energy_cutoff) {
	    for (row=new_row;row<new_row+g_ct_diag[location].length;row++) {
	      if ((row >= display_t) && (row <= display_b)
		  && (col >= display_l) && (col <= display_r)) {
		if ((energy == g_optimal_energy) ||
		    (g_ct_diag[location].plotlen >= g_chain_len)) {
		  ct_type = 
		    set_ct_color(energy, g_optimal_energy,
				 g_worst_energy, g_close_optimal);
		  if (point_size * g_dot_size[1] > 3.1)
		    plot_ij_zoom(row, col, -5,
				 FALSE, g_dot_size[0]);
		  /* plot black gray on top of green, yellow, red */
		  if (ct_type == 2)
		    g_red_gray_overlap++;
		  else if (ct_type == 1)
		    g_yellow_gray_overlap++;
		}
	      }
	      col--;
	    }
	  }
	}
      }
    }
  }
  if ((display_w > 200) && (display_h > 200))
    glEnd();
  sniff_for_opengl_errors("At end of plot points zoom");
}

void opt_plot_points_zoom(float point_size) {
  int   row, col, new_row, count;
  int   diag, position, old_color, location, new_color;
  int   start_diag, end_diag;
  int   color_optimal;	/* for the optimal dots of plot file */
  float size_multiplier;
  float big_dot_size;
  int   energy;
  int   plot_display_b, plot_display_r;
  int   ct_display_b, ct_display_r;
  /* int ct_type; */
  g_zoom_ct_missed = 0;
  g_zoom_ct_red = 0;
  g_zoom_ct_green = 0;
  g_zoom_ct_yellow = 0;
  old_color = COLOR_PLOT_OPTIMAL;
  color_optimal = COLOR_NOT_OPTIMAL;	/* were missing ct dots before */
  fixcolor_plot(color_optimal);
  points_plotted_zoom = 0;
  /* keep these within bounds of plot file */
  if (display_b > g_plot_len)
    plot_display_b = g_plot_len;
  else
    plot_display_b = display_b;
  if (display_r > g_plot_len)
    plot_display_r = g_plot_len;
  else
    plot_display_r = display_r;
  g_zoom_optimal_dots = 0;
  start_diag = display_t + display_l - 1;
  end_diag = plot_display_b + plot_display_r - 1;
  /* plot gray in upper triangle */
  glPointSize(point_size * g_dot_size[0]);
  sniff_for_opengl_errors("Just before plot gray of  opt plot points zoom");
  if ((display_w > 200) && (display_h > 200))
    glBegin(GL_POINTS);
  for (diag = start_diag; diag <= end_diag; diag++) {
    position = g_diag_start[diag];
    for (count = 0; ((count < g_diag_count[diag])
		     && (g_diag[position + count].row <= display_b));
	 count++) {
      location = position + count;
      if (g_diag[location].energy != g_optimal_energy) {
	if ((g_diag[location].energy <= g_energy_cutoff) &&
	    (g_diag[location].length >= g_chain_len)) {
	  new_row = g_diag[location].row;
	  col = g_diag[location].column;
	  if (col >= display_l) {
	    new_color = g_diag[location].color;
	    if (new_color != old_color) {
	      fixcolor_plot(new_color);
	      old_color = new_color;
	    }
	    for (row=new_row;row<new_row+g_diag[location].length;row++) {
	      if ((row >= display_t) && (row <= display_b)
		  && (col >= display_l) && (col <= display_r)) {
		plot_ij_zoom(row, col,
			     g_diag[location].energy, TRUE, g_dot_size[0]);
		points_plotted_zoom++;
	      }
	      col--;
	    }
	  }
	}
      }
    }
  }
  if ((display_w > 200) && (display_h > 200))
    glEnd();
  /* plot black in lower triangle */
  glPointSize(point_size * g_dot_size[0]);
  if ((display_w > 200) && (display_h > 200))
    glBegin(GL_POINTS);
  fixcolor_plot(COLOR_PLOT_OPTIMAL);
  old_color = 1;
  for (diag = start_diag; diag <= end_diag; diag++) {
    position = g_diag_start[diag];
    for (count = 0; ((count < g_diag_count[diag])
		     && (g_diag[position + count].row <= display_r));
	 count++) {
      location = position + count;
      if (g_diag[location].energy == g_optimal_energy) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	if (col >= display_t) {
	  for (row=new_row; row<new_row+g_diag[location].length; row++) {
	    if ((col >= display_t) && (col <= display_b)
		&& (row >= display_l) && (row <= display_r)) {
	      plot_ij_zoom(col,row,g_diag[location].energy,TRUE,g_dot_size[0]);
	    }
	    col--;
	  }
	}
      }
    }
  }
  if ((display_w > 200) && (display_h > 200))
    glEnd();
  /* keep these within bounds of plot file */
  if (display_b > g_ct_len)
    ct_display_b = g_ct_len;
  else
    ct_display_b = display_b;
  if (display_r > g_ct_len)
    ct_display_r = g_ct_len;
  else
    ct_display_r = display_r;
  end_diag = ct_display_b + ct_display_r - 1;
  /* for ct dots, red, yellow, green */
  sniff_for_opengl_errors("Just before plot red, yellow,green of  plot points zoom");
  size_multiplier = g_dot_size[0];
  fixcolor_plot(COLOR_CT_MISSED_OPTIMAL);
  glEnable(GL_POINT_SMOOTH);
  /* make small circles instead of squares */
  big_dot_size = point_size * .5;
  if (big_dot_size < 1.0)
    big_dot_size = 1.0;
  glPointSize(big_dot_size);
  glBegin(GL_POINTS);
  for (diag = start_diag; diag <= end_diag; diag++) {
    position = g_ct_diag_start[diag];
    for (count = 0; ((count < g_ct_diag_count[diag])
		     && (g_ct_diag[position + count].row <= display_b));
	 count++) {
      location = position + count;
      new_row = g_ct_diag[location].row;
      col = g_ct_diag[location].column;
      energy = g_ct_diag[location].energy;
      if (energy != g_optimal_energy) {
	if (col >= display_l) {
	  for (row = new_row; row < new_row + g_ct_diag[location].length;
	       row++) {
	    if ((row >= display_t) && (row <= display_b)
		&& (col >= display_l) && (col <= display_r)) {	
	      opt_plot_ij_zoom(row, col, -5, FALSE, size_multiplier);
	      g_zoom_ct_missed++;
	    }
	    col--;
	  }
	}
      }
    }
  }
  glEnd();
  glDisable(GL_POINT_SMOOTH);
  /* plot optimal in upper triangle */
  glPointSize(point_size * g_dot_size[1]);
  size_multiplier = g_dot_size[3];
  sniff_for_opengl_errors("Just before plot optimal of plot points zoom");
  if ((display_w > 200) && (display_h > 200))
    glBegin(GL_POINTS);
  fixcolor_plot(COLOR_NOT_OPTIMAL);
  old_color = COLOR_NOT_OPTIMAL;
  end_diag = plot_display_b + plot_display_r - 1;
  for (diag = start_diag; diag <= end_diag; diag++) {
    position = g_diag_start[diag];
    for (count = 0; ((count < g_diag_count[diag])
		     && (g_diag[position + count].row <= display_b));
	 count++) {
      location = position + count;
      if (g_diag[location].energy == g_optimal_energy) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	/* for exact green, use yellow if close, others are red */
	if (col >= display_l) {
	  for (row=new_row; row<new_row+g_diag[location].length; row++) {
	    if ((row >= display_t) && (row <= display_b)
		&& (col >= display_l) && (col <= display_r)) {
	      if (row <= g_ct_len) {
		if (g_ct_basepair[row] == col) {
		  new_color = COLOR_OPTIMAL;
		  size_multiplier = g_dot_size[1];
		  g_zoom_ct_green++;
		} else {
		  if (distance_plot_ct(g_ct_len, row, col, g_opt_distance)) {
		    new_color = COLOR_NEAR_OPTIMAL;
		    size_multiplier = g_dot_size[2];
		    g_zoom_ct_yellow++;
		  } else {
		    new_color = COLOR_NOT_OPTIMAL;
		    size_multiplier = g_dot_size[3];
		    g_zoom_ct_red++;
		  }
		}
	      } else {
		new_color = COLOR_NOT_OPTIMAL;
		size_multiplier = g_dot_size[3];
		g_zoom_ct_red++;
	      }
	      if (new_color != old_color) {
		fixcolor_plot(new_color);
		old_color = new_color;
	      }
	      plot_ij_zoom(row, col,
			   g_diag[location].energy, TRUE, size_multiplier);
	      g_zoom_optimal_dots++;
	      points_plotted_zoom++;
	    }
	    col--;
	  }
	}
      }
    }
  }
  if ((display_w > 200) && (display_h > 200))
    glEnd();
  sniff_for_opengl_errors("At end of opt plot points zoom");
}

void plot_points(float point_size) {
  int   row, col, new_row;
  int   old_color, location, new_color;
  float dot_size;
  int   energy;
  old_color = 0;
  g_ct_red = 0;
  g_ct_green = 0;
  g_ct_yellow = 0;
  fixcolor_plot(old_color);
  glBegin(GL_POINTS);
  points_plotted = 0;
  g_optimal_dots = 0;
  /* draw plot file dots */
  for (location = 0; location < g_helices_in_plot_file; location++) {
    if (((g_diag[location].energy <= g_energy_cutoff) &&
	 (g_diag[location].length >= g_chain_len)) ||
	(g_diag[location].energy == g_optimal_energy)) {
      new_color = g_diag[location].color;
      if (new_color != old_color) {
	fixcolor_plot(new_color);
	old_color = new_color;
      }
      new_row = g_diag[location].row;
      col = g_diag[location].column;
      for (row = new_row; row < new_row+g_diag[location].length; row++) {
	if (new_color == 1) {
	  plot_ij(col, row);
	  g_optimal_dots++;
	} else
	  plot_ij(row, col);
	col--;
	points_plotted++;
      }
    }
  }
  glEnd();
  /* for ct file */
  dot_size = point_size * g_dot_size[1];
  if (dot_size < 3.1)
    dot_size = 3.1;
  glPointSize(dot_size);
  glBegin(GL_POINTS);
  /* draw red,yellow,green dots */
  for (old_color = 7; old_color >= 5; old_color--) {
    fixcolor_plot(old_color);
    for (location = 0; location < g_helices_in_ct_file; location++) {
      energy = g_ct_diag[location].energy;
      new_color = set_ct_color(energy, g_optimal_energy,
			       g_worst_energy, g_close_optimal) + 5;
      if (new_color == old_color) {
	new_row = g_ct_diag[location].row;
	col = g_ct_diag[location].column;
	for (row = new_row; row < new_row + g_ct_diag[location].length; row++) {
	  if (new_color == 5)
	    g_ct_green++;
	  else {
	    if (new_color == 6)
	      g_ct_yellow++;
	    else
	      g_ct_red++;
	  }
	  plot_ij(row, col);
	  col--;
	}
      }
    }
  }
  glEnd();
  /* color optimal dots green, yellow or red */
  sniff_for_opengl_errors("At end of plot points");
}

void opt_plot_points(float point_size) {
  int   row, col, new_row;
  int   old_color, location, new_color;
  float dot_size;
  int   energy;
  old_color = 0;
  g_ct_red = 0;
  g_ct_green = 0;
  g_ct_yellow = 0;
  g_ct_missed = 0;
  fixcolor_plot(old_color);
  glBegin(GL_POINTS);
  points_plotted = 0;
  g_optimal_dots = 0;
  /* draw gray plot file dots in upper triangle */
  for (location = 0; location < g_helices_in_plot_file; location++) {
    if (((g_diag[location].energy <= g_energy_cutoff) &&
	(g_diag[location].length >= g_chain_len)) ||
	(g_diag[location].energy == g_optimal_energy)) {
      new_color = g_diag[location].color;
      if (new_color != old_color) {
	fixcolor_plot(new_color);
	old_color = new_color;
      }
      new_row = g_diag[location].row;
      col = g_diag[location].column;
      for (row = new_row; row < new_row + g_diag[location].length; row++) {
	if (new_color == 1) {
	  plot_ij(col, row);
	  g_optimal_dots++;
	} else {
	  plot_ij(row, col);
	  points_plotted++;
	}
	col--;
      }
    }
  }
  glEnd();
  points_plotted = points_plotted + g_optimal_dots;
  sniff_for_opengl_errors("After suboptimal, before miss ct plot points opt");
  /* for ct file */
  dot_size = point_size * g_dot_size[0] * .5;
  if (dot_size < 1.0)
    dot_size = 1.0;
  glPointSize(dot_size);
  if (g_len < 300)
    glEnable(GL_POINT_SMOOTH);
  glBegin(GL_POINTS);
  /* draw cyan dots that miss */
  fixcolor_plot(COLOR_CT_MISSED_OPTIMAL);
  for (location = 0; location < g_helices_in_ct_file; location++) {
    energy = g_ct_diag[location].energy;
    if (energy != g_optimal_energy) {
      new_row = g_ct_diag[location].row;
      col = g_ct_diag[location].column;
      for (row = new_row; row < (new_row + g_ct_diag[location].length); row++) {
	g_ct_missed++;
	plot_ij(row, col);
	col--;
      }
    }
  }
  glEnd();
  if (g_len < 300)
    glDisable(GL_POINT_SMOOTH);
  /* plot the optimal points */
  fixcolor_plot(COLOR_NOT_OPTIMAL);
  old_color = COLOR_NOT_OPTIMAL;
  dot_size = point_size * g_dot_size[1];
  if (dot_size < 1.5)
    dot_size = 1.5;
  glPointSize(dot_size);
  glBegin(GL_POINTS);
  for (location = 0; location < g_helices_in_plot_file; location++) {
    if (g_diag[location].energy == g_optimal_energy) {
      new_row = g_diag[location].row;
      col = g_diag[location].column;
      for (row = new_row; row < new_row + g_diag[location].length; row++) {
	if (row <= g_ct_len) {
	  if (g_ct_basepair[row] == col) {
	    new_color = COLOR_OPTIMAL;
	    g_ct_green++;
	  } else {	/* check close */
	    if (distance_plot_ct(g_ct_len, row, col, g_opt_distance)) {
	      new_color = COLOR_NEAR_OPTIMAL;
	      g_ct_yellow++;
	    } else {
	      new_color = COLOR_NOT_OPTIMAL;
	      g_ct_red++;
	    }
	  }
	} else {
	  new_color = COLOR_NOT_OPTIMAL;
	  g_ct_red++;
	}
	if (new_color != old_color) {
	  fixcolor_plot(new_color);
	  old_color = new_color;
	}
	plot_ij(row, col);
	col--;
      }
    }
  }
  glEnd();
  sniff_for_opengl_errors("At end of plot points opt");
}

void output_main(int x, int y, char *string) {	
  /* display string at real coordinates  x,y  */
  int   len1, i;
  glRasterPos2f(x, y);
  len1 = (int) strlen(string);
  for (i = 0; i < len1; i++) {
    glutBitmapCharacter(g_font_main, string[i]);
  }
}

void output_zoom(int x, int y, char *string) {
  /* display string at real coordinates  x,y  */
  int   len2, i;
  glRasterPos2f(x, y);
  len2 = (int) strlen(string);
  for (i = 0; i < len2; i++) {
    glutBitmapCharacter(g_font_zoom, string[i]);
  }
}

void output_text(int x, int y, char *string) {
  /* display string at real coordinates  x,y  */
  int   len3, i;
  glRasterPos2f(x, y);
  len3 = (int) strlen(string);
  for (i = 0; i < len3; i++) {
    glutBitmapCharacter(g_font_text, string[i]);
  }
}

void single_hor_tic(int j_col, int grid_flag) {
  char  string2[20];
  float x;
  x = (float) plot_j(j_col);
  glBegin(GL_LINES);
  glVertex2f(x, 751.);
  glVertex2f(x, 758.);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COLOR_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(x, 750);
    glVertex2f(x, 40);
    glEnd();
    fixcolor_plot(COLOR_TEXT);
  }
  strcpy(string2, num_string_int(j_col));
  output_main(plot_j(j_col) - 10, 762, string2);
}

void plot_hor_tics(void) {
  int   j_col, step, start;
  step = step_fun(1, g_len);
  start = start_fun(1, step);
  single_hor_tic(g_len, FALSE);
  single_hor_tic(1, FALSE);
  for (j_col = start; j_col < (g_len - 2 * step / 3); j_col = j_col + step) {
    single_hor_tic(j_col, g_grid_flag);
  }
}

void single_ver_tic(int i_row, int grid_flag) {
  char  string2[20];
  float y;
  y = (float) plot_i(i_row);
  glBegin(GL_LINES);
  glVertex2f(727., y);
  glVertex2f(736., y);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COLOR_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(20, y);
    glVertex2f(725., y);
    glEnd();
    fixcolor_plot(COLOR_TEXT);
  }
  strcpy(string2, num_string_int(i_row));
  output_main(737, plot_i(i_row) - 5, string2);
}

void plot_ver_tics(void) {
  int   i_row, step, start;
  step = step_fun(1, g_len);
  start = start_fun(1, step);
  single_ver_tic(1, FALSE);
  single_ver_tic(g_len, FALSE);
  for (i_row = start; i_row < (g_len - step / 2); i_row = i_row + step) {
    single_ver_tic(i_row, g_grid_flag);
  }
}

void single_hor_tic_zoom(int j_col, int grid_flag) {
  char  string2[10];
  float x;
  x = (float) plot_j_zoom(j_col);
  glBegin(GL_LINES);
  glVertex2f(x, 751);
  glVertex2f(x, 758);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COLOR_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(x, 749);
    glVertex2f(x, 50);
    glEnd();
    fixcolor_plot(COLOR_TEXT);
  }
  strcpy(string2, num_string_int(j_col));
  output_zoom(plot_j_zoom(j_col) - 10, 762, string2);
}

void plot_hor_tics_zoom(void) {
  int   j_col, step;
  int   start;
  step = step_fun(display_l, display_r);
  start = start_fun(display_l, step);
  single_hor_tic_zoom(display_r, FALSE);
  single_hor_tic_zoom(display_l, FALSE);
  for (j_col = start; j_col < (display_r - 2 * step / 3);
       j_col = j_col + step) {
    single_hor_tic_zoom(j_col, g_grid_flag);
  }
}

void single_ver_tic_zoom(int i_row, int grid_flag) {
  char  string2[20];
  float y;
  y = (float) plot_i_zoom(i_row);
  glBegin(GL_LINES);
  glVertex2f(727., y);
  glVertex2f(736., y);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COLOR_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(23., y);
    glVertex2f(723., y);
    glEnd();
    fixcolor_plot(COLOR_TEXT);
  }
  strcpy(string2, num_string_int(i_row));
  output_zoom(737, plot_i_zoom(i_row) - 5, string2);
}

void plot_ver_tics_zoom(void) {
  int   i_row, step, start;
  step = step_fun(display_t, display_b);
  start = start_fun(display_t, step);
  single_ver_tic_zoom(display_t, FALSE);
  single_ver_tic_zoom(display_b, FALSE);
  for (i_row = start; i_row < (display_b - step / 2); i_row = i_row + step) {
    single_ver_tic_zoom(i_row, g_grid_flag);
  }
}

void plot_tics_zoom(void) {
  plot_hor_tics_zoom();
  plot_ver_tics_zoom();
  return;
}

void plot_tics(void) {
  plot_hor_tics();
  plot_ver_tics();
  return;
}

/* converts window x coordinates to column coordinate of RNA
   top left for window is 0,0 
   x is right, y is down 
   i is down, j is right  */ 
int window_x_convert(int x) {
  float xf;
  xf = x - 23. * g_width / 800.;
  xf = xf * (g_len + 1) / 700. * 800. / g_width;
  return (int) (xf);
}

int window_y_convert(int y) {
  /* convert window y coordinates to */
  float yf;	/* to row coordinates */
  int   i_row;
  yf = (float) (y * 800. / g_height);	/* scale according to window size */
  yf = 799 - yf; /* origin is at top in window, bottom in opengl */
  i_row = (int) ((g_len + 1.) * (749. - yf)/700. + 1.0); /* inverse of plot_ij
							  * function */
  /* The 1.0 above should be .5 ? but 1.0 works much better */
  return i_row;
}

int window_x_convert_zoom(int x) {
  /* converts window x coordinates to */
  float xf;
  xf = x - 25. * g_width_z / 800.;	/* subtract offset to right
					 * depending on window */
  xf = xf * (display_w + 1) / 700.*800. / g_width_z + display_l - 1; /* size */
  return (int) (xf + .5);
}

int window_y_convert_zoom(int y) {
  /* convert window y coordinates to */
  float yf;	/* RNA i coordinates down */
  yf = y - 50. * g_height_z / 800;	/* subtract offset down
					 * depending on window */
  yf = yf * (display_h + 1) / 700*800. / g_height_z + display_t - 1; /* size */
  return (int) (yf + .5);
}

/* display functions ____________________________________________*/

void display_text(void) {
  float x_float;
  int   energy;
  int   plot_dot_color;
  int   ct_dot_color;
  int   color, offset;
  char  close_to_optimal[15];
  char  text_message[150];
  float close_float;
  char  location[80], energy_string[60], energy_long_string[60];
  glClear(GL_COLOR_BUFFER_BIT);
  fixcolor_plot(COLOR_TEXT);
  if (g_message_set == TRUE) {
    glutPopWindow();
    if (g_type_message == MESSAGE_ENERGY) {
      fixcolor_plot(COLOR_TEXT);
      strcpy(text_message, text_window_message);
      strcat(text_message, "_");
      output_text(10, 5, text_message);
      fixcolor_plot(COLOR_TEXT);
      if (data_entered == TRUE) {
	data_entered = FALSE;
	g_message_set = FALSE;
	if (g_prob != TRUE) {
	  finish_setting_energy_cutoff();
	  return;
	} else {
	  finish_setting_prob_cutoff();
	  return;
	}
      }
    }
    if (g_type_message == MESSAGE_CLOSE) {
      fixcolor_plot(COLOR_NEAR_OPTIMAL);
      output_text(12, 5, text_window_message);
      fixcolor_plot(COLOR_TEXT);
      if (data_entered == TRUE) {
	data_entered = FALSE;
	g_message_set = FALSE;
	if (g_opt_mode) {
	  finish_setting_close_distance();
	  return;
	} else {
	  finish_setting_close();
	  return;
	}
      }
    }
    if (g_type_message == MESSAGE_POST) {
      fixcolor_plot(COLOR_TEXT);
      output_text(12, 5, text_window_message);
      if (data_entered == TRUE) {
	data_entered = FALSE;
	g_message_set = FALSE;
	finish_making_post();
	return;
      }
    }
  } else if (g_i_row > 0) {
    energy = energy_read(g_i_row, g_j_col);
    strcpy(location, "(i,j) base pair =(");
    strcat(location, g_i_row_s);
    strcat(location, ",");
    strcat(location, g_j_col_s);
    strcat(location, ")");
    if (!g_opt_mode) {
      if (energy != INT_MIN) {
	fixcolor_plot(find_color(energy));
	output_text(2, 50, "**");
      }
      if (ct_read(g_i_row, g_j_col)) {
	fixcolor_plot(set_ct_color(energy, g_optimal_energy, g_worst_energy, 
				   g_close_optimal) + COLOR_OPTIMAL);
	output_text(13, 50, "**");
      }
    } else {
      if (energy != INT_MIN)
	plot_dot_color = find_color(energy);
      else
	plot_dot_color = INT_MIN;
      if (ct_read(g_i_row, g_j_col)) {
	ct_dot_color = set_ct_color(energy, g_optimal_energy,
				    g_worst_energy, g_close_optimal);
	if (ct_dot_color == 0) {
	  fixcolor_plot(COLOR_OPTIMAL);
	  output_text(2, 50, "****");
	} else {
	  fixcolor_plot(COLOR_CT_MISSED_OPTIMAL);
	  output_text(13, 50, "**");
	  if (plot_dot_color == COLOR_PLOT_OPTIMAL)
	    plot_dot_color = COLOR_NOT_OPTIMAL;
	  if (plot_dot_color != INT_MIN) {
	    fixcolor_plot(plot_dot_color);
	    output_text(2, 50, "**");
	  }
	}
      } else {
	if (plot_dot_color != INT_MIN) {
	  if (plot_dot_color == COLOR_PLOT_OPTIMAL) {
	    if (distance_plot_ct(g_ct_len, g_i_row, g_j_col, g_opt_distance))
	      plot_dot_color = COLOR_NEAR_OPTIMAL;
	    else
	      plot_dot_color = COLOR_NOT_OPTIMAL;
	  }
	  fixcolor_plot(plot_dot_color);
	  output_text(2, 50, "**");
	}
      }
    }
    fixcolor_plot(COLOR_TEXT);
    output_text(25, 50, location);
    if (g_prob == TRUE) {
      strcpy(energy_long_string, "probability = ");
    } else {
      strcpy(energy_long_string, "Energy = ");
    }
    if (energy == INT_MIN) {
      strcat(energy_long_string, "Undefined");
    } else {
      strcpy(energy_string,
	     num_string_fancy_int(energy, g_prob));
      strcat(energy_long_string, energy_string);
    }
    if (g_prob == FALSE)
      strcat(energy_long_string, "   kcal/mol");
    output_text(10, 10, energy_long_string);
  }
  if (g_prob == FALSE) {
    strcpy(location, "Optimal Energy:  ");
    strcat(location, num_string_fancy_int(g_optimal_energy, g_prob));
    strcat(location, " kcal/mol");
    if (g_opt_mode)
      fixcolor_plot(COLOR_NOT_OPTIMAL);
    else
      fixcolor_plot(COLOR_PLOT_OPTIMAL);
    output_text(10, 180, location);
    fixcolor_plot(COLOR_TEXT);
    strcpy(location, "Highest Energy:  ");
    strcat(location, num_string_fancy_int(g_worst_energy,
					  g_prob));
    strcat(location, " kcal/mol");
    output_text(10, 150, location);
  } else {
    strcpy(location, "Highest Prob.:  ");
    strcat(location, num_string_fancy_int(g_optimal_energy, g_prob));
    output_text(10, 180, location);
    strcpy(location, "Lowest Prob.:  ");
    strcat(location, num_string_fancy_int(g_worst_energy, g_prob));
    output_text(10, 150, location);
  }
  if (g_chain_len > 1) {
    strcpy(location, "Filter: ");
    strcat(location, num_string_int(g_chain_len));
    output_text(10, 90, location);
  }
  /* display energy information */
  strcpy(location, "Cutoff: ");
  strcat(location, num_string_fancy_int(g_energy_cutoff, g_prob));
  if (g_prob == FALSE) {
    strcat(location, "  or ");
    strcat(location, num_string_fancy_int(g_energy_cutoff - g_optimal_energy, 
					  g_prob));
    strcat(location, " kcal/mol");
  }
  output_text(10, 120, location);
  /* display color scheme */
  if (g_prob == FALSE) {
    x_float = (float) g_optimal_energy;
    for (color=2,offset=0;color<=g_number_of_colors;color++,offset=offset+25) {
      fixcolor_plot(color);
      strcpy(location,
	     num_string_fancy_float(x_float+(color-2)*g_color_increment,
				    g_prob));
      strcat(location, " < energy <=");
      strcat(location, 
	     num_string_fancy_float(x_float + (color - 1) * g_color_increment,
				    g_prob));
      output_text(350, 180 - offset, location);
    }
  } else {
    fixcolor_plot(COLOR_PLOT_OPTIMAL);
    strcpy(location, num_string_float(g__prob[g_number_of_colors - 4][0]));
    strcat(location, " < Prob.");
    output_text(350, 180, location);
    for (color = 2, offset = 25; color <= g_number_of_colors - 1;
	 color++, offset = offset + 24) {
      fixcolor_plot(color);
      strcpy(location,
	     num_string_float(g__prob[g_number_of_colors - 4][color - 1]));
      strcat(location, " < Prob. <=");
      strcat(location,
	     num_string_float(g__prob[g_number_of_colors - 4][color - 2]));
      output_text(350, 180 - offset, location);
    }
    fixcolor_plot(color);
    strcpy(location, "             Prob. <=");
    strcat(location,num_string_float(g__prob[g_number_of_colors-4]
				     [g_number_of_colors-2]));
    output_text(350, 180 - offset, location);
  }
  if (g_opt_mode) {
    fixcolor_plot(COLOR_OPTIMAL);
    strcpy(location, "CT Optimal Plot Overlap");
    output_text(350, 100, location);
    fixcolor_plot(COLOR_NEAR_OPTIMAL);

    if (g_prob == TRUE) {
      close_float = 0.5 ;
    } else {
      close_float = (float) (g_worst_energy - g_optimal_energy);
      close_float = close_float * g_close_optimal;
      close_float = close_float * .1;
    }
    sprintf(close_to_optimal, "%d", g_opt_distance);
    strcpy(location, "Optimal plot within ");
    strcat(location, close_to_optimal);
    strcat(location, " of CT");
    output_text(350, 75, location);
    if (g_opt_mode)
      fixcolor_plot(COLOR_CT_MISSED_OPTIMAL);
    else
      fixcolor_plot(COLOR_NOT_OPTIMAL);
    strcpy(location, "Other CT Base Pairs");
    output_text(350, 50, location);
    fixcolor_plot(COLOR_TEXT);
    strcpy(location, "For ct, ");
    strcat(location, file_data_ct[0]);
    output_text(350, 25, location);
    strcpy(location, "Optimal Plot Mode");
    output_text(10, 75, location);
  } else {
    fixcolor_plot(COLOR_OPTIMAL);
    strcpy(location, "CT Optimal Overlap");
    output_text(350, 100, location);
    fixcolor_plot(COLOR_NEAR_OPTIMAL);
    if (g_prob == TRUE) {
      close_float = 0.5 ;
    } else {
      close_float = (float) (g_worst_energy - g_optimal_energy);
      close_float = close_float * g_close_optimal;
    }
    if (g_prob == TRUE) {
      close_float /= 1000.0 ;
      sprintf(close_to_optimal, "%.3f", close_float);
    } else {
      close_float = close_float * .1;
      sprintf(close_to_optimal, "%.1f", close_float);
    }
    if (g_prob == TRUE) {
      strcpy(location, "CT prob >= ");
      strcat(location, close_to_optimal);
    } else {
      strcpy(location, "CT Within ");
      strcat(location, close_to_optimal);
      strcat(location, " kcal of optimal");
    }
    output_text(350, 75, location);
    fixcolor_plot(COLOR_NOT_OPTIMAL);
    strcpy(location, "Other CT Base Pairs");
    output_text(350, 50, location);
    fixcolor_plot(COLOR_TEXT);
    strcpy(location, "For ct, ");
    strcat(location, file_data_ct[0]);
    output_text(350, 25, location);
    strcpy(location, "CT mode");
    output_text(10, 75, location);
  }
  sniff_for_opengl_errors("at end of display text");
  glutSwapBuffers();
}

void display_main(void) {
  char  location[60];
  float point_size;	/* size of points to plot within box */
  glClear(GL_COLOR_BUFFER_BIT);
  fixcolor_plot(COLOR_TEXT);
  glLineWidth(1.0);
  plot_tics();
  glLineWidth(1.0);
  glBegin(GL_LINES);	/* draw diagonal line */
  glVertex2f(23.0, 752.0);
  glVertex2f(726.0, 50.0);
  glEnd();
  glLineWidth(2.0);
  fixcolor_plot(COLOR_TEXT);
  glBegin(GL_LINE_LOOP);	/* draw triangle */
  glVertex2f(18.0, 752.0);
  glVertex2f(18.0, 43.0);
  glVertex2f(726.0, 43.0);
  glVertex2f(726.0, 752.0);
  glEnd();
  point_size = 700. / (g_len + 1) * g_height / 800. + .5;
  if (point_size < 1.5)
    point_size = 1.5;
  glPointSize(point_size);
  sniff_for_opengl_errors(" in Display main before plotting points");
  if (g_opt_mode)
    opt_plot_points(point_size);
  else
    plot_points(point_size);
  sniff_for_opengl_errors("in display main after plotting points");
  fixcolor_plot(COLOR_TEXT);
  /* display the optimal energy */
  strcpy(location, "Base Pairs Plotted   Plot file: ");
  strcat(location, num_string_int(points_plotted));
  strcat(location, ",");
  output_main(15, 5, location);
  strcpy(location, num_string_int(g_ct_green));
  fixcolor_plot(5);
  output_main(350, 5, location);
  fixcolor_plot(1);
  strcpy(location, " of ");
  strcat(location, num_string_int(g_optimal_dots));
  strcat(location, ",");
  output_main(425, 5, location);
  if (g_close_optimal != 0.0) {
    strcpy(location, num_string_int(g_ct_yellow));
    strcat(location, ",");
    fixcolor_plot(COLOR_NEAR_OPTIMAL);
    output_main(535, 5, location);
  }
  if (g_opt_mode) {
    strcpy(location, num_string_int(g_ct_missed));
    fixcolor_plot(COLOR_CT_MISSED_OPTIMAL);
    output_main(715, 5, location);
  }
  strcpy(location, num_string_int(g_ct_red));
  fixcolor_plot(7);
  output_main(630, 5, location);
  glLineWidth(2.0);
  if ((g_region_set == TRUE) || (g_region_exists == TRUE)) {	/* draw zoom */
    fixcolor_plot(COLOR_LABEL);
    glBegin(GL_LINE_LOOP);
    glVertex2f(region_real_l, region_real_t);
    glVertex2f(region_real_r, region_real_t);
    glVertex2f(region_real_r, region_real_b);
    glVertex2f(region_real_l, region_real_b);
    glEnd();
  }
  glutSwapBuffers();
  sniff_for_opengl_errors("at end of display main");
}

void display_zoom(void) {
  float point_size;
  char  location[60];
  int   right, bottom, top, left, i;
  void *temp_font;
  glClear(GL_COLOR_BUFFER_BIT);
  glLineWidth(1.0);
  fixcolor_plot(COLOR_TEXT);
  plot_tics_zoom();
  point_size = 700. / ((display_w + display_h) / 2) * g_width_z / 
    800. * g_height_z / 800. + .60;
  if (point_size < 1.5)
    point_size = 1.5;
  glPointSize(point_size);
  if (g_opt_mode)
    opt_plot_points_zoom(point_size);
  else
    plot_points_zoom(point_size); 
  fixcolor_plot(COLOR_TEXT);
  glLineWidth(2.0);
  glBegin(GL_LINE_LOOP);
  glVertex2f(22.0, 751.0);
  glVertex2f(22.0, 48.0);
  glVertex2f(724.0, 48.0);
  glVertex2f(724.0, 751.0);
  glEnd();
  glLineWidth(1.0);
  strcpy(location, "Base Pairs: ");
  strcat(location, num_string_int(points_plotted_zoom));
  strcat(location, ", ");
  output_zoom(10, 5, location);
  strcpy(location, num_string_int(g_zoom_ct_green));
  fixcolor_plot(5);
  output_zoom(260, 5, location);
  fixcolor_plot(1);
  strcpy(location, " of ");
  strcat(location, num_string_int(g_zoom_optimal_dots));
  strcat(location, " ");
  output_zoom(330, 5, location);
  if (!g_opt_mode) {
    if (g_close_optimal != 0.0) {
      strcpy(location, num_string_int(g_zoom_ct_yellow));
      strcat(location, ", Overlap: ");
      strcat(location, num_string_int(g_yellow_gray_overlap));
      fixcolor_plot(6);
      output_zoom(450, 5, location);
    }
  } else {
    strcpy(location, num_string_int(g_zoom_ct_missed));
    fixcolor_plot(COLOR_CT_MISSED_OPTIMAL);
    output_zoom(715, 5, location);
    strcpy(location, num_string_int(g_zoom_ct_yellow));
    fixcolor_plot(COLOR_NEAR_OPTIMAL);
    output_zoom(530, 5, location);
  }
  strcpy(location, num_string_int(g_zoom_ct_red));
  if (!g_opt_mode) {
    strcat(location, ", Overlap: ");
    strcat(location, num_string_int(g_red_gray_overlap));
  }
  fixcolor_plot(7);
  output_zoom(610, 5, location);
  fixcolor_plot(COLOR_TEXT);
  /* draw diagional line to indicate no points below */
  if ((display_r >= display_t) && (display_b >= display_l)) {
    if (display_r > display_b) {
      bottom = display_b;
      right = display_b;
    } else {
      right = display_r;
      bottom = display_r;
    }
    if (display_l > display_t) {
      left = display_l;
      top = display_l;
    } else {
      top = display_t;
      left = top;
    }
    glBegin(GL_LINES);
    glVertex2f(plot_j_zoom(left - 1), plot_i_zoom(top - 1));
    glVertex2f(plot_j_zoom(right + 1), plot_i_zoom(bottom + 1));
    glEnd();
  }
  fixcolor_plot(COLOR_LABEL);
  if (g_region_set == TRUE) {	/* draw zoom box */
    glBegin(GL_LINE_LOOP);
    glVertex2f(region_real_l_zoom, region_real_t_zoom);
    glVertex2f(region_real_l_zoom, region_real_b_zoom);
    glVertex2f(region_real_r_zoom, region_real_b_zoom);
    glVertex2f(region_real_r_zoom, region_real_t_zoom);
    glEnd();
  }
  /* draw the labels */
  if (g_zoom_labels_count > 0) {
    temp_font = g_font_zoom;
    g_font_zoom = GLUT_BITMAP_TIMES_ROMAN_10;
    glEnable(GL_POINT_SMOOTH);
    for (i = 0; i < g_zoom_labels_count; i++) {
      output_zoom(g_zoom_labels_x[i], g_zoom_labels_y[i],
		  g_zoom_labels_string[i]);
      
    }
    point_size = 700. / ((display_w + display_h) / 2) * g_width_z / 
      800. * g_height_z / 800. + .50;
    point_size = point_size * .5;
    glPointSize(point_size);
    glEnable(GL_POINT_SMOOTH);
    glBegin(GL_POINTS);
    for (i = 0; i < g_zoom_labels_count; i++) {
      glVertex2f((float) g_zoom_labels_xl[i],
		 (float) g_zoom_labels_y[i]);
    }
    glEnd();
    glDisable(GL_POINT_SMOOTH);
    g_font_zoom = temp_font;
  }
  glutSwapBuffers();
  sniff_for_opengl_errors("at end of display zoom");
}

void text_reshape(int w, int h) {
  if (w > 1000) {
    w = 1000;
    glutReshapeWindow(w, h);
  }
  if (h > 300) {
    h = 300;
    glutReshapeWindow(w, h);
  }
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, 600.0, 0.0, 200.0, -1.0, 1.0);
  glMatrixMode(GL_MODELVIEW);
  if (w < 700)
    select_font_text(0);
  else
    select_font_text(1);
}

void reshape(int w, int h) {
  int   current_window, font;
  current_window = glutGetWindow();
  if (w > h) {
    h = w;
    glutReshapeWindow(w, h);
    glutPostRedisplay();
  }
  if (h > w) {
    w = h;
    glutReshapeWindow(w, h);
    glutPostRedisplay();
  }
  if (w > 990) {
    w = 990;
    h = 990;
    glutReshapeWindow(w, h);
  }
  if (current_window == win_main) {
    if (w < h)
      h = w;
    else
      w = h;
  }
  if (current_window == win_main) {
    g_width = w;
    g_height = h;
  } else {
    g_width_z = w;
    g_height_z = h;
  }
  if (w < 650)
    font = 0;
  else {
    if (w < 900)
      font = 1;
    else
      font = 2;
  }
  if (current_window == win_main)
    select_font_main(font);
  else
    select_font_zoom(font);
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, 800.0, 0.0, 800.0, -1.0, 1.0); /* 0,0 is origin for drawing */
  /* lower left is 0,0   up is y, right is x */
  glMatrixMode(GL_MODELVIEW);
}

/* Functions to set zoom rectangle */
void fix_zoom_box(void) {	/* region_real_r, region_r; */
  region_real_l = 23.0 + (float) ((region_l + .5) * 700. / (g_len + 1));
  region_real_r = 23.0 + (float) ((region_r + .5) * 700. / (g_len + 1));
  region_real_b = 749.0 - (float) ((region_b - .5) * 700. / (g_len + 1));
  region_real_t = 749.0 - (float) ((region_t - .5) * 700. / (g_len + 1));
}

void finish_fixing_zoom_box(void) {
  int   temp;	/* make left less than right */
  if (region_r < region_l) {
    temp = region_r;
    region_r = region_l;
    region_l = temp;
    fix_zoom_box();
  }
  if (region_b < region_t) {	/* make top less than bottom */
    temp = region_t;
    region_t = region_b;
    region_b = temp;
    fix_zoom_box();
  }
  if (first_zoom > 0) {
    saved_display_l = display_l;
    saved_display_r = display_r;
    saved_display_t = display_t;
    saved_display_b = display_b;
  } else {
    first_zoom = 1;
    saved_display_l = region_l;
    saved_display_r = region_r;
    saved_display_t = region_t;
    saved_display_b = region_b;
  }
  display_l = region_l;
  display_r = region_r;
  display_t = region_t;
  display_b = region_b;
  display_w = display_r - display_l + 1;
  display_h = display_b - display_t + 1;
}

void fix_zoom_box_zoom(void) {
  region_real_l = 25.0 + (float) region_l *700. / g_len;
  region_real_r = 25.0 + (float) region_r *700. / g_len;
  region_real_b = 50.0 + (float) (g_len - region_b) * 700. / g_len;
  region_real_t = 50.0 + (float) (g_len - region_t) * 700 / g_len;
  region_real_l_zoom = 25.4 + (float)(region_l-display_l+1)*700./(display_w+1);
  region_real_r_zoom = 25.4 + (float)(region_r-display_l+1)*700./(display_w+1);
  region_real_t_zoom = 50.0 + (float)(display_b-region_t+1)*700./(display_h+1);
  region_real_b_zoom = 50.0 + (float)(display_b-region_b+1)*700./(display_h+1);
  if (region_real_l_zoom < 0.) {
    region_real_l_zoom = 0.;
  }
  if (region_real_r_zoom < 0.) {
    region_real_r_zoom = 0.;
  }
  if (region_real_r_zoom > 799.) {
    region_real_r_zoom = 800.;
  }
  if (region_real_l_zoom > 799.) {
    region_real_l_zoom = 799.;
  }
  if (region_real_t_zoom < 0) {
    region_real_t_zoom = 0;
  }
  if (region_real_t_zoom > 799.) {
    region_real_t_zoom = 799.;
  }
  if (region_real_b_zoom < 0.) {
    region_real_b_zoom = 0.;
  }
  if (region_real_b_zoom > 799.) {
    region_real_b_zoom = 799.;
  }
}

void finish_fixing_zoom_box_zoom(void) {
  int   temp;	/* make left less than right */
  if (region_r < region_l) {
    temp = region_r;
    region_r = region_l;
    region_l = temp;
  }
  if (region_b < region_t) {	/* make top less than bottom */
    temp = region_t;
    region_t = region_b;
    region_b = temp;
  }
  saved_display_l = display_l;
  saved_display_r = display_r;
  saved_display_t = display_t;
  saved_display_b = display_b;
  display_l = region_l;
  display_r = region_r;
  display_t = region_t;
  display_b = region_b;
  display_w = display_r - display_l + 1;
  display_h = display_b - display_t + 1;
  fix_zoom_box_zoom();
}

/* Mouse Functions and special keys  ___________ */

void fix_key_zoom_window(int x_change, int y_change, int zoom_direction) {
  int   x_offset, y_offset;
  g_zoom_labels_count = 0;	/* reset labels to none */
  if (x_change != 0) {
    x_offset = step_fun(display_l, display_r);
    if (x_offset < 1)
      x_offset = 1;
    x_offset = x_offset * x_change;
    if (x_change < 0) {
      if (display_l == 1)
	return;	/* do not go left when at left edge */
      region_l = display_l + x_offset;
      if (region_l < 1)
	region_l = 1;
      region_r = display_r + x_offset;
      if (region_r < 1)
	region_r = 1;
    }
    if (x_change > 0) {
      if (display_r == g_len)
	return;	/* do go right when at right edge */
      region_r = region_r + x_offset;
      region_l = region_l + x_offset;
      if (region_r > g_len)
	region_r = g_len;
      if (region_l > g_len)
	region_l = g_len;
    }
  }
  if (y_change != 0) {
    y_offset = step_fun(display_t, display_b);
    if (y_offset < 1)
      y_offset = 1;
    y_change = y_change * y_offset;
    if (y_change < 0) {
      if (display_t == 1)
	return;	/* do not go up when at top */
      region_t = display_t + y_change;
      if (region_t < 1)
	region_t = 1;
      region_b = display_b + y_change;
      if (region_b < 1)
	region_b = 1;
    }
    if (y_change > 0) {
      if (display_b == g_len)
	return;	/* do not go up when at top */
      region_t = region_t + y_change;
      region_b = region_b + y_change;
      if (region_b > g_len)
	region_b = g_len;
      if (region_t > g_len)
	region_t = g_len;
    }
  }
  if (zoom_direction) {
    y_offset = step_fun(display_t, display_b);
    if (y_offset < 1)
      y_offset = 1;
    x_offset = step_fun(display_l, display_r);
    if (x_offset < 1)
      x_offset = 1;
    if (zoom_direction < 0) {
      if ((display_b - display_t) < 4) {
	y_offset = 0;
      }
      if ((display_r - display_l) < 4)
	x_offset = 0;
      x_offset = x_offset * -1;
      y_offset = y_offset * -1;
    }
    region_l = display_l - x_offset;
    region_r = display_r + x_offset;
    region_t = display_t - y_offset;
    region_b = display_b + y_offset;
    if (region_l < 1)
      region_l = 1;
    if (region_t < 1)
      region_t = 1;
    if (region_b > g_len)
      region_b = g_len;
    if (region_r > g_len)
      region_r = g_len;
  }
  fix_zoom_box();
  finish_fixing_zoom_box();
  if (g_zoom_window_displayed==TRUE) {
    zoomSetWindow();
    glutPostRedisplay();
  }
  glutSetWindow(win_main);
  glutPostRedisplay();
}

static void Key2(int key, int x, int y) {
  int   x_change;
  int   y_change;
  int   zoom_direction;
  if (!g_zoom_window_displayed)
    return;
  x_change = 0;
  y_change = 0;
  zoom_direction = 0;
  switch (key) {
  case GLUT_KEY_LEFT:
    x_change = -1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_RIGHT:
    x_change = 1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_UP:
    y_change = -1.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_DOWN:
    y_change = 1.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);;
    break;
  case GLUT_KEY_PAGE_DOWN:
    zoom_direction = +1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_PAGE_UP:
    zoom_direction = -1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  default:
    return;
  }
}

static void Key2_fast(int key, int x, int y) {
  int   x_change;
  int   y_change;
  int   zoom_direction;
  if (!g_zoom_window_displayed)
    return;
  x_change = 0;
  y_change = 0;
  zoom_direction = 0;
  switch (key) {
  case GLUT_KEY_LEFT:
    x_change = -3;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_RIGHT:
    x_change = 3;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_UP:
    y_change = -3.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_DOWN:
    y_change = 3.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);;
    break;
  case GLUT_KEY_PAGE_DOWN:
    zoom_direction = +1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_PAGE_UP:
    zoom_direction = -1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  default:
    return;
  }
}

void mouse(int button, int state, int x, int y) {
  int   i_row, j_col;
  j_col = window_x_convert(x);
  i_row = window_y_convert(y);
  if (i_row < 1) {
    i_row = 1;
  }
  if (i_row > g_len)
    i_row = g_len;
  if (j_col < 1)
    j_col = 1;
  if (j_col > g_len)
    j_col = g_len;
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
      g_i_row = i_row;
      g_j_col = j_col;
      strcpy(g_i_row_s, num_string_int(i_row));
      strcpy(g_j_col_s, num_string_int(j_col));
      glutSetWindow(win_text);
      glutPostRedisplay();
    }
  }
  if (button == GLUT_MIDDLE_BUTTON) {
    if (state == GLUT_DOWN) {
      if (g_region_set == FALSE) {
	region_l = j_col;
	region_t = i_row;
	region_b = i_row;
	region_r = j_col;
	g_region_set = TRUE;
      } else {
	printf("Warning!\tWithin no motion mouse with g_region_set=1 ****\n");
	region_r = j_col;	/* This code is never executed ? */
	region_b = i_row;
      }
      fix_zoom_box();
    } else {
      g_region_set = FALSE;
      g_region_exists = TRUE;
      finish_fixing_zoom_box();
      g_zoom_labels_count = 0;
      g_zoom_window_displayed = TRUE;
      zoomSetWindow();
      glutShowWindow();
      glutPopWindow();
      glutPostRedisplay();
    }
  }
}

void mouse_motion(int x, int y) { 
/* returns second corner of zoom rectangle as mouse is moved */
  int   i_row, j_col;
  if (g_region_set == TRUE) {
    j_col = window_x_convert(x);
    i_row = window_y_convert(y);
    if (i_row < 1) {
      i_row = 1;
    }
    if (j_col > g_len)
      j_col = g_len;
    if (i_row > g_len)
      i_row = g_len;
    if (j_col < 1)
      j_col = 1;
    region_r = j_col;
    region_b = i_row;
    fix_zoom_box();
    glutSetWindow(win_main);
    glutPostRedisplay();
  }
}

void mouse_zoom(int button, int state, int x, int y) {
  int   energy;
  int   i_row, j_col;
  int   place;
  char  rec[30];
  j_col = window_x_convert_zoom(x);
  i_row = window_y_convert_zoom(y);
  if (i_row < display_t) {
    i_row = display_t;
  }
  if (i_row > display_b)
    i_row = display_b;
  if (j_col < display_l)
    j_col = display_l;
  if (j_col > display_r)
    j_col = display_r;
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
      g_i_row = i_row;
      g_j_col = j_col;
      strcpy(g_i_row_s, num_string_int(i_row));
      strcpy(g_j_col_s, num_string_int(j_col));
      glutSetWindow(win_text);
      glutPostRedisplay();
      energy = energy_read(i_row, j_col);
      if (((energy <= g_energy_cutoff) && (energy != INT_MIN))
	  || (ct_read(i_row, j_col))) {
	if (g_zoom_labels_count < 49)
	  g_zoom_labels_count++;
	place = g_zoom_labels_count - 1;
	strcpy(rec, "<");
	strcat(rec, g_i_row_s);
	strcat(rec, ",");
	strcat(rec, g_j_col_s);
	strcat(rec, ">");
	strcpy(g_zoom_labels_string[place], rec);
	g_zoom_labels_x[place] =
	  plot_j_zoom(j_col + 1);
	g_zoom_labels_xl[place] =
	  plot_j_zoom(j_col);
	g_zoom_labels_y[place] =
	  plot_i_zoom(i_row);
	g_zoom_labels_row[place] = i_row;
	g_zoom_labels_col[place] = j_col;
	if (g_zoom_window_displayed==TRUE) {
	  zoomSetWindow();
	  glutPostRedisplay();
	}
      }
    }
  }
  if (button == GLUT_MIDDLE_BUTTON) {
    if (state == GLUT_DOWN) {
      if (g_region_set == FALSE) {
	region_l = j_col;
	region_t = i_row;
	region_b = i_row;
	region_r = j_col;
	g_region_set = TRUE;
      } else {
	printf("Warning!\tWithin no motion mouse with g_r_s=1\n");
	region_r = j_col; /* This code is never executed ? */
	region_b = i_row;
      }
      fix_zoom_box_zoom();
    } else {
      g_region_set = FALSE;
      g_region_exists = TRUE;
      g_zoom_labels_count = 0;
      finish_fixing_zoom_box_zoom();
      if (g_zoom_window_displayed==TRUE) {
	zoomSetWindow();
	glutShowWindow();
	glutPostRedisplay();
      }
      glutSetWindow(win_main);
      glutPostRedisplay();
    }
  }
}

void mouse_motion_zoom(int x, int y) {
  int   i_row, j_col;
  if (g_region_set == TRUE) {
    j_col = window_x_convert_zoom(x);
    i_row = window_y_convert_zoom(y);
    if (i_row < display_t) {
      i_row = display_t;
    }
    if (i_row > display_b)
      i_row = display_b;
    if (j_col < display_l)
      j_col = display_l;
    if (j_col > display_r)
      j_col = display_r;
    region_r = j_col;
    region_b = i_row;
    fix_zoom_box_zoom();
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutPostRedisplay();
    }
  }
}

int   g_which_window_i_am_in;

void No_Action_Menu(int msg) {
  /* this values should match those in make_menu() */
  if (msg == 18)
    set_energy_cutoff1();
  if (msg == 19)
    set_energy_cutoff2();
  if (msg == 1)
    try_exit(0);
  if (msg == 5) {
    glutDestroyWindow(win_zoom);
    g_zoom_window_exists = FALSE;
    g_zoom_window_displayed = FALSE;
  }
  if (msg == 2) {
    glutSetWindow(g_which_window_i_am_in);
    glutPopWindow();
  }
  if (msg == 12) {
    if (g_opt_mode) {
      g_opt_mode = FALSE;
      printf("Switching to CT Mode\n");
    } else {
      g_opt_mode = TRUE;
      printf("Switching to Optimal Plot Mode\n");
    }
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutSetCursor(GLUT_CURSOR_WAIT);
    }
    glutSetWindow(win_main);
    glutSetCursor(GLUT_CURSOR_WAIT);
    glutPostRedisplay();
    glutSetWindow(win_text);
    glutPostRedisplay();
    fix_cursor();
  }
  if (msg == 3) {
    if (g_grid_flag == TRUE)
      g_grid_flag = FALSE;
    else
      g_grid_flag = TRUE;
    glutSetWindow(win_main);	/* display main */
    glutPostRedisplay();
    if (g_zoom_window_displayed==TRUE) { /* display zoom */
      zoomSetWindow();
      glutPostRedisplay();
    }
  }
  if (msg == 4) {
    set_close_range();
  }
  if (msg == 7) { /* previous zoom window */
    region_l = saved_display_l;
    region_r = saved_display_r;
    region_t = saved_display_t;
    region_b = saved_display_b;
    fix_zoom_box();
    finish_fixing_zoom_box();
    g_zoom_labels_count = 0;
    if (g_zoom_window_displayed==TRUE) {
      zoomSetWindow();
      glutPostRedisplay();
    }
    glutSetWindow(win_main);
    glutPostRedisplay();
  }
}

void main_entry(int state) {
  if (state == GLUT_ENTERED) {
    g_i_row = 0;
    g_which_window_i_am_in = win_main;
  }
}

void zoom_entry(int state) {
  if (state == GLUT_ENTERED) {
    g_i_row = 0;
    g_which_window_i_am_in = win_zoom;
  }
}

void text_entry(int state) {
  if (state == GLUT_ENTERED)
    g_which_window_i_am_in = win_text;
}

void make_menu(int window) {  /* initialize graphics for each window */
  int    font_submenu, chain_len_submenu = 0, postscript_submenu;
  int    img_submenu = 0;
  glClearColor(g_color[COLOR_BACK][0], g_color[COLOR_BACK][1], 
	       g_color[COLOR_BACK][2], 1.0);
  postscript_submenu = glutCreateMenu(create_post1);
  glutAddMenuEntry("Create Postscript for Main Window", MAIN_POST);
  if (g_zoom_window_displayed == TRUE)
    glutAddMenuEntry("Create Postscript for Zoom Window", ZOOM_POST);
  if (g_prob != TRUE) {
    chain_len_submenu = glutCreateMenu(set_chain_len);
    glutAddMenuEntry("Include all", 1);
    glutAddMenuEntry("Include chains at least 2 long", 2);
    glutAddMenuEntry("Include chains at least 3 long", 3);
    glutAddMenuEntry("Include chains at least 4 long", 4);
    glutAddMenuEntry("Include chains at least 5 long", 5);
    glutAddMenuEntry("Include chains at least 6 long", 6);
    glutAddMenuEntry("Include chains at least 7 long", 7);
    glutAddMenuEntry("Include chains at least 8 long", 8);
    glutAddMenuEntry("Include chains at least 9 long", 9);
    glutAddMenuEntry("Include chains at least 10 long", 10);
  }
  font_submenu = glutCreateMenu(select_font);
  glutAddMenuEntry("Helvetica         12", 0);
  glutAddMenuEntry("Helvetica         18", 1);
  glutAddMenuEntry("Times Roman    24", 2);
  img_submenu = glutCreateMenu(img_menu);
  glutAddMenuEntry("Res. 72    or 612  x 792 ", 1);
  glutAddMenuEntry("Res. 110   or 935  x 1210", 2);
  glutAddMenuEntry("Res. 200   or 1700 x 2200", 3);
  glutAddMenuEntry("Res. 300   or 2550 x 3300", 4);
  glutCreateMenu(No_Action_Menu);
  if (g_zoom_window_displayed == TRUE)
    glutAddMenuEntry("UnZoom", 7);
  if (g_prob != TRUE)
    glutAddSubMenu("Filter", chain_len_submenu);
  if (g_prob == TRUE) {
    glutAddMenuEntry("Enter Prob. cutoff", 18);
  } else {
    glutAddMenuEntry("Enter energy increment value", 19);
  }
  glutAddMenuEntry("Enter Close  value", 4);
  glutAddMenuEntry("Toggle Optimal / CT mode", 12);
  glutAddSubMenu("Create Postscript file", postscript_submenu);
#if defined(HAVE_LIBGD) && defined(HAVE_GD_H)
  if (g_zoom_window_displayed == TRUE) {
    glutAddSubMenu("Create GIF for Zoom Window", img_submenu);
  }
#endif
  glutAddSubMenu("Font", font_submenu);
  glutAddMenuEntry("Toggle Grid lines On/Off", 3);
  glutAddMenuEntry("Pop", 2);
  if (g_zoom_window_displayed == TRUE)
    glutAddMenuEntry("Close zoom window", 5);
  glutAddMenuEntry("Exit", 1);
  glutAttachMenu(GLUT_RIGHT_BUTTON);
}

void gfxinit_nontext(int window) { /* initalize graphics for each window */
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutReshapeFunc(reshape);
  make_menu(window);
}

void memory_error(void) {
  printf("Error!\tInsufficient memory to execute with sequence len = %d", 
	 g_len);
  try_exit(6);
}

void zoomSetWindow(void)  {
  if (g_zoom_window_exists==FALSE) {
    g_zoom_window_exists = TRUE;
    glutInitWindowSize(750, 750);
    glutInitWindowPosition(6, 262);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
    win_zoom = glutCreateWindow(PROGRAM_NAME" Zoom Window");
    gfxinit_nontext(win_zoom);	/* initialize graphics for zoom window */
    glutDisplayFunc(display_zoom);
    glutEntryFunc(zoom_entry);
    glutMouseFunc(mouse_zoom);
    glutMotionFunc(mouse_motion_zoom);
    glutSpecialFunc(Key2);
    g_width_z = 750;
    g_height_z = 750;
  }
  if (g_zoom_window_displayed==TRUE) {
    glutSetWindow(win_zoom);
  } else {
    glutDestroyWindow(win_zoom);
    g_zoom_window_exists = FALSE;
  }
}
#endif

int main(int argc, char **argv) {
#ifdef HAVE_GLUTBITMAPCHARACTER
  char  title[150];
  int   i;
  initialize_data1(argc, argv);
  glutInit(&argc, argv);
  for (i = 1; i < argc; i++) {
    if (!strcmp(argv[i], "-mono")) {
      g_font_main = GLUT_BITMAP_HELVETICA_18;
      g_font_text = GLUT_BITMAP_HELVETICA_18;
      g_font_zoom = GLUT_BITMAP_HELVETICA_18;
    }
  }
  /* Zoom window */
  display_l = 1;
  display_t = 1;
  display_r = g_ct_len;
  display_b = g_ct_len;
  display_w = g_ct_len;
  display_h = g_ct_len;
  zoomSetWindow();
  /* Main window */
  glutInitWindowSize(800, 800);
  glutInitWindowPosition(474, 218);
  strcpy(title, PROGRAM_NAME" for ");
  strcat(title, g_plot_filename);
  strcat(title, " and ");
  strcat(title, g_ct_filename);
  win_main = glutCreateWindow(title);
  glutMouseFunc(mouse);
  glutMotionFunc(mouse_motion);
  glutSpecialFunc(Key2_fast);
  gfxinit_nontext(win_main);	/* initialize graphics for Main Window */
  glutDisplayFunc(display_main);
  glutEntryFunc(main_entry);
  /* Text window */
  glutInitWindowSize(800, 200);
  glutInitWindowPosition(6, 31);
  win_text = glutCreateWindow(PROGRAM_NAME", " PACKAGE_STRING);
  make_menu(win_text);
  glutDisplayFunc(display_text);
  glutReshapeFunc(text_reshape);
  glutEntryFunc(text_entry);
  glutKeyboardFunc(keyboard);
  glutMainLoop();
#endif
  printf("%s not available because glut was not found.\n", PROGRAM_NAME);
  printf("Use %s_ng instead.\n", PROGRAM_NAME);
  try_exit(7);
  return 0; /* Keep the stupid compiler happy. */
}
