/* complot */
#define PROGRAM_NAME "complot"
#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h> /* for isspace() */
#include <time.h>
#include <limits.h>
#include "util.h"

/* On the Mac, libglut does not exist separately. Search instead for
   the function glutInit. Under Linux, glut.h should be in
   /usr/include/GL and /usr/include is automatically searched.
*/
#ifdef HAVE_GLUTBITMAPCHARACTER
#if HAVE_LIBGLUT
# include <GL/glut.h>
#else
# include <glut.h>
#endif

#if (defined(HAVE_LIBGD) || defined(HAVE_GDIMAGECREATE)) && defined(HAVE_GD_H)
# include "complot_img.h"
#else
# include "complot.h"
char img_filename[130];
#endif
#include "complot_ps.h"

#include "complot.inc"
#include "complot_setcolor.h"
#include "complot_ann.h"

#include <limits.h>

#define MAIN_DUMP    1		/* create common dump for main */
#define ZOOM_DUMP    2		/* create common dump for zoom */

#define MESSAGE_NONE       0	/* values for g_type_message */
#define MESSAGE_POST       1	/* see it below */
#define MESSAGE_ENERGY     2
#define MESSAGE_DUMP       3
#define MESSAGE_GIF        4
#define MESSAGE_LABEL      5
#define MESSAGE_ALIGN_PS   6
#define MESSAGE_ALIGN_MAL  7
#define MESSAGE_COORDINATE 8
#define MESSAGE_PNG        9
#define MESSAGE_JPG       10


/* determines when zoom boxes are gray or different sizes
 * If less than STRIPE_SCREEN pixels per color, use gray
 * Perhaps 4 is the best value 
 */
#define STRIPE_SCREEN 2.5	

/* ____________________________________________________________________ */

void initialize_data2(void) {
  int             i;
  g_zoom_labels_count = 0;	/* initialize to no labels */
  width = 800;		/* initialize size of window in pixels */
  height = 800;		/* needed for scale in drawing windows */
  for (i = 0; i <= g_last_used_sequence; i++)
    g_energy_cutoff[i] = g_worst_energy[i];
  /* for drag with more than 2 sequences */
  if (g_last_used_sequence > 3) {
    g_primary_helix_index = -1;
    for (i = 2; i <= g_last_used_sequence; i++) {
      g_drag_this_row_from[i] = -1;
    }
  }
  g_annotation_flag = FALSE;
  g_prob_annotation_flag = FALSE;
  g_no_partial_flag = FALSE;
  g_annotation_flag = FALSE;
  g_region_set = FALSE;
  g_unique_mode = FALSE;
  g_neighbor_distance = 0;
  g_col_insert_base_on = FALSE;
  g_row_insert_base_on = FALSE;
  g_base_color_mode = FALSE;
  g_drag_helix_sel_row = 0;
  g_drag_helix_sel_col = 0;
  g_drag_helix_set = FALSE;
  g_drag_helix_below_diagonal = FALSE;
  if (g_count_sequences == 1)
    g_this_sequence = 2;
  else
    g_this_sequence = 3;
  g_primary_sequence = 2;
  g_bigdot_mode = FALSE;
  g_message_set = FALSE;
  g_number_inserts = 1;
  g_grid_flag = TRUE;
  g_comp_flag = TRUE; /* display compensating base pairs the same */
  text_window_input[0] = '\0';
  text_window_message[0] = '\0';
  g_type_message = MESSAGE_NONE;
  g_i_row = 0;
}

/* Menu routines
 * Most are defined throught GLUT and called from the event queue
 * cursor functions
 */

void fix_cursor(void) {	/* set cursor back to crosshairs */
  glutSetWindow(win2);
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutPostRedisplay();
  glutSetWindow(win1);
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutPostRedisplay();
}

void set_cursor_help(void) {
  glutSetWindow(win1);
  glutSetCursor(GLUT_CURSOR_HELP);
  glutPostRedisplay();
  glutSetWindow(win2);
  glutSetCursor(GLUT_CURSOR_HELP);
  glutPostRedisplay();
}

int             g_which_dump_seq;

void keyboard(unsigned char key, int x, int y) {
  int             string_len1;	/* accepts keys from text window */
  int             string_len2;
  x = y;
  y = x;			/* get rid of warning message */
  if (key == 13) {
    data_entered = TRUE;	/* on <RETURN> mark end of text */
    glutSetWindow(win3);
    glutPostRedisplay();
    if (g_type_message == MESSAGE_DUMP) {
      strcpy(com_plot_filename[g_which_dump_seq], text_window_input);
      if (strlen(text_window_input) == 0) {
	strcpy(com_plot_filename[g_which_dump_seq], "default");
	text_window_input[0] = '\0';
      }
      text_window_input[0] = '\0';
    }
    return;
  }
  string_len1 = (int) strlen(text_window_input);
  string_len2 = (int) strlen(text_window_message);
  if (key == 8) {		/* process backspace or delete */
    if (string_len1 > 0) {
      text_window_input[string_len1 - 1] = '\0';
      text_window_message[string_len2 - 1] = '\0';
      glutSetWindow(win3);
      glutPostRedisplay();
    }
    return;
  }
  text_window_input[string_len1] = key;	/* add key to end of input */
  text_window_input[string_len1 + 1] = '\0';
  text_window_message[string_len2] = key;	/* add key to end of message */
  text_window_message[string_len2 + 1] = '\0';
  if (g_type_message == MESSAGE_DUMP) {
    if (text_window_input[0] == '/') {
      text_window_input[0] = '\0';
      g_message_set = FALSE;
      g_type_message = MESSAGE_NONE;
      text_window_message[0] = ' ';
      strcpy(text_window_message, " ");
      fix_cursor();
    }
  }
  glutSetWindow(win3);	/* show the text */
  glutPostRedisplay();
}

void neighbor_off(void) {
  if (g_count_neighbors > 0) {
    g_count_neighbors = 0;
    g_neighbor_distance = 0;
    free(g_neighbor_keeper_row);
    free(g_neighbor_keeper_column);
  }
}

void nearby_off(void) {
  if (g_count_nearby > 0) {
    g_count_nearby = -1;
    g_nearby_distance = -1;
    free(g_nearby_basepair);
  }
}

void show_align(void) {
  glutSetWindow(win4);
  glutPostRedisplay();
}

int             g_which_dump;
char           *dump_filename[TOTAL_SEQUENCES];

void set_message_dump(int which_seq) {
  char            message[90];
  g_which_dump_seq = which_seq;
  if (g_type_of_dump == PLOT)
    strcpy(message, "Plot ");
  else
    strcpy(message, "Forced ");
  strcat(message, "File name for ");
  strcat(message, g_sequence_name[which_seq]);
  strcat(message, " (/ to abort, default ");
  strcat(message, plot_filename_short[which_seq]);
  if (g_type_of_dump == PLOT)
    strcat(message, ".com.plot) ?");
  else
    strcat(message, ".com) ?");
  strcpy(text_window_message, message);
  g_type_message = MESSAGE_DUMP;
  g_message_set = TRUE;
  data_entered = FALSE;
  glutSetWindow(win3);
  glutPostRedisplay();
  set_cursor_help();
}

int             g_dump_comp_flag;

void create_comdump_generic(int message) {
  if ((message < 3) || (g_zoom_window_exists == FALSE))
    g_which_dump = MAIN_DUMP;
  else
    g_which_dump = ZOOM_DUMP;
  if ((message == 1) || (message == 3))
    g_dump_comp_flag = TRUE;
  else
    g_dump_comp_flag = FALSE;
  set_message_dump(2);	/* 2 is the first sequence a dump is needed for */
}

void create_comdump(int message) { /* for forced file output */
  g_type_of_dump = FORCE;
  create_comdump_generic(message);
}

void create_p_comdump(int message) {
  g_type_of_dump = PLOT;
  create_comdump_generic(message);
}

char            g_plot_def[] = "plot";
char            g_forced_def[] = "forced base pair";

void create_dump(int dump_l, int dump_r, int dump_t, int dump_b, int seq) {
  char            string[25];
  if (g_type_of_dump == PLOT) {
    strcpy(string, g_plot_def);
  } else {
    strcpy(string, g_forced_def);
  }
  printf("%s is the common file for %s (%s)\n",
	 com_plot_filename[seq], g_sequence_name[seq], string);
  if (g_dump_comp_flag)
    printf("For compensating bases only.\n");
  if ((com_plot_fp = fopen(com_plot_filename[seq], "w")) == NULL) {
    printf("Could not open %s.\n", g_sequence_name[seq]);
    printf("No common file created for %s.\n", g_sequence_name[seq]);
  } else {
    if (g_dot_lower_flag)
      common_convert_helices(dump_t, dump_b, dump_l, dump_r, seq,
			     g_dump_comp_flag, g_type_of_dump);
    else
      common_convert_helices(dump_l, dump_r, dump_t, dump_b, seq,
			     g_dump_comp_flag, g_type_of_dump);
    /* make the dump for this sequence */
  }
  fclose(com_plot_fp);
  fix_cursor();
}

void finish_making_dump(void) {
  int             dump_l, dump_r, dump_t, dump_b, seq;
  g_message_set = FALSE;
  text_window_message[0] = '\0';
  glutSetWindow(win3);
  glutPostRedisplay();
  if (g_which_dump == MAIN_DUMP) {
    dump_l = 1;
    dump_t = 1;
    dump_b = g_sequence_length;
    dump_r = g_sequence_length;
  } else {
    dump_l = g_display_l;
    dump_r = g_display_r;
    dump_t = g_display_t;
    dump_b = g_display_b;
  }
  for (seq = 2; seq <= g_adjust_last_used_sequence; seq++) {
    if (strcmp(com_plot_filename[seq], "default") == 0) {
      strcpy(com_plot_filename[seq], plot_filename_short[seq]);
      strcat(com_plot_filename[seq], ".com");
      if (g_type_of_dump == PLOT)
	strcat(com_plot_filename[seq], ".plot");
    }
    create_dump(dump_l, dump_r, dump_t, dump_b, seq);
  }
}

void fixcolor_plot(int color) {	/* used on the screen */
  if ((color >= 0) && (color < TOTAL_COLORS)) {
    glColor3f(g_color[color][0], g_color[color][1], g_color[color][2]);
  }
}

void fixcolor_plot_ann(int color) {
  if ((color >= 0) && (color < g_ann_num_colors))
    glColor3f(g_ann_color[color][0], g_ann_color[color][1],
	      g_ann_color[color][2]);
}

void fixcolor_plot_bad(int color) { /* used on the screen */
  if ((color >= 0) && (color < TOTAL_COLORS))
    glColor3f(g_color[color][0], g_color[color][1], g_color[color][2]);
  else
    printf("Warning!\tColor out of range.\n");
}

void select_font(int newfont) {
  g_font_main = fonts[newfont];
  g_font_zoom = fonts[newfont];
  g_font_text = fonts[newfont];
  glutSetWindow(win3);
  glutPostRedisplay();
  glutSetWindow(win2);
  glutPostRedisplay();
  glutSetWindow(win1);
  glutPostRedisplay();
}

void select_font_main(int newfont) {
  g_font_main = fonts[newfont];
  glutPostRedisplay();
}

void select_font_zoom(int newfont) {
  g_font_zoom = fonts[newfont];
  glutPostRedisplay();
}

void select_font_text(int newfont) {
  g_font_text = fonts[newfont];
  glutSetWindow(win3);
  glutPostRedisplay();
}

int open_post_label(void) { /* open specified output file */
  if ((postfp = fopen(g_post_filename, "a")) == NULL) {	/* open a file */
    printf("Error1\tCould not open file: %s.\n", g_post_filename);
    return (1);
  }
  return (0);
}

void make_zoom_label_file_each(int type_of_file) {
  data_entered = FALSE;
  g_message_set = TRUE;
  strcpy(text_window_message, label_message);
  strcat(text_window_message, " (default ");
  strcat(text_window_message, g_label_default[type_of_file + 1]);
  strcat(text_window_message, " ) for ");
  strcat(text_window_message, g_sequence_name[type_of_file + 1]);
  strcat(text_window_message, "? ");
  g_type_message = MESSAGE_LABEL;
  g_type_of_label_file = type_of_file;
  glutSetWindow(win3);
  glutPostRedisplay();
  set_cursor_help();
}

void make_zoom_label_file(int type_of_file) {
  data_entered = FALSE;
  g_message_set = TRUE;
  strcpy(text_window_message, label_message);
  g_type_message = MESSAGE_LABEL;
  g_type_of_label_file = type_of_file;
  strcat(text_window_message, " (default ");
  strcat(text_window_message, g_label_default[1]);
  strcat(text_window_message, " )");
  strcat(text_window_message, " for all sequences ?");
  g_type_message = MESSAGE_LABEL;
  g_type_of_label_file = type_of_file;
  glutSetWindow(win3);
  glutPostRedisplay();
  set_cursor_help();
}

void get_post_name(void) {
  data_entered = FALSE;
  g_message_set = TRUE;
  strcpy(text_window_message, file_message);
  g_type_message = MESSAGE_POST;
  glutSetWindow(win3);
  glutPostRedisplay();
  set_cursor_help();
}

void set_zoom_coordinates(void) {
  data_entered = FALSE;
  g_message_set = TRUE;
  strcpy(text_window_message, coordinate_message);
  g_type_message = MESSAGE_COORDINATE;
  glutSetWindow(win3);
  glutPostRedisplay();
  set_cursor_help();
}

void fix_zoom_box(void) {	/* region_real_r, region_r; */
  g_region_real_l = 23.0 + (float) ((g_region_l + .5) * 700. /
				    (g_sequence_length + 1));
  g_region_real_r = 23.0 + (float) ((g_region_r + .5) * 700. /
				    (g_sequence_length + 1));
  g_region_real_b = 749.0 - (float) ((g_region_b - .5) * 700. /
				     (g_sequence_length + 1));
  g_region_real_t = 749.0 - (float) ((g_region_t - .5) * 700. /
				     (g_sequence_length + 1));
}

void finish_fixing_zoom_box(void) { 
  int             temp;	/* make left less than right */
  if (g_region_r < g_region_l) {
    temp = g_region_r;
    g_region_r = g_region_l;
    g_region_l = temp;
    fix_zoom_box();
  }
  if (g_region_b < g_region_t) {	/* make top less than bottom */
    temp = g_region_t;
    g_region_t = g_region_b;
    g_region_b = temp;
    fix_zoom_box();
  }
  if (g_first_zoom > 0) {
    g_saved_display_l = g_display_l;
    g_saved_display_r = g_display_r;
    g_saved_display_t = g_display_t;
    g_saved_display_b = g_display_b;
  } else {
    g_first_zoom = 1;
    g_saved_display_l = g_region_l;
    g_saved_display_r = g_region_r;
    g_saved_display_t = g_region_t;
    g_saved_display_b = g_region_b;
  }
  g_display_l = g_region_l;
  g_display_r = g_region_r;
  g_display_t = g_region_t;
  g_display_b = g_region_b;
  g_display_w = g_display_r - g_display_l + 1;
  g_display_h = g_display_b - g_display_t + 1;
}

void finish_set_zoom_coordinates(void) {
  int             row, col;
  int             width;
  int             left=0, right=799, top=0, bottom=799;
  int             extra;
  int             len;
  int             i;
  int             total;
  int             temp;
  g_message_set = FALSE;
  text_window_message[0] = '\0';
  glutSetWindow(win3);
  glutPostRedisplay();
  len = strlen(text_window_input);
  if (len < 3) {
    fix_cursor();
    return;
  }
  for (i = 0; i < len; i++) {
    if (!isdigit(text_window_input[i]))
      text_window_input[i] = ' ';
  }
  total = sscanf(text_window_input, "%d%d%d", &row, &col, &width);
  if (total < 2) {
    fix_cursor();
    return;
  }
  if (total == 2)
    width = 100;
  if (row < 1)
    row = 1;
  if (row > g_sequence_length)
    row = g_sequence_length;
  if (col < 1)
    col = 1;
  if (col > g_sequence_length)
    col = g_sequence_length;
  if (g_dot_lower_flag) { /* switch to make zoom window in lower triangle */
    temp = row;
    row = col;
    col = temp;
  }
  g_i_row = row;
  g_j_col = col;
  if (width < 3)
    width = 3;
  if (width > g_sequence_length)
    width = g_sequence_length;
  extra = width % 2;
  g_row_line = TRUE;
  g_col_line = TRUE;
  width = width / 2;
  text_window_input[0] = '\0';
  left = col - width;
  right = col + width + extra;
  top = row - width;
  bottom = row + width + extra;
  if (left < 1)
    left = 1;
  if (top < 1)
    top = 1;
  if (bottom > g_sequence_length)
    bottom = g_sequence_length;
  if (right > g_sequence_length)
    right = g_sequence_length;
  g_region_l = left;
  g_region_t = top;
  g_region_set = TRUE;
  g_region_r = right;
  g_region_b = bottom;
  fix_zoom_box();
  glutSetWindow(win1);
  glutPostRedisplay();
  g_region_set = FALSE;
  g_region_exists = TRUE;
  finish_fixing_zoom_box();
  g_zoom_labels_count = 0;
  glutSetWindow(win2);
  glutShowWindow();
  glutPopWindow();
  g_zoom_window_exists = TRUE;
  fix_cursor();
}

void finish_making_post1(void) {
  int             name_len;
  g_message_set = FALSE;
  text_window_message[0] = '\0';
  glutSetWindow(win3);
  glutPostRedisplay();
  fix_cursor();
  if (strlen(text_window_input) == 0)
    strcpy(g_post_filename, PROGRAM_NAME);
  else
    strcpy(g_post_filename, text_window_input);
  text_window_input[0] = '\0';
  name_len = strlen(g_post_filename);
  if ((g_post_filename[0] == '/') && (name_len == 1))
    return;
  chop_suffix(g_post_filename, ".ps");
  strcat(g_post_filename, ".ps");
  general_post(g_post_filename, g_sequence_length, g_big_dots_flag, 
	       g_last_used_sequence, g_base_color_mode, g_display_l, 
	       g_display_r, g_display_t, g_display_b, g_sequence_data_same, 
	       g_sequence_data, g_count_sequences, g_grid_flag, 
	       g_last_diagonal, g_diag_count, g_diag_start, g_diag, 
	       g_unique_mode, g_count_nearby, g_nearby_basepair, g_comp_flag,
	       g_count_neighbors, g_neighbor_keeper_row, 
	       g_neighbor_keeper_column, g_post_file, g_display_w, g_display_h,
	       g_points_plotted_ind, g_chain_len, g_comp, g_zoom_comp, 
	       g_sequence_name, g_worst_energy, g_optimal_energy,
	       g_energy_cutoff, g_zoom_labels_count, g_zoom_labels_row,
	       g_zoom_labels_col, g_zoom_labels_string, g_color, g_ps_dot_min,
	       g_points_plotted, g_points_plotted_zoom,	
	       g_points_plotted_zoom_all, g_dot_lower_flag, 
	       g_ps_multicolor_dots_flag, g_no_partial_flag, 
	       g_annotation_flag, g_prob_annotation_flag, g_ann_color, 
	       g_ann_num_colors);
}

void general_label(int type_of_file, int find_max_min, int *min, int *max) {
  int             i, j, row, col, s_row, s_col;	/* shifted row and col
						 * for each sequence */
  int             energy;
  char            base1, base2;
  int             temp;
  int             sequence;
  int             new_min = 0;
  int             new_max = -100;
  if (find_max_min) {
    new_max = -100;
    new_min = g_sequence_length + 1;
  }
  if (type_of_file < 1) {
    fprintf(postfp, "Selected points for Zoom Window from %s\n", PROGRAM_NAME);
    fprintf(postfp, "    0 = Screen Label\n");
    for (i = 2; i <= g_last_used_sequence; i++) {
      fprintf(postfp, "    %d = %s\n", i - 1, g_sequence_name[i]);
    }
    fprintf(postfp, "\n");
    fprintf(postfp, "     ");
    fprintf(postfp, "%d            ", 0);
    for (i = 1; i <= g_count_sequences; i++) {
      fprintf(postfp, "%d                 ", i);
    }
    fprintf(postfp, "\n");
    fprintf(postfp, "  ");
    fprintf(postfp, " 5      3    ");
    for (i = 1; i <= g_count_sequences; i++) {
      fprintf(postfp, "  5     3         ");
    }
    fprintf(postfp, "\n");
  }
  if (type_of_file < 1) {
    for (j = 0; j < g_zoom_labels_count; j++) {
      row = g_zoom_labels_row[j];
      col = g_zoom_labels_col[j];
      if (col < row) {
	temp = row;
	row = col;
	col = temp;
      }
      fprintf(postfp, "%5d %5d  ", row, col);
      for (i = 2; i <= g_last_used_sequence; i++) {	/* shift each row and
							 * column depending on
							 * the sequence */
	s_row = row;
	s_col = col;
	/* translate back to original sequence */
	energy = energy_read(s_row, s_col, i);
	base1 = g_sequence_data[i][s_row - 1];
	base2 = g_sequence_data[i][s_col - 1];
	s_row = g_reverse_st[i][s_row];	/* translate back to
					 * original */
	s_col = g_reverse_st[i][s_col];	/* coordinates */
	if (energy == 0) {
	  s_row = 0;
	  s_col = 0;
	}
	fprintf(postfp, "%5d %5d (%c,%c) ", s_row, s_col, base1, base2);
      }
      fprintf(postfp, "\n");
    }
  } else {
    sequence = type_of_file + 1;
    for (j = 0; j < g_zoom_labels_count; j++) {
      row = g_zoom_labels_row[j];
      col = g_zoom_labels_col[j];
      if (col < row) {
	temp = row;
	row = col;
	col = temp;
      }
      /*
       * shift each row and column depending on the
       * sequence
       */
      s_row = row;
      s_col = col;
      /* translate back to original sequence */
      energy = energy_read(s_row, s_col, sequence);
      if (energy == 0) {
	s_row = 0;	/* no base pair for this sequence */
	s_col = 0;
      } else {
	s_row = g_reverse_st[sequence][s_row];
	/* translate back to original */
	s_col = g_reverse_st[sequence][s_col];	/* coordinates */
	fprintf(postfp, "F %5d %5d 1\n", s_row, s_col);
	if (find_max_min) {
	  if (s_row < new_min)
	    new_min = s_row;
	  if (s_col > new_max)
	    new_max = s_col;
	}
      }
    }
  }
  *min = new_min;
  *max = new_max;
  fclose(postfp);
}

void finish_making_label(void) {
  int             error;
  int             min, max;
  g_message_set = FALSE;
  text_window_message[0] = '\0';
  glutSetWindow(win3);
  glutPostRedisplay();
  fix_cursor();
  strcpy(g_post_filename, text_window_input);
  text_window_input[0] = '\0';
  if (strlen(g_post_filename) == 0)
    strcpy(g_post_filename, g_label_default[g_type_of_label_file + 1]);
  if ((g_post_filename[0] == '/') || (strlen(g_post_filename) == 0))
    return;
  /* remove .sel if it is there */
  chop_suffix(g_post_filename, ".sel");
  strcat(g_post_filename, ".sel");
  if (g_type_of_label_file == 0)
    printf("File of selected base pairs for %s is %s\n",
	   g_sequence_name[g_type_of_label_file], g_post_filename);
  else
    printf("File of selected base pairs for  %s is %s",
	   g_sequence_name[g_type_of_label_file + 1], g_post_filename);
  strcpy(g_label_default[g_type_of_label_file + 1], g_post_filename);
  error = open_post_label();
  if (error) {
    printf("Error creating zoom label file # %d\n", error);
    return;
  }
  general_label(g_type_of_label_file, FALSE, &min, &max);
}

/* from menu, indicate to  create postcript file for window */
void create_post(int which_window) {
  text_window_input[0] = '\0';
  if ((which_window < 2) || (g_zoom_window_exists == FALSE))
    g_post_file = MAIN_POST;	/* create post for main */
  else
    g_post_file = ZOOM_POST;	/* create post for zoom */
  if ((which_window == 1) || (which_window == 3))
    g_big_dots_flag = TRUE;
  else
    g_big_dots_flag = FALSE;
  get_post_name();
}

/* make the base pairs */
/* gif creation menu, called from pop up window */

void finish_making_img1(int file_type) {
  int   png_type = FALSE, jpg_type = FALSE;
  g_message_set = FALSE;
  text_window_message[0] = '\0';
  glutSetWindow(win3);
  glutPostRedisplay();
  glutSetWindow(win2);
  glutSetCursor(GLUT_CURSOR_WAIT);
  glutSetWindow(win1);
  glutSetCursor(GLUT_CURSOR_WAIT);
  glutPostRedisplay();
  if (strlen(text_window_input) == 0)
    strcpy(img_filename, PROGRAM_NAME);
  else
    strcpy(img_filename, text_window_input);
  text_window_input[0] = '\0';
  if ((img_filename[0] == '/') && (strlen(img_filename) == 1)) {
    fix_cursor();
    return; /* abort making file */
  }
  chop_suffix(img_filename, ".img"); /* Delete suffixes .gif, .jpg or .png */
  if (file_type == MESSAGE_GIF) {
    png_type = FALSE;
    jpg_type = FALSE;
    strcat(img_filename, ".gif");
  } else if (file_type == MESSAGE_PNG) {
    png_type = TRUE;
    jpg_type = FALSE;
    strcat(img_filename, ".png");
  } else if (file_type == MESSAGE_JPG) {
    png_type = FALSE;
    jpg_type = TRUE;
    strcat(img_filename, ".jpg");
  }
#if (defined(HAVE_LIBGD) || defined(HAVE_GDIMAGECREATE)) && defined(HAVE_GD_H)
  finish_making_img(img_filename, g_display_l, g_display_r, g_display_t, 
		    g_display_b, g_grid_flag, g_diag_start, g_diag_count,
		    g_count_nearby, g_unique_mode, g_diag, g_nearby_basepair,
		    g_last_used_sequence, g_comp_flag, g_sequence_length, 
		    g_zoom_comp, g_zoom_labels_count, g_zoom_labels_row,
		    g_zoom_labels_col, g_color, g_resolution, g_chain_len,
		    g_sequence_name, g_count_sequences, g_worst_energy, 
		    g_optimal_energy, g_energy_cutoff, g_points_plotted_zoom, 
		    g_points_plotted_zoom_all, g_no_partial_flag, 
		    g_annotation_flag, g_prob_annotation_flag, g_ann_color, 
		    g_ann_num_colors, g_dot_lower_flag, png_type, jpg_type);
#endif
  fix_cursor();		/* fix cursor */
}

void img_menu(int choice) {
  if (choice == 1)
    g_resolution = 72;
  else if (choice == 2)
    g_resolution = 110;
  else if (choice == 3)
    g_resolution = 200;
  else
    g_resolution = 300;
  strcpy(text_window_message, gif_message);
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_GIF;
  glutSetWindow(win3);
  glutPostRedisplay();
  set_cursor_help();
}

void jpg_menu(int choice) {
  if (choice == 1)
    g_resolution = 72;
  else if (choice == 2)
    g_resolution = 110;
  else if (choice == 3)
    g_resolution = 200;
  else
    g_resolution = 300;
  strcpy(text_window_message, jpg_message);
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_JPG;
  glutSetWindow(win3);
  glutPostRedisplay();
  set_cursor_help();
 }

void png_menu(int choice) {
  if (choice == 1)
    g_resolution = 72;
  else if (choice == 2)
    g_resolution = 110;
  else if (choice == 3)
    g_resolution = 200;
  else
    g_resolution = 300;
  strcpy(text_window_message, png_message);
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_PNG;
  glutSetWindow(win3);
  glutPostRedisplay();
  set_cursor_help();
}

void nearby_menu_function(int choice) {
  if (g_unique_mode)
    choice = -1;
  if ((choice == -1) && (g_count_nearby > 0)) {
    g_count_nearby = 0;
    g_nearby_distance = 0;
    free(g_nearby_basepair);
    glutSetWindow(win2);
    glutPostRedisplay();
    glutSetWindow(win1);
    glutPostRedisplay();
    return;
  }
  neighbor_off();
  nearby_off();
  glutSetWindow(win2);
  glutSetCursor(GLUT_CURSOR_WAIT);
  glutSetWindow(win1);
  glutSetCursor(GLUT_CURSOR_WAIT);
  if (g_this_sequence == g_primary_sequence) {
    printf("Error!\tCannot compute close for a sequence and itself\n");
  } else {
    g_nearby_distance = choice;
    if (g_nearby_distance > 0)
      printf("For sequence %s, computing close within %d of %s.\n",
	     g_sequence_name[g_this_sequence], choice,
	     g_sequence_name[g_primary_sequence]);
    create_nearby(g_this_sequence, choice);
  }
  fix_cursor();
}

void neighbor_menu_function(int choice) {
  nearby_off();
  if ((choice == -1) && (g_count_neighbors > 0)) {
    g_count_neighbors = 0;
    g_neighbor_distance = 0;
    free(g_neighbor_keeper_row);
    free(g_neighbor_keeper_column);
    glutSetWindow(win2);
    glutPostRedisplay();
    return;
  }
  glutSetWindow(win2);
  glutSetCursor(GLUT_CURSOR_WAIT);
  if (g_this_sequence == g_primary_sequence) {
    printf("Error!\tCannot create neighbors for a sequence and itself.\n");
  } else {
    printf("Creating neighbors for sequence %s within %d of %s.\n",
	   g_sequence_name[g_this_sequence], choice,
	   g_sequence_name[g_primary_sequence]);
    g_neighbor_distance = choice;
    create_neighbors(g_this_sequence, choice);
  }
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutPostRedisplay();
  glutSetWindow(win1);
  glutPostRedisplay();
}

void label_menu_function(int choice) {
  if (g_zoom_labels_count < 1) {
    printf("No labels exist.  No file created.\n");
    return;
  } else {
    if (choice == 0)
      make_zoom_label_file(choice);
    else
      make_zoom_label_file_each(choice);
  }
}

void set_energy_cutoff1(int cutoff_level) {   /* for probabilities */
  int             i;
  neighbor_off();
  g_zoom_labels_count = 0; /* reset labeling of points in zoom window */
  for (i = 2; i <= g_last_used_sequence; i++) {
    g_energy_cutoff[i] = 10000;
    switch (cutoff_level) {
    case 0:
      g_energy_cutoff[i] = g_worst_energy[i];
      break;
    case 1:
      g_energy_cutoff[i] = -10;
      break;
    case 2:
      g_energy_cutoff[i] = -100;
      break;
    case 3:
      g_energy_cutoff[i] = -1000;
      break;
    case 4:
      g_energy_cutoff[i] = -3500;
      break;
    case 5:
      g_energy_cutoff[i] = -6500;
      break;
    case 6:
      g_energy_cutoff[i] = -9000;
      break;
    case 7:
      g_energy_cutoff[i] = -9990;
      break;
    }
  }
  make_overlaps_initialize();
  glutSetWindow(win3);	/* redisplay all windows with new cutoff */
  glutPostRedisplay();
  glutSetWindow(win2);
  glutPostRedisplay();
  glutSetWindow(win1);
  glutPostRedisplay();
}

int             g_which_sequence;	/* the sequence to set the cutoff */

void set_energy_cutoff2(int seq) { /* for energies */
  g_zoom_labels_count = 0; /* reset labeling of points in zoom window */
  strcpy(text_window_message, increment_message);
  strcat(text_window_message, g_sequence_name[seq]);
  strcat(text_window_message, "? ");
  g_which_sequence = seq;
  g_message_set = TRUE;
  data_entered = FALSE;
  g_type_message = MESSAGE_ENERGY;
  glutSetWindow(win3);
  glutPostRedisplay();
  set_cursor_help();
}

void finish_setting_energy_cutoff(void) {
  int             seq;
  float           temp_filter;
  neighbor_off();
  seq = g_which_sequence;
  text_window_message[0] = '\0';
  g_message_set = FALSE;
  if (text_window_input[0] == '/') {
    text_window_message[0] = '\0';
    glutSetWindow(win3);
    glutPostRedisplay();
    fix_cursor();
    return;
  }
  sscanf(text_window_input, "%f", &temp_filter);
  g_energy_cutoff[seq] = (int) (temp_filter * 10);
  /* values greater than 0 are relative; values < 0 are absolute */
  if (g_energy_cutoff[seq] >= 0)
    g_energy_cutoff[seq] = g_optimal_energy[seq] +
      g_energy_cutoff[seq];
  text_window_message[0] = '\0';
  if (g_energy_cutoff[seq] > g_worst_energy[seq])
    g_energy_cutoff[seq] = g_worst_energy[seq];
  if (g_energy_cutoff[seq] < g_optimal_energy[seq])
    g_energy_cutoff[seq] = g_optimal_energy[seq];
  glutSetWindow(win3);
  glutPostRedisplay();
  fix_cursor();
}

/* This function is called from the menu to set the filter */
void set_chain_len(int chain_len_choice) {
  neighbor_off();
  g_zoom_labels_count = 0;	/* Reset to no labels */
  g_chain_len = chain_len_choice;
  make_overlaps_initialize();	/* refigure intersecting helices */
  if (g_nearby_distance > 0)
    create_nearby(g_this_sequence, g_nearby_distance);
  glutSetWindow(win1);	/* show the main window after changes */
  glutPostRedisplay();
  glutSetWindow(win2);	/* show zoom window */
  glutPostRedisplay();
  glutSetWindow(win3);	/* show text window to update filter line */
  glutPostRedisplay();
}

/* Plotting functions _____________________ */

void sniff_for_opengl_errors(void) {
  GLenum          errCode;
  const GLubyte  *errString;
  const char     *my_string;
  if ((errCode = glGetError()) != GL_NO_ERROR) {
    errString = gluErrorString(errCode);
    my_string = (char *) errString;
    fprintf(stderr, "%s OpenGL Error: %s\n", PROGRAM_NAME, my_string);
  }
}

void plot_ij(int i_row, int j_col) {	/* converterts i,j base pair to */
  float           real_x, real_y;	/* real coordinates */
  int             row, col;
  if (g_dot_lower_flag) {
    row = j_col;	/* draw dots in lower triangle instead */
    col = i_row;
  } else {
    row = i_row;
    col = j_col;
  }
  real_x = 23.0 + (float) ((col + .5) * 700. / (g_sequence_length + 1));
  /* and plots them */
  real_y = 749.0 - (float) ((row - .5) * 700. / (g_sequence_length + 1));
  glVertex2f(real_x, real_y);
}

void plot_ij_zoom_comp(int i_row, int j_col) {	/* same for zoom window */
  int             row, col;
  float           real_x, real_y;
  if (g_dot_lower_flag) {
    row = j_col;	/* draw dots in lower triangle instead */
    col = i_row;
  } else {
    row = i_row;
    col = j_col;
  }
  real_x = 25. + (float) (col - g_display_l + 1) * 700. / (g_display_w + 1);
  real_y = 50. + (float) (g_display_b - row + 1) * 700. / (g_display_h + 1);
  glVertex2f(real_x, real_y);
}

/* same for zoom window */
void plot_ij_zoom(int i_row, int j_col, int stripe_flag, int color)  {
  float           real_x, real_y, x_offset, y_offset, mag, adjuster;
  int             hor_insert_adjustment;
  int             ver_insert_adjustment;
  int             row, col;
  if ((stripe_flag != 0) && (color == 1))
    return;
  if (g_dot_lower_flag) {
    row = j_col;	/* draw dots in lower triangle instead */
    col = i_row;
  } else {
    row = i_row;
    col = j_col;
  }
  real_x = 25. + (float) (col - g_display_l + 1) * 700. / (g_display_w + 1);
  real_y = 50. + (float) (g_display_b - row + 1) * 700. / (g_display_h + 1);
  hor_insert_adjustment = 0;
  ver_insert_adjustment = 0;
  if (!g_dot_lower_flag) {
    if ((g_col_insert_base_on) && (color == g_col_insert_sequence)) {
      if (real_x >= (g_col_insert_base_x_start_true))
	hor_insert_adjustment = g_col_insert_base_x_offset;
      if (real_y <= (g_col_insert_row_start_y) &&
	  (color == g_col_insert_sequence))
	ver_insert_adjustment = hor_insert_adjustment * -1.0 *
	  ((float) g_display_w + 1.) / (g_display_h + 1.);
    }
    if ((g_row_insert_base_on) && (color == g_row_insert_sequence)) {
      if (real_y <= (g_row_insert_base_y_start_true)) {
	ver_insert_adjustment = g_row_insert_base_y_offset;
	hor_insert_adjustment = ver_insert_adjustment * -1.0 *
	  ((float) (g_display_h + 1.0) / (float) (g_display_w + 1));
      } else {
	if (real_x >= (g_row_insert_col_start_x))
	  hor_insert_adjustment = g_row_insert_base_y_offset * -1.0 *
	    ((float) (g_display_h + 1.0) / (float) (g_display_w + 1));
      }
    }
  } else {
    if ((g_col_insert_base_on) && (color == g_col_insert_sequence)) {
      if (real_x >= (g_col_insert_base_x_start_true)) {
	hor_insert_adjustment = g_col_insert_base_x_offset;
	ver_insert_adjustment = -1.0 * g_col_insert_base_x_offset *
	  ((float) (g_display_w + 1.0) / (float) (g_display_h + 1));
      } else {
	if (real_y <= (g_col_insert_row_start_y) &&
	    (color == g_col_insert_sequence)) {
	  ver_insert_adjustment = g_col_insert_base_x_offset * -1.0 *
	    ((float) g_display_h + 1.) / (g_display_w + 1.);
	}
      }
    }
    if ((g_row_insert_base_on) && (color == g_row_insert_sequence)) {
      if (real_x >= (g_row_insert_col_start_x)) {
	ver_insert_adjustment = g_row_insert_base_y_offset;
	hor_insert_adjustment = g_row_insert_base_y_offset * -1. *
	  ((float) (g_display_h + 1.0) / (float) (g_display_w + 1));
      } else {
	if (real_y <= g_row_insert_base_y_start_true)
	  ver_insert_adjustment = g_row_insert_base_y_offset;
      }
    }
  }
  if (!stripe_flag) {
    if ((g_display_w > 180) && (g_display_h > 180)) {
      glVertex2f(real_x + hor_insert_adjustment, real_y);
    } else {
      x_offset = (25. + (float) (col + 1 - g_display_l + 1) * 700. / (g_display_w + 1) - real_x) / 2;
      y_offset = (50. + (float) (g_display_b - (row - 1) + 1) * 700. / (g_display_h + 1) - real_y) / 2;
      glBegin(GL_POLYGON);
      glVertex2f(real_x + hor_insert_adjustment,
		 real_y - y_offset + ver_insert_adjustment);
      glVertex2f(real_x - x_offset + hor_insert_adjustment,
		 real_y - y_offset + ver_insert_adjustment);
      glVertex2f(real_x - x_offset + hor_insert_adjustment,
		 real_y + ver_insert_adjustment);
      glVertex2f(real_x + hor_insert_adjustment,
		 real_y + y_offset + ver_insert_adjustment);
      glVertex2f(real_x + x_offset + hor_insert_adjustment,
		 real_y + y_offset + ver_insert_adjustment);
      glVertex2f(real_x + x_offset + hor_insert_adjustment,
		 real_y + ver_insert_adjustment);
      glEnd();/* ? this statement produces errors */
    }
  } else {		/* stripe option */
    x_offset = (25. + (float) (col + 1 - g_display_l + 1) * 700. / (g_display_w + 1) - real_x) / 2;
    y_offset = (50. + (float) (g_display_b - (row - 1) + 1) * 700. / (g_display_h + 1) - real_y) / 2;
    if (color == 0)
      adjuster = 1.0;
    else
      adjuster = (color - 1) * 1. / (float) (g_count_sequences);
    mag = 1.0;
    glBegin(GL_POLYGON);	/* bot,mid */
    glVertex2f(real_x + x_offset*adjuster * mag + hor_insert_adjustment,
	       real_y - y_offset*adjuster * mag + ver_insert_adjustment);
    glVertex2f(real_x - x_offset*1.0 + hor_insert_adjustment,
	       real_y - y_offset*mag + ver_insert_adjustment);	/* bot,left */
    glVertex2f(real_x - x_offset*adjuster*mag + hor_insert_adjustment,
	       real_y+y_offset*adjuster*mag+ver_insert_adjustment);/* top,mid */
    glVertex2f(real_x + x_offset*mag + hor_insert_adjustment,
	       real_y + y_offset*mag + ver_insert_adjustment); /* top,right */
    glEnd();
    sniff_for_opengl_errors();
  }
}

void draw_zoom_dragged_helix(int color, int x, int y, int length, 
			     int front_length) {
  float           row_width;
  float           column_width;
  float           hor_offset, ver_offset;
  float           start_x, start_y;
  float           converted_x, converted_y;
  int             offset_to_window;
  int             i;
  int             mult_fac;
  if (g_drag_helix_below_diagonal)
    mult_fac = -1;
  else
    mult_fac = 1;
  if (g_count_sequences < 3)
    offset_to_window = 0;
  else
    offset_to_window = (g_count_sequences - 2) * 15;
  converted_x = x - (25+offset_to_window)*width_z / (800. + offset_to_window);
  /* subtract offset to right depending on window */
  converted_x = converted_x * (800. + offset_to_window) / width_z; /* size */
  /* subtract offset to right depending on window */
  converted_x = converted_x + 25.0;
  if (g_count_sequences < 3)
    offset_to_window = 0;
  else
    offset_to_window = (g_count_sequences - 2) * 15;
  converted_y = y - (50.+offset_to_window)*height_z/(800 + 2*offset_to_window);
  /* subtract offset down depending on window */
  converted_y = 700 - converted_y * (800. + 2 * offset_to_window) / height_z;
  converted_y = converted_y + 50.0;
  row_width = 700. / (g_display_h + 1.0);
  column_width = 700.0 / (g_display_w + 1.0);
  hor_offset = column_width / 2;
  ver_offset = row_width / 2;
  fixcolor_plot(color);
  for (i = 0; i < length; i++) {
    start_x = converted_x - i * column_width * mult_fac;
    start_y = converted_y - i * row_width * mult_fac;
    glBegin(GL_POLYGON);
    glVertex2f(start_x - hor_offset, start_y + ver_offset);
    glVertex2f(start_x - hor_offset, start_y - ver_offset);
    glVertex2f(start_x + hor_offset, start_y - ver_offset);
    glVertex2f(start_x + hor_offset, start_y + ver_offset);
    glEnd();
  }
  /* draw portion of helix above clicked base pair */
  for (i = 1; i <= front_length; i++) {
    start_x = converted_x + i * column_width * mult_fac;
    start_y = converted_y + i * row_width * mult_fac;
    glBegin(GL_POLYGON);
    glVertex2f(start_x - hor_offset, start_y + ver_offset);
    glVertex2f(start_x - hor_offset, start_y - ver_offset);
    glVertex2f(start_x + hor_offset, start_y - ver_offset);
    glVertex2f(start_x + hor_offset, start_y + ver_offset);
    glEnd();
  }
 }

int plot_i_zoom(int i_row) { /* convert i_row to y coordinate */
  return (int) (50.2 + (float) ((g_display_b-i_row+1)*700./(g_display_h + 1)));
}

int plot_j_zoom(int j_col) { /* convert col to x coordinate */
  return (int) (25.2 + (float) (j_col - g_display_l+1)*700./(g_display_w + 1));
}

int plot_i(int i_row) {	/* converts row to y coordinate */
  return (int) (749.0 - (float) ((i_row-.5)*700./(g_sequence_length+1)) + .5);
}

int plot_j(int j_col) { /* converts col to x coordinate */
  return (int) (23.0 + (float) ((j_col + .5)*700./(g_sequence_length+1)) + .5);
}

float plot_i_float(float i_row) { /* converts row to y coordinate */
  return 749.0 - (float) ((i_row - .5) * 700. / (g_sequence_length + 1)) + .5;
}

float plot_j_float(float j_col) { /* converts col to x coordinate */
  return 23.0 + (float) ((j_col + .5) * 700. / (g_sequence_length + 1)) + .5;
}

int find_color_prob(int energy) {
  if (energy == -5)
    return 4;
  if (energy == -30)
    return 3;
  return 2;
}

void plot_points_zoom_compensate(float point_size) {
  int     row, col, new_row, count;
  int     diag, position, current_color, location;
  int     start_diag, end_diag, seq;
  int     gl_display_b, gl_display_r, gl_display_t, gl_display_l;
  int     gl_display_w, gl_display_h;
  start_diag = g_display_t + g_display_l - 1;
  end_diag = g_display_b + g_display_r - 1;
  if (g_dot_lower_flag) {
    gl_display_b = g_display_r;
    gl_display_t = g_display_l;
    gl_display_l = g_display_t;
    gl_display_r = g_display_b;
  } else {
    gl_display_b = g_display_b;
    gl_display_t = g_display_t;
    gl_display_l = g_display_l;
    gl_display_r = g_display_r;
  }
  gl_display_w = gl_display_r - gl_display_l;
  gl_display_h = gl_display_b - gl_display_t;
  if ((gl_display_w > 180) && (gl_display_h > 180)) {
    glPointSize(point_size * 1.6);
    glBegin(GL_POINTS);
  }
  current_color = 0;
  fixcolor_plot(COLOR_COMP);
  for (diag = start_diag; diag <= end_diag; diag++) {
    position = g_diag_start[diag];
    for (count = 0; ((count < g_diag_count[diag])
		     && (g_diag[position + count].row <= g_display_b));
	 count++) {
      location = position + count;
      seq = g_diag[location].color;
      if (seq == current_color) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	if (col >= gl_display_l) {
	  for (row = new_row; row < new_row + g_diag[location].length;
	       row++) {
	    if ((row >= gl_display_t) && (row <= gl_display_b)
		&& (col >= gl_display_l) && (col <= gl_display_r)) {
	      if (check_compensate(row, col)) {
		g_zoom_comp++;
		plot_ij_zoom(row, col, FALSE, 0);
	      }
	    }
	    col--;
	  }
	}
      }
    }
  }
  if ((gl_display_w > 180) && (gl_display_h > 180))
    glEnd();
}

void plot_points_unique_zoom(float point_size) {
  int             row, col, new_row;
  int             current_color, location;
  int             diag, position, count;
  int            *holder;
  int             gl_display_b, gl_display_t, gl_display_l, gl_display_r;
  int             gl_display_w, gl_display_h;
  int             size;
  int             start_diag, end_diag;
  if (g_dot_lower_flag) {
    gl_display_b = g_display_r;
    gl_display_t = g_display_l;
    gl_display_l = g_display_t;
    gl_display_r = g_display_b;
  } else {
    gl_display_b = g_display_b;
    gl_display_t = g_display_t;
    gl_display_l = g_display_l;
    gl_display_r = g_display_r;
  }
  gl_display_w = gl_display_r - gl_display_l;
  gl_display_h = gl_display_b - gl_display_t;
  if ((gl_display_w > 180) && (gl_display_h > 180)) {
    glPointSize(point_size * 1.4);
    glBegin(GL_POINTS);	/* plot points if they are small */
  }
  holder = malloc((int) (1.5 * g_sequence_length + 1) * sizeof(int));
  if (holder == NULL) {
    printf("Error!\tInsufficient memory for unique dots ");
    printf("within plot_points_unique of %s.\n", PROGRAM_NAME);
    try_exit(1);
  }
  size = (int) (1.5 * g_sequence_length + 1);
  for (row = 0; row < (size - 1); row++)
    holder[row] = 0;
  current_color = 2;
  fixcolor_plot(current_color);
  g_points_plotted_zoom = 0;
  start_diag = g_display_t + g_display_l - 1;
  end_diag = g_display_b + g_display_r - 1;
  /* set black,gray */
  for (diag = start_diag; diag <= end_diag; diag++) {
    /* set black,gray */
    position = g_diag_start[diag];
    for (count = 0; count < g_diag_count[diag]; count++) {
      location = position + count;
      if (g_diag[location].color < 2) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	if ((new_row <= gl_display_b) && (col >= gl_display_l)) {
	  for (row=new_row; row < new_row + g_diag[location].length; row++) {
	    if ((row >= gl_display_t) && (row <= gl_display_b)
		&& (col >= gl_display_l) && (col <= gl_display_r)) {
	      holder[row] = 1;
	    }
	    col--;
	  }
	}
      }
    }
    /* plot other colors */
    for (count = 0; count < g_diag_count[diag]; count++) {
      location = position + count;
      if ((g_diag[location].row <= gl_display_b) &&
	  (g_diag[location].column >= gl_display_l)) {
	if (g_diag[location].color > 1) {
	  if (current_color != g_diag[location].color) {
	    current_color = g_diag[location].color;
	    fixcolor_plot(current_color);
	  }
	  for (row = g_diag[location].row,
		 col = g_diag[location].column;
	       row < g_diag[location].row + g_diag[location].length;
	       row++) {
	    if ((row >= gl_display_t) && (row <= gl_display_b)
		&& (col >= gl_display_l) && (col <= gl_display_r)) {
	      if (holder[row] == 0) {
		plot_ij_zoom(row, col, FALSE, 0);
		g_points_plotted_zoom++;
	      }
	    }
	    col--;
	  }
	}
      }
    }
    /* zero out black,gray */
    for (count = 0; count < g_diag_count[diag]; count++) {
      location = position + count;
      if (g_diag[location].color < 2) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	if ((new_row <= gl_display_b) && (col >= gl_display_l)) {
	  for (row = new_row; row < new_row + g_diag[location].length; row++) {
	    if ((row >= gl_display_t) && (row <= gl_display_b)
		&& (col >= gl_display_l) && (col <= gl_display_r)) {
	      holder[row] = 0;
	    }
	    col--;
	  }
	}
      }
    }
  }
  if ((gl_display_w > 180) && (gl_display_h > 180))
    glEnd();
  free(holder);
}

void plot_points_zoom_close(void) { /* plot the dots with the close distance
				     * specified */
  int             i, row, col;
  int             gl_display_b, gl_display_t, gl_display_l, gl_display_r;
  if (g_dot_lower_flag) {
    gl_display_b = g_display_r;
    gl_display_t = g_display_l;
    gl_display_l = g_display_t;
    gl_display_r = g_display_b;
  } else {
    gl_display_b = g_display_b;
    gl_display_t = g_display_t;
    gl_display_l = g_display_l;
    gl_display_r = g_display_r;
  }
  fixcolor_plot(COLOR_CLOSE);
  for (i = 0; i < g_count_nearby; i++) {
    row = g_nearby_basepair[i].row;
    col = g_nearby_basepair[i].column;
    if ((row >= gl_display_t) && (row <= gl_display_b)
	&& (col >= gl_display_l) && (col <= gl_display_r)) {
      plot_ij_zoom(row, col, FALSE, COLOR_CLOSE);
    }
    /* lower triangle */
    if ((col >= gl_display_t) && (col <= gl_display_b)
	&& (row >= gl_display_l) && (row <= gl_display_r)) {
      plot_ij_zoom(col, row, FALSE, COLOR_CLOSE);
    }
  }
}

int             get_ann_color(int, int);

void plot_points_zoom(float point_size, int stripe_flag) {
  int row, col, new_row, count;
  int diag, position, current_color, location;
  int start_diag, end_diag, seq;
  int gl_display_b, gl_display_t, gl_display_l, gl_display_r;
  int gl_display_w, gl_display_h;
  int             ann_color;
  if (g_dot_lower_flag) {
    gl_display_b = g_display_r;
    gl_display_t = g_display_l;
    gl_display_l = g_display_t;
    gl_display_r = g_display_b;
  } else {
    gl_display_b = g_display_b;
    gl_display_t = g_display_t;
    gl_display_l = g_display_l;
    gl_display_r = g_display_r;
  }
  gl_display_w = gl_display_r - gl_display_l;
  gl_display_h = gl_display_b - gl_display_t;
  g_points_plotted_zoom = 0;
  g_points_plotted_zoom_all = 0;
  if (g_unique_mode) {
    if (gl_display_t <= gl_display_r)
      plot_points_unique_zoom(point_size);
  }
  start_diag = g_display_t + g_display_l - 1;
  end_diag = g_display_b + g_display_r - 1;
  if ((gl_display_t <= gl_display_r) && (!g_unique_mode)) {
    if ((gl_display_w > 180) && (gl_display_h > 180)) {
      glPointSize(point_size * 1.4);
      glBegin(GL_POINTS);	/* plot points if they are
				 * small */
    }
    for (current_color = g_adjust_last_used_sequence; current_color >= 0;
	 current_color--) {
      if ((current_color == 1) && (g_count_nearby > 0))
	plot_points_zoom_close();
      if ((g_adjust_count_sequences == 2) && (current_color == 1)) {
	current_color = 0;	/* skip grey when only 2
				 * sequences */
      }
      fixcolor_plot(current_color);
      for (diag = start_diag; diag <= end_diag; diag++) {
	position = g_diag_start[diag];
	for (count = 0; ((count < g_diag_count[diag])
			 && (g_diag[position + count].row <= gl_display_b));
	     count++) {
	  location = position + count;
	  seq = g_diag[location].color;
	  if (seq == current_color) {
	    new_row = g_diag[location].row;
	    col = g_diag[location].column;
	    if (col >= gl_display_l) {
	      for (row=new_row; row<new_row+g_diag[location].length; row++) {
		if ((row >= gl_display_t) && (row <= gl_display_b)
		    && (col >= gl_display_l) && (col <= gl_display_r)) {
		  plot_ij_zoom(row, col, stripe_flag, current_color);
		  if (seq > 1)
		    g_points_plotted_zoom++;
		  else if (seq == 0)
		    g_points_plotted_zoom_all++;
		}
		col--;
	      }
	    }
	  }
	}
      }
    }
    if ((gl_display_w > 180) && (gl_display_h > 180))
      glEnd();
    if (g_comp_flag)
      plot_points_zoom_compensate(point_size);
  }
  /* plot neighbors */
  if (g_count_neighbors > 0) {
    if ((point_size * .4) < 1) {
      glPointSize(1.0);
    } else {
      glPointSize(point_size * .4);
    }
    fixcolor_plot(COLOR_LABEL);
    glBegin(GL_POINTS);
    for (position = 0; position < g_count_neighbors; position++) {
      row = g_neighbor_keeper_row[position];
      col = g_neighbor_keeper_column[position];
      if ((row >= gl_display_t) &&
	  (col >= gl_display_l) &&
	  (col <= gl_display_r) && (row <= gl_display_b)
	  )
	plot_ij_zoom_comp(row, col);
    }
    glEnd();
  }
  /* draw the moving helix */
  if (g_drag_helix_set)
    draw_zoom_dragged_helix(g_this_sequence, g_drag_helix_mov_x, 
			    g_drag_helix_mov_y, g_drag_helix_sel_length,
			    g_drag_helix_front_length);
  /* plot overlap in lower triangle */
  if (gl_display_b < gl_display_l) {
    return;		/* none of region is below main diagonal */
  }
  if ((gl_display_w > 180) && (gl_display_h > 180)) {
    glPointSize(point_size * 1.4);
    glBegin(GL_POINTS);	/* plot points if they are small */
  }
  for (current_color = 1; current_color >= 0; current_color--) {
    if (((g_adjust_count_sequences == 2) && (current_color == 1)) ||
	(g_no_partial_flag)) {
      current_color = 0;	/* skip grey when only 2
				 * sequences */
    }
    fixcolor_plot(current_color);
    for (diag = start_diag; diag <= end_diag; diag++) {
      position = g_diag_start[diag];
      for (count = 0; ((count < g_diag_count[diag])
		       && (g_diag[position + count].row <= gl_display_b));
	   count++) {
	location = position + count;
	seq = g_diag[location].color;
	if (seq == current_color) {
	  new_row = g_diag[location].row;
	  col = g_diag[location].column;
	  if (col >= gl_display_t) {
	    for (row = new_row; row < new_row + g_diag[location].length;
		 row++) {
	      if ((col >= gl_display_t) && (col <= gl_display_b)
		  && (row >= gl_display_l) && (row <= gl_display_r)) {
		if (current_color == 0) {
		  if ((g_annotation_flag) ||
		      (g_prob_annotation_flag)) {
		    ann_color = get_ann_color(row, col);
		    fixcolor_plot_ann(ann_color);
		    plot_ij_zoom(col, row, FALSE, 0);
		  } else {
		    if ((g_comp_flag) &&
			(check_compensate(col, row))) {
		      fixcolor_plot(COLOR_COMP);
		      plot_ij_zoom(col, row, FALSE, 0);
		      fixcolor_plot(current_color);
		    } else
		      plot_ij_zoom(col, row, FALSE, 0);
		  }
		} else
		  plot_ij_zoom(col, row, FALSE, 0);
	      }
	      col--;
	    }
	  }
	}
      }
    }
  }
  if ((gl_display_w > 180) && (gl_display_h > 180))
    glEnd();
}

void plot_points_compensate(void) {
  int             row, col, new_row;
  int             current_color, seq, location;
  glBegin(GL_POINTS);
  current_color = 0;
  g_comp = 0;
  fixcolor_plot(COLOR_COMP);
  for (location = 0; location < g_helices_used; location++) {
    seq = g_diag[location].color;
    if (seq == current_color) {
      new_row = g_diag[location].row;
      col = g_diag[location].column;
      for (row = new_row; row < new_row + g_diag[location].length; row++) {
	if (check_compensate(row, col)) {
	  g_comp++;
	  if (!g_unique_mode) 
	    plot_ij(row, col);
	  if ((!g_annotation_flag) && (!g_prob_annotation_flag))
	    plot_ij(col, row);
	}
	col--;
      }
    }
  }
  glEnd();
}

void plot_points_unique(void) {
  int             row, col, new_row;
  int             current_color, location;
  int             diag, position, count;
  int            *holder;
  int             size;
  int             start_diag, end_diag;
  holder = malloc((int) (1.5 * g_sequence_length + 1) * sizeof(int));
  if (holder == NULL) {
    printf("Error!\tInsufficient memory for unique dots ");
    printf("within plot_points_unique of %s.\n", PROGRAM_NAME);
    try_exit(2);
  }
  size = (int) (1.5 * g_sequence_length + 1);
  for (row = 0; row < (size - 1); row++)
    holder[row] = 0;
  current_color = 2;
  fixcolor_plot(current_color);
  g_points_plotted = 0;
  glBegin(GL_POINTS);
  start_diag = 1 + 1 - 1;
  end_diag = g_last_diagonal;
  /* set black,gray */
  for (diag = start_diag; diag <= end_diag; diag++) {
    /* set black,gray */
    position = g_diag_start[diag];
    for (count = 0; count < g_diag_count[diag]; count++) {
      location = position + count;
      if (g_diag[location].color < 2) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	for (row = new_row; row < new_row + g_diag[location].length; row++) {
	  holder[row] = 1;
	  col--;
	}
      }
    }
    /* plot other colors */
    for (count = 0; count < g_diag_count[diag]; count++) {
      location = position + count;
      if (g_diag[location].color > 1) {
	if (current_color != g_diag[location].color) {
	  current_color = g_diag[location].color;
	  fixcolor_plot(current_color);
	}
	for (row = g_diag[location].row, col = g_diag[location].column;
	     row < g_diag[location].row + g_diag[location].length; row++) {
	  if (holder[row] == 0) {
	    plot_ij(row, col);
	    g_points_plotted_ind[current_color]++;
	    g_points_plotted++;
	  }
	  col--;
	}
      }
    }
    /* zero out black,gray */
    for (count = 0; count < g_diag_count[diag]; count++) {
      location = position + count;
      if (g_diag[location].color < 2) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	for (row = new_row; row < new_row + g_diag[location].length; row++) {
	  holder[row] = 0;
	  col--;
	}
      }
    }
  }
  glEnd();
  free(holder);
}

void plot_points_close(void) { /* plot the dots with the close
				  distance specified */ 
  int             i, row, col;
  fixcolor_plot(COLOR_CLOSE);
  for (i = 0; i < g_count_nearby; i++) {
    row = g_nearby_basepair[i].row;
    col = g_nearby_basepair[i].column;
    plot_ij(row, col);
    g_points_plotted_ind[7]++;
    plot_ij(col, row);
  }
}

void plot_points(void) {
  int             row, col, new_row;
  int             current_color, seq, location;
  int             i;
  int             ann_color;
  glBegin(GL_POINTS);
  g_points_plotted = 0;
  for (i = 0; i <= 7; i++)
    g_points_plotted_ind[i] = 0;
  for (current_color=g_last_used_sequence; current_color>=0; current_color--) {
    if ((current_color == 1) && (g_count_nearby > 0))
      plot_points_close();
    if ((g_adjust_count_sequences == 2) && (current_color == 1)) {
      current_color = 0;	/* skip grey when only 2
				 * sequences */
    }
    fixcolor_plot(current_color);
    for (location = 0; location < g_helices_used; location++) {
      seq = g_diag[location].color;
      if (seq == current_color) {
	new_row = g_diag[location].row;
	col = g_diag[location].column;
	for (row = new_row; row < new_row + g_diag[location].length; row++) {
	  if (!g_unique_mode) {
	    plot_ij(row, col);
	    g_points_plotted_ind[seq]++;
	    if (seq > 1)
	      g_points_plotted++;
	  }
	  if (seq < 2) {
	    g_points_plotted_ind[seq]++;
	    if (seq == 0) {
	      if ((g_annotation_flag) || (g_prob_annotation_flag)) {
		ann_color = get_ann_color(col, row);
		fixcolor_plot_ann(ann_color);
		plot_ij(col, row);
		fixcolor_plot(current_color);
	      } else
		plot_ij(col, row);
	    } else {
	      if (!g_no_partial_flag)
		plot_ij(col, row);
	    }
	  }
	  col--;
	}
      }
    }
  }
  glEnd();
  if (!g_unique_mode) {
    g_points_plotted_ind[0] = g_points_plotted_ind[0] / 2;
    g_points_plotted_ind[1] = g_points_plotted_ind[1] / 2;
  }
  if (g_comp_flag)
    plot_points_compensate();
  if (g_unique_mode)
    plot_points_unique();
}

void output_main(int x, int y, char *string) {
  /* display string at real coordinates  x,y  */
  int             len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(g_font_main, string[i]);
  }
}

/* display string at real coordinates  x,y  */
void output_align(int x, int y, char *string) {
  int             len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++)
    glutBitmapCharacter(g_font_align, string[i]);
}

void output_align_char(int x, int y, char c) {
  /* display char at real coordinates  x,y  */
  glRasterPos2f((float) x, (float) y);
  glutBitmapCharacter(g_font_align, c);
}

void output_main_small(int x, int y, char *string) {
  /* display string at real coordinates  x,y  */
  int             len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, string[i]);
  }
}

void output_zoom(int x, int y, char *string) { /* display string at
						  real coordinates x,y  */ 
  int             len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(g_font_zoom, string[i]);
  }
}

void output_text(int x, int y, char *string) {	/* display string at
						   real coordinates x,y  */ 
  int             len, i;
  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(g_font_text, string[i]);
  }
}

void single_hor_tic(int j_col, int grid_flag) {
  char            string2[20];
  float           x;
  x = (float) plot_j(j_col);
  glBegin(GL_LINES);
  glVertex2f(x, 751.);
  glVertex2f(x, 758.);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COLOR_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(x, 750);
    glVertex2f(x, 46);
    glEnd();
    fixcolor_plot(COLOR_TEXT);
  }
  strcpy(string2, num_string_int(j_col));
  output_main(plot_j(j_col) - 10, 762, string2);
}

void plot_hor_tics(void) {
  int             j_col, step, start;
  step = step_fun(1, g_sequence_length);
  start = start_fun(1, step);
  single_hor_tic(g_sequence_length, FALSE);
  single_hor_tic(1, FALSE);
  for (j_col=start; j_col<(g_sequence_length-2*step/3); j_col=j_col+step) {
    single_hor_tic(j_col, g_grid_flag);
  }
}

void single_ver_tic(int i_row, int grid_flag) {
  char            string2[20];
  float           y;
  y = (float) plot_i(i_row);
  glBegin(GL_LINES);
  glVertex2f(727., y);
  glVertex2f(736., y);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COLOR_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(21, y);
    glVertex2f(725., y);
    glEnd();
    fixcolor_plot(COLOR_TEXT);
  }
  strcpy(string2, num_string_int(i_row));
  output_main(737, plot_i(i_row) - 5, string2);
}

void plot_ver_tics(void) {
  int             i_row, step, start;
  step = step_fun(1, g_sequence_length);
  start = start_fun(1, step);
  single_ver_tic(1, FALSE);
  single_ver_tic(g_sequence_length, FALSE);
  for (i_row = start; i_row < (g_sequence_length-step/2); i_row=i_row+step) {
    single_ver_tic(i_row, g_grid_flag);
  }
}

void single_hor_tic_zoom(int j_col, int grid_flag) {
  char            string2[10];
  float           x;
  int             sequence;
  int             ver_offset;
  x = (float) plot_j_zoom(j_col);
  glBegin(GL_LINES);
  glVertex2f(x, 751);
  glVertex2f(x, 758);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COLOR_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(x, 749);
    glVertex2f(x, 50);
    glEnd();
    fixcolor_plot(COLOR_TEXT);
  }
  strcpy(string2, num_string_int(j_col));
  output_zoom(plot_j_zoom(j_col) - 10, 758, string2);
  if (g_count_sequences < 3)
    ver_offset = 789;
  else
    ver_offset = 789 + 15 * (g_count_sequences - 2);
  for (sequence = 2; sequence <= g_last_used_sequence; sequence++) {
    strcpy(string2, num_string_int(g_reverse_st[sequence][j_col]));
    if (!strcmp(string2, "0"))
      strcpy(string2, "-");
    fixcolor_plot(sequence);
    output_main_small(plot_j_zoom(j_col)-10, ver_offset - 15*(sequence-2), 
		      string2);
  }
  fixcolor_plot(COLOR_TEXT);
}

void single_hor_tic_zoom_extras(int j_col, int j_col_next) {
  float           x;
  float           x_next;
  float           dif;
  int             lines;
  if (j_col_next > g_display_r)
    j_col_next = g_display_r;
  if ((j_col_next - j_col) < 5)
    return;
  lines = 10;
  x = (float) plot_j_zoom(j_col);
  x_next = (float) plot_j_zoom(j_col_next);
  fixcolor_plot(COLOR_GRID_FINE);
  dif = (x_next - x) / (float) lines;
  while (x < (x_next - dif / 2.)) {
    x += dif;
    glBegin(GL_LINES);
    glVertex2f(x, 749);
    glVertex2f(x, 50);
    glEnd();
  }
  fixcolor_plot(COLOR_TEXT);
}

void plot_hor_tics_zoom(void) {
  int             j_col, step;
  int             start;
  step = step_fun(g_display_l, g_display_r);
  start = start_fun(g_display_l, step);
  single_hor_tic_zoom(g_display_r, g_grid_flag);
  single_hor_tic_zoom(g_display_l, g_grid_flag);
  if (g_grid_flag)
    single_hor_tic_zoom_extras(g_display_l, start);
  for (j_col = start; j_col <= (g_display_r - 2 * step / 3);
       j_col = j_col + step) {
    single_hor_tic_zoom(j_col, g_grid_flag);
    if (g_grid_flag)
      single_hor_tic_zoom_extras(j_col, j_col + step);
  }
}

void single_ver_tic_zoom(int i_row, int grid_flag) {
  char            string2[20];
  int             i, max_seq;
  float           y;
  y = (float) plot_i_zoom(i_row);
  glBegin(GL_LINES);
  glVertex2f(727., y);
  glVertex2f(736., y);
  glEnd();
  if (grid_flag == TRUE) {
    fixcolor_plot(COLOR_GRID);	/* sets color to gray */
    glBegin(GL_LINES);
    glVertex2f(23., y);
    glVertex2f(723., y);
    glEnd();
    fixcolor_plot(COLOR_TEXT);
  }
  strcpy(string2, num_string_int(i_row));
  output_zoom(737, plot_i_zoom(i_row) - 5, string2);
  max_seq = g_last_used_sequence;
  if (max_seq > 4)
    max_seq = 4;
  for (i = 2; i <= max_seq; i++) {
    fixcolor_plot(i);
    strcpy(string2, num_string_int(g_reverse_st[i][i_row]));
    if (!strcmp(string2, "0"))
      strcpy(string2, "-");
    output_main_small(739, plot_i_zoom(i_row) - 17 - 13 * (i - 2), string2);
  }
  fixcolor_plot(COLOR_TEXT);
}

void single_ver_tic_zoom_extras(int i_row, int i_row_next) {
  int             lines;
  float           dif;
  float           y;
  float           y_next;
  if (i_row_next > g_display_b)
    i_row_next = g_display_b;
  if ((i_row_next - i_row) < 5)
    return;
  y = (float) plot_i_zoom(i_row);
  y_next = (float) plot_i_zoom(i_row_next);
  lines = 10;
  dif = (y - y_next) / (float) (lines);
  fixcolor_plot(COLOR_GRID_FINE);
  while (y > (y_next + dif / 2.)) {
    y -= dif;
    glBegin(GL_LINES);
    glVertex2f(23., y);
    glVertex2f(723., y);
    glEnd();
  }
  fixcolor_plot(COLOR_TEXT);
}

void plot_ver_tics_zoom(void) {
  int             i_row, step, start;
  step = step_fun(g_display_t, g_display_b);
  start = start_fun(g_display_t, step);
  single_ver_tic_zoom(g_display_t, g_grid_flag);
  if (g_grid_flag)
    single_ver_tic_zoom_extras(g_display_t, start);
  single_ver_tic_zoom(g_display_b, g_grid_flag);
  for (i_row = start; i_row < (g_display_b - step/2); i_row = i_row + step) {
    single_ver_tic_zoom(i_row, g_grid_flag);
    if (g_grid_flag)
      single_ver_tic_zoom_extras(i_row, i_row + step);
  }
}

void plot_tics_zoom(void) {
  plot_hor_tics_zoom();
  plot_ver_tics_zoom();
  return;
}

void plot_tics(void) {
  plot_hor_tics();
  plot_ver_tics();
  return;
}

int window_x_convert(int x) {
  /* converts window x coordinates to column coordinate of RNA
   * top left for window is 0,0;  x is right; y is down;
   * i is down; j is right for RNA */ 
  float xf;
  xf = x - 23. * width / 800.;	/* subtract offset to right depending
				 * on window */
  xf = xf * (g_sequence_length + 1) / 700. * 800. / width;	/* size */
  return (int) xf;
}

int window_y_convert(int y) {	/* convert window y coordinates to */
  float           yf;	/* RNA i coordinates down */
  int             i_row;
  yf = (float) (y * 800. / height);	/* scale according to window
					 * size */
  yf = 799 - yf;		/* top is 0 or 800, different from window
				 * coordinates */
  i_row = (int) ((g_sequence_length + 1.) * (749. - yf) / 700. + 1.0);
  /* inverse of plot_ij function */
  return i_row;
}

int window_x_convert_zoom(int x) {	/* converts window x coordinates to */
  float           xf;	/* a column */
  float           offset_to_window;
  if (g_count_sequences < 3)
    offset_to_window = 5;
  else
    offset_to_window = (g_count_sequences - 2) * 15;
  xf = x - (25 + offset_to_window) * width_z / (800. + offset_to_window);
  /* subtract offset to right depending on window size */
  xf = xf*(g_display_w+1)/700.*(800.+offset_to_window)/width_z + g_display_l-1;
  return (int) (xf + .5);
}

int window_y_convert_zoom(int y) {
  /* convert window y coordinates to RNA i coordinates down */
  float           yf;	
  float           offset_to_window;
  if (g_count_sequences < 3)
    offset_to_window = 0;
  else
    offset_to_window = (g_count_sequences - 2) * 15;
  yf = y - (50. + offset_to_window) * height_z / (800 + 2 * offset_to_window);
  /* subtract offset down depending on window */
  yf = yf*(g_display_h+1)/700*(800.+2*offset_to_window)/height_z+g_display_t-1;
  /* size */
  return (int) (yf + .5);
}

/* display functions ____________________ */
void display_text_filter(void) {
  char            text[25];
  if (g_chain_len > 1) {
    strcpy(text, "Filter: ");
    strcat(text, num_string_int(g_chain_len));
    output_text(100, 5, text);
  }
}

int get_ann_color(int row, int column) {
  int             seq;
  int             color;
  int             pos1, pos2;
  float           annotation_average;
  float           annotation_average_total = 0.0;
  float           annotation_percent;

  if (g_prob_annotation_flag) {
    for (seq = 2; seq <= (g_adjust_last_used_sequence); seq++) {
      annotation_average_total += (energy_read(row, column, seq) / -10000.);
    }
    annotation_average_total = annotation_average_total /
      (g_adjust_last_used_sequence - 1);
    return (get_color_prob(annotation_average_total));
  }
  for (seq = 2; seq <= (g_adjust_last_used_sequence); seq++) {
    pos1 = g_reverse_st[seq][row];
    pos2 = g_reverse_st[seq][column];
    annotation_average = (g_annotation_base[seq][pos1] +
			  g_annotation_base[seq][pos2]) / 2.;
    annotation_average_total += annotation_average;
  }
  annotation_percent = annotation_average_total /
    (float) g_annotation_max_total;
  color = (int) (g_ann_num_colors * annotation_percent);
  if (color >= g_ann_num_colors)
    color = g_ann_num_colors - 1;
  if (color < 0)
    color = 0;	
  return (color);
}

void display_text_data(void) {
  float  annotation_average, annotation_average_total = 0.0;
  float  annotation_percent;
  int    i, i_row, j_col,  pos1, pos2,  row, column, energy;
  char   base1, base2;
  char   bases[10];
  char   text[80];
  strcpy(text, "  Base pair");
  if (g_prob == FALSE)
    strcat(text, "          .plot Energy");
  else
    strcat(text, "                Prob. ");
  strcat(text, "              Name ");
  fixcolor_plot(COLOR_TEXT);
  output_text(2, 182, text);
  if (g_i_row > 0) {
    if (g_annotation_flag)
      annotation_average_total = 0.;
    if (g_i_row > g_j_col) {
      i_row = g_j_col;
      j_col = g_i_row;
    } else {
      j_col = g_j_col;
      i_row = g_i_row;
    }
  } else {
    i_row = g_i_row;
    j_col = g_j_col;
  }
  if (g_prob == TRUE) {
    strcpy(text, "Range");
    output_text(540, 182, text);
  } else {
    strcpy(text, "Optimal ");
    output_text(510, 182, text);
  }
  for (i = 0; i <= g_last_used_sequence; i++) {
    if (i == g_primary_sequence) {
      strcpy(text, "P>> ");
      if (i == g_this_sequence) {
	strcat(text, "A>> ");
      }
    } else {
      if (i == g_this_sequence) {
	strcpy(text, "A>> ");
      } else
	strcpy(text, "       ");
    }
    if ((g_primary_helix_index >= 0) && (i > 1)) {
      if ((i == g_primary_sequence) || (g_drag_this_row_from[i] > 0))
	strcat(text, "D ");
    }
    strcat(text, g_sequence_name[i]);
    fixcolor_plot(i);
    output_text(250, 165 - i * 22, text);
  }
  fixcolor_plot(COLOR_TEXT);
  strcpy(text, "Cutoff");
  output_text(730, 182, text);
  if (g_prob == TRUE) {
    for (i = 2; i <= g_last_used_sequence; i++) {
      strcpy(text, num_string_fancy_int(g_worst_energy[i]));
      strcat(text, " <= Prob <= ");
      strcat(text, num_string_fancy_int(g_optimal_energy[i]));
      output_text(510, 165 - i * 22, text);
      strcpy(text, num_string_fancy_int(g_energy_cutoff[i]));
      output_text(750, 165 - i * 22, text);
    }
  } else {
    strcpy(text, "   Highest");
    output_text(590, 182, text);
    for (i = 2; i <= g_last_used_sequence; i++)
      output_text(510, 165-i*22, num_string_fancy_int(g_optimal_energy[i]));
    for (i = 2; i <= g_last_used_sequence; i++)
      output_text(600, 165 - i * 22, num_string_fancy_int(g_worst_energy[i]));
    for (i = 2; i <= g_last_used_sequence; i++) {
      strcpy(text, num_string_fancy_int(g_energy_cutoff[i]));
      strcat(text, " or ");
      strcat(text, num_string_fancy_int(g_energy_cutoff[i] - 
					g_optimal_energy[i]));
      output_text(690, 165 - i * 22, text);
    }
  }
  fixcolor_plot(COLOR_TEXT);
  if (i_row > 0) {
    energy = 0;	/* take care of grid position */
    strcpy(text, "(");
    strcat(text, num_string_int(i_row));
    strcat(text, ",");
    strcat(text, num_string_int(j_col));
    strcat(text, ")");
    output_text(5, 165, text);
    for (i = 2; i <= g_last_used_sequence; i++) {
      /* display base pair and energy for sequences */
      row = i_row;
      column = j_col;
      energy = energy_read(row, column, i);
      strcpy(text, "(");
      pos1 = g_reverse_st[i][row];
      base1 = g_sequence_data[i][row - 1];
      strcat(text, num_string_int(pos1));
      strcat(text, ",");
      pos2 = g_reverse_st[i][column];
      base2 = g_sequence_data[i][column - 1];
      strcat(text, num_string_int(pos2));
      strcat(text, ")  ");
      bases[0] = '(';
      bases[1] = base1;
      bases[2] = ',';
      bases[3] = base2;
      bases[4] = ')';
      bases[5] = '\0';
      output_text(5, 165 - i * 22, text);
      output_text(105, 165 - i * 22, bases);
      strcpy(text, num_string_fancy_int(energy));
      output_text(155, 165 - i * 22, text);
      if ((g_annotation_flag) && (i <= g_adjust_last_used_sequence)) {
	annotation_average = (g_annotation_base[i][pos1] +
			      g_annotation_base[i][pos2]) / 2.;
	annotation_average_total += annotation_average;
	printf("Sequence %d: pnums for(%d,%d) are %d & %d\n", i-1, pos1, 
	       pos2, g_annotation_base[i][pos1], g_annotation_base[i][pos2]);
	printf("Unused percentage is %.2f\n", 
	       annotation_average/g_annotation_max[i] * 100.);
	printf("Average=%.2f of maximum, %d\n",
	       annotation_average, g_annotation_max[i]);
      }
    }
  }
  if ((g_annotation_flag) && (g_i_row > 0)) {
    annotation_percent =
      annotation_average_total / ((float) g_annotation_max_total);
    printf("pnum average total =%.2f of possible %d\n",
	   annotation_average_total, g_annotation_max_total);
    printf("Color value is %d\n",
	   (int) (40 * annotation_percent));
    printf("Percentage pnum-ann value is %.2f percent\n",
	   annotation_percent * 100);
  }
}

void display_text(void) {
  int             i;
  char            text_message[150];
  sniff_for_opengl_errors();
  glClear(GL_COLOR_BUFFER_BIT);
  display_text_data();
  fixcolor_plot(COLOR_TEXT);
  if (g_message_set == TRUE) {
    glutPopWindow();
    if (g_type_message != MESSAGE_DUMP)
      fixcolor_plot(COLOR_TEXT);
    else
      fixcolor_plot(g_which_dump_seq);
    strcpy(text_message, text_window_message);
    strcat(text_message, "_");
    output_text(10, 5, text_message);
    if (data_entered == TRUE) {
      data_entered = FALSE;
      g_message_set = FALSE;
      if (g_type_message == MESSAGE_ALIGN_PS) {
	finish_making_align_ps();
      }
      if (g_type_message == MESSAGE_COORDINATE) {
				finish_set_zoom_coordinates();
      }
      if (g_type_message == MESSAGE_ALIGN_MAL) {
	finish_making_align_mal(FALSE);
      }
      if (g_type_message == MESSAGE_POST) {
	finish_making_post1();
      }
      if (g_type_message == MESSAGE_LABEL) {
	if (g_type_of_label_file == 0)
	  finish_making_label();
	else {
	  finish_making_label();
	  if (g_type_of_label_file < g_count_sequences) {
	    g_type_of_label_file++;
	    make_zoom_label_file_each(g_type_of_label_file);
	  }
	}
      }
      if (g_type_message == MESSAGE_DUMP) {
	if (g_which_dump_seq == g_adjust_last_used_sequence) {
	  finish_making_dump();
	} else {
	  g_which_dump_seq++;
	  set_message_dump(g_which_dump_seq);
	}
      }
      if (g_type_message == MESSAGE_GIF)
	finish_making_img1(g_type_message);
      if (g_type_message == MESSAGE_PNG)
	finish_making_img1(g_type_message);
      if (g_type_message == MESSAGE_JPG)
	finish_making_img1(g_type_message);
      if (g_type_message == MESSAGE_ENERGY) {
	if (g_which_sequence != 0) {
	  finish_setting_energy_cutoff();	/* only set one */
	  make_overlaps_initialize();
	  if (g_nearby_distance > 0)
	    create_nearby(g_this_sequence, g_nearby_distance);
	  text_window_input[0] = '\0';
	} else {
	  for (i = 2; i <= g_last_used_sequence; i++) {
	    g_which_sequence = i;
	    finish_setting_energy_cutoff();
	  }
	  if (text_window_input[0] != '/') {
	    make_overlaps_initialize();
	    if (g_nearby_distance > 0)
	      create_nearby(g_this_sequence, g_nearby_distance);
	  }
	  text_window_input[0] = '\0';
	}
      }
    }
  } else {
    fixcolor_plot(COLOR_TEXT);
    display_text_filter();
  }
  sniff_for_opengl_errors();
  glutSwapBuffers();
}

void            align_check_make_col_insert(int, int, int);
void            auto_insertion_column(int, int, int);
void            auto_deletion_column(int, int, int);

void mouse_align(int button, int state, int x, int y) {
  int             font_width, first_base_x;
  int             j_col;
  int             number_inserts;
  if (g_font_align == GLUT_BITMAP_HELVETICA_12) {
    font_width = 10;
    first_base_x = 75;
  } else {
    font_width = 15;
    first_base_x = 75;
  }
  j_col = x - first_base_x;
  j_col = (int) (j_col / ((float) font_width));
  j_col = j_col + g_align_start_base;
  if (j_col < 1)
    j_col = 1;
  if (j_col > g_sequence_length)
    j_col = g_sequence_length;
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
      g_i_row = j_col;
      g_j_col = j_col;
      glutSetWindow(win3);
      glutPostRedisplay();
      align_check_make_col_insert(x, y, g_i_row);
    } else {
      if (g_align_col_insert_base_on) {
	g_align_col_insert_base_on = FALSE;
	g_align_col_insert_base_x_offset = 0;
	number_inserts = g_align_col_insert_base_final -
	  g_align_col_insert_base_start;
	if (number_inserts > 0) {
	  auto_insertion_column(g_align_col_insert_sequence, 
				g_align_col_insert_base_start,number_inserts);
	  show_align();
	}
	if (number_inserts < 0) {
	  auto_deletion_column(g_align_col_insert_sequence - 1,
			       g_align_col_insert_base_final,-number_inserts);
	  show_align();
	}
	if (number_inserts == 0) {
	  show_align();
	}
      }
    }
  }
}

void            align_check_update_col_insert(int);

void mouse_motion_align(int x, int y) {
  /* int i_row,j_col; */
  if (g_align_col_insert_base_on) {
    align_check_update_col_insert(x);
    glutSetWindow(win4);
    glutPostRedisplay();
  }
}

int find_same_base_offset(int i, int mov_sequence, int base_offset) {
  char            c,c2;
  int                s;
  if ((i < 1) || (i > g_sequence_length))
    return FALSE;
  if (((i - base_offset) < 1) || ((i - base_offset) > g_sequence_length))
    return FALSE;
  if (mov_sequence > 1)
    c = g_sequence_data[mov_sequence][i - 1 - base_offset];
  else
    c = g_sequence_data[2][i];
  if (c == 'T')
    c = 'U';
  for (s = 2; s <= g_last_used_sequence; s++) {
    if (s != mov_sequence) {
      c2 = g_sequence_data[s][i - 1];
      if (c2 == 'T')
	c2 = 'U';
      if (c != c2)
	return FALSE;
    }
  }
  return TRUE;
}

int fixcolor_base(char c,int i,int sequence,int mov_sequence,int base_offset) {
  int             color;
  int             index;
  int             helix_here;
  int             compensate_offset_for_mov;
  helix_here = g_sequence_helix_used[sequence][g_reverse_st[sequence][i]];
  if (g_base_color_mode) {
    if (!helix_here) {
      color = find_base_color(c);
    } else {
      color = COLOR_HELIX_ALIGN;
    }
  } else {
    if (base_offset == 0) {
      if (helix_here)
	color = COLOR_HELIX_ALIGN;
      else {
	if (g_sequence_data_same[i])
	  color = COLOR_BACK;
	else
	  color = sequence;
      }
    } else {
      if (mov_sequence == sequence)
	compensate_offset_for_mov = base_offset;
      else
	compensate_offset_for_mov = 0;
      if (helix_here)
	color = COLOR_HELIX_ALIGN;
      else {
	if (find_same_base_offset(i + compensate_offset_for_mov, mov_sequence,
				  base_offset))
	  color = COLOR_BACK;
	else
	  color = sequence;
      }
    }
  }
  fixcolor_plot_bad(color);
  if ( (i == g_i_row) || ((i == g_j_col) && (i > 0))) {
    index = index_energy_read(g_i_row, g_j_col, sequence);
    if (index >= 0) {
      if ((g_diag[index].length >= g_chain_len) &&
	  (g_diag[index].energy <= g_energy_cutoff[sequence]))
	return TRUE;
      else
	return FALSE;
    } else
      return FALSE;
  }
  return FALSE;
}

void display_align() {
  int             font_width, i;
  int             x, y;
  int             first_label;
  int             first_base_x;
  int             test;
  int             last_label;
  int             last_base;
  int            *old_font;
  char            c;
  char            name[40];
  int             base;
  char            string[120];
  glClear(GL_COLOR_BUFFER_BIT);
  fixcolor_plot(COLOR_TEXT);
  old_font = g_font_align;
  if (g_font_align == GLUT_BITMAP_HELVETICA_12) {
    font_width = 10;
    first_base_x = 75;
  } else {
    font_width = 15;
    first_base_x = 75;
  }
  g_align_first_base_x = first_base_x;
  g_align_font_width = font_width;
  g_font_align = GLUT_BITMAP_HELVETICA_12;
  for (i = 2; i <= g_last_used_sequence; i++) {
    x = 2;
    y = g_align_height - 10 - i * 20;
    strcpy(name, g_sequence_name[i]);
    name[10] = '\0';
    fixcolor_plot(i);
    output_align(x, y, name);
  }
  g_font_align = old_font;
  fixcolor_plot(COLOR_TEXT);
  if (g_align_start_base < 1) {
    g_align_start_base = g_sequence_length -
      (g_align_width - first_base_x - 15) / font_width + 1;
    if (g_align_start_base < 1)
      g_align_start_base = 1;
  }
  first_label = g_align_start_base - g_align_start_base % 10 + 10;
  last_base = g_align_start_base +
    (g_align_width - first_base_x - 15) / (font_width);
  if (last_base > g_sequence_length)
    last_base = g_sequence_length;
  g_align_end_base = last_base;
  last_label = last_base - last_base % 10;
  /* labels */
  for (i = first_label; i <= last_label; i += 10) {
    strcpy(string, num_string_int(i));
    x = first_base_x + (i - g_align_start_base) * (font_width);
    y = g_align_height - 15;
    output_align(x, y, string);
    glBegin(GL_LINES);
    glVertex2f((float) (x + 5), (float) (y - 5));
    glVertex2f((float) (x + 5), (float) (y - 15));
    glEnd();
  }
  /* display bases */
  for (i = 2; i <= g_last_used_sequence; i++) {
    for (base = g_align_start_base; base <= last_base; base++) {
      y = g_align_height - 10 - i * 20;
      x = first_base_x + (base - g_align_start_base) * (font_width);
      c = g_sequence_data[i][base - 1];
      if ((g_align_col_insert_base_on) &&
	  ((base >= g_align_col_insert_base_start) || 
	   (base >= (g_align_col_insert_base_start +
		     (g_align_col_insert_base_final -
		      g_align_col_insert_base_start))))) {
	test = fixcolor_base(c, base, i, g_align_col_insert_sequence,
			     g_align_col_insert_base_final -
			     g_align_col_insert_base_start);
      } else {
	test = fixcolor_base(c, base, i, g_align_col_insert_sequence, 0);
      }
      if ((i == g_align_col_insert_sequence) &&
	  (g_align_col_insert_base_on) &&
	  (base >= g_align_col_insert_base_start))
	x = x + g_align_col_insert_base_x_offset;
      glBegin(GL_POLYGON);
      glVertex2f((float) x, (float) (y - 2));
      glVertex2f((float) x, (float) (y + 18));
      glVertex2f((float) (x + font_width), (float) (y + 18));
      glVertex2f((float) (x + font_width), (float) (y - 2));
      glEnd();
      if (test) {
	fixcolor_plot(COLOR_LABEL);
	glBegin(GL_LINE_LOOP);
	glVertex2f((float) x, (float) (y - 2));
	glVertex2f((float) x, (float) (y + 18));
	glVertex2f((float) (x + font_width), (float) (y + 18));
	glVertex2f((float) (x + font_width), (float) (y - 2));
	glEnd();
      }
      fixcolor_plot(COLOR_TEXT_BACKGROUND);
      output_align_char(x, y, c);
    }
  }
  if (!g_col_insert_base_on) {
    fixcolor_plot(COLOR_LABEL);
    glLineWidth(2.0);
    if (g_i_row > 0) {
      if ((g_i_row > 0) && (g_i_row >= g_align_start_base) &&
	  (g_i_row <= g_align_end_base)) {
	x = first_base_x +
	  (g_i_row - g_align_start_base) * (font_width) + 1;
	y = g_align_height - 15;
	glBegin(GL_LINES);
	glVertex2f((float) (x + 5), (float) (y - 5));
	glVertex2f((float) (x + 5), (float) (y - 15));
	glEnd();
	y = g_align_height - 12 - g_last_used_sequence * 20;
	glBegin(GL_LINES);
	glVertex2f((float) (x + 5), (float) (y - 5));
	glVertex2f((float) (x + 5), (float) (y - 15));
	glEnd();
      }
      if ((g_j_col > 0) && (g_j_col >= g_align_start_base) &&
	  (g_j_col <= g_align_end_base)) {
	y = g_align_height - 15;
	x = first_base_x +
	  (g_j_col - g_align_start_base) * (font_width) + 1;
	glBegin(GL_LINES);
	glVertex2f((float) (x + 5), (float) (y - 5));
	glVertex2f((float) (x + 5), (float) (y - 15));
	glEnd();
	y = g_align_height - 12 - g_last_used_sequence * 20;
	glBegin(GL_LINES);
	glVertex2f((float) (x + 5), (float) (y - 5));
	glVertex2f((float) (x + 5), (float) (y - 15));
	glEnd();
      }
    }
  }
  glutSwapBuffers();
  sniff_for_opengl_errors();
}

void display_main(void) {
  float           min_point_size;
  int             i;
  char            location[60];
  float           line_t, line_l, line_r, line_b;
  float           point_size;	/* size of points to plot within box */
  glClear(GL_COLOR_BUFFER_BIT);
  glLineWidth(1.0);
  fixcolor_plot(COLOR_TEXT);
  plot_tics();
  glLineWidth(2.0);
  glBegin(GL_LINE_LOOP);	/* draw big box */
  glVertex2f(21.0, 752.0);
  glVertex2f(21.0, 46.0);
  glVertex2f(726.0, 46.0);
  glVertex2f(726.0, 752.0);
  glEnd();
  glLineWidth(1.0);
  line_l = plot_j_float(.5);
  line_r = plot_j_float(g_sequence_length + .5);
  line_t = plot_i_float(.5);
  line_b = plot_i_float(g_sequence_length + .5);
  glBegin(GL_LINES);	/* draw triangle */
  glVertex2f(line_l, line_t);
  glVertex2f(line_r, line_b);
  glEnd();
  glLineWidth(2.0);
  point_size = 700./(g_sequence_length + 1)*width/800. * height / 800. + .4;
  if (g_bigdot_mode)
    min_point_size = 2.0;
  else
    min_point_size = 1.0;
  if (point_size < min_point_size)
    point_size = min_point_size;
  glPointSize(point_size);
  plot_points();
  fixcolor_plot(COLOR_TEXT);
  if (g_unique_mode) {
    strcpy(location, "Unique Mode");
    output_main(680, 5, location);
  }
  if (g_comp_flag) {
    fixcolor_plot(COLOR_COMP);
    strcpy(location, "Compensating: ");
    strcat(location, num_string_int(g_comp));
    output_main(10, 5, location);
    fixcolor_plot(COLOR_TEXT);
  }
  strcpy(location, "Base Pairs:");
  output_main(10, 23, location);
  for (i = 0; i <= g_last_used_sequence; i++) {
    if ((i != 1) || (g_count_sequences > 2)) {
      fixcolor_plot(i);
      strcpy(location, num_string_int(g_points_plotted_ind[i]));
      output_main(125 + i * 100, 23, location);
    }
  }
  if (g_nearby_distance > 0) {
    fixcolor_plot(COLOR_CLOSE);
    strcpy(location, num_string_int(g_count_nearby));
    strcat(location, " Close within ");
    strcat(location, num_string_int(g_nearby_distance));
    strcat(location, " of ");
    output_main(300, 5, location);
    strcpy(location, g_sequence_name[g_primary_sequence]);
    fixcolor_plot(g_primary_sequence);
    output_main(535, 5, location);
  }
  if (g_region_set == TRUE || (g_region_exists == TRUE)) {
    /* draw zoom */
    fixcolor_plot(COLOR_LABEL);
    glBegin(GL_LINE_LOOP);
    glVertex2f(g_region_real_l, g_region_real_t);
    glVertex2f(g_region_real_r, g_region_real_t);
    glVertex2f(g_region_real_r, g_region_real_b);
    glVertex2f(g_region_real_l, g_region_real_b);
    glEnd();
  }
  glutSwapBuffers();
  sniff_for_opengl_errors();
}

void display_zoom_bases_hor(void) {
  int             i, sequence;
  char            c;
  float           x, y = 0.0;
  float           width;
  int             hor_offset_insert;
  int             start_base = 1;
  int             mov_sequence = 1;
  int             extra;
  int             test;
  float           x_box;
  float           character_height;
  int             base_offset;
  extra = FALSE;
  base_offset = 0;
  hor_offset_insert = 0;
  character_height = 13;
  glLineWidth(1.0);
  if (g_col_insert_base_on) {
    hor_offset_insert = g_col_insert_base_x_offset;
    mov_sequence = g_col_insert_sequence;
    start_base = g_col_insert_base_start;
    base_offset = g_col_insert_base_final - g_col_insert_base_start;
    extra = TRUE;
  }
  if (g_row_insert_base_on) {
    hor_offset_insert = -g_row_insert_base_y_offset * 
      ((float) (g_display_h + 1.0) / (float) (g_display_w + 1));
    mov_sequence = g_row_insert_sequence;
    start_base = g_row_insert_base_start;
    base_offset = g_row_insert_base_final - g_row_insert_base_start;;
    extra = TRUE;
  }
  x = 15.;
  glLineWidth(2.0);
  width = (float) 700. / (g_display_w + 1) / 2;
  for (sequence = 2; sequence <= g_last_used_sequence; sequence++) {
    y = 35 - (sequence - 2) * 12;
    fixcolor_plot(sequence);
    glRasterPos2f(x, y);
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, '>');
    glRasterPos2f(5, y);
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, '0' + sequence - 1);
  }
  for (i = g_display_l; i <= g_display_r; i++) {
    x = 21. + (float) (i - g_display_l + 1) * 700. / (g_display_w + 1);
    if ((i == g_j_col) && (g_col_line)) {
      fixcolor_plot(COLOR_LABEL);
      glBegin(GL_POLYGON);
      glVertex2f(x - 1.5 + 4, 35 - (0 - 2) * 12 - 2);
      glVertex2f(x + 1 + 4, 35 - (0 - 2) * 12 - 2);
      glVertex2f(x + 1. + 4, 35 - (0 - 1) * 12 - 1);
      glVertex2f(x - 1.5 + 4, 35 - (0 - 1) * 12 - 1);
      glEnd();
    }
    for (sequence = 2; sequence <= g_last_used_sequence; sequence++) {
      y = 35 - (sequence - 2) * 12;
      c = g_sequence_data[sequence][i - 1];
      if ((extra) && ((i >= start_base) || (i >= start_base + base_offset))) {
	test = fixcolor_base(c, i, sequence, mov_sequence, base_offset);
      } else {
	test = fixcolor_base(c, i, sequence, mov_sequence, 0);
      }
      /* draw box here */
      if ((extra) && (sequence == mov_sequence) && (i >= start_base))
	x_box = x + hor_offset_insert;
      else
	x_box = x;
      glBegin(GL_POLYGON);
      glVertex2f(x_box - width + 2.5, y - 2.5);
      glVertex2f(x_box + width + 2.8, y - 2.5);
      glVertex2f(x_box + width + 2.8, y + character_height - 3.2);
      glVertex2f(x_box - width + 2.5, y + character_height - 3.2);
      glEnd();
      if (test && ((i == g_j_col)) && (g_col_line)) {
	fixcolor_plot(COLOR_LABEL);
	glBegin(GL_LINE_LOOP);
	glVertex2f(x_box - width + 2.5, y - 2.5);
	glVertex2f(x_box + width + 2.8, y - 2.5);
	glVertex2f(x_box + width + 2.8, y + character_height - 3.2);
	glVertex2f(x_box - width + 2.5, y + character_height - 3.2);
	glEnd();
      }
      fixcolor_plot(COLOR_TEXT_BACKGROUND);
      if ((extra) && (sequence == mov_sequence) && (i >= start_base)) {
	glRasterPos2f(x + hor_offset_insert, y);
      } else {
	glRasterPos2f(x, y);
      }
      if ((test != 0) || (g_display_w > 80) || (width_z < 650))
	glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_10, c);
      else
	glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, c);
    }
    if (g_base_color_mode) {
      if (((hor_offset_insert == 0) && g_sequence_data_same[i]) ||
	  ((hor_offset_insert != 0) && ((i < start_base) && 
					g_sequence_data_same[i])) ||
	  (((i >= start_base) || (i >= start_base + base_offset)) &&
	   (find_same_base_offset(i, mov_sequence, base_offset)))) {
	fixcolor_plot(0);
	glBegin(GL_LINES);
	glVertex2f(x - width + 4, y - 3);
	glVertex2f(x + width + 4, y - 3);
	glEnd();
      }
    }
  }
}

void display_zoom_bases_ver(void) {
  int             i, sequence;
  char            c;
  float           x, y = 0.0;
  float           hor_offset;
  float           width;
  float           y_box;
  int             ver_offset_insert;
  int             start_base = 1;
  int             mov_sequence = 1;
  int             test;
  int             extra;
  int             base_offset;
  base_offset = 0;
  extra = FALSE;
  glLineWidth(1.0);
  ver_offset_insert = 0;
  if (g_row_insert_base_on) {
    ver_offset_insert = g_row_insert_base_y_offset;
    mov_sequence = g_row_insert_sequence;
    start_base = g_row_insert_base_start;
    extra = TRUE;
    base_offset = g_row_insert_base_final - g_row_insert_base_start;
  }
  if (g_col_insert_base_on) {
    ver_offset_insert = g_col_insert_base_x_offset * -1.0 *
      ((float) (g_display_w + 1.0) / (float) (g_display_h + 1));
    mov_sequence = g_col_insert_sequence;
    start_base = g_col_insert_base_start;
    base_offset = g_col_insert_base_final - g_col_insert_base_start;;
    extra = TRUE;
  }
  width = (float) (700. / (g_display_h + 1) / 2.0);
  glLineWidth(2.0);
  if (g_count_sequences < 3)
    hor_offset = 0;
  else
    hor_offset = 1 - (g_count_sequences - 2) * 12;
  for (sequence = 2; sequence <= g_last_used_sequence; sequence++) {
    x = hor_offset + (g_count_sequences - (sequence - 1)) * 10 + 2;
    y = 758;
    fixcolor_plot(sequence);
    glRasterPos2f(x, y - 13);
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, '^');
    glRasterPos2f(x, y);
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, '0' + sequence - 1);
  }
  for (i = g_display_t; i <= g_display_b; i++) {
    y = 46 + (float) (g_display_b - i + 1) * 700. / (g_display_h + 1);
    if (((i == g_i_row) && (g_row_line))) {
      fixcolor_plot(COLOR_LABEL);
      glBegin(GL_POLYGON);
      glVertex2f(hor_offset + g_last_used_sequence * 10 - 9, y + 2.0 + 2);
      glVertex2f(hor_offset + g_last_used_sequence * 10, y + 2.0 + 2);
      glVertex2f(hor_offset + g_last_used_sequence * 10, y - 1 + 2);
      glVertex2f(hor_offset + g_last_used_sequence * 10 - 9, y - 1 + 2);
      glEnd();
    }
    for (sequence = 2; sequence <= g_last_used_sequence; sequence++) {
      x = hor_offset + (g_count_sequences - (sequence - 1)) * 10;
      c = g_sequence_data[sequence][i - 1];
      if ((extra) && ((i >= start_base) || (i >= start_base + base_offset)))
	test = fixcolor_base(c, i, sequence, mov_sequence, base_offset);
      else
	test = fixcolor_base(c, i, sequence, mov_sequence, 0);
      /* draw box here */
      if ((extra) && (sequence == mov_sequence) && (i >= start_base)) {
	y_box = y + ver_offset_insert;
      } else {
	y_box = y;
      }
      glBegin(GL_POLYGON);
      glVertex2f(x - 5 + 4, y_box - width + 3.8);
      glVertex2f(x - 5 + 4, y_box + width + 3.5);
      glVertex2f(x + 5 + 4, y_box + width + 3.5);
      glVertex2f(x + 5 + 4, y_box - width + 3.8);
      glEnd();
      if (test && ((i == g_i_row) && (g_row_line))) {
	fixcolor_plot(COLOR_LABEL);
	glBegin(GL_LINE_LOOP);
	glVertex2f(x - 5 + 4, y_box - width + 3.8);
	glVertex2f(x - 5 + 4, y_box + width + 3.5);
	glVertex2f(x + 5 + 4, y_box + width + 3.5);
	glVertex2f(x + 5 + 4, y_box - width + 3.8);
	glEnd();
      }
      fixcolor_plot(COLOR_TEXT_BACKGROUND);
      if ((extra) && (sequence == mov_sequence) && (i >= start_base)) {
	glRasterPos2f(x, y + ver_offset_insert);
      } else {
	glRasterPos2f(x, y);
      }
      if ((test) || (g_display_h > 70) || (height_z < 700))
	glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_10, c);
      else
	glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, c);
    }
    if (g_base_color_mode) {
      if (((ver_offset_insert == 0) && g_sequence_data_same[i]) ||
	  ((ver_offset_insert != 0) && ((i < start_base) && 
					g_sequence_data_same[i])) ||
	  (((i >= start_base) || (i >= start_base + base_offset)) &&
	   (find_same_base_offset(i, mov_sequence, base_offset)))) {
	fixcolor_plot(0);
	x = hor_offset - 2;
	glBegin(GL_LINES);
	glVertex2f(x, y - width + 5.0);
	glVertex2f(x, y + width + 5.0);
	glEnd();
      }
    }
  }
}

void display_zoom(void) {
  float           min_point_size;
  void           *temp_font;
  float           point_size;
  char            location[60];
  int             right = 750, bottom = 750, top = 1, left = 1, i;
  int             y_location;
  int             stripe_flag;
  /* This is false unless there are more than */
  /* 2 sequences and the image is zoomed enought to show multiple */
  /* squares at each spot */
  glClear(GL_COLOR_BUFFER_BIT);
  fixcolor_plot(COLOR_TEXT);
  glLineWidth(1.0);
  plot_tics_zoom();
  point_size=700./((g_display_w+g_display_h)/2)*width_z/800.* height_z/800.+.60;
  if (g_bigdot_mode)
    min_point_size = 2.0;
  else
    min_point_size = 1.0;
  if ((g_adjust_count_sequences > 2) &&
      (point_size > (STRIPE_SCREEN * g_count_sequences))) {
    stripe_flag = TRUE;
  } else
    stripe_flag = FALSE;
  g_ps_multicolor_dots_flag = stripe_flag;
  if (point_size < min_point_size)
    point_size = min_point_size;
  glPointSize(point_size);
  g_zoom_comp = 0;
  plot_points_zoom(point_size, stripe_flag);
  glLineWidth(2.0);
  fixcolor_plot(g_this_sequence);
  glBegin(GL_LINE_LOOP);
  glVertex2f(22.0, 751.0);
  glVertex2f(22.0, 48.0);
  glVertex2f(724.0, 48.0);
  glVertex2f(724.0, 751.0);
  glEnd();
  fixcolor_plot(COLOR_TEXT);
  glLineWidth(1.0);
  /*
   * if(g_number_inserts>1) { strcpy(location,"Ins=");
   * strcat(location,num_string_int(g_number_inserts));
   * output_zoom(740,782,location); }
   */
  strcpy(location, "Base Pairs: ");
  strcat(location, num_string_int(g_points_plotted_zoom));
  strcat(location, "  Common Base Pairs: ");
  strcat(location, num_string_int(g_points_plotted_zoom_all));
  if (g_count_sequences < 3)
    y_location = 3;
  else
    y_location = 5 - (g_count_sequences - 2) * 15;
  output_zoom(10, y_location, location);
  fixcolor_plot(COLOR_COMP);
  if (g_comp_flag) {
    strcpy(location, "   Compensate: ");
    strcat(location, num_string_int(g_zoom_comp));
    output_zoom(580, y_location, location);
  }
  if (g_neighbor_distance > 0) {
    fixcolor_plot(COLOR_LABEL);
    strcpy(location, "N. Within ");
    strcat(location, num_string_int(g_neighbor_distance));
    output_zoom(450, y_location, location);
  }
  if (g_nearby_distance > 0) {
    fixcolor_plot(COLOR_CLOSE);
    strcpy(location, "C. within ");
    strcat(location, num_string_int(g_nearby_distance));
    output_zoom(450, y_location, location);
  }
  fixcolor_plot(COLOR_TEXT);
  /* draw diagional line to indicate no points below */
  if ((g_display_r >= g_display_t) && (g_display_b >= g_display_l)) {
    if (g_display_r > g_display_b) {
      bottom = g_display_b;
      right = g_display_b;
    } else {
      right = g_display_r;
      bottom = g_display_r;
    }
    if (g_display_l > g_display_t) {
      left = g_display_l;
      top = g_display_l;
    } else {
      top = g_display_t;
      left = top;
    }
    glBegin(GL_LINES);
    glVertex2f(plot_j_zoom(left - 1), plot_i_zoom(top - 1));
    glVertex2f(plot_j_zoom(right + 1), plot_i_zoom(bottom + 1));
    glEnd();
  }
  fixcolor_plot(COLOR_LABEL);
  if (g_region_set) {/* draw zoom box */
    fixcolor_plot(COLOR_LABEL);
    glBegin(GL_LINE_LOOP);
    glVertex2f(g_region_real_l_zoom, g_region_real_t_zoom);
    glVertex2f(g_region_real_l_zoom, g_region_real_b_zoom);
    glVertex2f(g_region_real_r_zoom, g_region_real_b_zoom);
    glVertex2f(g_region_real_r_zoom, g_region_real_t_zoom);
    glEnd();
  }
  /* draw the labels */
  if (g_zoom_labels_count > 0) {
    fixcolor_plot(COLOR_LABEL);
    temp_font = g_font_zoom;
    g_font_zoom = GLUT_BITMAP_TIMES_ROMAN_10;
    glEnable(GL_POINT_SMOOTH);
    for (i = 0; i < g_zoom_labels_count; i++) {
      output_zoom(g_zoom_labels_x[i], g_zoom_labels_y[i],
		  g_zoom_labels_string[i]);
    }
    point_size=700./((g_display_w+g_display_h)/2)*width_z/800.*height_z/800.+.5;
    point_size = point_size * .5;
    glPointSize(point_size);
    glEnable(GL_POINT_SMOOTH);
    glBegin(GL_POINTS);
    for (i = 0; i < g_zoom_labels_count; i++) {
      glVertex2f((float) g_zoom_labels_xl[i],
		 (float) g_zoom_labels_y[i]);
    }
    glEnd();
    glDisable(GL_POINT_SMOOTH);
    g_font_zoom = temp_font;
  }
  if (g_display_h < 150) {
    display_zoom_bases_ver();
  }
  if (g_display_w < 150)
    display_zoom_bases_hor();
  sniff_for_opengl_errors();
  glutSwapBuffers();
}

void text_reshape(int w, int h) {
  if (w > 1200) {
    w = 1100;
    glutReshapeWindow(w, h);
  }
  if (h > 300) {
    h = 300;
    glutReshapeWindow(w, h);
  }
  if (w > 500)
    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  else
    glViewport(0, 0, (GLsizei) 800, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, 880.0, 0.0, 200.0, -1.0, 1.0);
  glMatrixMode(GL_MODELVIEW);
  if (w < 850)
    select_font_text(0);
  else
    select_font_text(1);
}

void reshape(int w, int h) {
  int             current_window;
  int             font;
  float           offset;
  if (w > h) {
    h = w;
    glutReshapeWindow(w, h);
    glutPostRedisplay();
  }
  if (h > w) {
    w = h;
    glutReshapeWindow(w, h);
    glutPostRedisplay();
  }
  if (w > 990) {
    w = 990;
    h = 990;
    glutReshapeWindow(w, h);
  }
  current_window = glutGetWindow();
  if (current_window == win1) {
    if (w < h)
      h = w;
    else
      w = h;
  }
  if (current_window == win1) {
    width = w;
    height = h;
  } else {
    width_z = w;
    height_z = h;
  }
  if (w < 700)
    font = 0;
  else {
    if (w < 900)
      font = 1;
    else
      font = 2;
  }
  if (current_window == win1)
    select_font_main(font);
  else
    select_font_zoom(font);
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  if (current_window == win1) {
    glOrtho(0, 800.0, 0.0, 800.0, -1.0, 1.0);	
    /* 0,0 is origin for drawing
     * lower left is 0,0   up is y, right is x */
  } else {
    if (g_count_sequences < 3)
      offset = -4.0;
    else {
      offset = (g_count_sequences - 2) * -15.;
    }
    glOrtho(offset, 800, offset, 800 - offset, -1.0, 1.0);
  }
  sniff_for_opengl_errors();
  glMatrixMode(GL_MODELVIEW);
}

void reshape_align(int w, int h) {
  if (h != g_align_height) {
    h = g_align_height;
    glutReshapeWindow(w, h);
    glutPostRedisplay();
  }
  g_align_width = w;
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, (float) w, 0, (float) h, -1.0, 1.0);
  /* 0,0 is origin for drawing
   * lower left is 0,0   up is y, right is x */
  sniff_for_opengl_errors();
  glMatrixMode(GL_MODELVIEW);
}

/* Functions to set zoom rectangle */

void fix_zoom_box_zoom(void) {
  float           left, right, top, bottom;
  g_region_real_l = 25.0 + (float) g_region_l *700. / g_sequence_length;
  g_region_real_r = 25.0 + (float) g_region_r *700. /  g_sequence_length;
  g_region_real_b = 50.0 + (float) (g_sequence_length - g_region_b) *
    700. / g_sequence_length;
  g_region_real_t = 50.0 + (float) (g_sequence_length - g_region_t) *
    700 / g_sequence_length;
  left = 25.4 + (float) (g_region_l-g_display_l+1)*700./(g_display_w + 1);
  right = 25.4 + (float) (g_region_r-g_display_l+1)*700./(g_display_w + 1);
  top = 50. + (float) (g_display_b-g_region_t+1)*700./(g_display_h + 1);
  bottom = 50. + (float) (g_display_b-g_region_b+1)*700./(g_display_h + 1);
  if (left < 0.) {
    left = 0.;
  };
  if (right < 0.) {
    right = 0.;
  }
  if (right > 799.) {
    right = 800.;
  }
  if (left > 799.) {
    left = 799.;
  }
  if (top < 0) {
    top = 0;
  }
  if (top > 799.) {
    top = 799.;
  }
  if (bottom < 0.) {
    bottom = 0.;
  }
  if (bottom > 799.) {
    bottom = 799.;
  }
  g_region_real_l_zoom = left;
  g_region_real_r_zoom = right;
  g_region_real_t_zoom = top;
  g_region_real_b_zoom = bottom;
}

void extra_finish_fixing_zoom_box_zoom(void) {
  int             temp;	/* make left less than right */
  if (g_region_r < g_region_l) {
    temp = g_region_r;
    g_region_r = g_region_l;
    g_region_l = temp;
  }
  if (g_region_b < g_region_t) {	/* make top less than bottom */
    temp = g_region_t;
    g_region_t = g_region_b;
    g_region_b = temp;
  }
  fix_zoom_box_zoom();
}

void finish_fixing_zoom_box_zoom(void) {
  int             temp;	/* make left less than right */
  if (g_region_r < g_region_l) {
    temp = g_region_r;
    g_region_r = g_region_l;
    g_region_l = temp;
  }
  if (g_region_b < g_region_t) {  /* make top less than bottom */
    temp = g_region_t;
    g_region_t = g_region_b;
    g_region_b = temp;
  }
  g_saved_display_l = g_display_l;
  g_saved_display_r = g_display_r;
  g_saved_display_t = g_display_t;
  g_saved_display_b = g_display_b;
  g_display_l = g_region_l;
  g_display_r = g_region_r;
  g_display_t = g_region_t;
  g_display_b = g_region_b;
  g_display_w = g_display_r - g_display_l + 1;
  g_display_h = g_display_b - g_display_t + 1;
  fix_zoom_box_zoom();
}

/* Mouse Functions ___________ */

void set_helix_use_row_column(int row, int col) {
  int             index_for_helix, sequence;
  int             pos;
  int             temp;
  int             length, start_row, start_column;
  if (row > col) {
    temp = row;
    row = col;
    col = temp;
  }
  for (sequence = 2; sequence <= g_last_used_sequence; sequence++) {
    index_for_helix = index_energy_read(row, col, sequence);
    if (index_for_helix >= 0) {
      length = g_sequence_helix_used_length[sequence];
      if (length > 0) {
	start_row = g_sequence_helix_used_row[sequence];
	start_column = g_sequence_helix_used_column[sequence];
	for (pos = start_row; pos <= (start_row + length - 1); pos++)
	  g_sequence_helix_used[sequence][pos] = FALSE;
	for (pos = start_column; pos >= (start_column - length + 1); pos--)
	  g_sequence_helix_used[sequence][pos] = FALSE;
      }
      length = g_diag[index_for_helix].length;
      start_row =
	g_reverse_st[sequence][g_diag[index_for_helix].row];
      start_column =
	g_reverse_st[sequence][g_diag[index_for_helix].column];
      for (pos = start_row; pos <= (start_row + length - 1); pos++) {
	g_sequence_helix_used[sequence][pos] = TRUE;
      }
      for (pos = start_column; pos >= (start_column - length + 1); pos--) {
	g_sequence_helix_used[sequence][pos] = TRUE;
      }
      g_sequence_helix_used_length[sequence] = length;
      g_sequence_helix_used_row[sequence] = start_row;
      g_sequence_helix_used_column[sequence] = start_column;
    }
  }
}

void mouse(int button, int state, int x, int y) {
  int             i_row, j_col;
  j_col = window_x_convert(x);
  i_row = window_y_convert(y);
  if (i_row < 1) {
    i_row = 1;
  }
  if (i_row > g_sequence_length)
    i_row = g_sequence_length;
  if (j_col < 1)
    j_col = 1;
  if (j_col > g_sequence_length)
    j_col = g_sequence_length;
  if ((button == GLUT_LEFT_BUTTON) || (button == GLUT_MIDDLE_BUTTON)) {
    if (state == GLUT_DOWN) {
      g_mouse_motion = FALSE;
      g_region_l = j_col;
      g_region_t = i_row;
      g_region_b = i_row;
      g_region_r = j_col;
      g_region_set = TRUE;
      fix_zoom_box();
    } else {
      if (g_mouse_motion) {
	g_mouse_motion = FALSE;
	g_region_set = FALSE;
	g_region_exists = TRUE;
	finish_fixing_zoom_box();
	g_zoom_labels_count = 0;
	glutSetWindow(win2);
	glutShowWindow();
	glutPopWindow();
	g_zoom_window_exists = TRUE;
	glutPostRedisplay();
      } else {
	g_region_set = FALSE;
	g_region_exists = FALSE;
	g_i_row = i_row;
	g_j_col = j_col;
	set_helix_use_row_column(g_i_row, g_j_col);
	glutSetWindow(win3);
	glutPostRedisplay();
	show_align();
      }
    }
  }
}

void mouse_motion(int x, int y) {
  /* returns second corner of zoom rectangle as mouse is moved */
  int             i_row, j_col;
  if (g_region_set == TRUE) {
    j_col = window_x_convert(x);
    i_row = window_y_convert(y);
    if ((j_col != g_region_l) && (i_row != g_region_t))
      g_mouse_motion = TRUE;
    if (i_row < 1) {
      i_row = 1;
    }
    if (j_col > g_sequence_length)
      j_col = g_sequence_length;
    if (i_row > g_sequence_length)
      i_row = g_sequence_length;
    if (j_col < 1)
      j_col = 1;
    g_region_r = j_col;
    g_region_b = i_row;
    fix_zoom_box();
    glutSetWindow(win1);
    glutPostRedisplay();
  }
}

void check_display_zoom_labels(int i_row, int j_col) {
  int             i, place;
  int             label;
  int             row, col;
  if (i_row > j_col) {
    row = j_col;
    col = i_row;
  } else {
    row = i_row;
    col = j_col;
  }
  if ((!g_col_line) || (!g_row_line))
    return;
  for (label = 0;
       label < g_zoom_labels_count;
       label++) {
    if ((g_zoom_labels_row[label] == i_row) &&
	(g_zoom_labels_col[label] == j_col)) {
      g_zoom_labels_count--;
      if (g_zoom_labels_count > 0) {
	g_zoom_labels_row[label] =
	  g_zoom_labels_row[g_zoom_labels_count];
	g_zoom_labels_col[label] =
	  g_zoom_labels_col[g_zoom_labels_count];
	strcpy(g_zoom_labels_string[label],
	       g_zoom_labels_string[g_zoom_labels_count]);
	g_zoom_labels_x[label] =
	  g_zoom_labels_x[g_zoom_labels_count];
	g_zoom_labels_xl[label] =
	  g_zoom_labels_xl[g_zoom_labels_count];
	g_zoom_labels_y[label] =
	  g_zoom_labels_y[g_zoom_labels_count];
      }
      return;
    }
  }
  for (i = 2; i <= g_last_used_sequence; i++) {
    if (energy_read(row, col, i)
	<= g_energy_cutoff[i]) {
      /* label the point if any energies are in range */
      if (g_zoom_labels_count < 49)
	g_zoom_labels_count++;
      place = g_zoom_labels_count - 1;
      strcpy(rec, "<");
      strcat(rec, num_string_int(row));
      strcat(rec, ",");
      strcat(rec, num_string_int(col));
      strcat(rec, ">");
      strcpy(g_zoom_labels_string[place], rec);
      g_zoom_labels_x[place] =
	plot_j_zoom(j_col + 1);
      g_zoom_labels_xl[place] =
	plot_j_zoom(j_col);
      g_zoom_labels_y[place] =
	plot_i_zoom(i_row);
      g_zoom_labels_row[place] = i_row;
      g_zoom_labels_col[place] = j_col;
      glutSetWindow(win2);
      glutPostRedisplay();
      return;	/* only label the point once */
    }
  }
}

void check_to_drag_helix(int row, int column) {
  int             index;
  int             index2[TOTAL_SEQUENCES];
  int             sequence;
  int             found_sequences;
  int             the_found_sequence = 0;
  int             temp;
  int             gl_display_b, gl_display_t, gl_display_r, gl_display_l; 
  if (g_adjust_last_used_sequence != g_last_used_sequence) {
    printf("No helix dragging with view fewer sequences option\n");
    return;
  }
  if (row > column) {
    g_drag_helix_below_diagonal = TRUE;
    temp = row;
    row = column;
    column = temp;
  } else
    g_drag_helix_below_diagonal = FALSE;
  if (!g_dot_lower_flag) {
    gl_display_b = g_display_b;
    gl_display_t = g_display_t;
    gl_display_r = g_display_r;
    gl_display_l = g_display_l;
  } else {
    gl_display_b = g_display_r;
    gl_display_t = g_display_l;
    gl_display_r = g_display_b;
    gl_display_l = g_display_t;
  }
  if ((row > gl_display_b) || (row < gl_display_t)) {
    g_drag_helix_set = FALSE;
    return;
  }
  if ((column < gl_display_l) || (column > gl_display_r)) {
    g_drag_helix_set = FALSE;
    return;
  }
  if (g_this_sequence == g_primary_sequence) {
    g_drag_helix_set = FALSE;
    return;
  }
  index = index_energy_read(row, column, g_this_sequence);
  /* if the sequence has a dot here, define the helix to be the 
   * remainder of the length */
  if ((index >= 0) &&
      (g_diag[index].energy <= g_energy_cutoff[g_this_sequence])) {
    g_drag_helix_sel_row = row;
    g_drag_helix_sel_col = column;
    g_drag_helix_set = TRUE;
    g_drag_helix_sel_length = g_diag[index].length - row + g_diag[index].row;
    g_drag_helix_front_length = g_diag[index].length - g_drag_helix_sel_length;
  } else {		/* check_other_non_primary_sequences */
    if (g_last_used_sequence < 4)
      return;
    found_sequences = 0;
    for (sequence = 2; sequence <= g_last_used_sequence; sequence++) {
      if ((sequence != g_primary_sequence) && (sequence != g_this_sequence)) {
	index2[sequence] = index_energy_read(row, column, sequence);
	if ((index2[sequence] >= 0) &&
	    (g_diag[index2[sequence]].energy <= g_energy_cutoff[sequence])) {
	  found_sequences++;
	  the_found_sequence = sequence;
	}
      }
    }
    if (found_sequences == 1) {
      index = index2[the_found_sequence];
      g_this_sequence = the_found_sequence;
      g_drag_helix_sel_row = row;
      g_drag_helix_sel_col = column;
      g_drag_helix_set = TRUE;
      g_drag_helix_sel_length = g_diag[index].length - row + g_diag[index].row;
      g_drag_helix_front_length = g_diag[index].length-g_drag_helix_sel_length;
      glutSetWindow(win3);
      glutPostRedisplay();
    }
  }
}

int window_y_convert_zoom_sequence(int y) {
  /* convert window y coordinates to a sequence shown below box */
  float           yf;	
  float           offset_to_window;
  int             sequence;
  if (g_count_sequences < 3)
    offset_to_window = 0;
  else
    offset_to_window = (g_count_sequences - 2) * 15;
  yf = y - (50. + offset_to_window) * height_z / (800 + 2 * offset_to_window);
  /* subtract offset down depending on window
   * should be at top of box at this stage */
  yf = yf - 700 * height_z / (800. + 2 * offset_to_window);
  /* should be at bottom of box at this stage */
  if (yf < 0)
    return 0;
  yf = yf - 6 * height_z / (800. + 2 * offset_to_window);
  /* center of first sequence */
  yf = yf - 3.0;
  yf = yf / (12.0 * height_z / (800. + 2 * offset_to_window));
  sequence = (int) (yf + .5) + 2;
  if (sequence > g_last_used_sequence)
    return -1;
  return sequence;
}

int window_x_convert_zoom_sequence(int x) {
  /* convert window y coordinates to */
  float           offset_to_window;
  float           xf;
  int             sequence;
  if (g_count_sequences < 3)
    offset_to_window = -4;
  else
    offset_to_window = (g_count_sequences - 2) * -15;
  xf = (x - 2) * (800. - 2 * offset_to_window) / width_z + offset_to_window;
  if (g_count_sequences < 3)
    sequence = (int) ((xf + 17.) / 10. + .5);
  else
    sequence = (int) ((xf - 8 + 12. * g_count_sequences) / 10.0 + .5);
  sequence = 2 + g_last_used_sequence - sequence;
  if (sequence > g_last_used_sequence)
    return -1;
  if (sequence < 1)
    return -1;
  return sequence;
}

void check_make_row_insert(int x, int y) {
  int             row, sequence;
  /* convert x coordinate to a column  */
  row = window_y_convert_zoom(y);
  if ((row < g_display_t) || (row > g_display_b))
    return;
  /* convert y coordinate to sequence */
  sequence = window_x_convert_zoom_sequence(x);
  if ((sequence < 2) || (sequence > g_last_used_sequence))
    return;
  g_row_insert_base_on = TRUE;
  g_row_insert_base_start = row;
  g_row_insert_base_final = row;
  g_row_insert_base_y_start = y;
  g_row_insert_base_y_start_true = 50. + (float) (g_display_b - row + 1) *
    700. / (g_display_h + 1);
  g_row_insert_col_start_x = 25. + (float) (row - g_display_l + 1)
    * 700. / (g_display_w + 1);
  g_row_insert_base_y_offset = 0;
  g_row_insert_sequence = sequence;
  g_row_insert_base_y_safe = y;
  g_row_insert_base_safe = row;
}

void check_update_row_insert(int y) {
  int             row, offset_to_window;
  row = window_y_convert_zoom(y);
  if (row > g_display_b)
    row = g_display_b;
  if (row < g_display_t)
    row = g_display_t;
  if (row < g_row_insert_base_start) {
    if (g_sequence_data[g_row_insert_sequence][row - 1] != '.') {
      y = g_row_insert_base_y_safe;
      row = g_row_insert_base_safe;
    }
  }
  g_row_insert_base_y_offset = (g_row_insert_base_y_start - y);
  if (g_count_sequences < 3)
    offset_to_window = -4.0;
  else
    offset_to_window = (g_count_sequences - 2) * -15.;
  g_row_insert_base_y_offset =
    g_row_insert_base_y_offset * ((800. - 2 * offset_to_window) / height_z);
  g_row_insert_base_final = row;
  g_row_insert_base_y_safe = y;
  g_row_insert_base_safe = row;
}

void check_make_col_insert(int x, int y) {
  int             column, sequence;
  /* convert x coordinate to column */
  column = window_x_convert_zoom(x);
  if ((column < g_display_l) || (column > g_display_r))
    return;
  /* convert y coordinate to sequence */
  sequence = window_y_convert_zoom_sequence(y);
  if ((sequence < 2) || (sequence > g_last_used_sequence))
    return;
  g_col_insert_base_on = TRUE;
  g_col_insert_base_start = column;
  g_col_insert_base_final = column;
  g_col_insert_base_x_start = x;
  g_col_insert_base_x_start_true = 25. + (float) (column - g_display_l + 1)
    * 700. / (g_display_w + 1);
  g_col_insert_row_start_y = 50. + (float) (g_display_b - column + 1) * 700. /
    (g_display_h + 1);
  g_col_insert_base_x_offset = 0;
  g_col_insert_sequence = sequence;
  g_col_insert_base_x_safe = x;
  g_col_insert_base_safe = column;
}

void align_check_make_col_insert(int x, int y, int column) {
  int             sequence;
  /* convert x coordinate to column */
  if ((column < g_align_start_base) || (column > g_align_end_base))
    return;
  /* convert y coordinate to sequence */
  sequence = (y - 12) / 20 + 1;
  if ((sequence < 2) || (sequence > g_last_used_sequence))
    return;
  g_align_col_insert_base_on = TRUE;
  g_align_col_insert_base_start = column;
  g_align_col_insert_base_final = column;
  g_align_col_insert_base_x_start = x;
  g_align_col_insert_base_x_offset = 0;
  g_align_col_insert_sequence = sequence;
  g_align_col_insert_base_x_safe = x;
  g_align_col_insert_base_safe = column;
}

void check_update_col_insert(int x) {
  int             column, offset_to_window;
  column = window_x_convert_zoom(x);
  if (column > g_display_r)
    column = g_display_r;
  if (column < g_display_l)
    column = g_display_l;
  if (column < g_col_insert_base_start) {
    if (g_sequence_data[g_col_insert_sequence][column - 1] != '.') {
      x = g_col_insert_base_x_safe;
      column = g_col_insert_base_safe;
    }
  }
  g_col_insert_base_x_offset = (x - g_col_insert_base_x_start);
  if (g_count_sequences < 3)
    offset_to_window = -4;
  else
    offset_to_window = (g_count_sequences - 2) * 15;
  g_col_insert_base_x_offset =
    g_col_insert_base_x_offset / (width_z / (800. + offset_to_window));
  g_col_insert_base_final = column;
  g_col_insert_base_x_safe = x;
  g_col_insert_base_safe = column;
}

void align_check_update_col_insert(int x) { 
  int             column;
  int             font_width, first_base_x;
  if (g_font_align == GLUT_BITMAP_HELVETICA_12) {
    font_width = 10;
    first_base_x = 75;
  } else {
    font_width = 15;
    first_base_x = 75;
  }
  column = x - first_base_x;
  column = (int) (column / ((float) font_width));
  column = column + g_align_start_base;
  if (column < g_align_start_base)
    column = g_align_start_base;
  if (column > g_sequence_length)
    column = g_sequence_length;
  if (column < g_align_col_insert_base_start) {
    if (g_sequence_data[g_align_col_insert_sequence][column - 1] != '.') {
      x = g_align_col_insert_base_x_safe;
      column = g_align_col_insert_base_safe;
    }
  }
  g_align_col_insert_base_x_offset = x - g_align_col_insert_base_x_start;
  g_align_col_insert_base_final = column;
  g_align_col_insert_base_x_safe = x;
  g_align_col_insert_base_safe = column;
}

void drag_helix_more_than2(int from_row,int from_col,int to_row,int to_col) {
  int             i, index;
  index = index_energy_read(to_row, to_col, g_primary_sequence);
  if ((index < 0) ||
      (g_diag[index].energy > g_energy_cutoff[g_primary_sequence])) {
    printf("No helix here for primary sequence.\n");
    return;	
  }
  if (index != g_primary_helix_index) {
    printf("New primary helix selected.\n");
    g_primary_helix_index = index;
    for (i = 2; i <= g_last_used_sequence; i++) {
      g_drag_this_row_from[i] = -1;
    }
  }
  g_drag_this_row_from[g_this_sequence] = from_row;
  g_drag_this_column_from[g_this_sequence] = from_col;
  g_drag_this_row_to[g_this_sequence] = to_row;
  g_drag_this_column_to[g_this_sequence] = to_col;
  printf("Set helix for sequence %d\n", g_this_sequence);
  /* automatic adjust advancement */
  g_this_sequence++;
  if (g_this_sequence == g_primary_sequence)
    g_this_sequence++;
  if (g_this_sequence > g_last_used_sequence)
    g_this_sequence = 2;
  if (g_this_sequence == g_primary_sequence)
    g_this_sequence++;
  glutSetWindow(win3);
  glutPostRedisplay();
}

void mouse_zoom(int button, int state, int x, int y) {
  int             number_inserts;
  int             i_row, j_col;
  int             i_row_original, j_col_original;
  g_col_line = TRUE;
  g_row_line = TRUE;
  j_col = window_x_convert_zoom(x);
  i_row = window_y_convert_zoom(y);
  i_row_original = i_row;
  j_col_original = j_col;
  if (i_row < g_display_t) {
    i_row = g_display_t;
    g_row_line = FALSE;
  }
  if (i_row > g_display_b) {
    i_row = g_display_b;
    g_row_line = FALSE;
  }
  if (j_col < g_display_l) {
    j_col = g_display_l;
    g_col_line = FALSE;
  }
  if (j_col > g_display_r) {
    j_col = g_display_r;
    g_col_line = FALSE;
  }
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
      g_i_row = i_row;
      g_j_col = j_col;
      set_helix_use_row_column(g_i_row, g_j_col);
      glutSetWindow(win3);
      glutPostRedisplay();
      show_align();
      check_display_zoom_labels(i_row, j_col);
      g_drag_helix_mov_x = x;
      g_drag_helix_mov_y = y;
      check_to_drag_helix(i_row_original, j_col_original);
      check_make_col_insert(x, y);
      check_make_row_insert(x, y);
      glutSetWindow(win2);
      glutPostRedisplay();
    } else {
      if (g_col_insert_base_on) {
	g_col_insert_base_on = FALSE;
	g_col_insert_base_x_offset = 0;
	number_inserts = g_col_insert_base_final -
	  g_col_insert_base_start;
	if (number_inserts > 0) {
	  auto_insertion_column(g_col_insert_sequence, g_col_insert_base_start,
				number_inserts);
	  show_align();
	}
	if (number_inserts < 0) {
	  auto_deletion_column(g_col_insert_sequence - 1,
			       g_col_insert_base_final, -number_inserts);
	  show_align();
	}
	if (number_inserts == 0) {
	  glutSetWindow(win2);
	  glutPostRedisplay();
	}
      }
      if (g_row_insert_base_on) {
	g_row_insert_base_on = FALSE;
	g_row_insert_base_y_offset = 0;
	number_inserts = g_row_insert_base_final -
	  g_row_insert_base_start;
	if (number_inserts > 0) {
	  auto_insertion_column(g_row_insert_sequence, g_row_insert_base_start,
				number_inserts);
	  show_align();
	}
	if (number_inserts < 0) {
	  auto_deletion_column(g_row_insert_sequence - 1,
			       g_row_insert_base_final, -number_inserts);
	  show_align();
	}
	if (number_inserts == 0) {
	  glutSetWindow(win2);
	  glutPostRedisplay();
	}
      }
      if (g_drag_helix_set) {
	g_drag_helix_set = FALSE;
	if ((g_drag_helix_sel_row != i_row) ||
	    (g_drag_helix_sel_col != j_col)) {
	  glutSetWindow(win2);
	  glutSetCursor(GLUT_CURSOR_WAIT);
	  glutSetWindow(win1);
	  glutSetCursor(GLUT_CURSOR_WAIT);
	  if (g_count_sequences == 2) {
	    drag_helix(g_drag_helix_sel_row,g_drag_helix_sel_col,i_row,j_col);
	    show_align();
	  } else {
	    drag_helix_more_than2(g_drag_helix_sel_row, g_drag_helix_sel_col,
				  i_row, j_col);
	  }
	  if (g_nearby_distance > 0)
	    create_nearby(g_this_sequence, g_nearby_distance);
	}
	fix_cursor();
      }
    }
  }
  if (button == GLUT_MIDDLE_BUTTON) {
    if (state == GLUT_DOWN) {
      if (g_region_set == FALSE) {
	g_region_l = j_col;
	g_region_t = i_row;
	g_region_b = i_row;
	g_region_r = j_col;
	g_region_set = TRUE;
      } else {
	printf("Within no motion mouse with g_region_set = 1\n");
	g_region_r = j_col;	/* This code is never executed ? */
	g_region_b = i_row;
      }
      fix_zoom_box_zoom();
    } else {
      g_region_set = FALSE;
      g_region_exists = TRUE;
      g_zoom_labels_count = 0;
      finish_fixing_zoom_box_zoom();
      glutSetWindow(win2);
      glutShowWindow();
      glutPostRedisplay();
      glutSetWindow(win1);
      glutPostRedisplay();
    }
  }
}

void mouse_motion_zoom(int x, int y) {
  int             i_row, j_col;
  if (g_col_insert_base_on) {
    check_update_col_insert(x);
    glutSetWindow(win2);
    glutPostRedisplay();
  }
  if (g_row_insert_base_on) {
    check_update_row_insert(y);
    glutSetWindow(win2);
    glutPostRedisplay();
  }
  if (g_region_set == TRUE) {
    j_col = window_x_convert_zoom(x);
    i_row = window_y_convert_zoom(y);
    if (i_row < g_display_t) {
      i_row = g_display_t;
    }
    if (i_row > g_display_b)
      i_row = g_display_b;
    if (j_col < g_display_l)
      j_col = g_display_l;
    if (j_col > g_display_r)
      j_col = g_display_r;
    g_region_r = j_col;
    g_region_b = i_row;
    fix_zoom_box_zoom();
    glutSetWindow(win2);
    glutPostRedisplay();
  }
  if (g_drag_helix_set) { /* perform extra drawing for moving helix */
    /* draw the extra helix here */
    g_drag_helix_mov_x = x;
    g_drag_helix_mov_y = y;
    glutSetWindow(win2);
    glutPostRedisplay();
  }
}

int             g_which_window_i_am_in;

void main_entry(int state) {
  if (state == GLUT_ENTERED) {
     g_which_window_i_am_in = win1;
  }
}

void zoom_entry(int state) {
  if (state == GLUT_ENTERED) {
    g_which_window_i_am_in = win2;
  }
}

void text_entry(int state) {
  if (state == GLUT_ENTERED)
    g_which_window_i_am_in = win3;
}

void            undo_insertions(void);

void drag_more_than_2(void) {
  int             i;
  int             row;
  int             length;
  int             not_found;
  int             new_index;
  int             start_row;
  int             column;
  if (g_primary_helix_index < 0) {
    printf("No helix chosen for drag\n");
    return;
  }
  for (i = 2; i <= g_last_used_sequence; i++) {
    if ((g_primary_sequence != i) && (g_drag_this_row_from[i] < 1)) {

      /* No helix has been set here, see if one is under the primary
	 sequence 
	 search for helix here for this sequence */
      printf("Looking for helix under the primary sequence.\n");
      start_row = g_diag[g_primary_helix_index].row;
      column = g_diag[g_primary_helix_index].column;
      length = g_diag[g_primary_helix_index].length;
      not_found = TRUE;
      row = start_row;
      printf("Checking %d,%d\n", start_row, column);
      while ((not_found == TRUE) && (row <= start_row + length - 1)) {
	printf("Checking %d,%d\n", row, column);
	new_index = index_energy_read(row, column, i);
	if (!((new_index<0)||(g_diag[new_index].energy>g_energy_cutoff[i]))) {
	  not_found = FALSE;
	  g_drag_this_column_from[i] = column;
	  g_drag_this_row_from[i] = row;
	  if (!g_dot_lower_flag) {
	    g_drag_this_row_to[i] = row;
	    g_drag_this_column_to[i] = column;
	  } else {
	    g_drag_this_row_to[i] = column;
	    g_drag_this_column_to[i] = row;
	  }
	}
	row++;
	column--;
      }
      if (not_found) {
	printf("No helix selected and no helix here for sequence %d\n", i-1);
	return;
      }
    }
  }
  printf("Data for drag is OK\n");
  execute_drag_more_than_2();
  /* restart */
  g_primary_helix_index = -1;
  for (i = 2; i <= g_last_used_sequence; i++)
    g_drag_this_row_from[i] = -1;
  glutSetWindow(win3);
  glutPostRedisplay();
}

void ann_menu(int msg) {
  int             seq;
  if (g_annotation_flag) {
    for (seq = 2; seq <= g_last_used_sequence; seq++) {
      free(g_annotation_base[seq]);
    }
  }
  if (msg == 0) {
    if ((g_annotation_flag) || (g_prob_annotation_flag)) {
      g_annotation_flag = FALSE;
      g_prob_annotation_flag = FALSE;
      glutSetWindow(win2);
      glutPostRedisplay();
      glutSetWindow(win1);
      glutPostRedisplay();
    }
    return;
  }
  if (msg == 1) {
    g_prob_annotation_flag = FALSE;
    if (compute_ann(g_nucleotide_total,  &g_ann_num_colors))
      g_annotation_flag = FALSE;
    else {
      g_annotation_flag = TRUE;
      g_annotation_max_total = 0;
      for (seq = 2; seq <= g_adjust_last_used_sequence; seq++)
	g_annotation_max_total += g_annotation_max[seq];
    }
  }
  if (msg == 2) {
    g_annotation_flag = FALSE;
    if (compute_ann_prob(&g_ann_num_colors))
      g_prob_annotation_flag = FALSE;
    else
      g_prob_annotation_flag = TRUE;
  }
  glutSetWindow(win2);
  glutPostRedisplay();
  glutSetWindow(win1);
  glutPostRedisplay();
}

void clean_function(int msg) {
  neighbor_off();
  if (msg == 3) {
    glutSetWindow(win2);
    glutSetCursor(GLUT_CURSOR_WAIT);
    glutSetWindow(win1);
    glutSetCursor(GLUT_CURSOR_WAIT);
    clean_inserts2();
  } else {
    glutSetWindow(win2);
    glutSetCursor(GLUT_CURSOR_WAIT);
    glutSetWindow(win1);
    glutSetCursor(GLUT_CURSOR_WAIT);
    clean_inserts_no_base();
  }
  if (g_display_r > g_sequence_length) {
    g_display_r = g_sequence_length;
    g_display_w = g_display_r - g_display_l + 1;
  }
  if (g_display_b > g_sequence_length) {
    g_display_b = g_sequence_length;
    g_display_h = g_display_b - g_display_t + 1;
  }
  if (g_nearby_distance > 0)
    create_nearby(g_this_sequence, g_nearby_distance);
  fix_cursor();
  show_align();
}

void No_Action_Menu(int msg) {
  /* numbers used here,
   * 1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,51,52,6 & 4 */
  int             left, right, top, bottom;
  if (msg == 1) {
    printf("Adjusting alignment to remove empty columns before exiting.\n");
    clean_function(1);
    try_exit(0);
  }
  if (msg == 14) {
    if (g_no_partial_flag)
      g_no_partial_flag = FALSE;
    else
      g_no_partial_flag = TRUE;
    glutSetWindow(win2);
    glutPostRedisplay();
    glutSetWindow(win1);
    glutPostRedisplay();
  }
  if (msg == 2) {
    glutSetWindow(g_which_window_i_am_in);
    glutPopWindow();
  }
  if (msg == 51) {
    neighbor_off();
    drag_more_than_2();
    if (g_nearby_distance > 0)
      create_nearby(g_this_sequence, g_nearby_distance);
    glutSetWindow(win2);
    glutPostRedisplay();
    glutSetWindow(win1);
    glutPostRedisplay();
    glutSetWindow(win3);
    glutPostRedisplay();
    show_align();
  }
  if (msg == 5) {
    if (g_grid_flag == TRUE)
      g_grid_flag = FALSE;
    else
      g_grid_flag = TRUE;
    glutSetWindow(win1);	/* display main */
    glutPostRedisplay();
    glutSetWindow(win2);	/* display zoom */
    glutPostRedisplay();
  }
  if (msg == 64) {
    if (g_dot_lower_flag == TRUE)
      g_dot_lower_flag = FALSE;
    else
      g_dot_lower_flag = TRUE;
    g_zoom_labels_count = 0;
    glutSetWindow(win2);
    glutPostRedisplay();
    glutSetWindow(win1);
    glutPostRedisplay();
  }
  if (msg == 6) {
    if (g_comp_flag == TRUE)
      g_comp_flag = FALSE;
    else
      g_comp_flag = TRUE;
    glutSetWindow(win1);
    glutPostRedisplay();
    glutSetWindow(win2);
    glutPostRedisplay();
  }
  if (msg == 7) {		/* go back to previous zoom window */
    g_region_l = g_saved_display_l;
    g_region_r = g_saved_display_r;
    g_region_t = g_saved_display_t;
    g_region_b = g_saved_display_b;
    if (g_region_r > g_sequence_length)
      g_region_r = g_sequence_length;
    if (g_region_b > g_sequence_length)
      g_region_b = g_sequence_length;
    fix_zoom_box();
    finish_fixing_zoom_box();
    g_zoom_labels_count = 0;
    glutSetWindow(win2);
    glutPostRedisplay();
    glutSetWindow(win1);
    glutPostRedisplay();
  }
  if (msg == 8) {		/* undo last insertions */
    glutSetWindow(win2);
    glutSetCursor(GLUT_CURSOR_WAIT);
    glutSetWindow(win1);
    glutSetCursor(GLUT_CURSOR_WAIT);
    neighbor_off();
    undo_insertions();
    if (g_display_r > g_sequence_length) {
      g_display_r = g_sequence_length;
      g_display_w = g_display_r - g_display_l + 1;
    }
    if (g_display_b > g_sequence_length) {
      g_display_b = g_sequence_length;
      g_display_h = g_display_b - g_display_t + 1;
    }
    if (g_nearby_distance > 0)
      create_nearby(g_this_sequence, g_nearby_distance);
    glutSetWindow(win3);
    glutPostRedisplay();
    fix_cursor();
    show_align();
  }
  if (msg == 4) {
    if (g_unique_mode) {
      g_unique_mode = FALSE;
    } else {
      g_unique_mode = TRUE;
    }
    nearby_off();
    neighbor_off();
    glutSetWindow(win2);
    glutPostRedisplay();
    glutSetWindow(win1);
    glutPostRedisplay();
  }
  if (msg == 10) {
    if (g_bigdot_mode == TRUE)
      g_bigdot_mode = FALSE;
    else
      g_bigdot_mode = TRUE;
    glutSetWindow(win1);
    glutPostRedisplay();
    glutSetWindow(win2);
    glutPostRedisplay();
  }
  if (msg == 52) {
    if (g_base_color_mode == TRUE)
      g_base_color_mode = FALSE;
    else
      g_base_color_mode = TRUE;
    glutSetWindow(win2);
    glutPostRedisplay();
    show_align();
  }
  if (msg == 9) {		/* show original alignment */
    glutSetWindow(win2);
    glutSetCursor(GLUT_CURSOR_WAIT);
    glutSetWindow(win1);
    glutSetCursor(GLUT_CURSOR_WAIT);
    neighbor_off();
    set_original();
    if (g_display_r > g_sequence_length) {
      g_display_r = g_sequence_length;
      g_display_w = g_display_r - g_display_l + 1;
    }
    if (g_display_b > g_sequence_length) {
      g_display_b = g_sequence_length;
      g_display_h = g_display_b - g_display_t + 1;
    }
    if (g_nearby_distance > 0)
      create_nearby(g_this_sequence, g_nearby_distance);
    glutSetWindow(win3);
    glutPostRedisplay();
    fix_cursor();
    show_align();
  }
  if (msg == 13) {	/* show alignment from last save */
    glutSetWindow(win2);
    glutSetCursor(GLUT_CURSOR_WAIT);
    glutSetWindow(win1);
    glutSetCursor(GLUT_CURSOR_WAIT);
    neighbor_off();
    set_saved_align();
    if (g_display_r > g_sequence_length) {
      g_display_r = g_sequence_length;
      g_display_w = g_display_r - g_display_l + 1;
    }
    if (g_display_b > g_sequence_length) {
      g_display_b = g_sequence_length;
      g_display_h = g_display_b - g_display_t + 1;
    }
    if (g_nearby_distance > 0)
      create_nearby(g_this_sequence, g_nearby_distance);
    glutSetWindow(win3);
    glutPostRedisplay();
    fix_cursor();
    show_align();
  }
  if (msg == 3) {		/* find best insert */
    neighbor_off();
    left = g_display_l;
    right = g_display_r;
    top = g_display_t;
    bottom = g_display_b;
    if (right <= top) {
      printf("No computations are performed below main diagonal.\n");
      return;
    }
    if (bottom > right) {
      printf("Lowest row of region adjusted up to %d\n", right);
      bottom = right;
    }
    if (top > left) {
      printf("Left adjusted right to %d\n", top);
      left = top;
    }
    glutSetWindow(win2);
    glutSetCursor(GLUT_CURSOR_WAIT);
    glutSetWindow(win1);
    glutSetCursor(GLUT_CURSOR_WAIT);
    best_insert(g_this_sequence,
		left, right, top, bottom);
    if (g_nearby_distance > 0)
      create_nearby(g_this_sequence, g_nearby_distance);
    glutSetWindow(win3);
    glutPostRedisplay();
    fix_cursor();
    show_align();
  }
  if (msg == 11) {
    finish_making_align_mal(TRUE);
  }
  if (msg == 12) {
    set_zoom_coordinates();
  }
}

void auto_insertion_column(int sequence, int column, int inserts) {
  printf("%d inserts selected for column %d with sequence %d.\n",
	 inserts, column, sequence - 1);
  glutSetWindow(win2);
  glutSetCursor(GLUT_CURSOR_WAIT);
  glutSetWindow(win1);
  glutSetCursor(GLUT_CURSOR_WAIT);
  neighbor_off();
  place_insert(sequence - 1, column, inserts);
  if (g_nearby_distance > 0)
    create_nearby(g_this_sequence, g_nearby_distance);
  fix_cursor();
}

void auto_deletion_column(int sequence, int position, int deletes) {
  /* works fine for rows too */
  printf("%d deletions selected for position %d with sequence %d.\n",
	 deletes, position, sequence);
  neighbor_off();
  delete_insert(sequence, position, deletes);
  if (g_display_r > g_sequence_length) {
    g_display_r = g_sequence_length;
    g_display_w = g_display_r - g_display_l + 1;
  }
  if (g_display_b > g_sequence_length) {
    g_display_b = g_sequence_length;
    g_display_h = g_display_b - g_display_t + 1;
  }
  if (g_nearby_distance > 0)
    create_nearby(g_this_sequence, g_nearby_distance);
  glutSetWindow(win2);
  glutPostRedisplay();
  glutSetWindow(win1);
  glutPostRedisplay();
}

void make_switch_function(int msg) {
  neighbor_off();
  nearby_off();
  printf("Making %s Primary\n", g_sequence_name[msg + 1]);
  make_switch_with1(msg);
  glutSetWindow(win3);
  glutPostRedisplay();
}

void make_alignment_file(int msg) {
  if (msg == 1) {		/* create alignment file */
    strcpy(text_window_message, align_message_ps);
    strcat(text_window_message, "? ");
    g_message_set = TRUE;
    data_entered = FALSE;
    g_type_message = MESSAGE_ALIGN_PS;
    glutSetWindow(win3);
    glutPostRedisplay();
    set_cursor_help();
  }
  if (msg == 2) {		/* create alignment file */
    strcpy(text_window_message, align_message_mal);
    strcat(text_window_message, g_saved_alignment_name);
    strcat(text_window_message, ") ? ");
    g_message_set = TRUE;
    data_entered = FALSE;
    g_type_message = MESSAGE_ALIGN_MAL;
    glutSetWindow(win3);
    glutPostRedisplay();
    set_cursor_help();
  }
}

void adjust_count_function(int msg) {
  int             seq;
  neighbor_off();
  nearby_off();
  g_adjust_count_sequences = msg;
  g_adjust_last_used_sequence = msg + 1;
  if (g_annotation_flag) {
    g_annotation_max_total = 0;
    for (seq = 2; seq <= g_adjust_last_used_sequence; seq++)
      g_annotation_max_total += g_annotation_max[seq];
  }
  printf("Redoing display with %d sequences shown.\n", msg);
  redo_with_new_count();
  glutSetWindow(win2);
  glutPostRedisplay();
  glutSetWindow(win1);
  glutPostRedisplay();
}

void make_this_sequence(int msg) {
  neighbor_off();
  nearby_off();
  printf("Making %s the Adjustable Sequence\n", g_sequence_name[msg + 1]);
  g_this_sequence = msg + 1;
  glutSetWindow(win3);
  glutPostRedisplay();
  glutSetWindow(win2);
  glutPostRedisplay();
}

void NoFunction(int junk) {
  if (junk == 5)
    printf("Something weird happened\n");
}

int             g_main_menu[5];

void fix_key_zoom_window(int x_change, int y_change, int zoom_direction) {
  int             x_offset, y_offset;
  g_zoom_labels_count = 0;	/* reset labels to none */
  if (x_change != 0) {
    x_offset = step_fun(g_display_l, g_display_r);
    if (x_offset < 1)
      x_offset = 1;
    x_offset = x_offset * x_change;
    if (x_change < 0) {
      if (g_display_l == 1)
	return;	/* do not go left when at left edge */
      g_region_l = g_display_l + x_offset;
      if (g_region_l < 1)
	g_region_l = 1;
      g_region_r = g_display_r + x_offset;
      if (g_region_r < 1)
	g_region_r = 1;
    }
    if (x_change > 0) {
      if (g_display_r == g_sequence_length)
	return;	/* do go right when at right edge */
      g_region_r = g_region_r + x_offset;
      g_region_l = g_region_l + x_offset;
      if (g_region_r > g_sequence_length)
	g_region_r = g_sequence_length;
      if (g_region_l > g_sequence_length)
	g_region_l = g_sequence_length;
    }
  }
  if (y_change != 0) {
    y_offset = step_fun(g_display_t, g_display_b);
    if (y_offset < 1)
      y_offset = 1;
    y_change = y_change * y_offset;
    if (y_change < 0) {
      if (g_display_t == 1)
	return;	/* do not go up when at top */
      g_region_t = g_display_t + y_change;
      if (g_region_t < 1)
	g_region_t = 1;
      g_region_b = g_display_b + y_change;
      if (g_region_b < 1)
	g_region_b = 1;
    }
    if (y_change > 0) {
      if (g_display_b == g_sequence_length)
	return;	/* do not go up when at top */
      g_region_t = g_region_t + y_change;
      g_region_b = g_region_b + y_change;
      if (g_region_b > g_sequence_length)
	g_region_b = g_sequence_length;
      if (g_region_t > g_sequence_length)
	g_region_t = g_sequence_length;
    }
  }
  if (zoom_direction) {	
    if (zoom_direction < 0)
      y_offset = (int) ((g_display_b - g_display_t) / 14 + .5);
    else
      y_offset = (int) ((g_display_b - g_display_t) / 12 + .5);
    if (y_offset < 1)
      y_offset = 1;
    if (zoom_direction < 0)
      x_offset = (int) ((g_display_r - g_display_l) / 14. + .5);
    else
      x_offset = (int) ((g_display_r - g_display_l) / 12 + .5);
    if (x_offset < 1)
      x_offset = 1;
    if (zoom_direction < 0) {
      if ((g_display_b - g_display_t) < 4) {
	y_offset = 0;
      }
      if ((g_display_r - g_display_l) < 4)
	x_offset = 0;
      x_offset = x_offset * -1;
      y_offset = y_offset * -1;
    }
    g_region_l = g_display_l - x_offset;
    g_region_r = g_display_r + x_offset;
    g_region_t = g_display_t - y_offset;
    g_region_b = g_display_b + y_offset;
    if (g_region_l < 1)
      g_region_l = 1;
    if (g_region_t < 1)
      g_region_t = 1;
    if (g_region_b > g_sequence_length)
      g_region_b = g_sequence_length;
    if (g_region_r > g_sequence_length)
      g_region_r = g_sequence_length;
  }
  fix_zoom_box();
  finish_fixing_zoom_box();
  glutSetWindow(win2);
  glutPostRedisplay();
  glutSetWindow(win1);
  glutPostRedisplay();
}

static void Key2(int key, int x, int y) {
  int             x_change;
  int             y_change;
  int             zoom_direction;
  if (!g_zoom_window_exists)
    return;
  x_change = 0;
  y_change = 0;
  zoom_direction = 0;
  switch (key) {
  case GLUT_KEY_LEFT:
    x_change = -1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_RIGHT:
    x_change = 1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_UP:
    y_change = -1.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_DOWN:
    y_change = 1.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);;
    break;
  case GLUT_KEY_PAGE_DOWN:
    zoom_direction = +1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_PAGE_UP:
    zoom_direction = -1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  default:
    return;
  }
}

static void Key2_fast(int key, int x, int y) {
  int             x_change;
  int             y_change;
  int             zoom_direction;
  if (!g_zoom_window_exists)
    return;
  x_change = 0;
  y_change = 0;
  zoom_direction = 0;
  switch (key) {
  case GLUT_KEY_LEFT:
    x_change = -3;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_RIGHT:
    x_change = 3;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_UP:
    y_change = -3.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_DOWN:
    y_change = 3.0;
    fix_key_zoom_window(x_change, y_change, zoom_direction);;
    break;
  case GLUT_KEY_PAGE_DOWN:
    zoom_direction = +1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  case GLUT_KEY_PAGE_UP:
    zoom_direction = -1;
    fix_key_zoom_window(x_change, y_change, zoom_direction);
    break;
  default:
    return;
  }
}

/* M. Zuker, January 29, 2007.
 * This function is really ugly. It uses nafold and assumes the "late
 * 1990s" format. It must be replaced by the appropriate UNAFold
 * function. 
 * Also, the naming conventions are ugly and stupid.
 */
void create_structure(int i) {
  int             row, column;
  char            filename[120];
  char            old_ct_filename[120];
  char            action[120];
  char            number[10];
  int             error_occurred;
  int             folding_length, temp_length;
  FILE           *nafold_fp;
  char            nafold_filename[120];
  /* temp_for_sir_graph.ct is produced as a ct file 
   * nafold.input.temp is created and deleted as instructions for
   * creating a ct file by nafold */
  printf("Trying to create structure for sequence %d at (%d,%d)\n", i-1,
	 g_i_row, g_j_col);
  if (g_i_row < 1) {
    printf("No base pair defined.\n");
    return;
  }
  if (g_i_row < g_j_col) {
    row = g_reverse_st[i][g_i_row];
    column = g_reverse_st[i][g_j_col];
  } else {
    column = g_reverse_st[i][g_i_row];
    row = g_reverse_st[i][g_j_col];
  }
  printf("Transformed row,col are (%d,%d)\n", row, column);
  if ((row < 1) || (column < 1)) {
    printf("Cannot Create structure at an insertion.\n");
    return;
  }
  strcpy(filename, plot_filename_short[i]);
  strcat(filename, ".sav");
  printf("Filename is %s\n", filename);
  /* remove ct file first */
  printf("Attempting to remove old temp.ct file\n");
  strcpy(action, "rm -fv temp_for_sir_graph.ct");
  error_occurred = system(action);
  /* should use 2 for <=500
   * need length used for creation of save file */
  folding_length = g_nucleotide_total[i];
  /* try to get length from ct file */
  strcpy(old_ct_filename, plot_filename_short[i]);
  strcat(old_ct_filename, ".ct");
  if ((nafold_fp = fopen(old_ct_filename, "r")) == NULL) {
    printf("Could not get length from ct file, using %s, will not open.\n",
	   old_ct_filename);
  } else {
    if (fgets(action, 90, nafold_fp) != NULL) {
      sscanf(action, "%d", &temp_length);
      if (temp_length > 0) {
	printf("Retrieved length %d from ct file %s.\n",
	       temp_length, old_ct_filename);
	folding_length = temp_length;
      } else
	printf("Could not get the length from first line of ct file %s.\n",
	       old_ct_filename);
    } else {
      printf("Could get not get length from first line of ct file %s.\n",
	     old_ct_filename);
    }
    fclose(nafold_fp);
  }
  /* end of attemp to get length from ct file */
  strcpy(action, "nafold < ");
  printf("Using script %s \n", action);
  /* create input file */
  strcpy(nafold_filename, "nafold.input.temp");
  strcat(action, nafold_filename);
  strcat(action, "\n");
  if ((nafold_fp = fopen(nafold_filename, "w")) == NULL) {
    printf("Could not open file: %s.\n", nafold_filename);
    printf("Cannot generate structure.\n");
    return;
  }
  fprintf(nafold_fp, "2\n");
  fprintf(nafold_fp, "%s", filename);
  fprintf(nafold_fp, "temp_for_sir_graph.ct\n");
  strcpy(number, num_string_int(row));
  fprintf(nafold_fp, "%s", number);
  fprintf(nafold_fp, " ");
  strcpy(number, num_string_int(column));
  fprintf(nafold_fp, "%s\n", number);
  fclose(nafold_fp);
  error_occurred = system(action);
  if (error_occurred != 0) {
    printf("Error creating new ct file: %s for save file %s\n",
	   "temp_for_sir_graph.ct", filename);
    printf("%s should be deleted\n", nafold_filename);
  } else {
    strcpy(action, "sir_graph  -aj ");
    strcat(action, " -i ");
    strcat(action, num_string_int(row));
    strcat(action, " ");
    strcat(action, num_string_int(column));
    strcat(action, "  temp_for_sir_graph &");
    /* remove nafold_filename */
    printf("Action is %s\n", action);
    error_occurred = system(action);
    strcpy(action, "rm ");
    strcat(action, nafold_filename);
    system(action);
  }
}

void remove_temp_file(char *filename) {
  char            action[100];
  int             error_occurred;
  strcpy(action, "rm ");
  strcat(action, filename);
  error_occurred = system(action);
  if (error_occurred != 0) {
    printf("Error = %d while performing %s\n", error_occurred, action);
  }
}

/* M. Zuker, January 29, 2007.
 * This function is also really ugly. 
 * It needs a total rewrite using the latest software
 * May 16, 2007. M. Zuker
 * replace mfold by hybrid-ss-min
 * Create folding for entire sequences (ignore min and max)
 */
void create_structure2(int i) {
  char            forced_filename[120];
  char            filename[120];
  char            new_name[120];
  char            data_name[120];
  char            action[120];
  char            number[15];
  int             max;
  int             min;
  int             start, end;
  int             error_occurred;
  printf("###################################################\n");
  if (i == -10) {
    start = 2;
    end = g_last_used_sequence;
  } else {
    start = i;
    end = i;
  }
  for (i = start; i <= end; i++) {
    printf("#### Creating structure for sequence %s ###\n",
	   g_sequence_name[i]);
    printf("###################################################\n");
    strcpy(forced_filename, "temp.forced_complot-");
    strcpy(number, num_string_int(i - 1));
    strcat(forced_filename, number);
    strcat(forced_filename, ".sel");
    strcpy(g_post_filename, forced_filename);
    remove_temp_file(forced_filename);
    error_occurred = open_post_label();
    if (error_occurred) {
      printf("Error creating file of selected bases for hybrid-ss-min # %d\n",
	     error_occurred);
      continue;
    }
    general_label(i - 1, TRUE, &min, &max);
    if (max < 1) {
      printf("No selected bases for sequence %d\n", i - 1);
      printf("No structure to draw.\n");
      remove_temp_file(g_post_filename);
      continue;
    } else {
      /* sort the forced file */
      strcpy(action, "sort -n -o ");
      strcat(action, forced_filename);
      strcat(action, " ");
      strcat(action, forced_filename);
      error_occurred = system(action);
      if (error_occurred != 0) {
	printf("Error occurred trying to sort %s with %s.\n",
	       forced_filename, action);
      }
    }
    /* temp_for_sir_graph.ct is produced as a ct file
     * nafold.input.temp is created and deleted as instructions
     * for creating a ct file by nafold */
    /* copy old sequence to new copy of it */
    strcpy(new_name, "temp.for_complot-");
    strcat(new_name, number);
    strcpy(data_name, new_name);
    strcat(new_name, ".seq");
    strcpy(filename, plot_filename_short[i]);
    strcat(filename, ".seq");
    strcpy(action, "cp ");
    strcat(action, filename);
    strcat(action, " ");
    strcat(action, new_name);
    printf("Copying file with %s\n", action);
    error_occurred = system(action);
    if (error_occurred) {
      printf("Error!\tcould not copy sequence data to %s with %s.\n",
	     new_name, action);
      printf("Perhaps the sequence data file does not exist.\n");
      return;
    }
    printf("Copied sequence file: %s to: %s\n", filename, new_name);
    strcpy(action, "hybrid-ss-min -sDAT --constraints=");
    strcat(action, g_post_filename);
    strcat(action, " --output=");
    strcat(action, data_name);
    strcat(action, " ");
    strcat(action, new_name);
    printf("Executing: %s\n", action);
    error_occurred = system(action);
    if (error_occurred != 0) {
      printf("Error running hybrid-ss-min to create structure with %s\n", new_name);
      printf("%s files should be deleted.\n", data_name);
    } else {
      strcpy(action, "sir_graph -lab 10 -aj -f ");
      strcat(action, "-force ");
      strcat(action, forced_filename);
      if (g_dot_lower_flag)
	strcat(action, " -rot -45 ");	/* match dot plot */
      else
	strcat(action, " -cc -rot -45 ");
      strcat(action, "-fix ");
      strcat(action, data_name);
      strcat(action, " &");
      printf("Running sir_graph: %s \n", action);
      error_occurred = system(action);
      if (error_occurred) {
	printf("Error!\tCould not run sir_graph.\n");
      }
      /* Clean up useless files */
      system("rm -f temp.for_com*.run temp.for_com*.37.* temp.for_com*.dG");
    }
  }
}

void top_align_menu(int msg) {
  if (msg == 1)
    try_exit(0);
  if ((msg > 1) && (msg < 8)) {
    if (msg == 2)
      g_align_start_base = 1;
    if (g_zoom_window_exists) {
      if (msg == 3)
	g_align_start_base = g_display_t;
      if (msg == 4)
	g_align_start_base = g_display_l;
    }
    if (msg == 5) {
      if (g_i_row > 0) {
	if (g_i_row < g_j_col)
	  g_align_start_base = g_i_row - 10;
	else
	  g_align_start_base = g_j_col - 10;
	if (g_align_start_base < 1)
	  g_align_start_base = 1;
	if (g_align_start_base > g_sequence_length)
	  g_align_start_base = g_sequence_length;
      }
    }
    if (msg == 6) {
      if (g_j_col > 0) {
	if (g_j_col > g_i_row)
	  g_align_start_base = g_j_col - 10;
	else
	  g_align_start_base = g_i_row - 10;
	if (g_align_start_base < 1)
	  g_align_start_base = 1;
	if (g_align_start_base > g_sequence_length)
	  g_align_start_base = g_sequence_length;
      }
    }
    if (msg == 7)
      g_align_start_base = -1;
    glutSetWindow(win4);
    glutPostRedisplay();
  }
  if (msg == 10) {
    if (g_base_color_mode == TRUE)
      g_base_color_mode = FALSE;
    else
      g_base_color_mode = TRUE;
    glutSetWindow(win1);
    glutPostRedisplay();
    glutSetWindow(win2);
    glutPostRedisplay();
    show_align();
  }
  if (msg == 8) {		/* undo last insertions */
    glutSetWindow(win2);
    glutSetCursor(GLUT_CURSOR_WAIT);
    glutSetWindow(win1);
    glutSetCursor(GLUT_CURSOR_WAIT);
    neighbor_off();
    undo_insertions();
    if (g_display_r > g_sequence_length) {
      g_display_r = g_sequence_length;
      g_display_w = g_display_r - g_display_l + 1;
    }
    if (g_display_b > g_sequence_length) {
      g_display_b = g_sequence_length;
      g_display_h = g_display_b - g_display_t + 1;
    }
    if (g_nearby_distance > 0)
      create_nearby(g_this_sequence, g_nearby_distance);
    glutSetWindow(win3);
    glutPostRedisplay();
    fix_cursor();
    show_align();
  }
  return;
}

void set_align_font(int font) {
  if (font > 2)
    g_font_align = GLUT_BITMAP_HELVETICA_18;
  else
    g_font_align = GLUT_BITMAP_HELVETICA_12;
  glutSetWindow(win4);
  glutPostRedisplay();
}


static void key_align(int key, int x, int y) {
  switch (key) {
  case GLUT_KEY_LEFT:
    g_align_start_base -= 10;
    if (g_align_start_base < 1)
      g_align_start_base = 1;
    show_align();
    break;
  case GLUT_KEY_RIGHT:
    g_align_start_base += 10;
    if (g_align_start_base > g_sequence_length)
      g_align_start_base = g_sequence_length;
    show_align();
    break;
  case GLUT_KEY_UP:
    g_align_start_base += 50;
    if (g_align_start_base > g_sequence_length)
      g_align_start_base = g_sequence_length;
    show_align();
    break;
  case GLUT_KEY_DOWN:
    g_align_start_base -= 50;
    if (g_align_start_base < 1)
      g_align_start_base = 1;
    show_align();
    break;
  case GLUT_KEY_PAGE_DOWN:
    g_align_start_base -= 1000;
    if (g_align_start_base < 1)
      g_align_start_base = 1;
    show_align();
    break;
  case GLUT_KEY_PAGE_UP:
    g_align_start_base += 1000;
    if (g_align_start_base > g_sequence_length)
      g_align_start_base = g_sequence_length;
    show_align();
    break;
  default:
    return;
  }
}

void create_align_menu(void) {
  int             font_submenu;
  int             alignment_submenu;
  font_submenu = glutCreateMenu(set_align_font);
  glutAddMenuEntry("Large", 5);
  glutAddMenuEntry("Normal", 0);
  alignment_submenu = glutCreateMenu(make_alignment_file);
  glutAddMenuEntry(".ps", 1);
  glutAddMenuEntry(".mal", 2);
  g_main_menu[4] = glutCreateMenu(top_align_menu);
  glutAddMenuEntry("Undo ", 8);
  glutAddSubMenu("Font Size", font_submenu);
  glutAddMenuEntry("Start", 2);
  glutAddMenuEntry("End", 7);
  glutAddMenuEntry("Zoom Vertical", 3);
  glutAddMenuEntry("Zoom Horizontal", 4);
  glutAddMenuEntry("Selected 5 prime", 5);
  glutAddMenuEntry("Selected 3 prime", 6);
  glutAddMenuEntry("Toggle Base Colors", 10);
  glutAddSubMenu("Alignment File", alignment_submenu);
  glutAddMenuEntry("Exit", 1);
  glutAttachMenu(GLUT_RIGHT_BUTTON);
}

void make_menu(int window) {
  int   i;
  int   chain_len_submenu, postscript_submenu;
  int   clean_submenu;
  int   makeprimary_submenu;
  int   makethis_submenu;
  int   neighbor_submenu, nearby_submenu, energy_cutoff_submenu;
  int   img_submenu, png_submenu, jpg_submenu, label_submenu;
  int   comdump_main_submenu, comdump_zoom_submenu;
  int   comdump_p_main_submenu;
  int   comdump_p_zoom_submenu;
  int   adjust_count_submenu;
  int   ann_submenu;
  int   alignment_submenu;
  int   create_structure_submenu;
  int   create_structure2_submenu;
  int   comdump_submenu;
  int   comdump_p_submenu;
  char  text[75];
  glClearColor(g_color[COLOR_BACK][0],
	       g_color[COLOR_BACK][1],
	       g_color[COLOR_BACK][2], 1.0);
  create_structure_submenu = glutCreateMenu(create_structure);
  for (i = 2; i <= g_last_used_sequence; i++) {
    glutAddMenuEntry(g_sequence_name[i], i);
  }
  create_structure2_submenu = glutCreateMenu(create_structure2);
  if (g_last_used_sequence > 2)
    glutAddMenuEntry("All ", -10);
  for (i = 2; i <= g_last_used_sequence; i++) {
    glutAddMenuEntry(g_sequence_name[i], i);
  }
  makethis_submenu = glutCreateMenu(make_this_sequence);
  for (i = 2; i <= g_last_used_sequence; i++) {
    glutAddMenuEntry(g_sequence_name[i], i - 1);
  }
  makeprimary_submenu = glutCreateMenu(make_switch_function);
  for (i = 2; i <= g_last_used_sequence; i++) {
    glutAddMenuEntry(g_sequence_name[i], i - 1);
  }
  comdump_main_submenu = glutCreateMenu(create_comdump);
  glutAddMenuEntry("Compensate Only", 1);
  glutAddMenuEntry("All Base pairs", 2);
  comdump_zoom_submenu = glutCreateMenu(create_comdump);
  glutAddMenuEntry("Compensate Only", 3);
  glutAddMenuEntry("All Base pairs", 4);
  comdump_p_main_submenu = glutCreateMenu(create_p_comdump);
  glutAddMenuEntry("Compensate Only", 1);
  glutAddMenuEntry("All Base pairs", 2);
  comdump_p_zoom_submenu = glutCreateMenu(create_p_comdump);
  glutAddMenuEntry("Compensate Only", 3);
  glutAddMenuEntry("All Base pairs", 4);
  comdump_submenu = glutCreateMenu(NoFunction);
  glutAddSubMenu("For Main Window", comdump_main_submenu);
  glutAddSubMenu("For Zoom Window", comdump_zoom_submenu);
  comdump_p_submenu = glutCreateMenu(NoFunction);
  glutAddSubMenu("For Main Window", comdump_p_main_submenu);
  glutAddSubMenu("For Zoom Window", comdump_p_zoom_submenu);
  clean_submenu = glutCreateMenu(clean_function);
  glutAddMenuEntry("No Base", 1);
  glutAddMenuEntry("Single Base", 3);
  postscript_submenu = glutCreateMenu(create_post);
  glutAddMenuEntry(".ps for Main Window", 0);
  glutAddMenuEntry("Alt .ps for Main Window", 1);
  glutAddMenuEntry(".ps for Zoom Window", 2);
  glutAddMenuEntry("Alt .ps for Zoom Window", 3);
  alignment_submenu = glutCreateMenu(make_alignment_file);
  glutAddMenuEntry(".ps", 1);
  glutAddMenuEntry(".mal", 2);
  adjust_count_submenu = glutCreateMenu(adjust_count_function);
  for (i = 1; i <= g_count_sequences; i++) {
    strcpy(text, num_string_int(i));
    glutAddMenuEntry(text, i);
  }
  if (g_prob == TRUE) {
    energy_cutoff_submenu = glutCreateMenu(set_energy_cutoff1);
    glutAddMenuEntry("include all ", 0);
    glutAddMenuEntry("include probabilities >= .001", 1);
    glutAddMenuEntry("include probabilities >= .010", 2);
    glutAddMenuEntry("include probabilities >= .100", 3);
    glutAddMenuEntry("include probabilities >= .350", 4);
    glutAddMenuEntry("include probabilities >= .650", 5);
    glutAddMenuEntry("include probabilities >= .900", 6);
    glutAddMenuEntry("include probabilities >= .990", 7);
  } else {
    energy_cutoff_submenu = glutCreateMenu(set_energy_cutoff2);
    strcpy(text, "For all");
    glutAddMenuEntry(text, 0);
    for (i = 2; i <= g_last_used_sequence; i++) {
      strcpy(text, "For ");
      strcat(text, g_sequence_name[i]);
      glutAddMenuEntry(text, i);
    }
  }
  chain_len_submenu = glutCreateMenu(set_chain_len);
  glutAddMenuEntry("All", 1);
  glutAddMenuEntry("Helices >=2", 2);
  glutAddMenuEntry("Helices >=3", 3);
  glutAddMenuEntry("Helices >=4", 4);
  glutAddMenuEntry("Helices >=5", 5);
  glutAddMenuEntry("Helices >=6", 6);
  glutAddMenuEntry("Helices >=7", 7);
  glutAddMenuEntry("Helices >=8", 8);
  glutAddMenuEntry("Helices >=9", 9);
  glutAddMenuEntry("Helices >=10", 10);
  label_submenu = glutCreateMenu(label_menu_function);
  glutAddMenuEntry("Detailed List of all", 0);
  glutAddMenuEntry("File for each sequence", 1);
  neighbor_submenu = glutCreateMenu(neighbor_menu_function);
  glutAddMenuEntry("Within 0", 0);
  glutAddMenuEntry("Within 1", 1);
  glutAddMenuEntry("Within 2", 2);
  glutAddMenuEntry("Within 3", 3);
  glutAddMenuEntry("Within 4", 4);
  glutAddMenuEntry("Within 5", 5);
  glutAddMenuEntry("Within 6", 6);
  glutAddMenuEntry("Within 7", 7);
  glutAddMenuEntry("Within 8", 8);
  glutAddMenuEntry("Within 9", 9);
  glutAddMenuEntry("Within 10", 10);
  glutAddMenuEntry("Off", -1);
  nearby_submenu = glutCreateMenu(nearby_menu_function);
  glutAddMenuEntry(" 1", 1);
  glutAddMenuEntry(" 2", 2);
  glutAddMenuEntry(" 3", 3);
  glutAddMenuEntry(" 4", 4);
  glutAddMenuEntry(" 5", 5);
  glutAddMenuEntry(" 6", 6);
  glutAddMenuEntry(" 7", 7);
  glutAddMenuEntry(" 8", 8);
  glutAddMenuEntry(" 9", 9);
  glutAddMenuEntry(" 10", 10);
  glutAddMenuEntry(" Off", -1);
  img_submenu = glutCreateMenu(img_menu);
  glutAddMenuEntry("612 x 792 ", 1);
  glutAddMenuEntry("935 x 1210", 2);
  glutAddMenuEntry("1700 x 2200", 3);
  glutAddMenuEntry("2550 x 3300", 4);
  png_submenu = glutCreateMenu(png_menu);
  glutAddMenuEntry("612 x 792 ", 1);
  glutAddMenuEntry("935 x 1210", 2);
  glutAddMenuEntry("1700 x 2200", 3);
  glutAddMenuEntry("2550 x 3300", 4);
  jpg_submenu = glutCreateMenu(jpg_menu);
  glutAddMenuEntry("612 x 792 ", 1);
  glutAddMenuEntry("935 x 1210", 2);
  glutAddMenuEntry("1700 x 2200", 3);
  glutAddMenuEntry("2550 x 3300", 4);
  ann_submenu = glutCreateMenu(ann_menu);
  glutAddMenuEntry("pnum", 1);
  if (g_prob)
    glutAddMenuEntry("prob", 2);
  glutAddMenuEntry("Off", 0);
  g_main_menu[window] = glutCreateMenu(No_Action_Menu);
  glutAddMenuEntry("Undo ", 8);
  if (window == win2)
    glutAddMenuEntry("UnZoom ", 7);
  glutAddMenuEntry("Save", 11);
  glutAddSubMenu("Filter", chain_len_submenu);
  glutAddSubMenu("Cutoff", energy_cutoff_submenu);
  if ((window == win2) && (g_count_sequences > 2))
    glutAddMenuEntry("Align", 51);
  if (g_count_sequences > 1) {
    glutAddSubMenu("Close Within ", nearby_submenu);
    glutAddSubMenu("View sequences", adjust_count_submenu);
    glutAddMenuEntry("Comp. B.P. On/Off", 6);
    if (g_count_sequences > 2)
      glutAddMenuEntry("Partial Overlap On/Off", 14);
    glutAddMenuEntry("Unique On/Off", 4);
  }
  glutAddSubMenu("Overlap Annotation", ann_submenu);
  glutAddMenuEntry("Enter Zoom Coordinates", 12);
  if (window == win2) {
    glutAddMenuEntry("-------------", 35);
    if (g_count_sequences > 1) {
      glutAddSubMenu("Make Primary", makeprimary_submenu);
      glutAddSubMenu("Adjust ", makethis_submenu);
    }
    if (g_count_sequences > 2) {
      glutAddMenuEntry("----------", 36);
    }
    glutAddMenuEntry("Back to Original", 9);
    glutAddMenuEntry("Back to Saved", 13);
    if (g_count_sequences > 1)
      glutAddSubMenu("Clean Inserts", clean_submenu);
    glutAddMenuEntry("-------------", 36);
    glutAddSubMenu("Selected BP File", label_submenu);
#if (defined(HAVE_LIBGD) || defined(HAVE_GDIMAGECREATE)) && defined(HAVE_GD_H)
    glutAddSubMenu("GIF File", img_submenu);
#endif
#if HAVE_GDIMAGEPNG
    glutAddSubMenu("PNG File", png_submenu);
#endif
#if HAVE_GDIMAGEJPEG
    glutAddSubMenu("JPG File", jpg_submenu);
#endif
  }
  glutAddSubMenu("Postscript File", postscript_submenu);
  glutAddSubMenu("Common Dump Files: Forced", comdump_submenu);
  glutAddSubMenu("Common Dump Files: Plot", comdump_p_submenu);
  glutAddSubMenu("Alignment File", alignment_submenu);
  glutAddSubMenu("Neighbor File", neighbor_submenu);
  glutAddMenuEntry("Grid lines On/Off", 5);
  glutAddMenuEntry("Big Dot On/Off", 10);
  glutAddMenuEntry("Upper/Lower Triangle", 64);
  if (window == win2)
    glutAddMenuEntry("Toggle Base Colors", 52);
  /* glutAddSubMenu("Font",font_submenu); */
  glutAddMenuEntry("Pop", 2);
  if (window == win2) {
    glutAddSubMenu("Draw Structure .sav", create_structure_submenu);
    glutAddSubMenu("Draw Structure new", create_structure2_submenu);
  }
  glutAddMenuEntry("Exit", 1);
  glutAttachMenu(GLUT_RIGHT_BUTTON);
}

char * fix_time(char *time_string) {
  time_t          lt;
  int             k;
  struct tm      *ptr;
  lt = time(NULL);
  ptr = localtime(&lt);
  k = (*ptr).tm_hour;
  strcpy(time_string, "  ");
  strcat(time_string, num_string_int2(k));
  strcat(time_string, ":");
  k = (*ptr).tm_min;
  strcat(time_string, num_string_int2(k));
  strcat(time_string, ":");
  k = (*ptr).tm_sec;
  strcat(time_string, num_string_int2(k));
  strcat(time_string, "  ");
  return time_string;
}

void gfxinit_nontext(int window) { /* initialize graphics for each window */
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutReshapeFunc(reshape);
  make_menu(window);
}
#endif

int main(int argc, char **argv) {
#ifdef HAVE_GLUTBITMAPCHARACTER
  char            data_filename[120];
  int             i;
  g_first_run = TRUE;
  initialize_colors();
  g_mouse_motion = FALSE;
  g_level_undos = 1;
  g_total_undos = 1;
  g_count_neighbors = 0;
  g_count_nearby = 0;
  g_nearby_distance = 0;
  g_dot_lower_flag = TRUE;
  g_type_of_dump = PLOT;
  g_ps_multicolor_dots_flag = FALSE;
  first_initialize();
  /* main_initialize will read alignment file, plot files */
  /* stores helices in memory according to alignment file */
  initialize_data2();	/* sets graphic global variables */
  /* set label count to zero, sets energy cutoff */
  glutInit(&argc, argv);
  for (i = 1; i < argc; i++) {
    if (!strcmp(argv[i], "-mono")) {
      g_font_main = GLUT_BITMAP_HELVETICA_18;
      g_font_text = GLUT_BITMAP_HELVETICA_18;
      g_font_zoom = GLUT_BITMAP_HELVETICA_18;
    }
  }
  glutInitWindowSize(750, 750);
  glutInitWindowPosition(6, 262);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
  strcpy(data_filename, PROGRAM_NAME" zoom for ");
  strcat(data_filename, g_in_align_filename);
  win2 = glutCreateWindow(data_filename);
  gfxinit_nontext(win2);	/* initialize graphics for zoom window */
  glutDisplayFunc(display_zoom);
  glutMouseFunc(mouse_zoom);
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutMotionFunc(mouse_motion_zoom);
  glutSpecialFunc(Key2);
  width_z = 750;
  height_z = 750;
  glutHideWindow();
  /* main window */
  glutInitWindowSize(800, 800);
  glutEntryFunc(zoom_entry);
  glutInitWindowPosition(474, 218);
  strcpy(data_filename, PROGRAM_NAME" for ");
  strcat(data_filename, g_in_align_filename);
  win1 = glutCreateWindow(data_filename);
  glutMouseFunc(mouse);
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutMotionFunc(mouse_motion);
  glutSpecialFunc(Key2_fast);
  gfxinit_nontext(win1);	/* initialize graphics for Main Window */
  glutDisplayFunc(display_main);
  glutEntryFunc(main_entry);
  /* Text window */
  glutInitWindowSize(880, 200);
  glutInitWindowPosition(6, 31);
  strcpy(data_filename, PROGRAM_NAME" Info. for ");
  strcat(data_filename, g_in_align_filename);
  win3 = glutCreateWindow(data_filename);
  make_menu(win3);
  glutDisplayFunc(display_text);
  glutReshapeFunc(text_reshape);
  glutKeyboardFunc(keyboard);
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutEntryFunc(text_entry);
  /* create alignment window */
  g_align_start_base = 1;
  glutInitWindowSize(750, 200);
  g_align_width = 750;
  g_align_height = 50 + g_last_used_sequence * 15;
  glutInitWindowPosition(6, 262);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
  strcpy(data_filename, "Alignment for ");
  strcat(data_filename, g_in_align_filename);
  win4 = glutCreateWindow(data_filename);
  glutMouseFunc(mouse_align);
  glutMotionFunc(mouse_motion_align);
  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
  glutSpecialFunc(key_align);
  glutDisplayFunc(display_align);
  glutReshapeFunc(reshape_align);
  glClearColor(g_color[COLOR_BACK][0], g_color[COLOR_BACK][1],
	       g_color[COLOR_BACK][2], 1.0);
  create_align_menu();
  /* end of create alignment window */
  glutSetWindow(win4);
  glutPopWindow();
  glutMainLoop();
#endif
  printf("%s not available because glut was not found.\n", PROGRAM_NAME);
  try_exit(3);
  return 0; /* Keep the stupid compiler happy. */
}
